/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact:
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.openrdf.sesame.query;

import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.openrdf.model.Graph;
import org.openrdf.model.Resource;
import org.openrdf.model.URI;
import org.openrdf.model.Value;
import org.openrdf.model.impl.GraphImpl;

import org.openrdf.sesame.sail.RdfRepository;
import org.openrdf.sesame.sail.SailInitializationException;
import org.openrdf.sesame.sail.SailUpdateException;

/**
 * An implementation of <tt>GraphQueryResultListener</tt> that can be used to
 * create a Graph from a query result.
 * 
 * @author Jeen Broekstra
 * @author Arjohn Kampman
 * @version $Revision: 1.4 $
 */
public class QueryResultsGraphBuilder implements GraphQueryResultListener {

/*----------+
| Variables |
+----------*/

	private RdfRepository _repository;
	private Map _namespacesTable;
	private Graph _graph;

/*-------------+
| Constructors |
+-------------*/

	/**
	 * Creates a new QueryResultsGraphBuilder.
	 **/
	public QueryResultsGraphBuilder() {
		_namespacesTable = new HashMap(16);
	}

/*--------+
| Methods |
+--------*/

	/**
	 * Gets the graph that has been build from the query result.
	 * 
	 * @return A <tt>Graph</tt> object, or <tt>null</tt> if the query result has
	 * not yet been processed.
	 */
	public Graph getGraph() {
		return _graph;
	}

	// implements GraphQueryResultListener.startGraphQueryResult()
	public void startGraphQueryResult()
		throws IOException
	{
		_graph = null;
		_repository = new org.openrdf.sesame.sailimpl.memory.RdfRepository();

		try {
			_repository.initialize(Collections.EMPTY_MAP);
		}
		catch (SailInitializationException e) {
			IOException ioe = new IOException("Unable to initialize memory sail");
			ioe.initCause(e);
			throw ioe;
		}

		_repository.startTransaction();
	}

	// implements GraphQueryResultListener.endGraphQueryResult()
	public void endGraphQueryResult() {
		Iterator iter = _namespacesTable.keySet().iterator();
		while (iter.hasNext()) {
			String namespace = (String)iter.next();
			String prefix = (String)_namespacesTable.get(namespace);
			try {
				_repository.changeNamespacePrefix(namespace, prefix);
			}
			catch (SailUpdateException e) {
				// ignore
			}
		}
		_namespacesTable.clear();

		_repository.commitTransaction();

		_graph = new GraphImpl(_repository);
	}

	// implements GraphQueryResultListener.namespace(String,String)
	public void namespace(String prefix, String name) {
		_namespacesTable.put(name, prefix);
	}

	// implements GraphQueryResultListener.triple(...)
	public void triple(Resource subj, URI pred, Value obj)
		throws IOException
	{
		try {
			_repository.addStatement(subj, pred, obj);
		}
		catch (SailUpdateException e) {
			IOException ioe = new IOException("Unable to add statement to memory sail");
			ioe.initCause(e);
			throw ioe;
		}
	}

	// implements GraphQueryResultListener.reportError(String)
	public void reportError(String msg)
		throws IOException
	{
		//FIXME do nothing
		//throw new QueryEvaluationException(msg);
	}
}
