/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2001-2005 Aduna
 *
 *  Contact: 
 *  	Aduna
 *  	Prinses Julianaplein 14 b
 *  	3817 CS Amersfoort
 *  	The Netherlands
 *  	tel. +33 (0)33 465 99 87
 *  	fax. +33 (0)33 465 99 87
 *
 *  	http://aduna.biz/
 *  	http://www.openrdf.org/
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.sail.query;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.openrdf.sesame.sail.RdfSource;

/**
 * A list of path expressions that can have a number of child lists
 * and boolean constraints.
 *
 * @author Arjohn Kampman
 */
public class PathExpressionList implements PathExpression {

/*------------------------+
| Variables               |
+------------------------*/

	/**
	 * A List of PathExpression objects indicating the paths in the
	 * RDF graph that are relevant for the query.
	 **/
	private List _pathExpressions;

	/**
	 * Boolean constraints on the (variable) values that were found
	 * as matching to the path expressions.
	 **/
	protected BooleanExpr _constraints;

	/**
	 * A mixed list of PathExpression and BooleanExpr objects that is
	 * used for query evaluation. Verification of parts of the boolean
	 * constraints can be done before all path expressions have been
	 * instantiated. The order of objects in this list indicate when
	 * this verification takes place.
	 **/
	protected List _expressions;

	/**
	 * A List of PathExpressionList objects, to be interpreted as
	 * optional path expressions.
	 **/
	private List _childLists;

	private boolean _firstCall;
	private boolean _exhausted;

/*------------------------+
| Constructors            |
+------------------------*/

	/**
	 * Creates a new PathExpressionList.
	 **/
	public PathExpressionList() {
		_pathExpressions = new ArrayList();
		_childLists = new ArrayList();
		_expressions = new ArrayList();
	}

/*------------------------+
| Methods                 |
+------------------------*/

	/**
	 * Adds a collection of PathExpression objects to this object.
	 **/
	public void addAll(Collection triplePatterns) {
		Iterator iter = triplePatterns.iterator();
		while (iter.hasNext()) {
			PathExpression pe = (PathExpression)iter.next();
			add(pe);
		}
	}

	/**
	 * Adds a path expression to the this object.
	 **/
	public void add(PathExpression pathExpr) {
		_pathExpressions.add(pathExpr);
	}

	/**
	 * Gets a list of path expressions contained in this object.
	 * @return An unmodifiable List of PathExpression objects.
	 **/
	public List getPathExpressions() {
		return Collections.unmodifiableList(_pathExpressions);
	}

	/**
	 * Sets the list of path expressions for this object.
	 **/
	public void setPathExpressions(List pathExpressions) {
		_pathExpressions.clear();
		_pathExpressions.addAll(pathExpressions);
	}

	/**
	 * Gets a list of path expressions contained in this object
	 * and all of its child lists recursively.
	 * @return An unmodifiable List of PathExpression objects.
	 **/
	public List getPathExpressionsRecursively() {
		List result = new ArrayList(_pathExpressions);

		Iterator iter = _childLists.iterator();
		while (iter.hasNext()) {
			PathExpressionList childList = (PathExpressionList)iter.next();
			result.addAll(childList.getPathExpressionsRecursively());
		}

		return Collections.unmodifiableList(result);
	}

	public BooleanExpr getConstraints() {
		return _constraints;
	}

	public void setConstraints(BooleanExpr constraints) {
		_constraints = constraints;
	}

	/**
	 * Gets the constraints that are joined using AND operators from the root
	 * constraint as a list of separate constraints. If the root constraint is
	 * not an AND operator, the list only contains the root constraint itself.
	 * If there are no constraints at all, an empty list will be returned.
	 *
	 * @return A List containing BooleanExpr objects that, when applied conjunctively,
	 * result in the same constraint as the root constraint.
	 **/
	public List getConjunctiveConstraints() {
		List result = new ArrayList();
		if (_constraints != null) {
			_getConjunctiveConstraints(_constraints, result);
		}
		return result;
	}

	protected void _getConjunctiveConstraints(BooleanExpr booleanExpr, List constraints) {
		if (booleanExpr instanceof And) {
			And and = (And)booleanExpr;
			_getConjunctiveConstraints(and.getLeftArg(), constraints);
			_getConjunctiveConstraints(and.getRightArg(), constraints);
		}
		else {
			constraints.add(booleanExpr);
		}
	}

	public void setConjunctiveConstraints(List conjunctiveConstraints) {
		BooleanExpr newConstraints = null;

		if (!conjunctiveConstraints.isEmpty()) {
			newConstraints = (BooleanExpr)conjunctiveConstraints.get(0);

			for (int i = 1; i < conjunctiveConstraints.size(); i++) {
				BooleanExpr rightArg = (BooleanExpr)conjunctiveConstraints.get(i);
				newConstraints = new And(newConstraints, rightArg);
			}
		}

		setConstraints(newConstraints);
	}

	public List getExpressions() {
		return Collections.unmodifiableList(_expressions);
	}

	public void setExpressions(List expressions) {
		_expressions.clear();
		_expressions.addAll(expressions);
	}

	public boolean findFirst(RdfSource rdfSource) 
		throws SailQueryException
	{
		initialize(rdfSource);
		return _findNext(rdfSource, true);
	}

	public boolean findNext(RdfSource rdfSource) 
		throws SailQueryException
	{
		return _findNext(rdfSource, false);
	}

	public void initialize(RdfSource rdfSource) {
		if (_expressions.size() == 0 && _pathExpressions.size() > 0) {
			_expressions.addAll(_pathExpressions);
			_expressions.addAll(_childLists);

			if (_constraints != null) {
				_expressions.add(_constraints);
			}
		}

		_firstCall = true;
		_exhausted = false;
	}

	public boolean selectNext(RdfSource rdfSource) 
		throws SailQueryException
	{
		if (_exhausted) {
			return false;
		}

		boolean result;

		if (_firstCall) {
			_firstCall = false;

			result = _findNext(rdfSource, true);

			if (result == false) {
				_exhausted = true;
				result = true;
			}
		}
		else {
			result = _findNext(rdfSource, false);

			if (result == false) {
				_exhausted = true;
			}
		}

		return result;
	}

	/**
	 * Clears the internal state of this object.
	 **/
	public void clear() {
		for (int i = _expressions.size() - 1; i >= 0; i--) {
			Object expression = _expressions.get(i);
			if (expression instanceof PathExpression) {
				((PathExpression)expression).clear();
			}
		}
	}

	public void getLocalVariables(Collection variables) {
		Iterator iter = _pathExpressions.iterator();
		while (iter.hasNext()) {
			PathExpression pe = (PathExpression)iter.next();
			pe.getVariables(variables);
		}
	}

	public void getVariables(Collection variables) {
		getLocalVariables(variables);

		Iterator iter = _childLists.iterator();
		while (iter.hasNext()) {
			PathExpressionList pel = (PathExpressionList)iter.next();
			pel.getVariables(variables);
		}
	}

	/**
	 * Finds the next result. If 'firstCall' is true, then this method will
	 * start with the initialization of the first path expression.
	 * Otherwise, this method will assume that all path expressions have
	 * been initialization before and will continue with the last path
	 * expression.
	 *
	 * @return true If a next result has been found, false otherwise.
	 **/
	private boolean _findNext(RdfSource rdfSource, boolean firstCall) 
		throws SailQueryException
	{
		int lastIndex = _expressions.size() - 1;
		int index;
		Object expr = null;
		PathExpression pathExpr = null;
		BooleanExpr boolExpr = null;

		if (firstCall) {
			// Start at first expression
			index = 0;
			if (_expressions.size() == 0) {
				// no path expressions
				return true;
			}
			expr = _expressions.get(index);
			if (expr instanceof PathExpression) {
				((PathExpression)expr).initialize(rdfSource);
			}
		}
		else {
			// Continue with last expression
			index = lastIndex;
			
			if (_expressions.size() == 0) {
				// no path expressions
				return false;
			}
			expr = _expressions.get(index);

			// Boolean expressions don't have a state and shouldn't be
			// re-evaluated. Go back in the recursion until we come
			// accross a PathExpression
			while (expr instanceof BooleanExpr && index > 0) {
				index--;
				expr = _expressions.get(index);
			}

			if (expr instanceof BooleanExpr) {
				// No PathExpression has been found
				return false;
			}
		}

		while (true) {
			// Check expression at index 'index'
			boolean success = false;

			if (expr instanceof PathExpression) {
				pathExpr = (PathExpression)expr;
				success = pathExpr.selectNext(rdfSource);
			}
			else if (expr instanceof BooleanExpr) {
				// BooleanExpr
				boolExpr = (BooleanExpr)expr;
				success = boolExpr.isTrue(rdfSource);
			}
			else {
				throw new RuntimeException("Unknown expression type: " +  expr.getClass().toString());
			}

			if (success) {
				// Expression evaluated successfully

				if (index == lastIndex) {
					// All expressions evaluated successfully
					return true;
				}
				else {
					// Evaluate next expression
					index++;
					expr = _expressions.get(index);
					if (expr instanceof PathExpression) {
						((PathExpression)expr).initialize(rdfSource);
					}
				}
			}
			else {
				// Expression evaluation failed
				if (expr instanceof PathExpression) {
					pathExpr.clear();
				}

				if (index == 0) {
					// First PathExpression exhausted, no more results
					return false;
				}
				else {
					index--;
					expr = _expressions.get(index);

					// Boolean expressions don't have a state and shouldn't be
					// re-evaluated. Go back in the recursion until we come
					// accross a PathExpression
					while (expr instanceof BooleanExpr && index > 0) {
						index--;
						expr = _expressions.get(index);
					}

					if (expr instanceof BooleanExpr) {
						// No PathExpression has been found
						return false;
					}
				}
			}
		}
	}

	/**
	 * Adds a child list to this object.
	 **/
	public void addChildList(PathExpressionList childList) {
		_childLists.add(childList);
	}

	/**
	 * Gets a list of child expression lists.
	 * @return An unmodifiable List of PathExpressionList objects.
	 **/
	public List getChildLists() {
		return Collections.unmodifiableList(_childLists);
	}

	/**
	 * Sets the list of child lists for this object.
	 **/
	public void setChildLists(List childLists) {
		_childLists.clear();
		_childLists.addAll(childLists);
	}

	public String toString() {
		List expressions = _expressions;

		if (expressions.size() == 0) {
			expressions = new ArrayList();

			expressions.addAll(_pathExpressions);
			expressions.addAll(_childLists);
			if (_constraints != null) {
				expressions.add(_constraints);
			}
		}

		String result = "[\n";

		Iterator iter = expressions.iterator();
		while (iter.hasNext()) {
			Object expr = iter.next();
			result += "\t" + expr.toString();

			if (iter.hasNext()) {
				result += ",\n";
			}
		}
		result += "\n]";

		return result;
	}
}
