/*  Sesame - Storage and Querying architecture for RDF and RDF Schema
 *  Copyright (C) 2003 OntoText Lab, Sirma AI OOD
 *
 *  Contact:
 *	Sirma AI OOD, OntoText Lab.
 *	38A, Christo Botev Blvd.
 *  1000 Sofia, Bulgaria
 *	tel. +359(2)981 00 18
 *	fax. +359(2)981 90 58
 *	info@ontotext.com
 *
 * 	http://www.ontotext.com/
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.openrdf.sesame.sailimpl.rdbms.rules;

import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Iterator;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import org.openrdf.util.xml.XMLReaderFactory;

/**
 *
 * <p>Title: CustomInference </p>
 * <p>Description: parse an XML inference configuration file</p>
 * <p>Company: Ontotext Lab.</p>
 * @author Damyan Ognyanoff
 * @version 1.0
 */
public class RuleParser implements ContentHandler {

	public static final String TAG_AXIOM = "axiom";
	public static final String TAG_RULE = "rule";
	public static final String ATTR_NAME = "name";
	public static final String TAG_TRIPLE = "premise";
	public static final String TAG_INFER = "consequent";
	public static final String TAG_S = "subject";
	public static final String TAG_P = "predicate";
	public static final String TAG_O = "object";
	public static final String TAG_TRIGGERS_RULE =	"triggers_rule";
	public static final String ATTR_VAR = "var";
	public static final String ATTR_URI = "uri";
	public static final String ATTR_TYPE = "type";
	public static final String ATTR_REGEXP = "pattern";
	public static final String ATTR_ESCAPE = "escape";

	public RuleParser() {
	}

	public void load(String filename) 
		throws IOException, SAXException
	{
		FileReader reader = null;
		reader = new FileReader(filename);
		try {
			parse(reader);
		}
		finally {
			reader.close();
		}
	}

	public void parse(Reader r)
		throws IOException, SAXException
	{
		InputSource isrc = new InputSource(r);

		// Try to create a SAX parser
		XMLReader reader = XMLReaderFactory.createXMLReader();
		reader.setContentHandler(this);
		reader.parse(isrc);
	}

	Locator locator = null;

	public void setDocumentLocator(Locator locator) {
		this.locator = locator;
	}

	ArrayList rules = null;
	ArrayList axioms = null;

	public void startDocument()
		throws SAXException
	{
		rules = new ArrayList();
		axioms = new ArrayList();
		parseMode = 0;
	}

	public void endDocument()
		throws SAXException
	{
	}

	Rule rule = null;
	TripleTemplate tripleTempl = null;
	int tripleCount;
	int inferCount;
	int sCount;
	int pCount;
	int oCount;
	int parseMode;

	public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
		throws SAXException
	{
		if (parseMode == 1 && 0==localName.compareTo(TAG_RULE)) { // in <triggered_by>
			String ruleName = atts.getValue(ATTR_NAME);
			rule.triggersRule(ruleName);
			return;
		}
		if (0==parseMode && 0==localName.compareTo(TAG_TRIGGERS_RULE)) {
			if (rule == null) {
				throw new SAXException(
						"ERROR: '"+TAG_TRIGGERS_RULE+"' is allowed only within '"+TAG_RULE+
						"' tag! Context:("+locator.getLineNumber()+","+locator.getColumnNumber()+")");
			}
			parseMode = 1;
			rule.initTriggersRuleCollection();
		}

		if (1 != parseMode && 0==localName.compareTo(TAG_RULE)) {
			String ruleName=atts.getValue(ATTR_NAME);
			rule = new Rule(ruleName);
			tripleCount=inferCount=sCount=pCount=oCount=0;
			return;
		}

		if (0==localName.compareTo(TAG_AXIOM)) {
			tripleTempl = new TripleTemplate();
			sCount=pCount=oCount=0;
			parseMode = 3; // force subject,object and predicate tags with 'uri' attribute only
			return;
		}
		if (0==localName.compareTo(TAG_TRIPLE)) {
			tripleTempl = new TripleTemplate();
			sCount=pCount=oCount=0;
			tripleCount++;
			return;
		}
		if (0==localName.compareTo(TAG_INFER)) {
			tripleTempl = new TripleTemplate();
			rule.infer = tripleTempl;
			sCount=pCount=oCount=0;
			inferCount++;
			return;
		}
		if (0==localName.compareTo(TAG_S)) {
			sCount++;
			tripleTempl.subject = createComponent(atts, parseMode==3);
			return;
		}
		if (0==localName.compareTo(TAG_P)) {
			pCount++;
			tripleTempl.predicate = createComponent(atts, parseMode==3);
			return;
		}
		if (0==localName.compareTo(TAG_O)) {
			oCount++;
			tripleTempl.object = createComponent(atts, parseMode==3);
			return;
		}
	}

	public void endElement(String namespaceURI, String localName, String qName)
		throws SAXException
	{
		if (0==localName.compareTo(TAG_TRIGGERS_RULE)) {
			if (parseMode != 1) {
				throw new SAXException(
						"ERROR: Unexpected closing of '"+TAG_TRIGGERS_RULE+
						"' tag! Context:("+locator.getLineNumber()+","+locator.getColumnNumber()+")");
			}
			parseMode = 0;
			return;
		}

		if (0==parseMode && 0==localName.compareTo(TAG_RULE)) {
			if (tripleCount < 1) {
				throw new SAXException(
						"ERROR: at least 1 '"+TAG_TRIPLE+"' element is expected under the '"+TAG_RULE+
						"' tag! Context:("+locator.getLineNumber()+","+locator.getColumnNumber()+")");
			}
			if (inferCount != 1) {
				throw new SAXException(
						"ERROR: exactly 1 '"+TAG_INFER+"' element is expected under '"+TAG_RULE+
						"' tag! Context:("+locator.getLineNumber()+","+locator.getColumnNumber()+")");
			}
			rules.add(rule);
			rule = null;
			return;
		}
		if (0==localName.compareTo(TAG_AXIOM)) {
			if (sCount != 1) {
				throw new SAXException(
						"ERROR: exactly 1 '"+TAG_S+"' element is expected under '"+localName+
						"' tag! Context:("+locator.getLineNumber()+","+locator.getColumnNumber()+")");
			}
			if (pCount != 1) {
				throw new SAXException(
						"ERROR: exactly 1 '"+TAG_P+"' element is expected under '"+localName+
						"' tag! Context:("+locator.getLineNumber()+","+locator.getColumnNumber()+")");
			}
			if (oCount != 1) {
				throw new SAXException(
						"ERROR: exactly 1 '"+TAG_O+"' element is expected under '"+localName+
						"' tag! Context:("+locator.getLineNumber()+","+locator.getColumnNumber()+")");
			}

			// add the used resources to the 'requred' list so to be able to assign their respective id's later
			Rule.getId(tripleTempl.subject);
			Rule.getId(tripleTempl.predicate);
			Rule.getId(tripleTempl.object);
			axioms.add(tripleTempl);
			tripleTempl = null;
			parseMode = 0;
			return;
		}
		if (0==localName.compareTo(TAG_TRIPLE)) {
			if (sCount != 1) {
				throw new SAXException(
						"ERROR: exactly 1 '"+TAG_S+"' element is expected under '"+localName+
						"' tag! Context:("+locator.getLineNumber()+","+locator.getColumnNumber()+")");
			}
			if (pCount != 1) {
				throw new SAXException(
						"ERROR: exactly 1 '"+TAG_P+"' element is expected under '"+localName+
						"' tag! Context:("+locator.getLineNumber()+","+locator.getColumnNumber()+")");
			}
			if (oCount != 1) {
				throw new SAXException(
						"ERROR: exactly 1 '"+TAG_O+"' element is expected under '"+localName+
						"' tag! Context:("+locator.getLineNumber()+","+locator.getColumnNumber()+")");
			}
			rule.addTemplate(tripleTempl);
			tripleTempl = null;
			return;
		}
		if (0==localName.compareTo(TAG_INFER)) {
			if (sCount != 1) {
				throw new SAXException(
						"ERROR: exactly 1 '"+TAG_S+"' element is expected under '"+localName+
						"' tag! Context:("+locator.getLineNumber()+","+locator.getColumnNumber()+")");
			}
			if (pCount != 1) {
				throw new SAXException(
						"ERROR: exactly 1 '"+TAG_P+"' element is expected under '"+localName+
						"' tag! Context:("+locator.getLineNumber()+","+locator.getColumnNumber()+")");
			}
			if (oCount != 1) {
				throw new SAXException(
						"ERROR: exactly 1 '"+TAG_O+"' element is expected under '"+localName+
						"' tag! Context:("+locator.getLineNumber()+","+locator.getColumnNumber()+")");
			}
			rule.addInfer(tripleTempl);
			tripleTempl = null;
			return;
		}
		if (0==localName.compareTo(TAG_S) ||
			0==localName.compareTo(TAG_P) ||
			0==localName.compareTo(TAG_O))
		{
			return;
		}
	}
	public void characters(char[] ch, int start, int length) throws SAXException {
	}
	public void startPrefixMapping(String prefix, String uri) throws SAXException {
	}
	public void endPrefixMapping(String prefix) throws SAXException {
	}
	public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
	}
	public void processingInstruction(String target, String data) throws SAXException {
	}
	public void skippedEntity(String name) throws SAXException {
	}

	/**
	 * createComponent() - create a new component instance from the attribute list
	 * of 'subject' 'object' or 'predicate' tags.
	 * @param attr - list of attributes of the tag
	 * @param forceUriOnly - if the context forces only 'uri' attribute to appear in
	 * the supplied list
	 * @return new Component instance
	 * @throws SAXException
	 */

	Component createComponent(Attributes attr, boolean forceUriOnly) throws SAXException {
		Component result = null;
		String attrUri = attr.getValue(ATTR_URI);
		if (forceUriOnly) {
			if (attrUri == null) {
				throw new SAXException(
					"'uri' attribute is required! Context:("+locator.getLineNumber()+","+locator.getColumnNumber()+")");
			}
			result = new Component(attrUri, Component.URI, 0);
		}
		else {
			if (attrUri == null) {
				// if there is no uri attribute	- ensure that at least 'var' is present
				String var = attr.getValue(ATTR_VAR);
				if (var == null) {
					throw new SAXException(
							"'uri' or 'var' attributes are required! Context:("+locator.getLineNumber()+","+locator.getColumnNumber()+")");
				}

				// retrieve 'type' attribute value (an optional resrtiction to resources only)
				String type = attr.getValue(ATTR_TYPE);
				if (type == null) {
					result = new Component(var, Component.VAR, 0);
				}
				else {
					result = new Component(var, Component.VAR, 1);
				}
				String regexp = attr.getValue(ATTR_REGEXP);
				String escape = attr.getValue(ATTR_ESCAPE);
				if (regexp != null) {
					result.setRegExpTemplate(regexp, escape);
				}
			}
			else {
				result = new Component(attrUri, Component.URI, 0);
			}
		} // if ('uri' attribute has a value)
		return result;
	} // createComponent()

	/**
	 * matchTriples - given a consequent and a premise it simply	match them
	 * and report that the consequent can be a premise for a rule
	 * @param t1
	 * @param t2
	 * @return true if it is applicable. the arguments are compared by-component.
	 * In case of a varible both componets of the triple can be unified.
	 * In case of an explicit URIs they should be exactly the same.
	 */
	protected boolean matchTriples(TripleTemplate t1, TripleTemplate t2) {
		// special case on transitivity rules
		if (t1.predicate.isUri() && t2.predicate.isUri() &&
				0==t1.predicate.value().compareTo(t2.predicate.value()))
		{
			if (t1.subject.isVar() && t1.object.isVar() &&
					0==t1.subject.value().compareToIgnoreCase(t1.object.value()))
			{
				if (t2.subject.isVar() && t2.object.isVar() &&
						0!=t2.subject.value().compareToIgnoreCase(t2.object.value()))
				{
					return false;
				}
			}
		} // end special case

		if (!t1.subject.isVar() && !t2.subject.isVar()) {
			if (0!=t1.subject.value().compareToIgnoreCase(t2.subject.value())) {
				return false;
			}
		}
		if (!t1.predicate.isVar() && !t2.predicate.isVar()) {
			if (0!=t1.predicate.value().compareToIgnoreCase(t2.predicate.value())) {
				return false;
			}
		}
		if (!t1.object.isVar() && !t2.object.isVar()) {
			if (0!=t1.object.value().compareToIgnoreCase(t2.object.value())) {
				return false;
			}
		}
		return true;
	} //matchTriples()

	/**
	 * method buildTriggers - given a list of rules it builds the inter-rule dependancy table.
	 * e.g. If a rule inferrs some new statemnents which rules should be applied on next iteration.
	 * This method is implementation specific because it deals with several variants of a rule.
	 * @param theRules ArrayList with the Rule instances
	 * @return square array of boolean values. each row holds the flags that indicate which rules(
	 * rule variants) are trigered by this rule(rule variant)
	 */
	protected boolean[][] buildTriggers(ArrayList theRules) {
		boolean[][] result= null;
		// toral number of rules(variants)
		// it is equl to the sum of allpremices (implementation specific)
		int num_rule_variants = 0;
		for (int i=0; i < theRules.size(); i++) {
			Rule r = (Rule)theRules.get(i);
			num_rule_variants+= r.getPremiseCount();
		}
		// creates a square array of boolean values
		result = new boolean[num_rule_variants][num_rule_variants];

		// each
		int rowIndex = 0;
		for (int i=0; i < theRules.size(); i++) {
			int colIndex = 0;
			Rule activator = (Rule)theRules.get(i);

			for (int j=0; j < theRules.size(); j++) {
				Rule activated = (Rule)theRules.get(j);
				String aName = activated.getName();
				boolean inTrigersList = false;

				if (activator.getTriggersRule() != null) {
					Iterator trIter = activator.getTriggersRule().iterator();
					while (trIter.hasNext()) {
						String toCheck = (String)trIter.next();
						if (0==toCheck.compareToIgnoreCase(aName)) {
							inTrigersList = true;
							break;
						}
					} //while
				} // triggersList is not empty

				if (inTrigersList) {
					TripleTemplate consequent = activator.getConsequent();

					for (int indexActivated = 0; indexActivated < activated.getPremiseCount(); indexActivated++) {
						TripleTemplate premiceToCheck = (TripleTemplate)activated.getPremiseCollection().get(indexActivated);
						boolean valueToSet =	matchTriples(consequent, premiceToCheck);

						for (int activatorSize = 0; activatorSize < activator.getPremiseCount(); activatorSize++) {
							result[rowIndex+activatorSize][colIndex+indexActivated] = valueToSet;
						}
					}// for each premice of the rule
				}
				colIndex+=activated.getPremiseCount();
			}// for each activated rule

			rowIndex+=activator.getPremiseCount();
		} // for each activator

		return result;
	} // buildTriggers();

	public ArrayList getRules() {
		return rules;
	}

	public ArrayList getAxioms() {
		return axioms;
	}
}
