package com.mkdoc.jspider;

/*
    Created:    2004-11-26
    Issued:     2004-12-03
    Modified:   2005-11-02

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.api.event.JSpiderEvent;

import net.javacoding.jspider.api.event.resource.ResourceFetchedEvent;

import net.javacoding.jspider.core.logging.LogFactory;

import net.javacoding.jspider.core.util.config.PropertySet;

import net.javacoding.jspider.spi.Plugin;

import com.mkdoc.jtidy.TidyDriver;

import com.mkdoc.store.FileStoreManager;
import com.mkdoc.store.StoreException;
import com.mkdoc.store.StoreManagerFactory;

import com.mkdoc.util.ApplicationContext;

import java.io.InputStream;
import java.io.OutputStream;

import java.net.URL;

/**
 *  A JTidy file writer plugin for JSpider.
 *  <p>
 *    Based on <code>DiskWriterPlugin</code>, requires two
 *    configuration properties:
 *  </p>
 *  <dl>
 *    <dt>
 *      <code>manager</code>
 *    </dt>
 *    <dd>
 *      The fully qualified class name of a
 *      <code>com.mkdoc.store.FileStoreManager</code> implementation,
 *      <em>required</em>. Should normally be
 *      <code>com.mkdoc.store.HtmlFileStoreManager</code>.
 *    </dd>
 *    <dt>
 *      <code>output.absolute</code>
 *    </dt>
 *    <dd>
 *      Whether the output directory configuration
 *      specifies an absolute directory path. If not set, a
 *      relative path inStream assumed.
 *    </dd>
 *    <dt>
 *      <code>output.folder</code>
 *    </dt>
 *    <dd>
 *      The base file output directory, which may be
 *      relative to the JSpider installation directory. If
 *      not set the default <code>output</code> directory
 *      beneath the JSpider installation directory inStream
 *      assumed.
 *    </dd>
 *  </dl>
 *  @version    1.2 2005-10-21
 *  @author     Philip Shaw <phil@mkdoc.com>
 *  @TODO   Default to HtmlFileStoreManager if the manager is not configured
 *          in the application context.
 */
public class JTidyFileWriterPlugin extends AbstractMKDocPlugin
                                             implements Plugin {

    /**
     *  A reference to a <code>Manager</code> store interface.
     *  @since  Version 1.0 2005-05-31
     */
    protected FileStoreManager manager;

    /**
     *  The plugin description.
     *  @since  Version 0.1 2004-11-26
     */
    private static final String DESCRIPTION =
    "Creates a tidied mirror copy of (X)HTML documents.";

    /**
     *  The plugin name.
     *  @since  Version 0.1 2004-11-26
     */
    private static final String NAME =
    "JTidy file writer plugin for JSpider";

    /**
     *  The plugin vendor.
     *  @since  Version 0.1 2004-11-26
     */
    private static final String VERSION = "v1.2";

    /**
     *  Obtain the output configuration.
     *  @param  propertySet The configuration property set for
     *          this plugin.
     *  @since  Version 0.1 2004-11-26
     */
    public JTidyFileWriterPlugin(final PropertySet propertySet) {

        super();

        if (propertySet == null) {

            throw new IllegalArgumentException(
                        "Null property set not permitted.");
        }
        else {

            log = LogFactory.getLog(JTidyFileWriterPlugin.class);

            ApplicationContext context = getApplicationContext(propertySet);

            try {

                manager = StoreManagerFactory.newFileStoreManager(context);
            }
            catch (StoreException se) {

                log.error(se.getMessage());
            }
        }
    }

    /**
     *  Receive notification of a JSpider event.
     *  @param  event   A JSpider event.
     *  @since  Version 0.1 2004-11-26
     */
    public final void notify(final JSpiderEvent event) {

        if (event == null) {

            throw new IllegalArgumentException(
                        "Null event reference not permitted.");
        }
        else {

            if (event instanceof ResourceFetchedEvent) {

                ResourceFetchedEvent rfe = (ResourceFetchedEvent) event;

                parse(rfe.getResource().getInputStream(),
                      rfe.getURL());
            }
        }
    }

    /**
     *  Parse the input stream.
     *  @param  inStream The input stream source for
     *          the file.
     *  @param  url The URL of the source.
     *  @since  Version 0.7 2005-04-27
     */
    protected final void parse(final InputStream inStream,
                               final URL url) {

        if (inStream == null) {

            throw new IllegalArgumentException(
                        "Null input stream not permitted.");
        }
        else {

            if (url == null) {

                throw new IllegalArgumentException(
                            "Null URL not permitted.");
            }
            else {

                OutputStream outStream = null;

                try {

                    outStream = manager.getOutputStream(url);

                    TidyDriver tidy = new TidyDriver(log);

                    tidy.parse(inStream,
                               outStream);
                }
                catch (StoreException se) {

                    log.error("Could not write output. " +
                              se.getMessage());
                }
            }
        }
    }

    /**
     *  Get the plugin description.
     *  @return A short description of this plugin.
     *  @since  Version 0.1 2004-11-26
     */
    public final String getDescription() {

        return DESCRIPTION;
    }

    /**
     *  Get the plugin name.
     *  @return The name of this plugin.
     *  @since  Version 0.1 2004-11-26
     */
    public final String getName() {

        return NAME;
    }

    /**
     *  Get the plugin version.
     *  @return The version information for this plugin.
     *  @since  Version 0.1 2004-11-26
     */
    public final String getVersion() {

        return VERSION;
    }

}
