package com.mkdoc.jtidy;

/*
    Created:    2004-12-09
    Issued:     2004-12-09
    Modified:   2005-01-07

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.core.logging.Log;

import org.w3c.tidy.Configuration;
import org.w3c.tidy.Tidy;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;

/**
 *  A JTidy driver to encapsulate HTML processing.
 *  @version    0.2 2004-12-10
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class TidyDriver {

    /**
     *  The default replacement <code>alt</code> text.
     *  @since  Version 0.1 2004-12-09
     */
    private static final String ALT = "Image";

    /**
     *  A wrapped JTidy instance.
     *  @since  Version 0.1 2004-12-09
     */
    private final Tidy tidy = new Tidy();

    /**
     *  An JSpider log.
     *  @since  Version 0.1 2004-12-09
     */
    private Log log;

    /**
     *  Default constructor without logging.
     *  @since  Version 0.1 2004-12-09
     */
    public TidyDriver() {

        // Empty
    }

    /**
     *  Constructor with JSpider logging.
     *  @param  log A JSpider <code>Log</code> instance to
     *          pass messages.
     *  @since  Version 0.1 2004-12-09
     */
    public TidyDriver(final Log log) {

        if (log != null) {

            this.log = log;
        }
        else {

            throw new IllegalArgumentException(
                        "Null log not permitted.");
        }
    }

    /**
     *  Write an tidied output stream.
     *  @param  outStream   The output stream to which to
     *          write the output.
     *  @param  inStream The (X)HTML input stream for to be
     *          tidied.
     *  @since  Version 0.1 2004-12-09
     */
    public final void parse(final InputStream inStream,
                            final OutputStream outStream) {

        if (inStream != null) {

            if (outStream != null) {

                configure(tidy);

                tidy.parse(inStream, outStream);

                try {

                    inStream.close();

                    outStream.flush();
                    outStream.close();
                }
                catch (IOException ioe) {

                    logError("Error closing streams. " +
                             ioe.getMessage());
                }
            }
            else {

                throw new IllegalArgumentException(
                            "Null output stream not permitted.");
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null input stream not permitted.");
        }
    }

    /**
     *  Get a tidied input stream in memory.
     *  @param  inStream The (X)HTML input stream for to be
     *          tidied.
     *  @return An input stream that has been passed
     *          through JTidy.
     *  @since  Version 0.2 2004-12-03
     */
    public final InputStream tidyInputStream(final InputStream inStream) {

        if (inStream != null) {

            configure(tidy);

            ByteArrayOutputStream outStream = null;

            outStream = new ByteArrayOutputStream();

            tidy.parse(inStream, outStream);

            byte[] tidyBytes = outStream.toByteArray();

            // Clear buffer
            outStream.reset();

            return new ByteArrayInputStream(tidyBytes);
        }
        else {

            throw new IllegalArgumentException(
                        "Null input stream not permitted.");
        }
    }

    /**
     *  Configure a JTidy instance.
     *  <p>
     *    Currently uses a fixed configuration.
     *  </p>
     *  @param  tidy    The JTidy instance to be
     *          configured.
     *  @since  Version 0.1 2004-12-09
     */
    private void configure(final Tidy tidy) {

        tidy.setCharEncoding(Configuration.UTF8);

        tidy.setQuiet(true);

        tidy.setXHTML(true);

        tidy.setXmlPi(false);

        tidy.setNumEntities(true);

        tidy.setQuoteMarks(true);

        tidy.setQuoteNbsp(true);

        tidy.setQuoteAmpersand(true);

        tidy.setAltText(ALT);

        tidy.setDropEmptyParas(true);

        tidy.setFixComments(true);

        tidy.setFixBackslash(true);

        tidy.setEncloseText(true);

        tidy.setEncloseBlockText(true);
    }

    /**
     *  Log an error message.
     *  @param  message The message to log.
     *  @since  Version 0.1 2004-12-09
     */
    private void logError(final String message) {

        if (log != null) {

            log.error(message);
        }
    }

}
