package com.mkdoc.query;

/*
    Created:    2005-07-05
    Issued:     2005-07-06
    Modified:   2005-09-15

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.sax.AbstractRdfContentHandler;

import com.mkdoc.schema.ApplicationProfile;
import com.mkdoc.schema.DublinCoreProfile;
import com.mkdoc.schema.SchemaProperty;

import java.util.Collection;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 *  A base implementation of a query interpreter.
 *  @version    0.7 2005-09-14
 *  @author     Philip Shaw <phil@mkdoc.com>
 *  @TODO       Reduce the complexity of the getQueryTerms method
 */
public abstract class AbstractQueryBuilder implements QueryBuilder {

    /**
     *  A null schema property error message.
     *  @since  Version 0.2 2005-07-26
     */
    private static final String NULL_PROP =
    "Null schema property not permitted.";

    /**
     *  A list of the variable values to get from the query.
     *  @since  Version 0.1 2005-01-24
     */
    private Set variables = new HashSet();

    /**
     *  A list of the path expressions for the query.
     *  @since  Version 0.1 2005-01-24
     */
    private List expressions = new Vector();

    /**
     *  A list of the optional path expressions for the query.
     *  @since  Version 0.3 2005-07-27
     */
    private List options = new Vector();

    /**
     *  A list of the conditions to apply to the query variables.
     *  @since  Version 0.1 2005-01-24
     */
    private List conditions = new Vector();

    /**
     *  A map of schema property to a List of query terms.
     *  @since  Version 0.1 2005-01-24
     */
    private Map properties = new Hashtable();

    /**
     *  A list of the namespaces used in the query.
     *  @since  Version 0.1 2005-01-24
     */
    private Set namespaces = new HashSet();

    /**
     *  The query type, defalts to <code>SELECT</code>.
     *  @since  Version 0.1 2005-01-24
     */
    private String queryType = SELECT;

    /**
     *  Whether the <code>DISTINCT</code> keyword is applied to the query.
     *  @since  Version 0.1 2005-01-24
     */
    private boolean distinct;

    /**
     *  Whether to use extended bNode queries.
     *  @since  Version 0.6 2005-09-13
     */
    protected boolean extended = false;

    /**
     *  Whether any properties have optional encoding schemes.
     *  @since  Version 0.6 2005-09-14
     */
    protected boolean extendable = false;

    /**
     *  The application profile to use for interpreting queries.
     *  @since  Version 0.1 2005-07-05
     */
    protected ApplicationProfile profile = new DublinCoreProfile();

    /**
     *  Set the application profile to use for interpreting queries.
     *  @param  profile The application profile to use to prepare the query.
     *  @since  Version 0.1 2005-07-05
     */
    public final void setApplicationProfile(final ApplicationProfile profile) {

        if (profile != null) {

            this.profile = profile;
        }
        else {

            throw new IllegalArgumentException(
                        "Null application profile not permitted.");
        }
    }

    /**
     *  Get query terms from a string input.
     *  <p>
     *    The parse rules for query parameters are as follows:
     *  </p>
     *  <dl>
     *    <dt>
     *      White space
     *    </dt>
     *    <dd>
     *      <p>
     *        White space includes the single space, tab, new line, carriage
     *        return, and form feed characters. Any white space after the
     *        first, and before the last, non-white space character is
     *        interpreted as a single space, which is a word separator.
     *    </dd>
     *    <dt>
     *      Leading and trailing white space
     *    </dt>
     *    <dd>
     *      <p>
     *        All leading and trailing white space characters are trimmed and
     *        discarded.
     *      </p>
     *    </dd>
     *    <dt>
     *      Single word
     *    </dt>
     *    <dd>
     *      <p>
     *        A single word is interpreted as a match against the single word
     *        content of a metadata field. The SeRQL condition for the query
     *        &quot;cat&quot; is:
     *      </p>
     *      <pre>
     * WHERE label(varX) LIKE "cat"
     *      </pre>
     *      <p>
     *        This query would only match the literal word &quot;cat&quot;, not
     *        a phrase that included the word, such as &quot;the cat sat on the
     *        mat&quot;. <em>See &quot;Wildcard before and after&quot;.</em>
     *      </p>
     *    </dd>
     *    <dt>
     *      Multiple words
     *    </dt>
     *    <dd>
     *      <p>
     *        Multiple words are interpreted as if each word is an alternative
     *        match against the single word content of a metadata field. The
     *        SeRQL condition for the query &quot;cat dog&quot; is:
     *      </p>
     *      <pre>
     * WHERE label(varX) LIKE "cat" OR label(varX) LIKE "dog"
     *      </pre>
     *      <p>
     *        This would only match the literal word &quot;cat&quot; or
     *        &quot;dog&quot;, not a phrase that included either word, such as
     *        &quot;fought like cat and dog&quot;.  <em>See &quot;Wildcard
     *        before and after&quot;.</em>
     *      </p>
     *      <p>
     *        <em>The wildcard schemes described below can be applied to
     *        multiple query terms.</em> For example, the query &quot;*cat*
     *        *dog*&quot; would match &quot;fought like cat and dog&quot;,
     *        &quot;I like cats&quot; and &quot;I like dogs&quot;.
     *      </p>
     *    </dd>
     *    <dt>
     *      Wildcard before
     *    </dt>
     *    <dd>
     *      <p>
     *        A word with an asterisk before it is interpreted as the end of a
     *        phrase, or the suffix to a single word, where any number of
     *        characters may be matched before it, but not after it. The SeRQL
     *        condition for the query &quot;*cat&quot; is:
     *      </p>
     *      <pre>
     * WHERE label(varX) LIKE "*cat"
     *      </pre>
     *      <p>
     *        This would match &quot;cat&quot;, &quot;senses of a cat&quot; and
     *        &quot;polecat&quot;, but not &quot;cats&quot; or &quot;fought
     *        like cat and dog&quot;.
     *      </p>
     *    </dd>
     *    <dt>
     *      Wildcard after
     *    </dt>
     *    <dd>
     *      <p>
     *        A word with an asterisk after it is interpreted as the start of a
     *        phrase, or the prefix to a single word, where any number of
     *        characters may be matched after it, but not before it. The SeRQL
     *        condition for the query &quot;cat*&quot; is:
     *      </p>
     *      <pre>
     * WHERE label(varX) LIKE "cat*"
     *      </pre>
     *      <p>
     *        This would match &quot;cat&quot;, &quot;cats&quot; and &quot;cats
     *        and dogs&quot;, but not &quot;polecat&quot; or  &quot;senses of a
     *        cat&quot;.
     *      </p>
     *    </dd>
     *    <dt>
     *      Wildcard before and after
     *    </dt>
     *    <dd>
     *      <p>
     *        A word with an asterisk before and after it is interpreted as the
     *        middle of a phrase, or part of a compound word, where any number
     *        of characters may be matched before and after it. The SeRQL
     *        condition for the query &quot;*cat*&quot; is:
     *      </p>
     *      <pre>
     * WHERE label(varX) LIKE "*cat*"
     *      </pre>
     *      <p>
     *        This would match &quot;cat&quot;, &quot;cats&quot;, &quot;cats
     *        and dogs&quot;, &quot;polecat&quot; and &quot;senses of a
     *        cat&quot;.
     *      </p>
     *    </dd>
     *    <dt>
     *      Quoted phrase
     *    </dt>
     *    <dd>
     *      <p>
     *        A phrase enclosed by double quote characters, <code>"</code>, is
     *        interpreted as a match against the whole content of a single
     *        metadatum field. The SeRQL condition for &quot;cat litter&quot;
     *        is:
     *      </p>
     *      <pre>
     * WHERE label(varX) LIKE "cat litter"
     *      </pre>
     *      <p>
     *        This would only match &quot;cat litter&quot;, not &quot;cat
     *        litter tray&quot;, &quot;use cat litter&quot;, &quot;cat&quot; or
     *        &quot;litter&quot;.
     *      </p>
     *      <p>
     *        Quoted phrases can also be include wildcard asterisks at
     *        the beginning and end of the phrase. The query &quot;*cat
     *        litter&quot; would match &quot;use cat litter&quot; and &quot;buy
     *        cat litter&quot;, for example.
     *      </p>
     *      <p>
     *        Un-terminated quoted phrases are interpreted as everything
     *        following an opening double quote to the end of the query
     *        parameter.
     *      </p>
     *    </dd>
     *  </dl>
     *  @param  query   The contents of a single metadata query field, which
     *          may include multiple search terms separated by delimiters.
     *  @return A <code>Vector</code> containing all phrase- and word-query
     *          tokens. If the query is the empty string or pure white space,
     *          the list will be empty, so clients should check its size
     *          before further processing.
     *  @since  Version 0.1 2005-07-05
     */
    public final List getQueryTerms(final String query) {

        if (query != null) {

            String trimmed = query.trim();

            List list = new Vector();

            if (!trimmed.equals("")) {

                StringTokenizer tokens = new StringTokenizer(trimmed,
                                                             DELIMITERS,
                                                             true);
                String token = null;

                String term = null;

                boolean phrase = false;

                boolean first = true;

                while (tokens.hasMoreTokens()) {

                    token = tokens.nextToken();

                    if (token.equals(QUOTE)) {

                        if (!phrase) {

                            // Start of a phrase
                            phrase = true;

                            first = true;

                            term = "";
                        }
                        else {

                            // End of a phrase
                            phrase = false;

                            list.add(term);
                        }
                    }
                    else {

                        if (!isDelimiter(token)) {

                            if (phrase) {

                                if (!first) {

                                    term += " ";

                                    term += token;
                                }
                                else {

                                    term += token;

                                    first = false;
                                }
                            }
                            else {

                                list.add(token);
                            }
                        }
                        // Else a delimiter, ignore
                    }
                }
                if (phrase) {

                    // An un-termintated phrase remains
                    list.add(term);
                }
            }
            // Else an empty string, list is empty

            return list;
        }
        else {

            throw new IllegalArgumentException(
                        "Null query not permitted.");
        }
    }

    /**
     *  Check whether a string token is a query delimiter.
     *  @param  token   The string to test.
     *  @return <code>true</code> if the input is a single character from the
     *          set of query delimiters.
     *  @since  Version 0.4 2005-01-27
     */
    protected final boolean isDelimiter(final String token) {

        return ((token.length() == 1) &&
                (DELIMITERS.indexOf(token) > -1));
    }

    /**
     *  Add a variable name.
     *  @param  property    The schema property for which to add a query
     *          variable.
     *  @since  Version 0.2 2005-07-26
     */
    protected final void addVariable(final SchemaProperty property) {

        if (property != null) {

            variables.add(Query.VAR_URI);

            variables.add(property.getNameVariable());
        }
        else {

            throw new IllegalArgumentException(NULL_PROP);
        }
    }

    /**
     *  Add a path expression to the query.
     *  @param  property    The schema property for which to add a query
     *          expression.
     *  @since  Version 0.2 2005-07-26
     */
    protected final void addExpression(final SchemaProperty property) {

        if (property != null) {

            variables.add(Query.VAR_URI);

            expressions.add(getQueryExpression(property));
        }
        else {

            throw new IllegalArgumentException(NULL_PROP);
        }
    }

    /**
     *  Add an optional summary expression to the query.
     *  @param  property    The schema property for which to add an optional
     *          summary expression.
     *  @since  Version 0.3 2005-07-27
     */
    protected final void addSummaryExpression(final SchemaProperty property) {

        if (property != null) {

            variables.add(Query.VAR_SUMMARY);

            options.add("[ {" + Query.VAR_URI + "} " +
                        property.getQualifiedName() +
                        " {" + Query.VAR_SUMMARY + "} ]");

            addNamespaceDeclaration(property);
        }
        else {

            throw new IllegalArgumentException(NULL_PROP);
        }
    }

    /**
     *  Add an optional title expression to the query.
     *  @param  property    The schema property for which to add an optional
     *          title expression.
     *  @since  Version 0.3 2005-07-27
     */
    protected final void addTitleExpression(final SchemaProperty property) {

        if (property != null) {

            variables.add(Query.VAR_TITLE);

            options.add("[ {" + Query.VAR_URI + "} " +
                        property.getQualifiedName() +
                        " {" + Query.VAR_TITLE + "} ]");

            addNamespaceDeclaration(property);
        }
        else {

            throw new IllegalArgumentException(NULL_PROP);
        }
    }

    /**
     *  Add a condition to the query selection.
     *  @param  property    The schema property for which to add a query
     *          condition.
     *  @param  term    The query term.
     *  @since  Version 0.2 2005-07-26
     */
    protected final void addCondition(final SchemaProperty property,
                                      final String term) {

        if (property != null) {

            if (term != null) {

                addVariable(property);

                addNamespaceDeclaration(property);

                conditions.add("label(" +
                               property.getNameVariable() +
                               ")" +
                               LIKE +
                               QUOTE +
                               term +
                               QUOTE);

                if (property.hasEncodingScheme()) {

                    extendable = true;
                }

                List terms = (List) properties.get(property);

                if (terms != null) {

                    terms.add(term);
                }
                else {

                    terms = new Vector();

                    terms.add(term);

                    properties.put(property, terms);
                }
            }
            else {

                throw new IllegalArgumentException(
                            "Null query term not permitted.");
            }
        }
        else {

            throw new IllegalArgumentException(NULL_PROP);
        }
    }

    /**
     *  Add a namespace declaration to the query selection.
     *  @param  property    The schema property for which to add a query
     *          namespace declaration.
     *  @since  Version 0.2 2005-07-26
     */
    protected final void addNamespaceDeclaration(
                                               final SchemaProperty property) {

        if (property != null) {

            namespaces.add(getNamespaceDeclaration(property));
        }
        else {

            throw new IllegalArgumentException(NULL_PROP);
        }
    }

    /**
     *  Set whether the <code>DISTINCT</code> keyword is applied.
     *  @param  distinct    Whether the query interpreter should suppress
     *          duplicate results.
     *  @since  Version 0.2 2005-07-26
     */
    public final void setDistinct(final boolean distinct) {

        this.distinct = distinct;
    }

    /**
     *  Set query type.
     *  @param  queryType   <code>SELECT</code> or <code>CONSTRUCT</code>.
     *  @since  Version 0.2 2005-07-26
     */
    protected final void setQueryType(final String queryType) {

        if (queryType != null) {

            this.queryType = queryType;
        }
        else {

            throw new IllegalArgumentException(
                        "Null query type not permitted.");
        }
    }

    /**
     *  Get a string representation of the query.
     *  <p>
     *    <em>At least one variable declaration and one path expression must be
     *    added to the query before calling this method.</em>
     *  </p>
     *  @return A SeRQL query string generated from the current builder
     *          configuration.
     *  @since  Version 0.2 2005-07-26
     */

/*
    public final String toString() {

        if (variables.size() > 0) {

            if (expressions.size() > 0) {

                StringBuffer query = new StringBuffer();

                query.append(queryType);

                if (distinct) {

                    query.append(DISTINCT);
                }

                query.append(SPACE);

                appendList(variables, query, APPEND_COMMA);

                query.append(FROM);

                appendList(expressions, query, APPEND_COMMA);

                if (options.size() > 0) {

                    query.append(APPEND_COMMA);

                    appendList(options, query, APPEND_COMMA);
                }

                if (conditions.size() > 0) {

                    query.append(WHERE);

                    appendList(conditions, query, APPEND_OR);

                }

                if (namespaces.size() > 0) {

                    query.append(USING);

                    appendList(namespaces, query, APPEND_COMMA);
                }

                return query.toString();
            }
            else {

                throw new IllegalStateException(
                            "No expressions declared.");
            }
        }
        else {

            throw new IllegalStateException(
                        "No variables declared.");
        }
    }
*/

    /**
     *  Get a SeRQL query string.
     *  @param  extended    Whether an extended bNode union query is requested.
     *  @return An SeRQL query that may use a union join to combine simple and
     *          extended bNode path expressions. If none of the query terms
     *          support encoding schemes that would be expressed in a bNode
     *          graph, no union is applied.
     *  @since  Version 0.7 2005-09-14
     */
    public final String getSeRQLQuery(final boolean extended) {

        if (properties.size() > 0) {

            StringBuffer query = new StringBuffer();

            query.append(queryType);

            if (distinct) {

                query.append(DISTINCT);
            }

            query.append(SPACE);

            appendList(variables, query, APPEND_COMMA);

            query.append(FROM);

            appendPrimaryExpressions(query);

            if (options.size() > 0) {

                query.append(APPEND_COMMA);

                appendList(options, query, APPEND_COMMA);
            }

            query.append(WHERE);

            appendConditions(query);

            if ((extended) &&
                (hasExtendableProperties())) {

                query.append(" UNION ");

                query.append(queryType);

                if (distinct) {

                    query.append(DISTINCT);
                }

                query.append(SPACE);

                appendList(variables, query, APPEND_COMMA);

                query.append(FROM);

                appendSecondaryExpressions(query);

                if (options.size() > 0) {

                    query.append(APPEND_COMMA);

                    appendList(options, query, APPEND_COMMA);
                }

                query.append(WHERE);

                appendConditions(query);
            }

            query.append(USING);

            appendList(namespaces, query, APPEND_COMMA);

            return query.toString();

        }
        else {

            throw new IllegalStateException(
                        "No query properties defined.");
        }
    }

    /**
     *  Check whether the query has properties that may have encoded schema.
     *  @return <code>true</code> if any property registered for the query has
     *          an optional encoding scheme.
     *  @since  Version 0.7 2005-09-14
     */
    protected final boolean hasExtendableProperties() {

        return extendable;
    }

    /**
     *  Append path expressions for the first selection in a union query.
     *  @param  query   The query buffer to which to append the expressions.
     *  @since  0.7 2005-09-14
     */
    protected final void appendPrimaryExpressions(final StringBuffer query) {

        Set propertySet = properties.keySet();

        Iterator iterator = propertySet.iterator();

        SchemaProperty property = null;

        boolean first = true;

        while (iterator.hasNext()) {

            if (first) {

                first = false;
            }
            else {

                query.append(", ");
            }

            property = (SchemaProperty) iterator.next();

            query.append(getQueryExpression(property));
        }
    }

    /**
     *  Append conditions for the first selection in a union query.
     *  @param  query   The query buffer to which to append the expressions.
     *  @since  0.7 2005-09-14
     */
    protected final void appendConditions(final StringBuffer query) {

        Set propertySet = properties.keySet();

        Iterator propItems = propertySet.iterator();

        SchemaProperty property = null;

        boolean firstProp = true;

        List terms = null;

        while (propItems.hasNext()) {

            if (firstProp) {

                firstProp = false;
            }
            else {

                query.append(" AND ");
            }

            property = (SchemaProperty) propItems.next();

            terms = (List) properties.get(property);

            Iterator termItems = terms.iterator();

            boolean firstTerm = true;

            String term = null;

            while (termItems.hasNext()) {

                if (firstTerm) {

                    if (terms.size() > 1) {

                        query.append("(");
                    }

                    firstTerm = false;
                }
                else {

                    query.append(" OR ");
                }

                term = (String) termItems.next();

                if (isUriQuery(property, term)) {

                    query.append(property.getNameVariable());
                }
                else {

                    query.append("label(");
                    query.append(property.getNameVariable());
                    query.append(")");
                }

                query.append(LIKE);
                query.append(QUOTE);
                query.append(term);
                query.append(QUOTE);
            }

            if (terms.size() > 1) {

                query.append(")");
            }
        }
    }

    /**
     *  Check whether this is a URI query.
     *  @param  property    The schema property against which the query term
     *          is to be tested.
     *  @param  term    The user query term to check.
     *  @return <code>true</code> if the schema property has a URI encoding
     *          scheme and the query term is a URI.
     *  @since  Version
     */
    protected final boolean isUriQuery(final SchemaProperty property,
                                       final String term) {

        return (property.hasUriEncodingScheme() &&
                AbstractRdfContentHandler.isUri(term));

    }

    /**
     *  Append path expressions for the second selection in a union query.
     *  @param  query   The query buffer to which to append the expressions.
     *  @since  0.7 2005-09-14
     */
    protected final void appendSecondaryExpressions(final StringBuffer query) {

        Set propertySet = properties.keySet();

        Iterator iterator = propertySet.iterator();

        SchemaProperty property = null;

        boolean first = true;

        while (iterator.hasNext()) {

            if (first) {

                first = false;
            }
            else {

                query.append(", ");
            }

            property = (SchemaProperty) iterator.next();

            if (property.hasEncodingScheme()) {

                query.append(getBNodeQueryExpression(property));
            }
            else {

                query.append(getQueryExpression(property));
            }
        }
    }


    /**
     *  Concatenate a collection of string values with the given separator.
     *  <p>
     *    If there is more than one item in the collection, the separator text
     *    is inserted before each subsequent item. No leading or trailing
     *    spaces are added.
     *  </p>
     *  @param  values  The list of string values to concatenate.
     *  @param  buffer  The string buffer to append values to.
     *  @param  separator   The separator characters to insert before each
     *          list item, e.g. &quot; AND &quot;, or &quot;, &quot;.
     *  @since  Version 0.2 2005-07-26
     */
    private final void appendList(final Collection values,
                                  final StringBuffer buffer,
                                  final String separator) {

        Iterator iterator = values.iterator();

        boolean first = true;

        while (iterator.hasNext()) {

            if (first) {

                first = false;
            }
            else {

                buffer.append(separator);
            }

            buffer.append((String) iterator.next());
        }
    }

    /**
     *  Get a query expression.
     *  @param  property    The schema property with which to prepare the
     *          query expression.
     *  @return A SeRQL query for the property.
     *  @since  Version 0.2 2005-07-26
     */
    private final String getQueryExpression(final SchemaProperty property) {

        return "{" + Query.VAR_URI + "} " +
               property.getQualifiedName() +
               " {" + property.getNameVariable() + "}";
    }

    /**
     *  Get a bNode query expression.
     *  @param  property    The schema property with which to prepare the
     *          query expression.
     *  @return A SeRQL query for the property.
     *  @since  Version 0.6 2005-09-13
     */
    private final String getBNodeQueryExpression(
                                               final SchemaProperty property) {

        return "{" + Query.VAR_URI + "} " +
               property.getQualifiedName() +
               " {} rdf:value {" +  property.getNameVariable() + "}";
    }

    /**
     *  Get a query expression.
     *  @param  property    The schema property with which to prepare the
     *          query expression.
     *  @return An SeRQL namespace declaration for the property.
     *  @since  Version 0.2 2005-07-26
     */
    private final String getNamespaceDeclaration(
                                               final SchemaProperty property) {

        return property.getNamespacePrefix() +
               " = <" + property.getNamespaceUri() + ">";

    }

}
