package com.mkdoc.query;

/*
    Created:    2005-07-29
    Issued:     2005-10-28
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

/**
 *  A result renderer for RSS output.
 *  @version    0.2 2005-10-11
 *  @author     Philip Shaw <phil@mkdoc.com>
 *  @TODO       Need some way to configure the output namespaces based on
 *              the application profile, so that results metadata can be
 *              inserted in the feed.
 */
public class RssResultRenderer extends AbstractResultRenderer
                                    implements ResultRenderer {

    /**
     *  The RSS content type.
     *  @since  Version 0.1 2005-07-29
     */
    public static final String CONTENT_TYPE = "application/rss+xml";

    /**
     *  The XML header and document element.
     *  @since  Version 0.1 2005-07-29
     */
    public static final String HEADER =
    "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n" +
    "<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">\n";

    /**
     *  The close RDF document element.
     *  @since  Version 0.2 2005-10-28
     */
    public static final String FOOTER = "</rdf:RDF>\n";

    /**
     *  Construct a renderer for a specific query.
     *  @param  query   The query against which the results were returned.
     *  @since  Version 0.1 2005-07-29
     */
    public RssResultRenderer(final Query query) {

        super();

        if (query == null) {

            throw new IllegalArgumentException(
                        "Null query not permitted.");
        }
        else {

            this.query = query;

        }
    }

    /**
     *  Render a query result to a string buffer.
     *  @param  queryResult   The query results to render.
     *  @param  mode    The output mode, either
     *          <code>ResultRenderer.FULL</code> or
     *          <code>ResultRenderer.INSERT</code>.
     *  @return A text formatted tabular layout of the query results.
     *  @since  Version 0.1 2005-07-29
     */
    protected final StringBuffer getQueryResult(final QueryResult queryResult,
                                                final int mode) {

        StringBuffer buffer = new StringBuffer();

        if (queryResult == null) {

            throw new IllegalArgumentException(
                        "Null query result not permitted.");
        }
        else {

            int rows = queryResult.getRows();

            if (rows > 0) {

                if (mode == FULL) {

                    buffer.append(getHeader());
                }

                buffer.append("<items>\n");
                buffer.append("<rdf:Seq>\n");

                for (int row = 0; row < rows; row++) {

                    buffer.append("<rdf:li rdf:resource=\"");
                    buffer.append(queryResult.getItemUri(row));
                    buffer.append("\"/>\n");
                }

                buffer.append("</rdf:Seq>\n");
                buffer.append("</items>\n");
                buffer.append("</channel>\n");

                for (int row = 0; row < rows; row++) {

                    buffer.append("<item xmlns=\"http://purl.org/rss/1.0/\" ");
                    buffer.append("rdf:about=\"");
                    buffer.append(queryResult.getItemUri(row));
                    buffer.append("\">\n");
                    buffer.append("<title>");
                    buffer.append(queryResult.getItemTitle(row));
                    buffer.append("</title>\n");
                    buffer.append("<description>");
                    buffer.append(queryResult.getItemSummary(row));
                    buffer.append("</description>\n");
                    buffer.append("<link>");
                    buffer.append(queryResult.getItemUri(row));
                    buffer.append("</link>\n");
                    buffer.append("</item>");
                }

                if (mode == FULL) {

                    buffer.append(FOOTER);
                }
            }
            else {

                buffer.append("No results");
            }

            return buffer;
        }
    }


    /**
     *  Get a basic RSS feed header.
     *  @return An RSS feed header with basic namespace declarations.
     *  @since  Version 0.1 2005-07-29
     */
    private String getHeader() {

        return HEADER +
               "<channel xmlns=\"http://purl.org/rss/1.0/\" rdf:about=\"" +
               query.getQueryUri() +
               "\">\n" +
               "<title>MKSearch result feed</title>\n<link>" +
               query.getQueryUri() +
               "</link>\n<description>A dynamically updated search query" +
               "</description>";
    }

    /**
     *  Get the content type for the renderer.
     *  @return The MIME type <code>text/html</code>.
     *  @since  Version 0.1 2005-07-29
     */
    public final String getContentType() {

        return CONTENT_TYPE;
    }

}
