package com.mkdoc.query;

/*
    Created:    2005-07-05
    Issued:     2005-07-06
    Modified:   2005-09-14

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.schema.SchemaProperty;

import com.mkdoc.servlet.AbstractMKDocServlet;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;

/**
 *  A query builder for servlets.
 *  @version    0.7 2005-09-12
 *  @author     Philip Shaw <phil@mkdoc.com>
 *  @TODO       Add state tracking to prevent repetition of build and a reset
 *              mechanism to re-use the builder for a different request.
 *              Internalise the queryUri buffer to avoid passing it around.
 *              Remove schema property arguments to avoid passing around.
 */
public class ServletQueryBuilder extends AbstractQueryBuilder
                                                      implements QueryBuilder {

    /**
     *  Adapt a SeRQL query builder.
     *  @since  Version 0.1 2005-07-05
     */
    private HttpServletRequest request;

    /**
     *  Whether this is the first query parameter.
     *  @since  Version 0.5 2005-08-01
     */
    private boolean firstParam = true;

    /**
     *  Construct from a servlet request.
     *  @param  request A servlet request object.
     *  @since  Version 0.1 2005-07-05
     */
    public ServletQueryBuilder(final HttpServletRequest request) {

        if (request != null) {

            this.request = request;
        }
        else {

            throw new IllegalArgumentException(
                        "Null servlet request not permitted.");
        }
    }

    /**
     *  Construct with extended bNode query support.
     *  @param  request A servlet request object.
     *  @param  extended    Whether to enable bNode query processing.
     *  @since  Version 0.1 2005-07-05
     */
    public ServletQueryBuilder(final HttpServletRequest request,
                               final boolean extended) {

        if (request != null) {

            this.request = request;
            this.extended = extended;
        }
        else {

            throw new IllegalArgumentException(
                        "Null servlet request not permitted.");
        }
    }

    /**
     *  Parse the query source and get a <code>Query</code> object.
     *  @return A query based on the input source.
     *  @since  Version 0.1 2005-07-05
     */
    public final Query getQuery() {

        addTitleExpression(profile.getTitleProperty());

        addSummaryExpression(profile.getSummaryProperty());

        Enumeration enumeration = request.getParameterNames();

        String name = null;

        String value = null;

        StringBuffer queryUri = queryUri = request.getRequestURL();

        queryUri.append("?");

        while (enumeration.hasMoreElements()) {

            name = (String) enumeration.nextElement();

            SchemaProperty property = null;

            property = profile.getPredicateProperty(name);

            if (property != null) {

                value = AbstractMKDocServlet.getFirstParam(request, name);

                if (value !=  null) {

                    parseParameter(value, property, queryUri);
                }
            }
        }

        return new SimpleQuery(getSeRQLQuery(extended),
                               queryUri.toString(),
                               getQueryContext());
    }

    /**
     *  Get the query output context.
     *  @return A suitable query context for the request.
     *  @since  Version 0.6 2005-08-02
     */
    protected final QueryContext getQueryContext() {

        String type = AbstractMKDocServlet.getFirstParam(request, "type");

        if (type != null) {

            if (type.equalsIgnoreCase(QueryContext.TEXT)) {

                return new TextQueryContext(getPageNumber(),
                                            getResultsPerPage());
            }
            else {

                if (type.equalsIgnoreCase(QueryContext.RSS)) {

                    return new RssQueryContext(getPageNumber(),
                                               getResultsPerPage());
                }
                // Else drop through
            }
        }

        return new XhtmlQueryContext(getPageNumber(),
                                     getResultsPerPage());
    }

    /**
     *  Parse a parameter value.
     *  @param  value   The query parameter value.
     *  @param  property    The schema property for this parameter.
     *  @param  queryUri    The query URI to which to append the parameter name
     *          and value.
     *  @see    java.util.StringTokenizer
     *  @since  Version 0.1 2005-01-24
     */
    protected final void parseParameter(final String value,
                                        final SchemaProperty property,
                                        final StringBuffer queryUri) {

        if (value != null) {

            if (property != null) {

                if (queryUri != null) {

                    List terms = getQueryTerms(value);

                    if (terms.size() > 0) {

                        addVariable(property);

                        addExpression(property);

                        addNamespaceDeclaration(property);

                        if (firstParam) {

                            firstParam = false;
                        }
                        else {

                            queryUri.append("&amp;");
                        }

                        queryUri.append(property.getParameterName());
                        queryUri.append("=");


                        boolean firstTerm = true;

                        Iterator iterator = terms.iterator();

                        String term = null;

                        while (iterator.hasNext()) {

                            if (firstTerm) {

                                firstTerm = false;
                            }
                            else {

                                queryUri.append("+");
                            }

                            term = (String) iterator.next();

                            appendTerm(term, queryUri);

                            addCondition(property, term);
                        }
                    }
                    // Else no query terms
                }
                else {

                    throw new IllegalArgumentException(
                                "Null query URI buffer not permitted.");
                }
            }
            else {

                throw new IllegalArgumentException(
                            "Null schema property not permitted.");
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null parameter value not permitted.");
        }
    }

    /**
     *  Append a query term to the query URI.
     *  @param  term    The query term, which may include quoted phrases.
     *  @param  queryUri    The query buffer to which to append the term.
     *  @since  Version 0.7 2005-09-12
     */
    protected final void appendTerm(final String term,
                                    final StringBuffer queryUri) {

        StringTokenizer tokenizer = new StringTokenizer(term,
                                                        DELIMITERS);

        if (tokenizer.countTokens() > 1) {

            queryUri.append("%22");

            boolean first = true;

            while (tokenizer.hasMoreTokens()) {

                if (first) {

                    first = false;
                }
                else {

                    queryUri.append("+");
                }

                queryUri.append(tokenizer.nextToken());

            }

            queryUri.append("%22");
        }
        else {

            queryUri.append(term);
        }
    }

    /**
     *  Get the page number
     *  @return The requested page number.
     *  @since  Version 0.5 2005-08-01
     */
    protected final int getPageNumber() {

        String page = AbstractMKDocServlet.getFirstParam(request, "page");

        try {

            return Integer.parseInt(page);
        }
        catch (NumberFormatException nfe) {

            return 1;
        }
    }

    /**
     *  Get the number of results per page.
     *  @return The number of results per page.
     *  @since  Version 0.5 2005-08-01
     */
    protected final int getResultsPerPage() {

        String limit = AbstractMKDocServlet.getFirstParam(request, "limit");

        try {

            return Integer.parseInt(limit);
        }
        catch (NumberFormatException nfe) {

            return QueryContext.LIMIT;
        }
    }

}
