package com.mkdoc.sax;

/*
    Created:    2004-12-10
    Issued:     2004-12-10
    Modified:   2005-09-09

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 *  An inert base implementation of a
 *  <code>ContentHandler</code>.
 *  <p>
 *    Subclasses must implement <code>startElement</code>
 *    and <code>endElement</code>.
 *  </p>
 *  @version    0.2 2005-09-09
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public abstract class AbstractContentHandler implements ContentHandler {

    /**
     *  The locator for this parse.
     *  @since  Version 0.1 2004-12-10
     */
    protected Locator locator;

    /**
     *  Set a <code>Locator</code> object for a parse.
     *  @param  locator The locator to use.
     *  @since  Version 0.1 2004-12-10
     */
    public final void setDocumentLocator(final Locator locator) {

        if (locator != null) {

            this.locator = locator;
        }
        else {

            throw new IllegalArgumentException("Null locator not permitted.");
        }
    }

    /**
     *  Process the start document event.
     *  @exception  SAXException    Never thrown by this
     *          implementation.
     *  @since  Version 0.1 2004-12-10
     */
    public void startDocument() throws SAXException {

        // Empty
    }

    /**
     *  Process the end document event.
     *  @exception  SAXException    Never thrown by this
     *          implementation.
     *  @since  Version 0.1 2004-12-10
     */
    public void endDocument() throws SAXException {

        // Empty
    }

    /**
     *  Process the start prefix mapping event.
     *  @param  prefix  The namespace prefix to map.
     *  @param  uri The namespace URI to alias.
     *  @exception  SAXException    Never thrown by this
     *          implementation.
     *  @since  Version 0.1 2004-12-10
     */
    public void startPrefixMapping(final String prefix,
                                   final String uri) throws SAXException {

        // Empty
    }

    /**
     *  Process the end prefix mapping event.
     *  @param  prefix  The namespace prefix that was being
     *          mapped.
     *  @exception  SAXException    Never thrown by this
     *          implementation.
     *  @since  Version 0.1 2004-12-10
     */
    public void endPrefixMapping(final String prefix) throws SAXException {

        // Empty
    }

    /**
     *  Process the start element event.
     *  @param  uri The namespace URI for the element.
     *  @param  localName   The local element name.
     *  @param  qName   The qualified element name.
     *  @param  atts    A set of attributes.
     *  @exception  SAXException    Never thrown by this
     *          implementation.
     *  @since  Version 0.1 2004-12-10
     */
    public void startElement(final String uri,
                             final String localName,
                             final String qName,
                             final Attributes atts)
        throws SAXException {

        // Empty
    }

    /**
     *  Process the end element event.
     *  @param  uri The namespace URI for the element.
     *  @param  localName   The local element name.
     *  @param  qName   The qualified element name.
     *  @exception  SAXException    Never thrown by this
     *          implementation.
     *  @since  Version 0.1 2004-12-10
     */
    public void endElement(final String uri,
                           final String localName,
                           final String qName)
        throws SAXException {

        // Empty
    }

    /**
     *  Process character content.
     *  @param  charArray   The character array to process.
     *  @param  start   The start index of the characters.
     *  @param  length  The number of characters to read.
     *  @exception  SAXException    Never thrown by this
     *          implementation.
     *  @since  Version 0.1 2004-12-10
     */
    public void characters(final char[] charArray,
                           final int start,
                           final int length) throws SAXException {

        // Empty
    }

    /**
     *  Process ignorable whitespace.
     *  @param  charArray   The character array to process.
     *  @param  start   The start index of the characters.
     *  @param  length  The number of characters to read.
     *  @exception  SAXException    Never thrown by this
     *          implementation.
     *  @since  Version 0.1 2004-12-10
     */
    public void ignorableWhitespace(final char[] charArray,
                                    final int start,
                                    final int length) throws SAXException {

        // Empty
    }


    /**
     *  Handle a processing instruction.
     *  @param  target  The target application identifier.
     *  @param  data    The data to pass to the
     *          application.
     *  @exception  SAXException    Never thrown by this
     *          implementation.
     *  @since  Version 0.1 2004-12-10
     */
    public void processingInstruction(final String target,
                                      final String data) throws SAXException {

        // Empty
    }

    /**
     *  Process a skipped entity.
     *  @param  name    The name of the skipped entity.
     *  @exception  SAXException    Never thrown by this
     *          implementation.
     *  @since  Version 0.1 2004-12-10
     */
    public void skippedEntity (final String name) throws SAXException {

        // Empty
    }

    /**
     *  Get a single attribute value from a set of attributes.
     *  @param  attributes  The attribute set from which to extract the value.
     *  @param  index   The index of the attribute to get.
     *  @return A trimmed string of the specified attribute value or
     *          <code>null</code> if out of range.
     *  @since  Version 0.2 2005-09-09
     *
     */
    protected final String getAttributeValue(final Attributes attributes,
                                             final int index) {

         String value = attributes.getValue(index);

         if (value != null) {

            return value.trim();
         }
         else {

            return null;
         }
    }

}
