package com.mkdoc.sax;

/*
    Created:    2004-10-19
    Issued:     2004-10-19
    Modified:   2004-10-25

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.IOException;

import java.util.Map;
import java.util.Hashtable;

import org.xml.sax.ContentHandler;
import org.xml.sax.DTDHandler;
import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.XMLReader;

/**
 *  A base class for custom XML readers.
 *  <p>
 *    Subclasses should initialise the feature set with the
 *    mandatory features for XML namespace and
 *    namespace prefix support, which must be mutable.
 *  </p>
 *  <pre>
 *    features.put(NAMESPACES,
 *                 new Feature());
 *
 *    features.put(PREFIXES,
 *                 new Feature());
 *  </pre>
 *  @version    0.2 2004-10-25
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public abstract class AbstractXMLReader implements XMLReader {

    /**
     *  The XML namespaces feature identifier.
     *  @since  Version 0.1 2004-10-25
     */
    public static final String NAMESPACES =
                              "http://xml.org/sax/features/namespaces";

    /**
     *  The XML namespace prefixes feature identifier.
     *  @since  Version 0.1 2004-10-25
     */
    public static final String PREFIXES =
                              "http://xml.org/sax/features/namespace-prefixes";

    /**
     *  A set of XML parser features.
     *  @since  Version 0.1 2004-10-20
     */
    protected Map features = new Hashtable();

    /**
     *  A set of XML parser properties.
     *  @since  Version 0.2 2004-10-25
     */
    protected Map properties = new Hashtable();

    /**
     *  An entity resolver.
     *  @since  Version 0.2 2004-10-25
     */
    protected EntityResolver resolver;

    /**
     *  A DTD handler.
     *  @since  Version 0.2 2004-10-25
     */
    protected DTDHandler dtdHandler;

    /**
     *  A content handler.
     *  @since  Version 0.2 2004-10-25
     */
    protected ContentHandler contHandler;

    /**
     *  An error handler.
     *  @since  Version 0.2 2004-10-25
     */
    protected ErrorHandler errorHandler;

    /**
     *  Get a parser feature.
     *  @param  name    A SAX parser feature name
     *          specified by a fully qualified URI.
     *  @return The value of the named feature property.
     *  @exception  SAXNotRecognizedException   If the
     *          feature name is not recognised.
     *  @exception  SAXNotSupportedException    If the
     *          named feature is not supported.
     *  @since  Version 0.1 2004-10-20
     */
    public final boolean getFeature(final String name)
        throws SAXNotRecognizedException, SAXNotSupportedException {

        if (name != null) {

            Feature feature = (Feature) features.get(name);

            if (feature != null) {

                return feature.getValue();
            }
            else {

                throw new SAXNotRecognizedException("SAX feature " +
                                                    name +
                                                    " not recognized.");
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null feature name not permitted.");
        }
    }

    /**
     *  Set a parser feature.
     *  @param  name    A SAX parser feature name
     *          specified by a fully qualified URI.
     *  @param  value   The setting to be assigned to the
     *          named property.
     *  @exception  SAXNotRecognizedException   If the
     *          feature name is not recognised.
     *  @exception  SAXNotSupportedException    If the
     *          named feature is not supported.
     *  @since  Version 0.1 2004-10-20
     */
    public final void setFeature(final String name,
                                 final boolean value)
        throws SAXNotRecognizedException, SAXNotSupportedException {

        if (name != null) {

            Feature feature = (Feature) features.get(name);

            if (feature != null) {

                feature.setValue(value);
            }
            else {

                throw new SAXNotRecognizedException("SAX feature " +
                                                    name +
                                                    " not recognized.");
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null feature name not permitted.");
        }
    }

    /**
     *  Get a parser property.
     *  @param  name    A SAX parser property name
     *          specified by a fully qualified URI.
     *  @return The value of the named parser property.
     *  @exception  SAXNotRecognizedException   If the
     *          property name is not recognised.
     *  @exception  SAXNotSupportedException    If the
     *          named property is not supported.
     *  @since  Version 0.2 2004-10-25
     */
    public final Object getProperty(final String name)
        throws SAXNotRecognizedException, SAXNotSupportedException {

        if (name != null) {

            Property property = (Property) properties.get(name);

            if (property != null) {

                return property.getValue();
            }
            else {

                throw new SAXNotRecognizedException("Property " + name +
                                                    " not recognized.");
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null property name not permitted.");
        }
    }

    /**
     *  Set a parser property.
     *  @param  name    A SAX parser property name
     *          specified by a fully qualified URI.
     *  @param  value   The setting to be assigned to the
     *          named property.
     *  @exception  SAXNotRecognizedException   If the
     *          property name is not recognised.
     *  @exception  SAXNotSupportedException    If the
     *          named property is not supported.
     *  @since  Version 0.2 2004-10-25
     */
    public final void setProperty(final String name,
                                  final Object value)
        throws SAXNotRecognizedException, SAXNotSupportedException {

        if (name != null) {

            if (value != null) {

                Property property = (Property) properties.get(name);

                if (property != null) {

                    property.setValue(value);
                }
                else {

                    throw new SAXNotRecognizedException("SAX property " +
                                                        name +
                                                        " not recognized.");
                }
            }
            else {

                throw new IllegalArgumentException(
                            "Null property value not permitted.");
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null property name not permitted.");
        }
    }

    /**
     *  Set an entity resolver.
     *  @param  resolver    An <code>EntityResolver</code>
     *          to be used when parsing input.
     *  @since  Version 0.2 2004-10-25
     */
    public final void setEntityResolver(final EntityResolver resolver) {

        if (resolver != null) {

            this.resolver = resolver;
        }
        else {

            throw new IllegalArgumentException(
                        "Null entity resolver not permitted.");
        }
    }

    /**
     *  Get the entity resolver.
     *  @return The current <code>EntityResolver</code>, or
     *          <code>null</code> if not set.
     *  @since  Version 0.2 2004-10-25
     */
    public final EntityResolver getEntityResolver() {

        return resolver;
    }

    /**
     *  Set a DTD handler.
     *  @param  handler A <code>DTDHandler</code>
     *          to be used when parsing input.
     *  @since  Version 0.2 2004-10-25
     */
    public final void setDTDHandler(final DTDHandler handler) {

        if (handler != null) {

            dtdHandler = handler;
        }
        else {

            throw new IllegalArgumentException(
                        "Null DTD handler not permitted.");
        }
    }

    /**
     *  Get the DTD handler.
     *  @return The current <code>DTDHandler</code>, or
     *          <code>null</code> if not set.
     *  @since  Version 0.2 2004-10-25
     */
    public final DTDHandler getDTDHandler() {

        return dtdHandler;
    }

    /**
     *  Set a content handler.
     *  @param  handler A <code>ContentHandler</code>
     *          to be used when parsing input.
     *  @since  Version 0.2 2004-10-25
     */
    public final void setContentHandler(final ContentHandler handler) {

        if (handler != null) {

            contHandler = handler;
        }
        else {

            throw new IllegalArgumentException(
                        "Null content handler not permitted.");
        }
    }

    /**
     *  Get the content handler.
     *  @return The current <code>ContentHandler</code>, or
     *          <code>null</code> if not set.
     *  @since  Version 0.2 2004-10-25
     */
    public final ContentHandler getContentHandler() {

        return contHandler;
    }

    /**
     *  Set an error handler.
     *  @param  handler An <code>ErrorHandler</code>
     *          to be used when parsing input.
     *  @since  Version 0.2 2004-10-25
     */
    public final void setErrorHandler(final ErrorHandler handler) {

        if (handler != null) {

            errorHandler = handler;
        }
        else {

            throw new IllegalArgumentException(
                        "Null error handler not permitted.");
        }
    }

    /**
     *  Get the error handler.
     *  @return The current <code>ErrorHandler</code>, or
     *          <code>null</code> if not set.
     *  @since  Version 0.2 2004-10-25
     */
    public final ErrorHandler getErrorHandler() {

        return errorHandler;
    }

    /**
     *  Parse input from a system ID.
     *  @param  systemId    A reference to the source to
     *          parse.
     *  @exception  IOException If there is a problem
     *          reading the input source.
     *  @exception  SAXException    If an error occurs
     *          parsing the input.
     *  @since  Version 0.2 2004-10-25
     */
    public final void parse(final String systemId)
        throws IOException, SAXException {

        if (systemId != null) {

            parse(new InputSource(systemId));
        }
        else {

            throw new IllegalArgumentException(
                        "Null system ID not permitted.");
        }
    }

    /**
     *  SAX parser feature.
     *  <p>
     *    <em>Feature support and feature values are true
     *    by default.</em>
     *  </p>
     *  @since  Version 0.1 2004-10-20
     */
    protected class Feature {

        /**
         *  The feature setting.
         *  @since  Version 0.1 2004-10-20
         */
        private boolean value = true;

        /**
         *  Whether the feature is mutable or accessible.
         *  @since  Version 0.1 2004-10-20
         */
        private boolean supported = true;

        /**
         *  Constructor with initial value.
         *  @param  value   The value of this feature.
         *  @since  Version 0.2 2004-10-25
         */
        protected Feature(final boolean value) {

            this.value = value;
        }

        /**
         *  Default constructor takes default true value.
         *  @since  Version 0.2 2004-10-25
         */
        protected Feature() {

            // Empty
        }

        /**
         *  Set the feature value.
         *  @param  value   The value of this feature.
         *  @exception  SAXNotSupportedException    If the
         *          feature is not supported; it is not
         *          mutable.
         *  @since  Version 0.1 2004-10-20
         */
        public final void setValue(final boolean value)
            throws SAXNotSupportedException {

            if (supported) {

                this.value = value;
            }
            else {

                throw new SAXNotSupportedException(
                            "This parser feature is not supported.");
            }
        }

        /**
         *  Get the feature value.
         *  @return The value of this feature.
         *  @exception  SAXNotSupportedException   If the
         *          feature value is not supported; it is
         *          not accessible.
         *  @since  Version 0.1 2004-10-20
         */
        public final boolean getValue()
            throws SAXNotSupportedException {

            if (supported) {

                return value;
            }
            else {

                throw new SAXNotSupportedException(
                            "This parser feature is not supported.");
            }
        }

        /**
         *  Control the accessibility of the feature.
         *  @param  supported   Whether this feature is
         *          supported.
         *  @since  Version 0.1 2004-10-20
         */
        protected final void setSupported(final boolean supported) {

            this.supported = supported;
        }

    }

    /**
     *  SAX parser property.
     *  <p>
     *    <em>Property support is true by default.</em> To
     *    create properties that are not supported, use the
     *    <code>setSupported</code> method.
     *  </p>
     *  @since  Version 0.2 2004-10-25
     */
    protected class Property {

        /**
         *  The property value.
         *  @since  Version 0.2 2004-10-25
         */
        private Object value = null;

        /**
         *  Whether the property is mutable or accessible.
         *  @since  Version 0.2 2004-10-25
         */
        private boolean supported = true;

        /**
         *  Constructor with initial object value.
         *  @param  value   The value of this feature.
         *  @since  Version 0.2 2004-10-25
         */
        protected Property(final Object value) {

            this.value = value;
        }

        /**
         *  Set the property value.
         *  @param  value   The value of this property.
         *  @exception  SAXNotSupportedException    If the
         *          feature is not supported; it is not
         *          mutable.
         *  @since  Version 0.2 2004-10-25
         */
        public final void setValue(final Object value)
            throws SAXNotSupportedException {

            if (supported) {

                this.value = value;
            }
            else {

                throw new SAXNotSupportedException(
                            "This parser property is not supported.");
            }
        }

        /**
         *  Get the property value.
         *  @return The value of this property.
         *  @exception  SAXNotSupportedException   If the
         *          property value is not supported; it is
         *          not accessible.
         *  @since  Version 0.2 2004-10-25
         */
        public final Object getValue()
            throws SAXNotSupportedException {

            if (supported) {

                return value;
            }
            else {

                throw new SAXNotSupportedException(
                            "This parser feature is not supported.");
            }
        }

        /**
         *  Control the accessibility of the property.
         *  @param  supported   Whether this property is
         *          supported.
         *  @since  Version 0.2 2004-10-25
         */
        protected final void setSupported(final boolean supported) {

            this.supported = supported;
        }

    }

}
