package com.mkdoc.sax;

/*
    Created:    2005-01-07
    Issued:     2005-01-13
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.model.BNode;
import org.openrdf.model.Graph;
import org.openrdf.model.URI;
import org.openrdf.model.Value;
import org.openrdf.model.ValueFactory;

import org.openrdf.model.impl.GraphImpl;

import com.mkdoc.schema.ApplicationProfile;

import com.mkdoc.store.StoreManager;

import java.net.URL;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 *  A Sesame repository writer for XHTML <code>meta</code>
 *  content.
 *  <p>
 *    Generates RDF <code>Graph</code> objects and adds
 *    them to a Sesame repository. Supports Dublin Core
 *    element and qualified terms schemata.
 *  </p>
 *  @version    0.8 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 *  @TODO   Reduce the complexity of the addTriples method.
 */
public class MetaStoreWriter extends AbstractStoreWriter
                             implements ContentHandler, ElementRdfWriter {

    /**
     *  The local name of the <code>meta</code> element.
     *  @since  Version 0.3 2005-02-15
     */
    public static final String META = "meta";

    /**
     *  Package private default constructor for composition, default schemas.
     *  @since  Version 0.3 2005-02-15
     */
    MetaStoreWriter() {

        super();
    }

    /**
     *  Package private constructor for custom application profile.
     *  @param  profile A custom application profile for the parse.
     *  @since  Version 0.6 2005-05-25
     */
    MetaStoreWriter(final ApplicationProfile profile) {

        super();

        if (profile == null) {

            throw new IllegalArgumentException(
                        "Null application profile not permitted.");
        }

        this.profile = profile;
    }

    /**
     *  Construct from a Sesame repository and initial URI.
     *  @param  manager A subject manager interface to store the
     *          RDF triples.
     *  @param  url An initial URI subject.
     *  @since  Version 0.1 2005-01-07
     */
    public MetaStoreWriter(final StoreManager manager,
                           final URL url) {

        super();

        if ((manager == null) ||
            (url == null)) {

            throw new IllegalArgumentException(
                        "Null arguments not permitted.");
        }

        this.manager = manager;

        graph = new GraphImpl();

        ValueFactory factory = graph.getValueFactory();

        subjectUri = factory.createURI(url.toExternalForm());
    }

    /**
     *  Report whether an instance accepts named element types.
     *  @param  uri The namespace URI for the document type.
     *  @param  localName   The local name of the element to check.
     *  @return <code>true</code> if this type is capable of handling the
     *          specified element.
     *  @since  Version 0.8 2005-02-15
     */
    public final boolean acceptElement(final String uri,
                                       final String localName) {

        if ((uri == null) ||
            (localName == null)) {

            throw new IllegalArgumentException(
                        "Null arguments not permitted.");
        }

        return ((uri.equals(NS_XHTML1)) &&
                (localName.equals(META)));
    }

    /**
     *  Extract <code>meta</code> element content.
     *  @param  uri The namespace URI for the element.
     *  @param  localName   The local element name.
     *  @param  qName   The qualified element name.
     *  @param  attributes    A set of attributes.
     *  @exception  SAXException    If the element is not a <code>meta</code>
     *          element or the instance is was not constructed with an output
     *          source and subject URI.
     *  @since  Version 0.1 2005-01-07
     */
    public final void startElement(final String uri,
                                   final String localName,
                                   final String qName,
                                   final Attributes attributes)
        throws SAXException {

        if (acceptElement(uri, localName)) {

            if ((subjectUri == null) ||
                (graph == null)) {

                throw new SAXException(
                            "Content handler not configured for this parse, " +
                            "has the start document event been called?");
            }

            addTriples(subjectUri, attributes, graph);
        }
        else {

            throw new SAXException("Cannot process element '" +
                                   qName + "'.");
        }
    }

    /**
     *  Add statements to an RDF graph for a source URI.
     *  @param  sourceUri   The URI for the source document.
     *  @param  attributes  The element attribute set to process.
     *  @param  graph   An RDF graph to add the statements to.
     *  @since  Version 0.1 2005-01-07
     */
    public final void addTriples(final URI sourceUri,
                                 final Attributes attributes,
                                 final Graph graph) {

        if ((sourceUri == null) ||
            (attributes == null) ||
            (graph ==  null)) {

            throw new IllegalArgumentException(
                        "Null arguments not permitted.");
        }

        int length = attributes.getLength();

        String name = null;

        String predicate = null;

        String object = null;

        String type = null;

        String value = null;

        for (int i = 0; i < length; i++) {

            name = attributes.getLocalName(i);

            value = getAttributeValue(attributes, i);

            if (name.equalsIgnoreCase("name")) {

                predicate = expandPredicate(value);
            }
            if (name.equalsIgnoreCase("content")) {

                object = value;
            }
            if (name.equalsIgnoreCase("scheme")) {

                type = expandScheme(value);
            }
        }
        if ((predicate != null) &&
            (isUri(predicate)) &&
            (object != null)) {

            ValueFactory factory = graph.getValueFactory();

            if (type == null) {

                graph.add(sourceUri,
                          factory.createURI(predicate),
                          getValue(object, factory));
            }
            else {

                BNode bNode = factory.createBNode(getBNodeId());

                graph.add(sourceUri,
                          factory.createURI(predicate),
                          bNode);

                graph.add(bNode,
                          factory.createURI(RDF_TYPE),
                          getValue(type, factory));

                graph.add(bNode,
                          factory.createURI(RDF_VALUE),
                          getValue(object, factory));

            }
        }
    }

    /**
     *  Get a <code>Value</code> type for an object.
     *  <p>
     *    Since values are only extracted from (X)HTML,
     *    and bNodes are generated explicitly, either
     *    <code>URI</code> or <code>Literal</code> types
     *    are returned.
     *  </p>
     *  @param  object  The triple object to type.
     *  @param  factory The factory from which to create the relevant value.
     *  @return A specific <code>Value</code> type representation of the
     *          object.
     *  @since  Version 0.1 2005-01-07
     */
    private Value getValue(final String object,
                           final ValueFactory factory) {

        if (isUri(object)) {

            return factory.createURI(object);
        }
        else {

            return factory.createLiteral(object);
        }
    }

}
