package com.mkdoc.sax;

/*
    Created:    2004-12-10
    Issued:     2004-12-17
    Modified:   2005-09-09

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.schema.ApplicationProfile;

import com.mkdoc.store.FileStoreManager;

import java.io.PrintWriter;

import java.net.URL;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 *  Triple writer for XHTML <code>meta</code> content.
 *  <p>
 *    Generates N-Triple style output <em>without</em>
 *    escaping for non-ASCII characters, see
 *    <a href="http://www.w3.org/TR/rdf-testcases/#ntriples">
 *    N-Triples</a>. If used for file output, should have a
 *    <code>.nt</code> extension. If used for HTTP output,
 *    should declare content type <code>text/plain</code>.
 *  </p>
 *  @version    1.3 2005-09-09
 *  @author     Philip Shaw <phil@mkdoc.com>
 *  @TODO   Reduce the complexity of the printTriples method.
 */
public class MetaTripleWriter extends AbstractTripleWriter
                           implements ContentHandler, ElementTripleWriter {

    /**
     *  The local name of the <code>meta</code> element.
     *  @since  Version 0.8 2005-02-15
     */
    public static final String META = "meta";

    /**
     *  Package private default constructor for composition, default schemas.
     *  @since  Version 0.8 2005-02-15
     */
    MetaTripleWriter() {

        // Empty
    }

    /**
     *  Package private constructor for custom application profile.
     *  @param  profile A custom application profile for the parse.
     *  @since  Version 1.1 2005-05-25
     */
    MetaTripleWriter(final ApplicationProfile profile) {

        if (profile != null) {

            this.profile = profile;
        }
        else {

            throw new IllegalArgumentException(
                        "Null application profile not permitted.");
        }
    }

    /**
     *  Construct with a subject manager and initial URI.
     *  @param  manager A subject manager interface to write the triples to.
     *  @param  url An initial URI subject.
     *  @since  Version 0.1 2004-12-10
     */
    public MetaTripleWriter(final FileStoreManager manager,
                            final URL url) {

        if (manager != null) {

            if (url != null) {

                this.manager = manager;

                subject = url.toExternalForm();
            }
            else {

                throw new IllegalArgumentException(
                            "Null URL not permitted.");
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null subject manager not permitted.");
        }
    }

    /**
     *  Report whether an instance accepts named element types.
     *  @param  uri The namespace URI for the document type.
     *  @param  localName   The local name of the element to check.
     *  @return <code>true</code> if this type is capable of handling the
     *          specified element.
     *  @since  Version 0.8 2005-02-15
     */
    public final boolean acceptElement(final String uri,
                                       final String localName) {

        if (uri != null) {

            if (localName != null) {

                return ((uri.equals(NS_XHTML1)) &&
                        (localName.equals(META)));
            }
            else {

                throw new IllegalArgumentException(
                            "Null local name not permitted.");
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null namespace URI not permitted.");
        }
    }

    /**
     *  Extract <code>meta</code> element content.
     *  @param  uri The namespace URI for the element.
     *  @param  localName   The local element name.
     *  @param  qName   The qualified element name.
     *  @param  attributes  A set of attributes.
     *  @exception  SAXException    If the element is not
     *          a <code>meta</code> element.
     *  @since  Version 0.1 2004-12-10
     */
    public final void startElement(final String uri,
                                   final String localName,
                                   final String qName,
                                   final Attributes attributes)
        throws SAXException {

        if (acceptElement(uri, localName)) {

            if ((subject != null) &&
                (out != null)) {

                printTriples(subject, attributes, out);
            }
            else {

                throw new SAXException(
                            "Content handler not configured for this parse, " +
                            "has the start document event been called?");
            }
        }
        else {

            out.close();

            throw new SAXException("Cannot process element '" +
                                   qName + "'.");
        }
    }

    /**
     *  Make a simple subject, predicate, object triple.
     *  <p>
     *    Outputs N-Triple comments for incomplete meta
     *    elements.
     *  </p>
     *  @param  sourceUri   The URI of the source document of the metadata.
     *  @param  attributes    The <code>link</code> element attributes.
     *  @param  printWriter The output stream for the triples.
     *  @since  Version 0.1 2004-12-10
     */
    public final void printTriples(final String sourceUri,
                                   final Attributes attributes,
                                   final PrintWriter printWriter) {

        if (sourceUri != null) {

            if (attributes != null) {

                if (printWriter != null) {

                    int length = attributes.getLength();

                    String name = null;

                    String predicate = null;

                    String object = null;

                    String type = null;

                    String value = null;

                    for (int i = 0; i < length; i++) {

                        name = attributes.getLocalName(i);

                        value = getAttributeValue(attributes, i);

                        if (name.equalsIgnoreCase("name")) {

                            predicate = expandPredicate(value);
                        }
                        if (name.equalsIgnoreCase("content")) {

                            object = value;
                        }
                        if (name.equalsIgnoreCase("scheme")) {

                            type = expandScheme(value);
                        }
                    }
                    if ((predicate != null) &&
                        (isUri(predicate)) &&
                        (object != null)) {

                        if (type != null) {

                            String bNode = getBNode();

                            printWriter.println(" " + escape(sourceUri) +
                                                " " + escape(predicate) +
                                                " " + bNode + " .");

                            printWriter.println(" " + bNode +
                                                " <" + RDF_TYPE + ">" +
                                                " " + escape(type) + " .");

                            printWriter.println(" " + bNode +
                                                " <" + RDF_VALUE + ">" +
                                                " " + escape(object) + " .");
                        }
                        else {

                            printWriter.println(" " + escape(sourceUri) +
                                                " " + escape(predicate) +
                                                " " + escape(object) + " .");
                        }
                    }
                }
                else {

                    throw new IllegalArgumentException(
                                "Null output stream not permitted.");
                }
            }
            else {

                throw new IllegalArgumentException(
                            "Null attributes not permitted.");
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null source URI not permitted.");
        }
    }

}
