package com.mkdoc.sax;

/*
    Created:    2005-02-16
    Issued:     2005-02-16
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.model.ValueFactory;

import org.openrdf.model.impl.GraphImpl;

import com.mkdoc.store.StoreManager;

import java.net.URL;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;

/**
 *  A Sesame repository writer for XHTML <code>meta</code> and
 *  <code>link</code> element content.
 *  <p>
 *    Generates RDF <code>Graph</code> objects and adds
 *    them to a Sesame repository. Supports Dublin Core
 *    element and qualified terms schemata.
 *  </p>
 *  @version    0.3 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class XhtmlStoreWriter extends AbstractStoreWriter
                              implements ContentHandler {

    /**
     *  A handler for XHTML <code>meta</code> elements.
     *  @since  Version 0.1 2005-02-16
     */
    private final ElementRdfWriter metaWriter = new MetaStoreWriter(profile);

    /**
     *  A handler for XHTML <code>link</code> elements.
     *  @since  Version 0.1 2005-02-16
     */
    private final ElementRdfWriter linkWriter = new LinkStoreWriter(profile);

    /**
     *  Construct from a Sesame repository and initial URI.
     *  @param  manager A subject manager interface to store the RDF triples.
     *  @param  url An initial URI subject.
     *  @since  Version 0.1 2005-02-16
     */
    public XhtmlStoreWriter(final StoreManager manager,
                            final URL url) {

        super();

        if ((manager == null) ||
            (url == null)) {

            throw new IllegalArgumentException(
                        "Null arguments not permitted.");
        }

        this.manager = manager;

        graph = new GraphImpl();

        ValueFactory factory = graph.getValueFactory();

        subjectUri = factory.createURI(url.toExternalForm());
    }

    /**
     *  Extract <code>meta</code> and <code>link</code> element content.
     *  @param  uri The namespace URI for the element.
     *  @param  localName   The local element name.
     *  @param  qName   The qualified element name.
     *  @param  attributes    A set of attributes.
     *  @exception  SAXException    If the element is not a <code>meta</code>
     *          or <code>link</code> element.
     *  @since  Version 0.1 2005-02-16
     */
    public final void startElement(final String uri,
                                   final String localName,
                                   final String qName,
                                   final Attributes attributes)
        throws SAXException {

        if (metaWriter.acceptElement(uri, localName)) {

            metaWriter.addTriples(subjectUri, attributes, graph);
        }
        else {

            if (linkWriter.acceptElement(uri, localName)) {

                linkWriter.addTriples(subjectUri, attributes, graph);
            }
            else {

                throw new SAXException("Cannot process element '" +
                                       qName + "'.");
            }
        }
    }

}
