package com.mkdoc.schema;

/*
    Created:    2005-05-24
    Issued:     2005-05-27
    Modified:   2005-06-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Hashtable;
import java.util.Map;

/**
 *  A base implementation of application profile methods.
 *  @version    0.4 2005-06-28
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public abstract class AbstractApplicationProfile {

    /**
     *  The Dublin Core metadata scheme prefix.
     *  @since  Version 0.2 2005-05-27
     */
    public static final String DCTERMS = "dcterms";

    /**
     *  A number of schemes that may be handled by a profile.
     *  <p>
     *    Schemas are stored by their encoding prefix in lower case. For
     *    example, metadata with the Dublin Core schema prefix &quot;DC&quot;
     *    are matched by the key &quot;dc&quot;.
     *  </p>
     *  @since  Version 0.1 2005-05-25
     */
    protected Map schemas = new Hashtable();

    /**
     *  Expand a schema encoded value to a URI.
     *  @param  predicate   The encoded triple predicate to
     *          expand in the form &quot;DC.Title&quot; for
     *          instance.
     *  @return The expanded URI reference for the triple
     *          predicate or the original string if it is
     *          not matched.
     *  @since  Version 0.1 2005-05-25
     */
    public final String expandPredicate(final String predicate) {

        if (predicate != null) {

            int dotIndex = predicate.indexOf(".");

            if (dotIndex > -1) {

                String prefix = predicate.substring(0, dotIndex);

                if (hasPrimarySchema(prefix)) {

                    Schema schema = getPrimarySchema(prefix);

                    String expanded = schema.expandPredicate(predicate);

                    if (!expanded.equals(predicate)) {

                        return expanded;
                    }
                    else {

                        return expandPredicateProfile(predicate);
                    }
                }
                // Fall through
            }
            // Fall through

            return predicate;
        }
        else {

            throw new IllegalArgumentException(
                        "Null predicate not permitted.");
        }
    }

    /**
     *  Template method for falling back to profile-specific algorithms.
     *  @param  predicate   A predicate that has not been matched using default
     *          schema prefixes.
     *  @return A predicate URI or <code>null</code> if not matched.
     *  @since  Version 0.2 2005-05-27
     */
    protected abstract String expandPredicateProfile(final String predicate);

    /**
     *  Get the primary schema signified by a metadata prefix.
     *  @param  prefix  The prefix for the metadata element.
     *  @return <code>true</code> if the profile includes a schema with this
     *          default prefix.
     *  @since  Version 0.2 2005-05-27
     */
    protected final boolean hasPrimarySchema(final String prefix) {

        if (prefix != null) {

            String key = prefix.toLowerCase();

            synchronized (schemas) {

                return schemas.containsKey(key);
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null metadata prefix not permitted.");
        }
    }

    /**
     *  Get the primary schema indicated by a metadata prefix.
     *  @param  prefix  The prefix for the metadata element.
     *  @return The schema whose default prefix matches this or
     *          <code>null</code> if not found.
     *  @since  Version 0.2 2005-05-27
     */
    protected final Schema getPrimarySchema(final String prefix) {

        if (prefix != null) {

            String key = prefix.toLowerCase();

            synchronized (schemas) {

                return (Schema) schemas.get(key);
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null metadata prefix not permitted.");
        }
    }

    /**
     *  Expand an encoding scheme to a URI.
     *  @param  scheme  The encoded triple component to
     *          expand in the form
     *          &quot;DCTERMS.W3CDTF&quot; for instance.
     *  @return The expanded URI reference for the triple
     *          component or the original string if it is
     *          not matched.
     *  @since  Version 0.1 2005-05-25
     */
    public final String expandScheme(final String scheme) {

        if (scheme != null) {

            int dotIndex = scheme.indexOf(".");

            if (dotIndex > -1) {

                String prefix = scheme.substring(0, dotIndex);

                if (hasPrimarySchema(prefix)) {

                    Schema schema = getPrimarySchema(prefix);

                    return schema.expandScheme(scheme);
                }
                else {

                    return expandSchemeProfile(scheme);
                }
            }
            else {

                return expandSchemeProfile(scheme);
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null scheme not permitted.");
        }
    }

    /**
     *  Template method for falling back to profile-specific algorithms.
     *  @param  scheme  A scheme that has not been matched using default
     *          schema prefixes.
     *  @return A scheme URI or <code>null</code> if not matched.
     *  @since  Version 0.2 2005-05-27
     */
    protected abstract String expandSchemeProfile(final String scheme);


    /**
     *  Add a schema to the handler.
     *  @param  schema  The schema instance to be added.
     *  @since  Version 1.0 2005-05-25
     */
    protected final void addSchema(final Schema schema) {

        if (schema != null) {

            synchronized (schemas) {

                String prefix = schema.getDefaultPrefix();

                schemas.put(prefix.toLowerCase(), schema);
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null schema not permitted.");
        }
    }

    /**
     *  Get a mapping of schemas with their default prefixes.
     *  @return A map in which lower case default schema prefixes are keys to
     *          the schemas themselves.
     *  @since  Version 0.7 2005-06-10
     */
    public final Map getSchemaMap() {

        synchronized (schemas) {

            return new Hashtable(schemas);
        }
    }

    /**
     *  Get the schema property for a given predicate.
     *  @param  predicate   The predicate name with schema prefix
     *  @return The schema property for the predicate or <code>null</code> if no
     *          mapping exists.
     *  @since  Version 0.7 2005-06-10
     */
    public final SchemaProperty getPredicateProperty(final String predicate) {

        if (predicate != null) {

            int dotIndex = predicate.indexOf(".");

            if (dotIndex > -1) {

                String prefix = predicate.substring(0, dotIndex);

                if (hasPrimarySchema(prefix)) {

                    Schema schema = getPrimarySchema(prefix);

                    String name = predicate.substring(dotIndex + 1);

                    return schema.getNamedPredicateProperty(name);
                }
                // Fall through
            }
            // Fall through

            return null;
        }
        else {

            throw new IllegalArgumentException(
                        "Null predicate not permitted.");
        }
    }


}
