package com.mkdoc.schema;

/*
    Created:    2005-06-15
    Issued:     2005-06-15
    Modified:   2005-10-19

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Locale;

/**
 *  A base class for schema resource types.
 *  @version    0.7 2005-10-19
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public abstract class AbstractSchemaProperty {

    /**
     *  The resource URI.
     *  @since  Version 0.1 2005-06-15
     */
    protected String uri;

    /**
     *  The resource label.
     *  @since  Version 0.1 2005-06-15
     */
    protected String label;

    /**
     *  The unique resource name.
     *  @since  Version 0.1 2005-06-15
     */
    protected String name;

    /**
     *  Whether the property has encoding schemes that may apply to it.
     *  @since  Version 0.5 2005-09-14
     */
    protected boolean encoding = false;

    /**
     *  Whether the property may have a URI encoding schemes.
     *  @since  Version 0.6 2005-09-14
     */
    protected boolean uriEncoding = false;

    /**
     *  Get the resource URI of the class.
     *  @return The URI of the class.
     *  @since  Version 0.1 2005-06-15
     */
    public final String getUri() {

        return uri;
    }

    /**
     *  Get the human-readable label for the class.
     *  @return The human-readable label for the class.
     *  @since  Version 0.1 2005-06-15
     */
    public final String getLabel() {

        return label;
    }

    /**
     *  Get a unique &quot;camel case&quot; name for the class.
     *  <p>
     *    This often, but not always, coincides with the URI suffix for the
     *    type.
     *  </p>
     *  @return A name that is unique within a given schema.
     */
    public final String getName() {

        return name;
    }

    /**
     *  Get the qualified name of the class, with namespace prefix.
     *  @return The qualified name of the class with a namespace prefix.
     *  @since  Version 0.2 2005-06-16
     */
    public final String getQualifiedName() {

        return getNamespacePrefix() + ":" + getName();
    }

    /**
     *  Get a unique name varible for query handling.
     *  @return A unique name variable for the class.
     *  @since  Version 0.2 2005-06-16
     */
    public final String getNameVariable() {

        String basicName = getName();

        String initial = basicName.substring(0, 1);

        return getNamespacePrefix() +
               initial.toUpperCase(getLocale()) +
               basicName.substring(1);
    }

    /**
     *  Get a unique varible name for extended bNode queries.
     *  @return A unique bNode variable name for the class.
     *  @since  Version 0.4 2005-09-13
     */
    public final String getBNodeVariable() {

        return getNameVariable() + "BNode";
    }

    /**
     *  Get the namespace prefix for the class.
     *  @return The namespace prefix for the class.
     *  @since  Version 0.2 2005-06-16
     */
    public abstract String getNamespacePrefix();

    /**
     *  Get a unique parameter name for HTML form handling.
     *  @return A unique parameter name for the class.
     *  @since  Version 0.3 2005-06-17
     */
    public final String getParameterName() {

        String basicName = getName();

        return getNamespacePrefix() +
               "." +
               basicName;
    }

    /**
     *  Check whether a schema property may have an encoding scheme.
     *  @return <code>true</code> if the property may have an encoding scheme.
     *  @since  Version 0.4 2005-09-14
     */
    public final boolean hasEncodingScheme() {

        return encoding;
    }

    /**
     *  Check whether a schema property may have a URI encoding scheme.
     *  @return <code>true</code> if the property may have a URI encoding
     *          scheme.
     *  @since  Version 0.6 2005-09-15
     */
    public final boolean hasUriEncodingScheme() {

        return uriEncoding;
    }

    /**
     *  Check equivalance of properties.
     *  @param  other   The object to compare with this.
     *  @return <code>true</code> if the other object is equivalent to this.
     *  @since  Version 0.3 2005-06-30
     */
    public abstract boolean equals(final Object other);

    /**
     *  Get a distinct hash code value.
     *  @return A distinct hash code value.
     *  @since  Version 0.3 2005-06-30
     */
    public abstract int hashCode();

    /**
     *  Get the locale of the property for case conversions, etc.
     *  @return The locale for the schema property.
     *  @since  Version 0.7 2005-10-19
     */
    public abstract Locale getLocale();

}
