package com.mkdoc.schema;

/*
    Created:    2005-02-24
    Issued:     2005-02-28
    Modified:   2005-09-15

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Collection;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Map;

/**
 *  The UK e-Government Metadtata Standard (e-GMS) schema.
 *  <p>
 *    This implementation is based on the application profile specified by the
 *    e-GMS  Application Profile version 1:
 *  http://www.govtalk.gov.uk/schemasstandards/metadata_document.asp?docnum=805
 *  </p>
 *  @version    1.0 2005-09-15
 *  @author     Philip Shaw <phil@mkdoc.com>
 *  @TODO   Update to e-GMS 3.0 changes (section 1.1), specifically:
 *          > GDSC encoding scheme.
 *          > PRONOM encoding scheme.
 *          > Verify specification for Case ID element refinement.
 *          > Verify specification for addresee copy element refinement.
 *          These details are described but not covered by any application
 *          profile. Current draft case ID and addressee copy refinements based
 *          on the unofficial schema at:
 *          http://www.esd.org.uk/standards/eGMS/egms.xml
 *          http://www.esd.org.uk/standards/egms/3.0/egms-schema/#
 */
public class UKeGMS extends AbstractSchema implements Schema {

    /**
     *  The common name of the schema.
     *  @since  Version 0.9 2005-06-17
     */
    public static final String NAME = "UK e-GMS Terms";

    /**
     *  The base URI for the e-GMS element set.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String BASE = "http://www.govtalk.gov.uk/terms/";

    /**
     *  The default namespace prefix for the e-GMS schema.
     *  @since  Version 0.3 2005-03-01
     */
    public static final String[] PREFIXES = new String[] {"eGMS",
                                                          "DC"};


/* ELEMENTS ---------------------------------------------------------------- */

    /**
     *  The Accessibility element URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_ACCESS = "accessibility";

    /**
     *  The Accessibility element label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_ACCESS = "accessibility";

    /**
     *  The Addressee element URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_ADDREE = "addressee";

    /**
     *  The Addressee element label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_ADDREE = "addressee";

    /**
     *  The Aggregation element label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_AGGRTN = "aggregation";

    /**
     *  The Aggregation element URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_AGGRTN = "aggregation";

    /**
     *  The Digital Signature element URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DIGSIG = "digitalSignature";

    /**
     *  The Digital Signature element label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DIGSIG = "digitalSignature";

    /**
     *  The Disposal element URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DISPOS = "disposal";

    /**
     *  The Disposal element label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DISPOS = "disposal";

    /**
     *  The Location element URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_LOCATN = "location";

    /**
     *  The Location element label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_LOCATN = "location";

    /**
     *  The Mandate element URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_MANDAT = "mandate";

    /**
     *  The Mandate element label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_MANDAT = "mandate";

    /**
     *  The Preservation element URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_PRESRV = "preservation";

    /**
     *  The Preservation element label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_PRESRV = "preservation";

    /**
     *  The Status element URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_STATUS = "status";

    /**
     *  The Status element label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_STATUS = "status";

/* REFINEMENTS ------------------------------------------------------------- */

    /**
     *  The Addressee.addresseeCopy refinement name.
     *  <p>
     *    Described in e-GMS, but not defined in application profile version 1.
     *  </p>
     *  @since  Version 0.2 2005-02-28
     */
    public static final String NAME_ADDCPY = "addresseeCopy";

    /**
     *  The Addressee.addresseeCopy refinement URI suffix.
     *  <p>
     *    Described in e-GMS, but not defined in application profile version 1.
     *  </p>
     *  @since  Version 0.2 2005-02-28
     */
    public static final String SUFF_ADDCPY = "addresseeAddresseeCopy";

    /**
     *  The Date acquired refinement name.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String SUFF_ACQUIR = "dateAcquired";

    /**
     *  The Date.Acquired refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_ACQUIR = "acquired";

    /**
     *  The &quot;cut-off date&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_CUTOFF = "cutOffDate";

    /**
     *  The Date &quot;declared&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DECLAR = "dateDeclared";

    /**
     *  The Date &quot;closed&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_CLOSED = "dateClosed";

    /**
     *  The Date &quot;next version due&quot; refinement name and URI suffix.
     *  <p>
     *    Note, the application profile has spaces between the words, which is
     *    assumed to by a setting error.
     *  </p>
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_VERDUE = "nextVersionDue";

    /**
     *  The Date &quot;updating frequency&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_UPFREQ = "updatingFrequency";

    /**
     *  The Disposal &quot;action&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DACTON = "disposalAction";

    /**
     *  The Disposal &quot;auto remove date&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_AUTREM = "disposalAutoRemoveDate";

    /**
     *  The Disposal &quot;conditions&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DCONDT = "disposalConditions";

    /**
     *  The Disposal &quot;time period&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DTIMEP = "disposalTimePeriod";

    /**
     *  The Disposal &quot;date&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DDATED = "disposalDate";

    /**
     *  The Disposal &quot;date of last review&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DLASTR = "disposalDateOfLastReview";

    /**
     *  The Disposal &quot;authorised by&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DAUTHB = "disposalAuthorisedBy";

    /**
     *  The Disposal &quot;comment&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DCOMMT = "disposalComment";

    /**
     *  The Disposal &quot;schedule ID&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DSCHID = "disposalScheduleId";

    /**
     *  The Disposal &quot;export destination&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DEXDES = "disposalExportDestination";

    /**
     *  The Disposal &quot;export status&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DEXSTA = "disposalExportStatus";

    /**
     *  The Disposal &quot;review&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DREVEW = "disposalReview";

    /**
     *  The Disposal &quot;review details&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DREVDT = "disposalReviewDetails";

    /**
     *  The Disposal &quot;reviewer details&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DREVWR = "disposalReviewerDetails";

    /**
     *  The file plan ID refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_FIPLID = "identifierFilePlanID";

    /**
     *  The system ID refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_SYSTID = "identifierSystemID";

    /**
     *  The case ID refinement name and URI suffix.
     *  <p>
     *    Described in e-GMS, but not defined in application profile version 1.
     *  </p>
     *  @since  Version 0.2 2005-02-28
     */
    public static final String NAME_CASEID = "identifierCaseID";

    /**
     *  The Location &quot;current location&quot; name.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_CURLOC = "locationCurrent";

    /**
     *  The Location &quot;current location&quot; refinement URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String SUFF_CURLOC = "locationCurrentLocation";

    /**
     *  The Location &quot;home location&quot; name.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_HOMLOC = "locationHome";

    /**
     *  The Location &quot;home location&quot; refinement URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String SUFF_HOMLOC = "locationHomeLocation";

    /**
     *  The Mandate &quot;authorising statute&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_AUTSTA = "mandateAuthorisingStatute";

    /**
     *  The Mandate &quot;data protection exempt category&quot; refinement
     *  name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DPEXMT = "dataProtectionExemptCategory";

    /**
     *  The Mandate &quot;personal data acquisition purpose&quot; refinement
     *  name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DPURPS = "personalDataAcquisitionPurpose";

    /**
     *  The Preservation &quot;original format&quot; name.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_OGNFMT = "originalFormat";

    /**
     *  The Preservation &quot;original format&quot; refinement URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String SUFF_OGNFMT = "preservationOriginalFormat";

    /**
     *  The Relation &quot;is defined by&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DEFNBY = "isDefinedBy";

    /**
     *  The Relation &quot;provides definition of&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_PRODEF = "providesDefinitionOf";

    /**
     *  The Relation &quot;reason for redaction&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_REARED = "reasonForRedaction";

    /**
     *  The Relation &quot;redaction&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_REDACT = "redaction";

    /**
     *  The deprecated Relation &quot;sequence no&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_SEQNNO = "sequenceNo";

    /**
     *  The Rights &quot;copyright&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_COPYRT = "rightsCopyright";

    /**
     *  The Rights &quot;custodian&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_CUSTDN = "rightsCustodian";

    /**
     *  The Rights &quot;descriptor&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DESCTR = "rightsDescriptor";

    /**
     *  The deprecated Rights &quot;disclosability to DPA data subject&quot;
     *  refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_DISCTS = "disclosabilityToDPADataSubject";

    /**
     *  The Rights &quot;DPA data subject access exemption&quot; refinement
     *  name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_SUBXMT = "DPADataSubjectAccessExemption";

    /**
     *  The Rights &quot;EIR disclosability indicator&quot; refinement name and
     *  URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_EIRDIS = "EIRDisclosabilityIndicator";

    /**
     *  The Rights &quot;EIR exemption&quot; refinement name and
     *  URI suffix.
     *  @since  Version 0.4 2005-03-11
     */
    public static final String NAME_EIRXMT = "EIRExemption";

    /**
     *  The Rights &quot;FOI disclosability indicator&quot; refinement name and
     *  URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_FOIDIS = "FOIDisclosabilityIndicator";

    /**
     *  The Rights &quot;FOI exemption&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_FOIXMT = "FOIExemption";

    /**
     *  The Rights &quot;FOI release details&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_FOIREL = "FOIReleaseDetails";

    /**
     *  The Rights &quot;FOI release date&quot; refinement name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_FOIDAT = "FOIReleaseDate";

    /**
     *  The Rights &quot;last FOI disclosability review&quot; refinement name
     *  and URI suffix.
     *  @since  Version 0.4 2005-03-11
     */
    public static final String NAME_FOIREV = "lastFOIDisclosabilityReview";

    /**
     *  The Rights &quot;group access&quot; name.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_GRPACC = "groupAccess";

    /**
     *  The Rights &quot;group access&quot; refinement URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String SUFF_GRPACC = "rightsGroupAccess";

    /**
     *  The Rights &quot;individual user access list&quot; name.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_ACCLST = "individualUserAccessList";

    /**
     *  The Rights &quot;individual user access list&quot; refinement URI
     *  suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String SUFF_ACCLST = "rightsIndividualUserAccessList";

    /**
     *  The Rights &quot;last FOIA disclosability review&quot; refinement name
     *  and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_FOILDR = "lastFoiaDisclosabilityReview";

    /**
     *  The Rights &quot;previous protective marking&quot; refinement name and
     *  URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_PREMRK = "previousProtectiveMarking";

    /**
     *  The Rights &quot;protective marking&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_PROMRK = "protectiveMarking";

    /**
     *  The Rights &quot;protective marking change date&quot; refinement name
     *  and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_CHGDAT = "protectiveMarkingChangeDate";

    /**
     *  The Rights &quot;protective marking expiry date&quot; refinement name
     *  and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_EXPDAT = "protectiveMarkingExpiryDate";

    /**
     *  The Subject &quot;category&quot; refinement name.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_CATGRY = "category";

    /**
     *  The Subject &quot;category&quot; refinement URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String SUFF_CATGRY = "subjectCategory";

    /**
     *  The Subject &quot;keyword&quot; refinement name.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_KEYWRD = "keyword";

    /**
     *  The Subject &quot;keyword&quot; refinement URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String SUFF_KEYWRD = "subjectKeyword";

    /**
     *  The Subject &quot;process identifier&quot; refinement name.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_PROCID = "processIdentifier";

    /**
     *  The Subject &quot;process identifier&quot; refinement URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String SUFF_PROCID = "subjectProcessIdentifier";

    /**
     *  The Subject &quot;programme&quot; refinement name.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_PROGRM = "programme";

    /**
     *  The Subject &quot;programme&quot; refinement URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String SUFF_PROGRM = "subjectProgramme";

    /**
     *  The Subject &quot;project&quot; refinement name.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_PROJCT = "project";

    /**
     *  The Subject &quot;project&quot; refinement URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String SUFF_PROJCT = "subjectProject";

    /**
     *  The Addressee.addresseeCopy refinement name.
     *  <p>
     *    Described in e-GMS, but not defined in application profile version 1.
     *  </p>
     *  @since  Version 0.2 2005-02-28
     */
    public static final String LABL_ADDCPY = "addresseeCopy";

    /**
     *  The Date.Acquired refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_ACQUIR = "acquired";

    /**
     *  The &quot;cut-off date&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_CUTOFF = "cutOffDate";

    /**
     *  The Date &quot;declared&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DECLAR = "dateDeclared";

    /**
     *  The Date &quot;closed&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_CLOSED = "dateClosed";

    /**
     *  The Date &quot;next version due&quot; refinement name and label.
     *  <p>
     *    Note, the application profile has spaces between the words, which is
     *    assumed to by a setting error.
     *  </p>
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_VERDUE = "nextVersionDue";

    /**
     *  The Date &quot;updating frequency&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_UPFREQ = "updatingFrequency";

    /**
     *  The Disposal &quot;action&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DACTON = "disposalAction";

    /**
     *  The Disposal &quot;auto remove date&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_AUTREM = "disposalAutoRemoveDate";

    /**
     *  The Disposal &quot;conditions&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DCONDT = "disposalConditions";

    /**
     *  The Disposal &quot;time period&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DTIMEP = "disposalTimePeriod";

    /**
     *  The Disposal &quot;date&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DDATED = "disposalDate";

    /**
     *  The Disposal &quot;date of last review&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DLASTR = "disposalDateOfLastReview";

    /**
     *  The Disposal &quot;authorised by&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DAUTHB = "disposalAuthorisedBy";

    /**
     *  The Disposal &quot;comment&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DCOMMT = "disposalComment";

    /**
     *  The Disposal &quot;schedule ID&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DSCHID = "disposalScheduleId";

    /**
     *  The Disposal &quot;export destination&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DEXDES = "disposalExportDestination";

    /**
     *  The Disposal &quot;export status&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DEXSTA = "disposalExportStatus";

    /**
     *  The Disposal &quot;review&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DREVEW = "disposalReview";

    /**
     *  The Disposal &quot;review details&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DREVDT = "disposalReviewDetails";

    /**
     *  The Disposal &quot;reviewer details&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DREVWR = "disposalReviewerDetails";

    /**
     *  The file plan ID refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_FIPLID = "identifierFilePlanID";

    /**
     *  The system ID refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_SYSTID = "identifierSystemID";

    /**
     *  The case ID refinement name and label.
     *  <p>
     *    Described in e-GMS, but not defined in application profile version 1.
     *  </p>
     *  @since  Version 0.2 2005-02-28
     */
    public static final String LABL_CASEID = "identifierCaseID";

    /**
     *  The Location &quot;current location&quot; name.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_CURLOC = "locationCurrent";

    /**
     *  The Location &quot;home location&quot; name.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_HOMLOC = "locationHome";

    /**
     *  The Mandate &quot;authorising statute&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_AUTSTA = "mandateAuthorisingStatute";

    /**
     *  The Mandate &quot;data protection exempt category&quot; refinement
     *  name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DPEXMT = "dataProtectionExemptCategory";

    /**
     *  The Mandate &quot;personal data acquisition purpose&quot; refinement
     *  name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DPURPS = "personalDataAcquisitionPurpose";

    /**
     *  The Preservation &quot;original format&quot; name.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_OGNFMT = "originalFormat";

    /**
     *  The Relation &quot;is defined by&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DEFNBY = "isDefinedBy";

    /**
     *  The Relation &quot;provides definition of&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_PRODEF = "providesDefinitionOf";

    /**
     *  The Relation &quot;reason for redaction&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_REARED = "reasonForRedaction";

    /**
     *  The Relation &quot;redaction&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_REDACT = "redaction";

    /**
     *  The deprecated Relation &quot;sequence no&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_SEQNNO = "sequenceNo";

    /**
     *  The Rights &quot;copyright&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_COPYRT = "rightsCopyright";

    /**
     *  The Rights &quot;custodian&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_CUSTDN = "rightsCustodian";

    /**
     *  The Rights &quot;descriptor&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DESCTR = "rightsDescriptor";

    /**
     *  The deprecated Rights &quot;disclosability to DPA data subject&quot;
     *  refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_DISCTS = "disclosabilityToDPADataSubject";

    /**
     *  The Rights &quot;DPA data subject access exemption&quot; refinement
     *  name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_SUBXMT = "DPADataSubjectAccessExemption";

    /**
     *  The Rights &quot;EIR disclosability indicator&quot; refinement name and
     *  label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_EIRDIS = "EIRDisclosabilityIndicator";

    /**
     *  The Rights &quot;EIR exemption&quot; refinement name and
     *  label.
     *  @since  Version 0.4 2005-03-11
     */
    public static final String LABL_EIRXMT = "EIRExemption";

    /**
     *  The Rights &quot;FOI disclosability indicator&quot; refinement name and
     *  label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_FOIDIS = "FOIDisclosabilityIndicator";

    /**
     *  The Rights &quot;FOI exemption&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_FOIXMT = "FOIExemption";

    /**
     *  The Rights &quot;FOI release details&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_FOIREL = "FOIReleaseDetails";

    /**
     *  The Rights &quot;FOI release date&quot; refinement name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_FOIDAT = "FOIReleaseDate";

    /**
     *  The Rights &quot;last FOI disclosability review&quot; refinement name
     *  and label.
     *  @since  Version 0.4 2005-03-11
     */
    public static final String LABL_FOIREV = "lastFOIDisclosabilityReview";

    /**
     *  The Rights &quot;group access&quot; name.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_GRPACC = "groupAccess";

    /**
     *  The Rights &quot;individual user access list&quot; name.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_ACCLST = "individualUserAccessList";

    /**
     *  The Rights &quot;last FOIA disclosability review&quot; refinement name
     *  and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_FOILDR = "lastFoiaDisclosabilityReview";

    /**
     *  The Rights &quot;previous protective marking&quot; refinement name and
     *  label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_PREMRK = "previousProtectiveMarking";

    /**
     *  The Rights &quot;protective marking&quot; refinement name and URI
     *  suffix.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_PROMRK = "protectiveMarking";

    /**
     *  The Rights &quot;protective marking change date&quot; refinement name
     *  and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_CHGDAT = "protectiveMarkingChangeDate";

    /**
     *  The Rights &quot;protective marking expiry date&quot; refinement name
     *  and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_EXPDAT = "protectiveMarkingExpiryDate";

    /**
     *  The Subject &quot;category&quot; refinement name.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_CATGRY = "category";

    /**
     *  The Subject &quot;keyword&quot; refinement name.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_KEYWRD = "keyword";

    /**
     *  The Subject &quot;process identifier&quot; refinement name.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_PROCID = "processIdentifier";

    /**
     *  The Subject &quot;programme&quot; refinement name.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_PROGRM = "programme";

    /**
     *  The Subject &quot;project&quot; refinement name.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_PROJCT = "project";

/* ENCODING SCHEMES -------------------------------------------------------- */

    /**
     *  The e-GMS Audience Encoding Scheme (AES) scheme name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_GMSAES = "egmsaes";

    /**
     *  The e-GMS Audience Encoding Scheme (AES) scheme name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_GMSAES = "egmsaes";

    /**
     *  The e-GMS Type Encoding Scheme (TES) scheme name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_GMSTES = "egmstes";

    /**
     *  The e-GMS Type Encoding Scheme (TES) scheme name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_GMSTES = "egmstes";

    /**
     *  The Educational Resources Information Centre scheme name and URI
     *  suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_ERICEN = "ERIC";

    /**
     *  The Educational Resources Information Centre scheme name and URI
     *  suffix.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_ERICEN = "ERIC";

    /**
     *  The Government Category List scheme name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_GCLENC = "GCL";

    /**
     *  The Government Category List scheme name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_GCLENC = "GCL";

    /**
     *  The IEEE Learning Object Metadata scheme name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_IEELOM = "IEEELOM";

    /**
     *  The IEEE Learning Object Metadata scheme name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_IEELOM = "IEEELOM";

    /**
     *  The Government Data Standards Catalogue scheme name and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_GDSCAT = "GovernmentDataStandardsCatalogue";

    /**
     *  The Government Data Standards Catalogue scheme name and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_GDSCAT = "GovernmentDataStandardsCatalogue";

    /**
     *  The Office of National Statistics Standard Names and Code scheme name
     *  and URI suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_ONSSNC = "ONSSNAC";

    /**
     *  The Office of National Statistics Standard Names and Code scheme name
     *  and label.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_ONSSNC = "ONSSNAC";

    /**
     *  The W3C Web Accessibility Initiative ratings scheme name and URI
     *  suffix.
     *  @since  Version 0.1 2005-02-24
     */
    public static final String NAME_W3CWAI = "W3CWAI";

    /**
     *  The W3C Web Accessibility Initiative ratings scheme name and URI
     *  suffix.
     *  @since  Version 0.7 2005-06-16
     */
    public static final String LABL_W3CWAI = "W3CWAI";

    /**
     *  The &quot;yes/no&quot; scheme name and URI suffix.
     *  @since  Version 0.4.1 2005-03-14
     */
    public static final String NAME_YES_NO = "YesNo";

    /**
     *  The &quot;yes/no&quot; scheme name and label.
     *  @since  Version 0.4.1 2005-03-14
     */
    public static final String LABL_YES_NO = "YesNo";

    /**
     *  A mapping between encoded metadata names and their URI.
     *  <p>
     *    Encoded names are stored in lower case without their schema prefixes.
     *  </p>
     *  @since  Version 0.1 2005-02-24
     */
    private static final Map PREDICATES = new Hashtable();

    /**
     *  A mapping between encoded metadata SCHEMES and their URI.
     *  <p>
     *    Encoded names are stored in lower case without their schema prefixes.
     *  </p>
     *  @since  Version 0.1 2005-02-24
     */
    private static final Map SCHEMES = new Hashtable();

    /**
     *  Populate the encoded metadata mapping.
     *  <p>
     *    In many cases the URI suffix is the same as the encoded metadata
     *    name and the same is used for the URI part.
     *  </p>
     *  @since  Version 0.1 2005-02-24
     */
    static {

        // Elements
        addPredicate(newSchemaProperty(BASE + NAME_ACCESS,
                                       LABL_ACCESS,
                                       NAME_ACCESS,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_ADDREE,
                                       LABL_ADDREE,
                                       NAME_ADDREE,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_AGGRTN,
                                       LABL_AGGRTN,
                                       NAME_AGGRTN,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DIGSIG,
                                       LABL_DIGSIG,
                                       NAME_DIGSIG,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DISPOS,
                                       LABL_DISPOS,
                                       NAME_DISPOS,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_LOCATN,
                                       LABL_LOCATN,
                                       NAME_LOCATN,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_MANDAT,
                                       LABL_MANDAT,
                                       NAME_MANDAT,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_PRESRV,
                                       LABL_PRESRV,
                                       NAME_PRESRV,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_STATUS,
                                       LABL_STATUS,
                                       NAME_STATUS,
                                       false));

        // Refinements
        addPredicate(newSchemaProperty(BASE + SUFF_ADDCPY,
                                       LABL_ADDCPY,
                                       NAME_ADDCPY,
                                       false));

        addPredicate(newSchemaProperty(BASE + SUFF_ACQUIR,
                                       LABL_ACQUIR,
                                       NAME_ACQUIR,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_CUTOFF,
                                       LABL_CUTOFF,
                                       NAME_CUTOFF,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DECLAR,
                                       LABL_DECLAR,
                                       NAME_DECLAR,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_CLOSED,
                                       LABL_CLOSED,
                                       NAME_CLOSED,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_VERDUE,
                                       LABL_VERDUE,
                                       NAME_VERDUE,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_UPFREQ,
                                       LABL_UPFREQ,
                                       NAME_UPFREQ,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DACTON,
                                       LABL_DACTON,
                                       NAME_DACTON,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_AUTREM,
                                       LABL_AUTREM,
                                       NAME_AUTREM,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DCONDT,
                                       LABL_DCONDT,
                                       NAME_DCONDT,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DTIMEP,
                                       LABL_DTIMEP,
                                       NAME_DTIMEP,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DDATED,
                                       LABL_DDATED,
                                       NAME_DDATED,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DLASTR,
                                       LABL_DLASTR,
                                       NAME_DLASTR,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DAUTHB,
                                       LABL_DAUTHB,
                                       NAME_DAUTHB,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DCOMMT,
                                       LABL_DCOMMT,
                                       NAME_DCOMMT,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DSCHID,
                                       LABL_DSCHID,
                                       NAME_DSCHID,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DEXDES,
                                       LABL_DEXDES,
                                       NAME_DEXDES,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DEXSTA,
                                       LABL_DEXSTA,
                                       NAME_DEXSTA,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DREVEW,
                                       LABL_DREVEW,
                                       NAME_DREVEW,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DREVDT,
                                       LABL_DREVDT,
                                       NAME_DREVDT,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DREVWR,
                                       LABL_DREVWR,
                                       NAME_DREVWR,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_FIPLID,
                                       LABL_FIPLID,
                                       NAME_FIPLID,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_SYSTID,
                                       LABL_SYSTID,
                                       NAME_SYSTID,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_CASEID,
                                       LABL_CASEID,
                                       NAME_CASEID,
                                       false));

        addPredicate(newSchemaProperty(BASE + SUFF_CURLOC,
                                       LABL_CURLOC,
                                       NAME_CURLOC,
                                       false));

        addPredicate(newSchemaProperty(BASE + SUFF_HOMLOC,
                                       LABL_HOMLOC,
                                       NAME_HOMLOC,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_AUTSTA,
                                       LABL_AUTSTA,
                                       NAME_AUTSTA,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DPEXMT,
                                       LABL_DPEXMT,
                                       NAME_DPEXMT,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DPURPS,
                                       LABL_DPURPS,
                                       NAME_DPURPS,
                                       false));

        addPredicate(newSchemaProperty(BASE + SUFF_OGNFMT,
                                       LABL_OGNFMT,
                                       NAME_OGNFMT,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DEFNBY,
                                       LABL_DEFNBY,
                                       NAME_DEFNBY,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_PRODEF,
                                       LABL_PRODEF,
                                       NAME_PRODEF,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_REARED,
                                       LABL_REARED,
                                       NAME_REARED,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_REDACT,
                                       LABL_REDACT,
                                       NAME_REDACT,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_SEQNNO,
                                       LABL_SEQNNO,
                                       NAME_SEQNNO,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_COPYRT,
                                       LABL_COPYRT,
                                       NAME_COPYRT,
                                       true,
                                       true));

        addPredicate(newSchemaProperty(BASE + NAME_CUSTDN,
                                       LABL_CUSTDN,
                                       NAME_CUSTDN,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DESCTR,
                                       LABL_DESCTR,
                                       NAME_DESCTR,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_DISCTS,
                                       LABL_DISCTS,
                                       NAME_DISCTS,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_SUBXMT,
                                       LABL_SUBXMT,
                                       NAME_SUBXMT,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_EIRDIS,
                                       LABL_EIRDIS,
                                       NAME_EIRDIS,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_EIRXMT,
                                       LABL_EIRXMT,
                                       NAME_EIRXMT,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_FOIDIS,
                                       LABL_FOIDIS,
                                       NAME_FOIDIS,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_FOIXMT,
                                       LABL_FOIXMT,
                                       NAME_FOIXMT,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_FOIREL,
                                       LABL_FOIREL,
                                       NAME_FOIREL,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_FOIDAT,
                                       LABL_FOIDAT,
                                       NAME_FOIDAT,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_FOIREV,
                                       LABL_FOIREV,
                                       NAME_FOIREV,
                                       false));

        addPredicate(newSchemaProperty(BASE + SUFF_GRPACC,
                                       LABL_GRPACC,
                                       NAME_GRPACC,
                                       false));

        addPredicate(newSchemaProperty(BASE + SUFF_ACCLST,
                                       LABL_ACCLST,
                                       NAME_ACCLST,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_FOILDR,
                                       LABL_FOILDR,
                                       NAME_FOILDR,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_PREMRK,
                                       LABL_PREMRK,
                                       NAME_PREMRK,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_PROMRK,
                                       LABL_PROMRK,
                                       NAME_PROMRK,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_CHGDAT,
                                       LABL_CHGDAT,
                                       NAME_CHGDAT,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + NAME_EXPDAT,
                                       LABL_EXPDAT,
                                       NAME_EXPDAT,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + SUFF_CATGRY,
                                       LABL_CATGRY,
                                       NAME_CATGRY,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + SUFF_KEYWRD,
                                       LABL_KEYWRD,
                                       NAME_KEYWRD,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + SUFF_PROCID,
                                       LABL_PROCID,
                                       NAME_PROCID,
                                       true,
                                       false));

        addPredicate(newSchemaProperty(BASE + SUFF_PROGRM,
                                       LABL_PROGRM,
                                       NAME_PROGRM,
                                       false));

        addPredicate(newSchemaProperty(BASE + SUFF_PROJCT,
                                       LABL_PROJCT,
                                       NAME_PROJCT,
                                       false));

        // Schemes
        addScheme(newSchemaProperty(BASE + NAME_GMSAES,
                                    LABL_GMSAES,
                                    NAME_GMSAES));
        addScheme(newSchemaProperty(BASE + NAME_GMSTES,
                                    LABL_GMSTES,
                                    NAME_GMSTES));
        addScheme(newSchemaProperty(BASE + NAME_ERICEN,
                                    LABL_ERICEN,
                                    NAME_ERICEN));
        addScheme(newSchemaProperty(BASE + NAME_GCLENC,
                                    LABL_GCLENC,
                                    NAME_GCLENC));
        addScheme(newSchemaProperty(BASE + NAME_GDSCAT,
                                    LABL_GDSCAT,
                                    NAME_GDSCAT));
        addScheme(newSchemaProperty(BASE + NAME_IEELOM,
                                    LABL_IEELOM,
                                    NAME_IEELOM));
        addScheme(newSchemaProperty(BASE + NAME_ONSSNC,
                                    LABL_ONSSNC,
                                    NAME_ONSSNC));
        addScheme(newSchemaProperty(BASE + NAME_W3CWAI,
                                    LABL_W3CWAI,
                                    NAME_W3CWAI));
        addScheme(newSchemaProperty(BASE + NAME_YES_NO,
                                    LABL_YES_NO,
                                    NAME_YES_NO));
    }

    /**
     *  Get a new schema property.
     *  @param  uri The resource URI.
     *  @param  label   The resource label.
     *  @param  name    The unique resource name.
     *  @return A new schema property constructed from the arguments.
     *  @since  Version 0.7 2005-06-15
     */
    protected static final SchemaProperty newSchemaProperty(final String uri,
                                                      final String label,
                                                      final String name) {

        return new UKeGMSProperty(uri, label, name, false, false);
    }


    /**
     *  Get a new schema property with optional encoding schemes.
     *  @param  uri The resource URI.
     *  @param  label   The resource label.
     *  @param  name    The unique resource name.
     *  @param  encoding    Whether the property has scheme encodings that may
     *          be applied.
     *  @return A new schema property constructed from the arguments.
     *  @since  Version 1.0 2005-09-15
     */
    protected static final SchemaProperty newSchemaProperty(
                                                      final String uri,
                                                      final String label,
                                                      final String name,
                                                      final boolean encoding) {

        return newSchemaProperty(uri, label, name, encoding, false);
    }

    /**
     *  Get a new schema property.
     *  @param  uri The resource URI.
     *  @param  label   The resource label.
     *  @param  name    The unique resource name.
     *  @param  encoding    Whether the property has scheme encodings that may
     *          be applied.
     *  @param  uriEncoding Whether the instance may have a URI encoding
     *          scheme.
     *  @return A new schema property constructed from the arguments.
     *  @since  Version 1.0 2005-09-15
     */
    protected static final SchemaProperty newSchemaProperty(
                                                   final String uri,
                                                   final String label,
                                                   final String name,
                                                   final boolean encoding,
                                                   final boolean uriEncoding) {

        return new UKeGMSProperty(uri, label, name, encoding, uriEncoding);
    }

    /**
     *  Add a predicate to the schema mapping.
     *  @param  predicate   The schema property to add to the mapping.
     *  @since  Version 0.7 2005-06-15
     */
    protected static final void addPredicate(final SchemaProperty predicate) {

        String name = predicate.getName();

        PREDICATES.put(name.toLowerCase(), predicate);
    }

    /**
     *  Add a predicate to the schema mapping.
     *  @param  scheme  The schema property to add to the mapping.
     *  @since  Version 0.7 2005-06-15
     */
    protected static final void addScheme(final SchemaProperty scheme) {

        String name = scheme.getName();

        SCHEMES.put(name.toLowerCase(), scheme);
    }

    /**
     *  Expand an encoded predicate name to a URI.
     *  <p>
     *    Allows recursive handling of schema prefixes.
     *  </p>
     *  @param  name    The predicate name to expand. Initially, this may
     *          include a schema prefix (i.e. <code>DC.Identifier</code>), but
     *          may be processed recursively without its schema prefix to
     *          handle refined elements that use the dot separator scheme
     *          <code>DC.Date.modified</code> or
     *          <code>eGMS.Addressee.addresseeCopy</code>.
     *  @param  predicate   The original predicate to fall back to if there is
     *          no match.
     *  @return The expanded URI reference for the predicate name, or the
     *          original predicate if the name part is not matched.
     *  @since  Version 0.2 2004-12-23
     */
    public final String expandPredicate(final String name,
                                        final String predicate) {

        if ((name != null) &&
            (predicate != null)) {

            int dotIdx = name.indexOf(".");

            if (dotIdx > -1) {

                String element = name.substring(dotIdx + 1,
                                                name.length());

                return expandPredicate(element, predicate);
            }
            else {

                // Element only
                if (hasPredicate(name)) {

                    return getPredicateUri(name);
                }
                // Fall through

                return predicate;
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null arguments not permitted.");
        }
    }

    /**
     *  Encapsulated check whether the schema has the named predicate.
     *  @param  predicate   The predicate name without any prefix.
     *  @return <code>true</code> if the schema has a URI mapping for the
     *          predicate.
     *  @since  Vesion 0.5 2005-05-27
     */
    private final boolean hasPredicate(final String predicate) {

        return PREDICATES.containsKey(predicate.toLowerCase());
    }

    /**
     *  Get the URI mapped to the predicate.
     *  @param  predicate   The predicate name without any prefix.
     *  @return The URI mapped to the predicate or <code>null</code> if not
     *          found.
     *  @since  Version 0.5 2005-05-27
     */
    protected final String getPredicateUri(final String predicate) {

        if (predicate != null) {

            SchemaProperty property = getNamedPredicateProperty(predicate);

            if (property != null) {

                return property.getUri();
            }
            else {

                return null;
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null predicate not permitted.");
        }
    }

    /**
     *  Get the schema property for a given predicate.
     *  @param  name    The predicate name without prefix.
     *  @return The schema property for the predicate or <code>null</code> if no
     *          mapping exists.
     *  @since  Version 0.8 2005-06-16
     */
    public final SchemaProperty getNamedPredicateProperty(final String name) {

        return (SchemaProperty) PREDICATES.get(name.toLowerCase());
    }

    /**
     *  Expand an encoded scheme value to a URI.
     *  @param  name    The encoding scheme name to expand. Initially, this may
     *          include a schema prefix (i.e. <code>DCTERMS.URI</code>),
     *          but may be processed recursively without its schema prefix.
     *  @param  scheme  The scheme to expand.
     *  @return The expanded URI reference for the scheme, or the original
     *          string if it is not matched.
     *  @since  Version 0.1 2005-02-24
     */
    protected final String expandScheme(final String name,
                                        final String scheme) {

        if (name != null) {

            if (hasScheme(name)) {

                return getSchemeUri(name);
            }
            // Fall through

            return scheme;
        }
        else {

            throw new IllegalArgumentException(
                        "Null component not permitted.");
        }
    }

    /**
     *  Encapsulated check whether the schema has the named scheme.
     *  @param  scheme  The scheme name without any prefix.
     *  @return <code>true</code> if the schema has a URI mapping for the
     *          predicate.
     *  @since  Vesion 0.5 2005-05-27
     */
    private final boolean hasScheme(final String scheme) {

        return SCHEMES.containsKey(scheme.toLowerCase());
    }

    /**
     *  Get the URI mapped to the scheme.
     *  @param  scheme  The scheme name without any prefix.
     *  @return The URI mapped to the scheme or <code>null</code> if not
     *          found.
     *  @since  Version 0.5 2005-05-27
     */
    protected final String getSchemeUri(final String scheme) {

        if (scheme != null) {

            SchemaProperty property = null;

            property = (SchemaProperty) SCHEMES.get(scheme.toLowerCase());

            if (property != null) {

                return property.getUri();
            }
            else {

                return null;
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null scheme not permitted.");
        }
    }

    /**
     *  Get the base URI for the metadata schema.
     *  @return The namespace URI for the metadata schema.
     *  @since  Version 0.2 2004-12-23
     */
    public final String getUri() {

        return BASE;
    }

    /**
     *  Get the default namespace prefix for the schema.
     *  @return The default namepace prefix for the schema.
     *  @since  Version 0.3 2005-03-01
     */
    public final String getDefaultPrefix() {

        return PREFIXES[0];
    }

    /**
     *  Get the HTML metadata prefixes recognised by this schema.
     *  @return An array of one or more schema prefixes in mixed case.
     *  @since  Version 0.6 2005-05-27
     */
    public final String[] getAllPrefixes() {

        return copyPrefixes(PREFIXES);
    }

    /**
     *  Signal whether the schema provides element qualifiers.
     *  @return <code>true</code> if the schema
     *  @since  Version 0.4 2005-03-10
     */
    public final boolean providesQualifiers() {

        return true;
    }

    /**
     *  Check whether the schema provides qualifiers for a specific schema.
     *  @param  uri The URI of another schema to check for extensions.
     *  @return <code>true</code> if the schema provieds qualifiers for the
     *        schema signified by the URI.
     *  @since  Version 0.6 2005-03-10
     */
    public final boolean providesQualifiersFor(final String uri) {

        if (uri != null) {

            return uri.equals(DublinCoreElements.BASE);
        }
        else {

            throw new IllegalArgumentException(
                        "Null schema URI not permitted.");
        }
    }

    /**
     *  Get a collection of schema property predicates for this schema.
     *  @return A collection of predicate names.
     *  @since  Version 0.6 2005-06-10
     */
    public final Collection getPredicates() {

        return Collections.unmodifiableCollection(PREDICATES.values());
    }

    /**
     *  Get the common name for the schema.
     *  @return The common name for the schema.
     *  @since  Version 0.9 2005-06-17
     */
    public final String getName() {

        return NAME;
    }

}
