package com.mkdoc.servlet;

/*
    Created:    2005-01-20
    Issued:     2005-01-25
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.query.Query;
import com.mkdoc.query.QueryBuilder;
import com.mkdoc.query.QueryFactory;
import com.mkdoc.query.QueryResult;
import com.mkdoc.query.ResultRenderer;
import com.mkdoc.query.ServletQueryBuilder;

import com.mkdoc.store.StoreManagerFactory;
import com.mkdoc.store.StoreManager;

import com.mkdoc.util.ApplicationContext;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.UnavailableException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 *  A working draft query results servlet.
 *  @version    1.5 2005-11-01
 *  @author     Philip Shaw
 *  @TODO       Get result and cache it before rendering or dispatching.
 */
public class HttpQuery extends AbstractMKDocServlet {

    /**
     *  The store interface.
     *  @since  Version 0.2 2005-01-21
     */
    protected static StoreManager manager;

    /**
     *  The text formatted result page.
     *  @since  Version 0.2 2005-01-21
     */
    private static String textResult;

    /**
     *  The (X)HTML formatted result page.
     *  @since  Version 0.2 2005-01-21
     */
    private static String htmlResult;

    /**
     *  Whether to use extended bNode queries.
     *  @since  Version 1.4 2005-09-13
     */
    private static boolean extended = false;

    /**
     *  Get the servlet configuration.
     *  <p>
     *    The servlet configuration must include the following result page
     *    parameters:
     *    <ul>
     *      <li>
     *        <code>textResult</code>
     *      </li>
     *      <li>
     *        <code>htmlResult</code>
     *      </li>
     *    </ul>
     *  <p>
     *    The respective results pages should include a
     *    <code>QueryResultTag</code> to handle the query and result.
     *  </p>
     *  <p>
     *    The servlet context must include the path of a Sesame
     *    <code>repository</code> and a <code>StoreManager</code> type
     *    with the parameter name <code>manager</code>.
     *  </p>
     *  <p>
     *    Set a parameter named <code>extended</code> to <code>true</code> to
     *    activate extended bNode queries.
     *  </p>
     *  @param  config  The servlet configuration object.
     *  @exception  ServletException    If critical configuration parameters
     *          are not set or invalid.
     *  @see    com.mkdoc.tag.QueryResultTag
     *  @since  Version 0.1 2005-01-21
     */
    public final void init(final ServletConfig config)
        throws ServletException {

        ServletContext context = config.getServletContext();

        ApplicationContext appContext = getApplicationContext(context);

        textResult = config.getInitParameter("textResult");

        htmlResult = config.getInitParameter("htmlResult");

        if ((textResult == null) ||
            (htmlResult == null)) {

            throw new UnavailableException(
                        "Missing result page parameter.");
        }
        else {

            try {

                manager = StoreManagerFactory.newStoreManager(appContext);

                extended = "true".equals(config.getInitParameter("extended"));
            }
            catch (Exception e) {

                throw new UnavailableException(e.getMessage());
            }
        }
    }

    /**
     *  Dispatch the query to the relevant results page.
     *  @param  request The HTTP servlet request object.
     *  @param  response    The HTTP servlet response object.
     *  @exception  IOException If there is a problem writing to the output
     *          stream or issuing an error response.
     *  @exception  ServletException    If there is a problem forwarding to the
     *          relevant result page.
     *  @since  Version 0.1 2005-01-20
     */
    public final void doGet(final HttpServletRequest request,
                            final HttpServletResponse response)
        throws IOException, ServletException {

        String echo = getFirstParam(request, "echo");

        if ((echo != null) &&
            (echo.equalsIgnoreCase("true"))) {

            echoQuery(request, response);
        }
        else {

            String type = getFirstParam(request, "type");

            if ((type == null) ||
                (type.equalsIgnoreCase("html")) ||
                (type.equalsIgnoreCase("text"))) {

                dispatchQuery(request, response);
            }
            else {

                handleDataQuery(request, response);
            }
        }
    }

    /**
     *  Handle a &quot;pure data&quot; type query.
     *  @param  request The servlet request object.
     *  @param  response    The servlet response object.
     *  @exception  IOException If there is a problem rendering the results to
     *          the output stream.
     *  @since  Version 1.3 2005-09-12
     */
    protected final void handleDataQuery(final HttpServletRequest request,
                                         final HttpServletResponse response)
        throws IOException {

        ServletOutputStream out = null;

        QueryBuilder builder = new ServletQueryBuilder(request, extended);

        builder.setDistinct(true);

        try {

            Query query = builder.getQuery();

            QueryResult result = manager.executeQuery(query);

            ResultRenderer renderer = QueryFactory.getResultRenderer(query);

            response.setContentType(renderer.getContentType());

            out = response.getOutputStream();

            renderer.renderQueryResult(out, result);
        }
        catch (IllegalStateException iae) {

            response.sendError(response.SC_BAD_REQUEST,
                               iae.getMessage());
        }
        catch (Exception e) {

            response.sendError(response.SC_INTERNAL_SERVER_ERROR,
                               e.getMessage());
        }
        finally {

            if (out != null) {

                out.flush();
                out.close();
            }
        }
    }

    /**
     *  Dispatch the query to the relevant results page.
     *  @param  request The servlet request object.
     *  @param  response    The servlet response object.
     *  @exception  IOException If there is a problem forwarding the query.
     *  @exception  ServletException If there is a problem with the servlet to
     *          which the query is dispatched.
     *  @since  Version 0.9 2005-07-22
     */
    protected final void dispatchQuery(final HttpServletRequest request,
                                       final HttpServletResponse response)
        throws IOException, ServletException {

        String type = getFirstParam(request, "type");

        RequestDispatcher dispatcher = null;

        if ((type != null) &&
            (type.equalsIgnoreCase("text"))) {

            dispatcher = request.getRequestDispatcher(textResult);

            dispatcher.forward(request, response);
        }
        else {

            dispatcher = request.getRequestDispatcher(htmlResult);

            dispatcher.forward(request, response);
        }
    }

    /**
     *  Echo the query parameters as an SeRQL query statement.
     *  @param  request The servlet request object.
     *  @param  response    The servlet response object.
     *  @exception  IOException If there is a problem handling a bad request
     *          or internal server error responses.
     *  @since  Version 0.9 2005-07-22
     */
    protected final void echoQuery(final HttpServletRequest request,
                                   final HttpServletResponse response)
        throws IOException {

        response.setContentType("text/plain");

        ServletOutputStream out = null;

        QueryBuilder builder = new ServletQueryBuilder(request, extended);

        builder.setDistinct(true);

        try {

            out = response.getOutputStream();

            Query query = builder.getQuery();

            out.print(query.getSeRQLQuery());
        }
        catch (IllegalStateException iae) {

            response.sendError(response.SC_BAD_REQUEST,
                               iae.getMessage());
        }
        catch (Exception e) {

            response.sendError(response.SC_INTERNAL_SERVER_ERROR,
                               e.getMessage());
        }
        finally {

            out.flush();
            out.close();
        }
    }

    /**
     *  Get a query result from a post submission.
     *  @param  request The HTTP servlet request object.
     *  @param  response    The HTTP servlet response object.
     *  @exception  IOException If there is a problem writing to the output
     *          stream or issuing an error response.
     *  @exception  ServletException    If there is a problem forwarding to the
     *          relevant result page.
     *  @since  Version 0.1 2005-01-20
     */
    public final void doPost(final HttpServletRequest request,
                             final HttpServletResponse response)
        throws IOException, ServletException {

        doGet(request, response);
    }

}
