package com.mkdoc.sesame;

/*
    Created:    2005-01-26
    Issued:     2005-01-26
    Modified:   2005-09-14

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.sesame.Sesame;

import org.openrdf.sesame.config.AccessDeniedException;
import org.openrdf.sesame.config.ConfigurationException;
import org.openrdf.sesame.config.RepositoryConfig;
import org.openrdf.sesame.config.SailConfig;

import org.openrdf.sesame.constants.RDFFormat;

import org.openrdf.sesame.constants.QueryLanguage;

import org.openrdf.sesame.query.MalformedQueryException;
import org.openrdf.sesame.query.QueryEvaluationException;
import org.openrdf.sesame.query.QueryResultsTable;

import org.openrdf.sesame.repository.local.LocalRepository;
import org.openrdf.sesame.repository.local.LocalService;

import org.openrdf.sesame.sailimpl.memory.RdfSchemaRepositoryConfig;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;

/**
 *  A command line tool for issuing SeRQL queries to a Sesame repository.
 *  @version    0.2 2005-09-13
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public final class RepositoryQuery {

    /**
     *  A query results renderer.
     *  @since  Version 0.1 2005-01-26
     */
    private static final ResultsRenderer RENDERER = new TextResultsRenderer();

    /**
     *  The Sesame repository.
     *  @since  Version 0.1 2005-01-26
     */
    private static LocalRepository repository;

    /**
     *  Private default constructor to prevent instantiation.
     *  @since  Version 0.1 2005-01-26
     */
    private RepositoryQuery() {

        // Empty
    }

    /**
     *  Process a repository query from the command line.
     *  <p>
     *    It's very difficult to write and edit a SeRQL query on the command
     *    line, so it is provided as a plain text file.
     *  </p>
     *  @param  args    The repository file name followed by a text file name
     *          that contains the SeRQL query.
     *  @since  Version 0.1 2005-01-26
     */
    public static final void main(final String[] args) {

        if (args.length > 1) {

            System.out.println("RepositoryQuery");
            System.out.println("========================================");

            QueryResultsTable table = null;

            try {

                loadRepository(args[0]);

                String query = getQuery(args);

                table = repository.performTableQuery(QueryLanguage.SERQL,
                                                     query);

                System.out.println(RENDERER.renderResults(table));
            }
            catch (IOException ioe) {

                System.err.println("Could not load the query file. ");
                System.err.println(ioe.getMessage());
            }
            catch (ConfigurationException ce) {

                System.err.println("Could not load the Sesame repository. ");
                System.err.println(ce.getMessage());
            }
            catch (MalformedQueryException mqe) {

                System.err.println("Malformed query. ");
                System.err.println(mqe.getMessage());
            }
            catch (QueryEvaluationException qee) {

                System.err.println("Query evaluation error. ");
                System.err.println(qee.getMessage());
            }
            catch (AccessDeniedException ade) {

                System.err.println("Repository access denied. ");
                System.err.println(ade.getMessage());
            }
            finally {

                System.out.println("========================================");
            }
        }
        else {

            System.err.println("Error: incomplete syntax, use:");
            System.err.println("com.mkdoc.sesame.RepositoryQuery " +
                               "<file-path> <query>");
        }
    }

    /**
     *  Load the repository.
     *  @param  path    The file path for the repository.
     *  @exception  ConfigurationException  If the repository cannot be loaded.
     *  @since  Version 0.1 2005-01-21
     */
    private static final void loadRepository(final String path)
        throws ConfigurationException {

        RepositoryConfig storeConfig = null;

        storeConfig = new RepositoryConfig("RepositoryQuery");

        SailConfig syncConfig = new SailConfig(
        "org.openrdf.sesame.sailimpl.sync.SyncRdfSchemaRepository");

        SailConfig memConfig = null;

        memConfig = new RdfSchemaRepositoryConfig(path,
                                                  RDFFormat.RDFXML);

        storeConfig.addSail(syncConfig);
        storeConfig.addSail(memConfig);
        storeConfig.setWorldReadable(true);
        storeConfig.setWorldWriteable(false);

        LocalService service = Sesame.getService();

        // May throw ConfigurationException
        repository = service.createRepository(storeConfig);

        System.out.println();
        System.out.println("Loaded repository at:");
        System.out.println(path);
    }

    /**
     *  Get a SeRQL query from the remaining arguments.
     *  @param  args    An array of query elements.
     *  @return A concatenated string of arguments, which is expected to be a
     *          SeRQL query.
     *  @exception  IOException If there is a problem reading the query file.
     *  @since  Version 0.1 2005-01-21
     */
    private static final String getQuery(final String[] args)
        throws IOException {

        File file = new File(args[1]);

        Reader fileReader = null;

        try {

            System.out.println("Loading query file " +
                               file.getAbsolutePath());

            fileReader = new FileReader(file);

            BufferedReader bufferReader = new BufferedReader(fileReader);

            String line = null;

            StringBuffer queryBuffer = new StringBuffer();

            while (true) {

                line = bufferReader.readLine();

                if (line == null) {

                    break;
                }

                line = line.trim();

                if (!line.startsWith("#") &&
                    !line.equals("")) {

                    queryBuffer.append(line);
                    queryBuffer.append(" ");
                }
            }

            return queryBuffer.toString();
        }
        finally {

            if (fileReader != null) {

                fileReader.close();
            }
        }
    }

/*
    private static final String getQuery(final String[] args) {

        StringBuffer buffer = new StringBuffer();

        for (int i = 1; i < args.length; i++) {

            buffer.append(args[i]);
            buffer.append(" ");
        }

        System.out.println();
        System.out.println("SeRQL query:");
        System.out.println();
        System.out.println(buffer.toString());
        System.out.println();

        return buffer.toString();
    }
*/

}
