package com.mkdoc.sesame;

/*
    Created:    2005-01-24
    Issued:     2005-01-24
    Modified:   2005-01-26

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Vector;

/**
 *  A builder for SeRQL query strings.
 *  <p>
 *    This initial draft is string-based with no substantial validation of
 *    variable and namespace usage versus declaration. Conditional clauses
 *    are combined as <code>OR</code> statements.
 *  </p>
 *  @version    0.3 2005-01-25
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class SeRQLQueryBuilder {

    /**
     *  A single space character.
     *  @since  Version 0.1 2005-01-24
     */
    protected static final String SPACE = " ";

    /**
     *  A comma separator.
     *  @since  Version 0.1 2005-01-24
     */
    protected static final String APPEND_COMMA = ", ";

    /**
     *  A <code>SELECT</code> query type.
     *  @since  Version 0.1 2005-01-24
     */
    public static final String SELECT = "SELECT";

    /**
     *  A <code>CONSTRUCT</code> query type.
     *  @since  Version 0.1 2005-01-24
     */
    public static final String CONSTRUCT = "CONSTRUCT";

    /**
     *  A <code>DISTINCT</code> keyword.
     *  @since  Version 0.1 2005-01-24
     */
    public static final String DISTINCT = " DISTINCT";

    /**
     *  A <code>FROM</code> keyword.
     *  @since  Version 0.1 2005-01-24
     */
    public static final String FROM = " FROM ";

    /**
     *  An <code>AND</code> keyword.
     *  @since  Version 0.1 2005-01-24
     */
    public static final String APPEND_AND = " AND ";

    /**
     *  An <code>OR</code> keyword.
     *  @since  Version 0.1 2005-01-24
     */
    public static final String APPEND_OR = " OR ";

    /**
     *  A <code>WHERE</code> keyword.
     *  @since  Version 0.1 2005-01-24
     */
    public static final String WHERE = " WHERE ";

    /**
     *  A <code>USING NAMESPACE</code> declaration.
     *  @since  Version 0.1 2005-01-24
     */
    public static final String USING = " USING NAMESPACE ";

    /**
     *  A list of the variable values to get from the query.
     *  @since  Version 0.1 2005-01-24
     */
    private Set variables = new HashSet();

    /**
     *  A list of the path expressions for the query.
     *  @since  Version 0.1 2005-01-24
     */
    private List expressions = new Vector();

    /**
     *  A list of the conditions to apply to the query variables.
     *  @since  Version 0.1 2005-01-24
     */
    private List conditions = new Vector();

    /**
     *  A list of the namespaces used in the query.
     *  @since  Version 0.1 2005-01-24
     */
    private Set namespaces = new HashSet();

    /**
     *  The query type, defalts to <code>SELECT</code>.
     *  @since  Version 0.1 2005-01-24
     */
    private String queryType = SELECT;

    /**
     *  Whether the <code>DISTINCT</code> keyword is applied to the query.
     *  @since  Version 0.1 2005-01-24
     */
    private boolean distinct;

    /**
     *  Add a variable name.
     *  @param  name    The name of a variable value to get from the query.
     *  @since  Version 0.1 2005-01-24
     */
    public final void addVariable(final String name) {

        if (name != null) {

            variables.add(name);
        }
        else {

            throw new IllegalArgumentException(
                        "Null variable name not permitted.");
        }
    }

    /**
     *  Add a path expression to the query.
     *  @param  expression  A single path expression, no trailing comma or
     *          semi-colon.
     *  @since  Version 0.1 2005-01-24
     */
    public final void addExpression(final String expression) {

        if (expression != null) {

            expressions.add(expression);
        }
        else {

            throw new IllegalArgumentException(
                        "Null expression not permitted.");
        }
    }

    /**
     *  Add a condition to the query selection.
     *  @param  condition   A condition, no trailing comma or
     *          semi-colon.
     *  @since  Version 0.1 2005-01-24
     */
    public final void addCondition(final String condition) {

        if (condition != null) {

            conditions.add(condition);
        }
        else {

            throw new IllegalArgumentException(
                        "Null condition not permitted.");
        }
    }

    /**
     *  Add a namespace declaration to the query selection.
     *  @param  namespace   A namespace, no trailing comma.
     *  @since  Version 0.1 2005-01-24
     */
    public final void addNamespace(final String namespace) {

        if (namespace != null) {

            namespaces.add(namespace);
        }
        else {

            throw new IllegalArgumentException(
                        "Null namespace not permitted.");
        }
    }

    /**
     *  Set whether the <code>DISTINCT</code> keyword is applied.
     *  @param  distinct    Whether the query interpreter should suppress
     *          duplicate results.
     *  @since  Version 0.1 2005-01-24
     */
    public final void setDistinct(final boolean distinct) {

        this.distinct = distinct;
    }

    /**
     *  Set query type.
     *  @param  queryType   <code>SELECT</code> or <code>CONSTRUCT</code>.
     *  @since  Version 0.1 2005-01-24
     */
    public final void setQueryType(final String queryType) {

        if (queryType != null) {

            this.queryType = queryType;
        }
        else {

            throw new IllegalArgumentException(
                        "Null query type not permitted.");
        }
    }

    /**
     *  Get a string representation of the query.
     *  <p>
     *    <em>At least one variable declaration and one path expression must be
     *    added to the query before calling this method.</em>
     *  </p>
     *  @return A SeRQL query string generated from the current builder
     *          configuration.
     *  @since  Version 0.1 2005-01-24
     */
    public final String toString() {

        if (variables.size() > 0) {

            if (expressions.size() > 0) {

                StringBuffer query = new StringBuffer();

                query.append(queryType);

                if (distinct) {

                    query.append(DISTINCT);
                }

                query.append(SPACE);

                appendList(variables, query, APPEND_COMMA);

                query.append(FROM);

                appendList(expressions, query, APPEND_COMMA);

                if (conditions.size() > 0) {

                    query.append(WHERE);

                    appendList(conditions, query, APPEND_OR);

                }

                if (namespaces.size() > 0) {

                    query.append(USING);

                    appendList(namespaces, query, APPEND_COMMA);
                }

                return query.toString();
            }
            else {

                throw new IllegalStateException(
                            "No expressions declared.");
            }
        }
        else {

            throw new IllegalStateException(
                        "No variables declared.");
        }
    }

    /**
     *  Concatenate a collection of string values with the given separator.
     *  <p>
     *    If there is more than one item in the collection, the separator text
     *    is inserted before each subsequent item. No leading or trailing
     *    spaces are added.
     *  </p>
     *  @param  values  The list of string values to concatenate.
     *  @param  buffer  The string buffer to append values to.
     *  @param  separator   The separator characters to insert before each
     *          list item, e.g. &quot; AND &quot;, or &quot;, &quot;.
     *  @since  Version 0.1 2005-01-24
     */
    protected final void appendList(final Collection values,
                                    final StringBuffer buffer,
                                    final String separator) {

        if ((values != null) &&
            (values.size() > 0)) {

            if (buffer != null) {

                if (separator != null) {

                    Iterator iterator = values.iterator();

                    boolean first = true;

                    while (iterator.hasNext()) {

                        if (first) {

                            first = false;
                        }
                        else {

                            buffer.append(separator);
                        }

                        buffer.append((String) iterator.next());
                    }
                }
                else {

                    throw new IllegalArgumentException(
                                "Null separator not permitted.");
                }
            }
            else {

                throw new IllegalArgumentException(
                            "Null buffer not permitted.");
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Null value list not permitted.");
        }
    }

}
