package com.mkdoc.store;

/*
    Created:    2005-04-26
    Issued:     2005-04-26
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import com.mkdoc.util.ApplicationContext;

import java.net.URL;
import java.net.MalformedURLException;

/**
 *  Common methods for managing file system resources.
 *  @version    0.7 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public abstract class AbstractFileStoreManager {

    /**
     *  The absolute file path property name.
     *  @since  Version 0.1 2005-04-26
     */
    public static final String ABSOLUTE = "output.absolute";

    /**
     *  The output file path property name.
     *  @since  Version 0.1 2005-04-26
     */
    public static final String FOLDER = "output.folder";

    /**
     *  The default output folder.
     *  @since  Version 0.1 2005-04-26
     */
    public static final String DOT = ".";

    /**
     *  The current output directory.
     *  @since  Version 0.1 2005-04-26
     */
    protected File outputDir;

    /**
     *  The base output directory.
     *  @since  Version 0.1 2005-04-26
     */
    //protected File baseDir;

    /**
     *  Configure the output directory from the plugin property set.
     *  @param  context The plugin application context.
     *  @return A status message giving the path of the output directory.
     *  @since  Version 0.1 2005-04-26
     */
    protected final String configureOutput(final ApplicationContext context) {

        if (context == null) {

            throw new IllegalArgumentException(
                        "Null application context set not permitted.");
        }
        else {

            outputDir = context.getStoreDirectory();

            return "Writing to output folder: " + outputDir;
        }
    }

    /**
     *  Issue an output stream to which RDF statements can be passed directly.
     *  <p>
     *    <em>Temporary measure to adapt plugin <code>writeFile</code> methods.
     *  </p>
     *  @param  subject The subject of the RDF statements.
     *  @exception  StoreException  If an error occurs with the triple store.
     *  @return An output stream on which to write RDF statements.
     *  @since  Version 0.1 2005-04-26
     */
    public final OutputStream getOutputStream(final URL subject)
        throws StoreException {

        try {

            File outputFile = getStorageFile(subject);

            createDirectories(outputFile);

            return new FileOutputStream(outputFile);
        }
        catch (IOException ioe) {

            throw new StoreException(ioe.getMessage());
        }
    }

    /**
     *  Purge statements about a document by subject URI.
     *  @param  subjectUri  The URI of the subject document to purge.
     *  @exception  StoreException  If an error occurs with the triple store.
     *  @since  Version 0.3 2005-05-12
     */
    protected final void purgeDocumentFile(final String subjectUri)
        throws StoreException {

        if (subjectUri == null) {

            throw new IllegalArgumentException(
                        "Null subject URI not permitted.");
        }
        else {

            try {

                File storeFile = getStorageFile(new URL(subjectUri));

                if (storeFile.exists()) {

                    storeFile.delete();
                }
            }
            catch (MalformedURLException mue) {

                throw new IllegalArgumentException("Invalid subject URI.");
            }
        }
    }

    /**
     *  Get the extension for storage files.
     *  @return An type-specific file extension.
     *  @since  Version 0.1 2005-04-26
     */
    protected abstract String getExtension();

    /**
     *  Get a storage file reference for management.
     *  @param  subject The URL of the subject resource.
     *  @return A reference to the storage file for the URL subject.
     *  @since  Version 0.1 2005-04-26
     */
    protected final File getStorageFile(final URL subject) {

        String path = subject.getHost() + subject.getPath();

        if (includesFile(subject)) {

            path = subject.getHost() + trimExtension(subject) + getExtension();
        }
        else {

            path = subject.getHost() +
                   subject.getPath() +
                   "/index" +
                   getExtension();
        }

        return new File(outputDir, path);
    }

    /**
     *  Create directories above a target output file.
     *  @param  outFile The target output file reference.
     *  @since  Version 0.1 2005-04-26
     */
    protected static final void createDirectories(final File outFile) {

        if (outFile == null) {

            throw new IllegalArgumentException(
                        "Null file not permitted.");
        }
        else {

            File parent = outFile.getParentFile();

            if (parent == null) {

                throw new IllegalArgumentException(
                            "File has no parent, " +
                            outFile.getAbsolutePath());
            }
            else {

                if (!parent.exists()) {

                    parent.mkdirs();
                }
            }
        }
    }

    /**
     *  Check whether a URL includes a file part.
     *  @param  url The URL to check.
     *  @return <code>true</code> if the part after the
     *          last directory separator includes a dot
     *          prefixed extension.
     *  @since  Version 0.1 2005-04-26
     */
    protected final boolean includesFile(final URL url) {

        String urlString = url.getPath();

        // @TODO 
        // A more robust check
        return (urlString.lastIndexOf(DOT) > urlString.lastIndexOf('/'));

    }

    /**
     *  Trim any extension part from a URL.
     *  @param  url The URL to process.
     *  @return The file part of the URL less any
     *          &quot;file&quot; extension.
     *  @since  Version 0.1 2005-04-26
     */
    protected final String trimExtension(final URL url) {

        String file = url.getFile();

        int dotIndex = file.lastIndexOf(DOT);

        if (dotIndex > -1) {

            return file.substring(0, dotIndex);
        }
        else {

            return file;
        }
    }

}
