package com.mkdoc.store;

/*
    Created:    2005-03-31
    Issued:     2005-03-31
    Modified:   2005-11-02

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.util.ApplicationContext;

/**
 *  A factory for store manager types.
 *  @version    0.6 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public final class StoreManagerFactory {

    /**
     *  A null property set error message.
     *  @since  Version 0.1 2005-03-31
     */
    public static final String NULL_ARG =
    "Null property set not permitted";

    /**
     *  The store manager class property name.
     *  @since  Version 0.1 2005-03-31
     */
    public static final String MANAGER_KEY = "manager";

    /**
     *  Help suggestion for exception cases.
     *  @since  Version 0.1 2005-03-31
     */
    public static final String SUGGESTION = "Check your configuration file. ";

    /**
     *  Private default constructor to prevent instantiation.
     *  @since  Version 0.1 2005-03-31
     */
    private StoreManagerFactory() {

        super();
    }

    /**
     *  Get a <code>StoreManager</code> instance.
     *  @param  context The application context for the manager.
     *  @return An initialized <code>StoreManager</code> type.
     *  @exception  StoreException  If the manager class cannot be
     *          instantiated for any reason.
     *  @since  Version 0.1 2005-03-31
     */
    public static StoreManager newStoreManager(
                                              final ApplicationContext context)
        throws StoreException {

        if (context == null) {

            throw new IllegalArgumentException(NULL_ARG);
        }
        else {

            try {

                return (StoreManager) newManager(context);
            }
            catch (Exception e) {

                throw new StoreException(
                            "Error casting to StoreManager. " +
                            SUGGESTION +
                            e.getMessage());
            }
        }
    }

    /**
     *  Get a <code>StoreManager</code> instance.
     *  @param  context The application context for the manager.
     *  @return An initialized <code>StoreManager</code> type.
     *  @exception  StoreException  If the manager class cannot be
     *          instantiated for any reason.
     *  @since  Version 0.1 2005-03-31
     */
    public static FileStoreManager newFileStoreManager(
                                              final ApplicationContext context)
        throws StoreException {

        if (context == null) {

            throw new IllegalArgumentException(NULL_ARG);
        }
        else {

            try {

                return (FileStoreManager) newManager(context);
            }
            catch (Exception e) {

                throw new StoreException(
                            "Error casting to FileStoreManager. " +
                            SUGGESTION +
                            e.getMessage());
            }
        }
    }

    /**
     *  Get a new, initialized <code>Manager</code> type.
     *  @param  context The application context for the manager.
     *  @return An initialized <code>Manager</code> type.
     *  @exception  StoreException  If the manager class cannot be
     *          instantiated for any reason.
     *  @since  Version 0.1 2005-03-31
     *
     */
    private static Manager newManager(final ApplicationContext context)
        throws StoreException {

        Class managerClass = context.getClassParameter(MANAGER_KEY);

        if (managerClass == null) {

            throw new StoreException("No store manager class configured.");
        }
        else {

            Manager manager = null;

            try {

                manager = (Manager) managerClass.newInstance();

                manager.initialize(context);

                return manager;
            }
            catch (Exception e) {

                throw new StoreException(e.getMessage());
            }
        }
    }

}
