package com.mkdoc.store;

/*
    Created:    2005-07-14
    Issued:     2005-10-26
    Modified:   2005-10-26

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.model.Value;

import org.openrdf.sesame.query.QueryResultsTable;

import com.mkdoc.query.Query;
import com.mkdoc.query.QueryResult;

/**
 *  A store query result.
 *  <p>
 *    A thin wrapper around a Sesame <code>QueryResultsTable</code>. The result
 *    rows use a zero based index.
 *  </p>
 *  @version    0.3 2005-10-26
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class StoreQueryResult implements QueryResult {

    /**
     *  A Sesame query results table to adapt.
     *  @since  Version 0.1 2005-07-14
     */
    private final QueryResultsTable resultsTable;

    /**
     *  The column index for item URIs.
     *  @since  Version 0.1 2005-07-14
     */
    private int uriIndex = -1;

    /**
     *  The column index for item titles.
     *  @since  Version 0.1 2005-07-14
     */
    private int titleIndex = -1;

    /**
     *  The column index for item summaries.
     *  @since  Version 0.1 2005-07-14
     */
    private int summaryIndex = -1;

    /**
     *  An array of column headings.
     *  @since  Version 0.1 2005-07-14
     */
    // private String[] headings;

    /**
     *  Adapt a Sesame query results table.
     *  @param  resultsTable    A Sesame query results table to adapt.
     *  @since  Version 0.1 2005-07-14
     */
    public StoreQueryResult(final QueryResultsTable resultsTable) {

        super();

        if (resultsTable == null) {

            throw new IllegalArgumentException(
                        "Null results table not permitted.");
        }
        else {

            this.resultsTable = resultsTable;

            registerColumns(resultsTable);
        }

    }

    /**
     *  Get the number of rows in the result set.
     *  @return The number of rows in the result set.
     *  @since  Version 0.1 2005-07-14
     */
    public final int getRows() {

        return resultsTable.getRowCount();
    }

    /**
     *  Get the number of columns in the result set.
     *  @return The number of columns in the result set.
     *  @since  Version 0.1 2005-07-14
     */
    public final int getColumns() {

        return resultsTable.getColumnCount();
    }

    /**
     *  Get the result at a coordinate.
     *  @param  row The row index.
     *  @param  column  The column index.
     *  @return The string value of the result at the given coordinate.
     *  @since  Version 0.1 2005-07-14
     */
    private String getValue(final int row,
                            final int column) {

        return getValue(row, column, null);
    }

    /**
     *  Get the result at a coordinate.
     *  @param  row The row index.
     *  @param  column  The column index.
     *  @param  fallback    The fallback value if null.
     *  @return The string value of the result at the given coordinate.
     *  @since  Version 0.1 2005-07-14
     */
    private String getValue(final int row,
                            final int column,
                            final String fallback) {

        try {

            Value value = resultsTable.getValue(row, column);

            return value.toString();
        }
        catch (Exception e) {

            return fallback;
        }
    }

    /**
     *  Check whether a row number is within range.
     *  @param  row The row number to check.
     *  @return <code>true</code> if the query result contains the specified
     *          row.
     *  @since  Version 0.3 2005-10-26
     */
    private boolean hasRow(final int row) {

        return ((row >= 0) &&
                (row <= getRows()));
    }

    /**
     *  Get the item URI for a row.
     *  @param  row The row index.
     *  @return The item URI.
     *  @since  Version 0.2 2005-07-28
     */
    public final String getItemUri(final int row) {

        if (hasRow(row)) {

            String uri = getValue(row, uriIndex);

            if (uri == null) {

                throw new IllegalStateException(
                            "Result item has no URI.");
            }
            else {

                return uri;
            }
        }
        else {

            throw new IllegalArgumentException(
                        "Results row index is out of range.");
        }
    }

    /**
     *  Get the item title for a row.
     *  @param  row The row index.
     *  @return The item title or a &quot;No title...&quot; statement.
     *  @since  Version 0.2 2005-07-28
     */
    public final String getItemTitle(final int row) {

        if (hasRow(row)) {

            return getValue(row,
                            titleIndex,
                            "No title available for this item.");
        }
        else {

            throw new IllegalArgumentException(
                        "Results row index is out of range.");
        }
    }

    /**
     *  Get the item summary for a row.
     *  @param  row The row index.
     *  @return The item summary or a &quot;No summary...&quot; statement.
     *  @since  Version 0.2 2005-07-28
     */
    public final String getItemSummary(final int row) {

        if (hasRow(row)) {

            return getValue(row,
                            summaryIndex,
                            "No summary available for this item.");
        }
        else {

            throw new IllegalArgumentException(
                        "Results row index is out of range.");
        }
    }

    /**
     *  Register the standard column indices.
     *  @param  resultsTable    The table of results.
     *  @since  Version 0.1 2005-07-14
     */
    private void registerColumns(final QueryResultsTable resultsTable) {

        String[] headings = resultsTable.getColumnNames();

        int size = headings.length;

        for (int i = 0; i < size; i++) {

            if (Query.VAR_URI.equals(headings[i])) {

                uriIndex = i;
            }
            if (Query.VAR_SUMMARY.equals(headings[i])) {

                summaryIndex = i;
            }
            if (Query.VAR_TITLE.equals(headings[i])) {

                titleIndex = i;
            }
        }
    }

}
