package com.mkdoc.io;

/*
    Created:    2004-12-17
    Issued:     2004-12-17
    Modified:   2005-01-13

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.ByteArrayInputStream;
import java.io.FilterInputStream;
import java.io.InputStream;
import java.io.IOException;

/**
 *  Mock input stream for testing.
 *  <p>
 *    Wraps a standard <code>InputStream</code> All methods
 *    delegate to the <code>FilterInputStream</code>
 *    superclass unless read or close failures are
 *    requested.
 *  </p>
 *  <p>
 *    The <code>mark</code> and <code>markSupported</code>
 *    methods are not overridden.
 *  </p>
 *  @version    0.2 2005-01-13
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockInputStream extends FilterInputStream {

    /**
     *  A default buffer size for the default constructor.
     *  @since  Version 0.2 2005-01-13
     */
    private static final int BUFFER_SIZE = 512;

    /**
     *  Whether throw exceptions with write operations.
     *  @since  Version 0.1 2004-12-17
     */
    private boolean failOnRead = false;

    /**
     *  Whether throw exceptions when closing.
     *  @since  Version 0.1 2004-12-17
     */
    private boolean failOnClose = false;

    /**
     *  Exception message.
     *  @since  Version 0.1 2004-12-03
     */
    private String message = "MockInputStream throws IOException.";


    /**
     *  Wrap a standard <code>InputStream</code>.
     *  @param  inStream    The primary input stream to
     *          wrap.
     *  @since  Version 0.1 2004-12-17
     */
    public MockInputStream(final InputStream inStream) {

        super(inStream);
    }

    /**
     *  Wrap an empty byte array.
     *  @since  Version 0.2 2005-01-13
     */
    public MockInputStream() {

        super(new ByteArrayInputStream(new byte[BUFFER_SIZE]));
    }

    /**
     *  Read a byte from the input or fail.
     *  @return The byte read from the stream.
     *  @exception  IOException Thrown by the underlying
     *          input stream or by request.
     *  @since  Version 0.1 2004-12-14
     */
    public final int read() throws IOException {

        if (failOnRead) {

            throw new IOException(message);
        }
        else {

            return super.read();
        }
    }

    /**
     *  Read into a byte array or fail.
     *  @return The number of bytes read into the buffer.
     *  @param  byteArray   The array to read the the data
     *          into.
     *  @exception  IOException Thrown by the underlying
     *          input stream or by request.
     *  @since  Version 0.1 2004-12-14
     */
    public final int read(final byte[] byteArray) throws IOException {

        if (failOnRead) {

            throw new IOException(message);
        }
        else {

            return super.read(byteArray);
        }
    }

    /**
     *  Read into a segment of a byte array or fail.
     *  @param  byteArray   The array to read the the data
     *          into.
     *  @param  off The offset index at which to start
     *          writing.
     *  @param  len The number of bytes to write.
     *  @return The number of bytes read into the buffer.
     *  @exception  IOException Thrown by the underlying
     *          input stream or by request.
     *  @since  Version 0.1 2004-12-14
     */
    public final int read(final byte[] byteArray,
                          final int off,
                          final int len) throws IOException {

        if (failOnRead) {

            throw new IOException(message);
        }
        else {

            return super.read(byteArray, off, len);
        }
    }

    /**
     *  Skip a number of bytes or fail.
     *  @param  skip    The number of bytes to skip.
     *  @return The number of bytes skipped.
     *  @exception  IOException Thrown by the underlying
     *          input stream or by request.
     *  @since  Version 0.1 2004-12-14
     */
    public final long skip(final long skip) throws IOException {

        if (failOnRead) {

            throw new IOException(message);
        }
        else {

            return super.skip(skip);
        }
    }

    /**
     *  Check whether bytes are available or fail.
     *  @return The number of bytes that can be read
     *          without blocking.
     *  @exception  IOException Thrown by the underlying
     *          input stream or by request.
     *  @since  Version 0.1 2004-12-14
     */
    public final int available() throws IOException {

        if (failOnRead) {

            throw new IOException(message);
        }
        else {

            return super.available();
        }
    }

    /**
     *  Check close the input stream or fail.
     *  @exception  IOException Thrown by the underlying
     *          input stream or by request.
     *  @since  Version 0.1 2004-12-14
     */
    public final void close() throws IOException {

        if (failOnClose) {

            throw new IOException(message);
        }
        else {

            super.close();
        }
    }

    /**
     *  Reset the stream to the marked position or fail.
     *  @exception  IOException Thrown by the underlying
     *          input stream or by request.
     *  @since  Version 0.1 2004-12-14
     */
    public final void reset()  throws IOException {

        if (failOnRead) {

            throw new IOException(message);
        }
        else {

            super.reset();
        }
    }

    /**
     *  Set the fail on read trigger.
     *  @param  setting Whether to fail on read actions.
     *  @since  Version 0.1 2004-12-17
     */
    public final void setFailOnRead(final boolean setting) {

        failOnRead = setting;
    }

    /**
     *  Set the fail on close trigger.
     *  @param  setting Whether to fail on close actions.
     *  @since  Version 0.1 2004-12-17
     */
    public final void setFailOnClose(final boolean setting) {

        failOnClose = setting;
    }

}
