package com.mkdoc.jspider;

/*
    Created:    2005-04-11
    Issued:     2005-04-11
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.api.model.Cookie;
import net.javacoding.jspider.api.model.Folder;
import net.javacoding.jspider.api.model.Site;
import net.javacoding.jspider.api.model.Resource;

import net.javacoding.jspider.spi.Rule;

import java.net.URL;
import java.net.MalformedURLException;

import java.util.Collection;
import java.util.List;
import java.util.Vector;

/**
 *  A mock <code>Site</code> for testing.
 *  @version    0.2 2005-10-28
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockSite implements Site {

    /**
     *  The default HTTP port.
     *  @since  Version 0.1 2005-04-11
     */
    public static final int HTTP_PORT = 80;

    /**
     *  All cookies.
     *  @since  Version 0.1 2005-04-11
     */
    private final List cookies = new Vector();

    /**
     *  A map of root folders.
     *  @since  Version 0.1 2005-04-11
     */
    private Folder folder;

    /**
     *  The state of the site.
     *  @since  Version 0.1 2005-04-11
     */
    private final int state = STATE_DISCOVERED;

    /**
     *  The host name of the site.
     *  @since  Version 0.1 2005-04-11
     */
    private String host = "MockSite.host";

    /**
     *  The port of the site.
     *  @since  Version 0.1 2005-04-11
     */
    private int port = HTTP_PORT;

    /**
     *  Whether robots.txt is obeyed.
     *  @since  Version 0.1 2005-04-11
     */
    private boolean obeyRobotsTxt;

    /**
     *  Whether robots.txt will be fetched.
     *  @since  Version 0.1 2005-04-11
     */
    private boolean fetchRobotsTxt;

    /**
     *  The host name of the site.
     *  @since  Version 0.1 2005-04-11
     */
    private URL url;

    /**
     *  The robots.txt rule.
     *  @since  Version 0.1 2005-04-11
     */
    private Rule robotsRule;

    /**
     *  Basic constructor with URL.
     *  @param  url The base URL for the site.
     *  @exception  MalformedURLException   If the URL is malformed.
     *  @since  Version 0.1 2005-04-11
     */
    public MockSite(final String url) throws MalformedURLException {

        super();

        this.url = new URL(url);
    }

    /**
     *  Basic constructor with URL.
     *  @param  url The base URL for the site.
     *  @since  Version 0.2 2005-10-28
     */
    public MockSite(final URL url) {

        super();

        this.url = url;
    }

    /**
     *  Get the state of the site.
     *  @return The state of the site.
     *  @since  Version 0.1 2005-04-11
     */
    public final int getState() {

        return state;
    }

    /**
     *  Get the host name of the site.
     *  @return The host name of the site.
     *  @since  Version 0.1 2005-04-11
     */
    public final String getHost() {

        return host;
    }

    /**
     *  Whether robots.txt has been handled.
     *  @return <code>true</code> if the robots.txt policy has been handled.
     *  @since  Version 0.1 2005-04-11
     */
    public final boolean isRobotsTXTHandled() {

        return state == STATE_ROBOTSTXT_HANDLED;
    }

    /**
     *  Whether robots.txt is obeyed.
     *  @return <code>true</code> if the robots.txt policy will be obeyed.
     *  @since  Version 0.1 2005-04-11
     */
    public final boolean getObeyRobotsTXT() {

        return obeyRobotsTxt;
    }

    /**
     *  Whether robots.txt will be fetched.
     *  @return <code>true</code> if the robots.txt policy will be fetched.
     *  @since  Version 0.1 2005-04-11
     */
    public final boolean getFetchRobotsTXT() {

        return fetchRobotsTxt;
    }

    /**
     *  Get the site URL.
     *  @return The URL of the site.
     *  @since  Version 0.1 2005-04-11
     */
    public final URL getURL() {

        return url;
    }

    /**
     *  Get root folders.
     *  @return An array of all the virtual directory at the root of a site.
     *  @since  Version 0.1 2005-04-11
     */
    public final Folder[] getRootFolders() {

        return folder.getFolders();
    }

    /**
     *  Get a specific root folder.
     *  @param  name    The name of the virtual directory to get.
     *  @return The root folder by the given name.
     *  @since  Version 0.1 2005-04-11
     */
    public final Folder getRootFolder(final String name) {

        return folder.getFolder(name);
    }

    /**
     *  Get root resources.
     *  @return An array of resources at the root level of the site.
     *  @since  Version 0.1 2005-04-11
     */
    public final Resource[] getRootResources() {

        return folder.getResources();
    }

    /**
     *  Get all resources.
     *  @return All resources for the site.
     *  @since  Version 0.1 2005-04-11
     */
    public final Resource[] getAllResources() {


        Collection all = getAllResources(folder);

        int size = all.size();

        return (Resource[]) all.toArray(new Resource[size]);
    }

    /**
     *  Get all resources in a folder hierarchy.
     *  @param  folder  The folder to start from.
     *  @return A collection view of all resources.
     *  @since  Version 0.1 2005-04-11
     */
    private Collection getAllResources(final Folder folder) {

        List all = new Vector();

        Resource[] resources = folder.getResources();

        for (int i = 0; i < resources.length; i++) {

            all.add(resources[i]);
        }

        Folder[] folders = folder.getFolders();

        for (int i = 0; i < folders.length; i++) {

            all.add(getAllResources(folders[i]));
        }

        return all;
    }

    /**
     *  Get the site cookies.
     *  @return An array of cookies for a site.
     *  @since  Version 0.1 2005-04-11
     */
    public final Cookie[] getCookies() {

        int size = cookies.size();

        return (Cookie[]) cookies.toArray(new Cookie[size]);
    }

    /**
     *  Get a cookie string.
     *  @return The literal cookie string for a site.
     *  @since  Version 0.1 2005-04-11
     */
    public final String getCookieString() {

        Cookie[] allCookies = getCookies();

        StringBuffer buffer = new StringBuffer();

        boolean first = true;

        for (int i = 0; i < allCookies.length; i++) {

            if (first) {

                first = false;
            }
            else {

                buffer.append("&");
            }
            buffer.append(allCookies[i].getName());
            buffer.append("=");
            buffer.append(allCookies[i].getValue());
        }

        return buffer.toString();
    }

    /**
     *  Whether cookies are used.
     *  @return <code>true</code> if cookies are in use.
     *  @since  Version 0.1 2005-04-11
     */
    public final boolean getUseCookies() {

        return cookies.size() > 0;
    }

    /**
     *  Whether a proxy is used, always false.
     *  @return <code>true</code> if a proxy is in use for the site.
     *  @since  Version 0.1 2005-04-11
     */
    public final boolean useProxy() {

        return false;
    }

    /**
     *  Get the user agent for this site.
     *  @return The user agent identifier used for the site.
     *  @since  Version 0.1 2005-04-11
     */
    public final String getUserAgent() {

        return "MockSite user agent";
    }

    /**
     *  Whether this is a base site, always false.
     *  @return Whether this is the base site for the crawl.
     *  @since  Version 0.1 2005-04-11
     */
    public final boolean isBaseSite() {

        return false;
    }

    /**
     *  Whether the site must be handled.
     *  @return <code>true</code> if the site must be handled.
     *  @since  Version 0.1 2005-04-11
     */
    public final boolean mustHandle() {

        return true;
    }

    /**
     *  Get the robots.txt rule for the site.
     *  @return The rule for handling robots.txt policy.
     *  @since  Version 0.1 2005-04-11
     */
    public final Rule getSiteRobotsTXTRule() {

        return robotsRule;
    }

    /**
     *  Check whether a proxy is in use.
     *  @return <code>true</code> if a proxy should be used for this site.
     *  @since  Version 0.1 2005-04-11
     */
    public final boolean getUseProxy() {

        return false;
    }

    /**
     *  Get the site port.
     *  @return The port number for the site.
     *  @since  Version 0.1 2005-04-11
     */
    public final int getPort() {

        return port;
    }

}
