package com.mkdoc.jtidy;

/*
    Created:    2004-12-09
    Issued:     2004-12-09
    Modified:   2005-01-07

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import net.javacoding.jspider.core.logging.Log;

import com.mkdoc.io.MockOutputStream;

import com.mkdoc.jspider.MockLog;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.OutputStream;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>TidyDriver</code>.
 *  @version    0.3 2005-01-07
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class TidyDriverTest extends TestCase {

    /**
     *  The project source directory.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final String HOME = System.getProperty("mkHome");

    /**
     *  The project test source directory.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final String TEST = System.getProperty("testDir") +
                                         File.separator +
                                         "html";

    /**
     *  The test source directory path.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final String PATH_TEST = HOME +
                                              File.separator +
                                              TEST;

    /**
     *  The project test output directory.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final String OUTPUT = System.getProperty("outputDir");

    /**
     *  An expected output directory for the test domain.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final String PATH_OUTPUT = HOME +
                                                File.separator +
                                                OUTPUT;

    /**
     *  An invalid source file for testing.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final String INVALID = "invalid-Basic.html";

    /**
     *  A file reference for invalid source for testing.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final File FILE_SRC_INVALID = new File(PATH_TEST +
                                                            File.separator +
                                                            INVALID);

    /**
     *  A file reference for invalid output file.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final File FILE_OUT_INVALID = new File(PATH_OUTPUT +
                                                            File.separator +
                                                            INVALID);
    /**
     *  An valid source file for testing.
     *  @since  Version 0.1 2004-11-26
     */
    protected static final String VALID = "valid-Basic.html";


    /**
     *  A file reference for valid output file.
     *  @since  Version 0.2 2004-12-03
     */
    protected static final File FILE_OUT_VALID = new File(PATH_OUTPUT +
                                                          File.separator +
                                                          VALID);

    /**
     *  Buffer byte size.
     *  @since  Version 0.3 2005-01-07
     */
    protected static final int BUFFER = 64;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2004-12-09
     */
    public TidyDriverTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2004-
    protected void setUp() {

    }
     */

    /**
     *  Get input stream with null input stream should
     *  throw an IllegalArgumentException.
     *  @since  Version 0.2 2004-12-10
     */
    public final void testTidyInputStreamNull() {

        TidyDriver tidy = new TidyDriver();

        try {

            tidy.tidyInputStream(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Get input stream with null input stream should
     *  throw an IllegalArgumentException.
     *  @exception  IOException Should not be thrown by
     *          this test.
     *  @since  Version 0.2 2004-12-10
     */
    public final void testTidyInputStreamReadFully() throws IOException {

        TidyDriver tidy = new TidyDriver();

        InputStream inStream = new FileInputStream(FILE_SRC_INVALID);

        InputStream tidyInStream = tidy.tidyInputStream(inStream);

        byte[] byteArray = new byte[BUFFER];

        int read = 0;

        try {

            while (true) {

                read = tidyInStream.read(byteArray);

                if (read == -1) {

                    break;
                }
            }
        }
        catch (IOException ioe) {

            fail("Should not throw an IOException.");
        }
        finally {

            inStream.close();
        }
        // No assertion, should not throw any exceptions
    }


    /**
     *  Logging constructor with null log should throw an
     *  IllegalArgumentException.
     *  @since  Version 0.1 2004-12-09
     */
    public final void testConstructorNullLog() {

        Log log = null;

        try {

            new TidyDriver(log);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  With IOException on output stream, should log and close.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2004-12-01
     */
    public final void testParse() throws IOException {

        InputStream fileInStream = null;

        OutputStream fileOutStream = null;

        MockLog log = new MockLog();

        try {

            fileInStream = new FileInputStream(FILE_SRC_INVALID);

            fileOutStream = new FileOutputStream(FILE_OUT_INVALID);

            TidyDriver tidy = new TidyDriver(log);

            tidy.parse(fileInStream, fileOutStream);
        }
        finally {

            fileInStream.close();

            fileOutStream.close();
        }

        assertTrue(FILE_OUT_INVALID.exists());

        assertTrue("".equals(log.getError()));
    }

    /**
     *  With null output stream, should log and close.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2004-12-01
     */
    public final void testParseNullOutputStream()
        throws IOException {

        InputStream fileInStream = null;

        OutputStream fileOutStream = null;

        try {

            fileInStream = new FileInputStream(FILE_SRC_INVALID);

            TidyDriver tidy = new TidyDriver();

            tidy.parse(fileInStream, fileOutStream);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

        finally {

            fileInStream.close();
        }

    }

    /**
     *  With null input stream, should log and close.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2004-12-01
     */
    public final void testParseNullInputStream()
        throws IOException {

        InputStream fileInStream = null;

        OutputStream fileOutStream = null;

        try {

            fileOutStream = new FileOutputStream(FILE_OUT_INVALID);

            TidyDriver tidy = new TidyDriver();

            tidy.parse(fileInStream, fileOutStream);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }

        finally {

            fileOutStream.close();
        }
    }

    /**
     *  If stream fails to close, should log and
     *  terminate cleanly.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2004-12-01
     */
    public final void testParseCloseThrowsIOExceptionLogged()
        throws IOException {

        InputStream fileInStream = null;

        MockOutputStream outStream = null;

        MockLog log = new MockLog();

        try {

            fileInStream = new FileInputStream(FILE_SRC_INVALID);

            outStream = new MockOutputStream();

            outStream.setFailOnClose(true);

            TidyDriver tidy = new TidyDriver(log);

            tidy.parse(fileInStream, outStream);

            String error = log.getError();

            assertTrue(error.indexOf("Error closing streams") > -1);

            assertTrue(error.indexOf("IOException") > -1);

        }
        catch (IOException iae) {

            fail("Should not throw an IOException closing the streams.");
        }

    }

    /**
     *  If stream fails to close, should terminate cleanly.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2004-12-01
     */
    public final void testParseCloseThrowsIOExceptionNoLog()
        throws IOException {

        InputStream fileInStream = null;

        MockOutputStream outStream = null;

        try {

            fileInStream = new FileInputStream(FILE_SRC_INVALID);

            outStream = new MockOutputStream();

            outStream.setFailOnClose(true);

            TidyDriver tidy = new TidyDriver();

            tidy.parse(fileInStream, outStream);

        }
        catch (IOException iae) {

            fail("Should not throw an IOException closing the streams.");
        }

    }


    /**
     *  Tear down a test scenario as required.
     *  @exception  IOException If canonical path check
     *          fails.
     *  @since  Version 0.1 2004-12-09
     */
    protected final void tearDown() throws IOException {

        if (FILE_OUT_VALID.exists()) {

            assertTrue("Could not delete valid output file '" +
                       FILE_OUT_VALID.getCanonicalPath() + "'",
                       FILE_OUT_VALID.delete());
        }
        if (FILE_OUT_INVALID.exists()) {

            assertTrue("Could not delete invalid output file '" +
                       FILE_OUT_INVALID.getCanonicalPath() + "'",
                       FILE_OUT_INVALID.delete());
        }
    }

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2004-
     */
    public static Test suite() {

        return new TestSuite(TidyDriverTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2004-
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
