package com.mkdoc.query;

/*
    Created:    2005-10-11
    Issued:     2005
    Modified:   2005-10-12

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.List;
import java.util.Vector;

/**
 *  A mock query result for testing.
 *  @version    0.1 2005-10-11
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockQueryResult implements QueryResult {

    /**
     *  The default number of columns.
     *  @since  Version 0.1 2005-10-11
     */
    private static final int COLUMNS = 3;

    /**
     *  A list of result items.
     *  @since  Version 0.1 2005-10-11
     */
    private final List results = new Vector();

    /**
     *  The number of columns.
     *  @since  Version 0.1 2005-10-11
     */
    private int columns = COLUMNS;

    /**
     *  Pre-populate the result set with duplicate items for convenience.
     *  @param  title   A common title for all result items.
     *  @param  summary A common summary for all result items.
     *  @param  uri A common URI for all result items.
     *  @param  rows    The number of items in the result set.
     *  @since  Version 0.1 2005-10-11
     */
    public MockQueryResult(final String title,
                           final String summary,
                           final String uri,
                           final int rows) {

        super();

        for (int i = 0; i < rows; i++) {

            results.add(new ResultItem(title, summary, uri));
        }
    }

    /**
     *  Get the column count.
     *  @return The number of field columns in the results.
     *  @since  Version 0.1 2005-10-11
     */
    public final int getColumns() {

        return columns;
    }

    /**
     *  Get the item summary for a row.
     *  @param  row The row index to report.
     *  @return The summary of the item at the specified index.
     *  @since  Version 0.1 2005-10-11
     */
    public final String getItemSummary(final int row) {

        ResultItem item = (ResultItem) results.get(row);

        if (item == null) {

            return "No item summary";
        }
        else {

            return item.getSummary();
        }
    }

    /**
     *  Get the item title for a row.
     *  @param  row The row index to report.
     *  @return The title of the item at the specified index.
     *  @since  Version 0.1 2005-10-11
     */
    public final String getItemTitle(final int row) {

        ResultItem item = (ResultItem) results.get(row);

        if (item == null) {

            return "No item title";
        }
        else {

            return item.getTitle();
        }
    }

    /**
     *  Get the item URI for a row.
     *  @param  row The row index to report.
     *  @return The URI of the item at the specified index.
     *  @since  Version 0.1 2005-10-11
     */
    public final String getItemUri(final int row) {

        ResultItem item = (ResultItem) results.get(row);

        if (item == null) {

            return null;
        }
        else {

            return item.getUri();
        }
    }

    /**
     *  Get the row count.
     *  @return The number of items in the results set.
     *  @since  Version 0.1 2005-10-11
     */
    public final int getRows() {

        return results.size();
    }

    /**
     *  An individual result.
     *  @since  Version 0.1 2005-10-11
     */
    class ResultItem {

        /**
         *  The item title.
         *  @since  Version 0.1 2005-10-11
         */
        private String title;

        /**
         *  The item summary.
         *  @since  Version 0.1 2005-10-11
         */
        private String summary;

        /**
         *  The item URI.
         *  @since  Version 0.1 2005-10-11
         */
        private String uri;

        /**
         *  A map of other named properties.
         *  @since  Version 0.1 2005-10-11
         */
        // private final Map fields = new Hashtable();

        /**
         *  Good citizen constructor.
         *  @param  title   The item title.
         *  @param  summary The item summary.
         *  @param  uri The item URI.
         *  @since  Version 0.1 2005-10-11
         */
        ResultItem(final String title,
                   final String summary,
                   final String uri) {

            super();

            this.title = title;
            this.summary = summary;
            this.uri = uri;
        }

        /**
         *  Get the item title.
         *  @return The item title.
         *  @since  Version 0.1 2005-10-11
         */
        public final String getTitle() {

            return title;
        }

        /**
         *  Get the item summary.
         *  @return The item summary.
         *  @since  Version 0.1 2005-10-11
         */
        public final String getSummary() {

            return summary;
        }

        /**
         *  Get the item URI.
         *  @return The item URI.
         *  @since  Version 0.1 2005-10-11
         */
        public final String getUri() {

            return uri;
        }

        /**
         *  Set the item title.
         *  @param  title   The item title.
         *  @since  Version 0.1 2005-10-11
         */
        public final void setTitle(final String title) {

            this.title = title;
        }

        /**
         *  Set the item summary.
         *  @param  summary The item summary.
         *  @since  Version 0.1 2005-10-11
         */
        public final void setSummary(final String summary) {

            this.summary = summary;
        }

        /**
         *  Set the item URI.
         *  @param  uri The item URI.
         *  @since  Version 0.1 2005-10-11
         */
        public final void getUri(final String uri) {

            this.uri = uri;
        }
    }



}
