package com.mkdoc.query;

/*
    Created:    2005-10-28
    Issued:     2005-10-28
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>SimpleQuery</code>.
 *  @version    0.1 2005-10-28
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class SimpleQueryTest extends TestCase {

    /**
     *  The default page index.
     *  @since  Version 0.1 2005-10-28
     */
    public static final int PAGE_INDEX = 1;

    /**
     *  The default results per page limit.
     *  <p>
     *    Low to make &quot;extreme&quot; paging possible with fewer results.
     *  </p>
     *  @since  Version 0.1 0.1 2005-10-28
     */
    public static final int RESULT_LIMIT = 2;


    /**
     *  An HTML type query.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String HTML_TYPE = "html";

    /**
     *  An expected IllegalArgumentException message.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";


    /**
     *  An SeRQL query.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String QUERY_STRING =
    "SELECT DISTINCT targetUri, createdDate, dcSubject, targetTitle, " +
    "targetSummary FROM {targetUri} dcterms:created {createdDate}, " +
    "     {targetUri} dc:subject {dcSubject}," +
    "     [ {targetUri} dc:title {targetTitle} ], " +
    "     [ {targetUri} dc:description {targetSummary} ] " +
    "WHERE (label(dcSubject) LIKE \"GCJ\" OR " +
    "       label(dcSubject) LIKE \"JSpider\") " +
    "AND label(createdDate) LIKE \"2005-*\" " +
    "UNION " +
    "SELECT DISTINCT targetUri, createdDate, dcSubject, targetTitle,  " +
    "targetSummary FROM {targetUri} dcterms:created {} rdf:value  " +
    "{createdDate}, {targetUri} dc:subject {dcSubject}, " +
    "     [ {targetUri} dc:title {targetTitle} ], " +
    "     [ {targetUri} dc:description {targetSummary} ] " +
    "WHERE (label(dcSubject) LIKE \"GCJ\" OR " +
    "       label(dcSubject) LIKE \"JSpider\") " +
    "AND label(createdDate) LIKE \"2005-*\" " +
    "USING NAMESPACE dc = <http://purl.org/dc/elements/1.1/>, " +
    "            dcterms = <http://purl.org/dc/terms/> ";

    /**
     *  A result key for testing.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String RESULT_KEY =
    "http://localhost/HttpQuery?dcterms.created=2005-*&amp;dc.subject=" +
    "GCJ+JSpider";

    /**
     *  The context parameters.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String PARAMS_1 = "&amp;page=" +
                                          PAGE_INDEX +
                                          "&amp;limit=" +
                                          RESULT_LIMIT +
                                          "&amp;type=" +
                                          HTML_TYPE;

    /**
     *  A custom page index.
     *  @since  Version 0.1 2005-10-28
     */
    public static final int INDEX_2 = 2;

    /**
     *  A custom result per page limit.
     *  @since  Version 0.1 2005-10-28
     */
    public static final int LIMIT_2 = 5;

    /**
     *  The context parameters.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String PARAMS_2 = "&amp;page=" +
                                          INDEX_2 +
                                          "&amp;limit=" +
                                          RESULT_LIMIT +
                                          "&amp;type=" +
                                          HTML_TYPE;

    /**
     *  The context parameters.
     *  @since  Version 0.1 2005-10-28
     */
    public static final String PARAMS_3 = "&amp;page=" +
                                          PAGE_INDEX +
                                          "&amp;limit=" +
                                          LIMIT_2 +
                                          "&amp;type=" +
                                          HTML_TYPE;


    /**
     *  An XHTML query context.
     *  @since  Version 0.1 2005-10-28
     */
    protected XhtmlQueryContext context;

    /**
     *  A query object.
     *  @since  Version 0.1 2005-10-28
     */
    protected SimpleQuery query;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-10-28
     */
    public SimpleQueryTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-10-28
     */
    protected final void setUp() {

        context = new XhtmlQueryContext(PAGE_INDEX,
                                        RESULT_LIMIT);

        query = new SimpleQuery(QUERY_STRING,
                                RESULT_KEY,
                                context);
    }

    /**
     *  Test the constructor.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testConstructorNullQueryString() {

        try {

            new SimpleQuery(null, RESULT_KEY, context);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test the constructor.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testConstructorNullResultKey() {

        try {

            new SimpleQuery(QUERY_STRING, null, context);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test the constructor.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testConstructorNullContext() {

        try {

            new SimpleQuery(QUERY_STRING, RESULT_KEY, null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test the get SeRQL method.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetSeRQLQuery() {

        String queryString = query.getSeRQLQuery();

        assertTrue(queryString.equals(QUERY_STRING));
    }

    /**
     *  Test the get result key method.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetResultKey() {

        String key = query.getResultKey();

        assertTrue(key.equals(RESULT_KEY));
    }

    /**
     *  Test query URI.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetQueryUri() {

        String uri = query.getQueryUri();

        assertTrue(uri.equals(RESULT_KEY + PARAMS_1));
    }

    /**
     *  Test query URI.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetQueryUriCustomPage() {

        String uri = query.getQueryUri(INDEX_2);

        assertTrue(uri.equals(RESULT_KEY + PARAMS_2));
    }

    /**
     *  Test query URI.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetQueryUriNegative() {

        try {

            query.getQueryUri(-1);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test query URI.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetQueryUriZero() {

        try {

            query.getQueryUri(0);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }


    /**
     *  Test query URI.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetQueryUriCustomResultsLimit() {

        String uri = query.getQueryUri(PAGE_INDEX, LIMIT_2);

        assertTrue(uri.equals(RESULT_KEY + PARAMS_3));
    }

    /**
     *  Test query URI.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetQueryUriCustomLimitZeroPage() {

        try {

            query.getQueryUri(0, LIMIT_2);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test query URI.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetQueryUriResultsLimitZero() {

        try {

            query.getQueryUri(PAGE_INDEX, 0);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

     /**
      *  Test query URI.
      *  @since  Version 0.1 2005-10-28
      */
     public final void testGetQueryUriResultsLimitNegative() {

         try {

             query.getQueryUri(PAGE_INDEX, -1);

             fail(EXPECTED_IAE);
         }
         catch (IllegalArgumentException iae) { }
     }

    /**
     *  Test the page number.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetPageNumber() {

        int page = query.getPageNumber();

        assertTrue(page == PAGE_INDEX);
    }

    /**
     *  Test the results per page
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetResultsPerPage() {

        int limit = query.getResultsPerPage();

        assertTrue(limit == RESULT_LIMIT);
    }

    /**
     *  Test the output type.
     *  @since  Version 0.1 2005-10-28
     */
    public final void testGetOutputType() {

        String type = query.getOutputType();

        assertTrue(type.equals(HTML_TYPE));
    }



    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-10-28
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-10-28
     */
    public static Test suite() {

        return new TestSuite(SimpleQueryTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-10-28
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
