package com.mkdoc.sax;

/*
    Created:    2004-12-22
    Issued:     2004-12-22
    Modified:   2005-06-15

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.schema.ApplicationProfile;
import com.mkdoc.schema.DublinCoreElements;
import com.mkdoc.schema.DublinCoreTerms;
import com.mkdoc.schema.Schema;
import com.mkdoc.schema.UKeGMS;
import com.mkdoc.schema.UKeGMSProfile;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>AbstractRdfContentHandler</code>.
 *  @version    1.3 2005-06-15
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class AbstractRdfContentHandlerTest extends TestCase {

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.9 2005-03-01
     */
    public static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  The e-GMS "egmsaes" scheme.
     *  @since  Version 0.9 2005-03-01
     */
    public static final String EGMS_AES = "egmsaes";

    /**
     *  The Dublin Core elements schema prefix.
     *  @since  Version 0.9 2005-03-01
     */
    public static final String DC_PREFIX = "dc";

    /**
     *  The Dublin Core terms schema prefix.
     *  @since  Version 0.9 2005-03-01
     */
    public static final String DCT_PREFIX = "dcterms";

    /**
     *  A generic metadata predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String GENERIC = "Description";

    /**
     *  An unrecognised predicate encoding.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String UNKNOWN = "BOGUS.Description";

    /**
     *  An Dublin Core encoded Title predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String DC_TITLE = "DC.Title";

    /**
     *  The Dublin Core W3CDTF term name.
     *  @since  Version 0.3 2005-01-06
     */
    private static final String W3CDTF = "W3CDTF";

    /**
     *  An Dublin Core encoded date scheme.
     *  @since  Version 0.3 2005-01-06
     */
    private static final String DCT_W3CDTF = "DCTERMS.W3CDTF";

    /**
     *  A Dublin Core encoded identifier predicate.
     *  @since  Version 0.7 2005-02-14
     */
    private static final String DC_ID = "DC.Identifier";

    /**
     *  A Dublin Core identifier URI.
     *  @since  Version 0.7 2005-02-14
     */
    private static final String URI_ID =
    "http://purl.org/dc/elements/1.1/identifier";

    /**
     *  A Dublin Core encoded relation predicate.
     *  @since  Version 0.7 2005-02-14
     */
    private static final String DC_RELATION = "DC.Relation";

    /**
     *  A Dublin Core relation URI.
     *  @since  Version 0.7 2005-02-14
     */
    private static final String URI_RELATION =
    "http://purl.org/dc/elements/1.1/relation";

    /**
     *  A Dublin Core encoded source predicate.
     *  @since  Version 0.7 2005-02-14
     */
    private static final String DC_SOURCE = "DC.Source";

    /**
     *  A Dublin Core source URI.
     *  @since  Version 0.7 2005-02-14
     */
    private static final String URI_SOURCE =
    "http://purl.org/dc/elements/1.1/source";

    /**
     *  A space separated list of generic link types.
     *  @since  Version 0.7 2005-02-14
     */
    private static final String LNK_GENERIC = " top copyright prev last ";

    /**
     *  A space separated list of Dublin Core link predicates.
     *  @since  Version 0.7 2005-02-14
     */
    private static final String LINKS_DC = " " +
                                           DC_ID +
                                           " " +
                                           DC_RELATION +
                                           " " +
                                           DC_SOURCE +
                                           " ";

    /**
     *  A space separated list of mixed Dublin Core predicates and generic link
     *  types.
     *  @since  Version 0.7 2005-02-14
     */
    private static final String LINKS_MIXED = " top " +
                                              DC_ID +
                                              " copyright " +
                                              DC_RELATION +
                                              " prev " +
                                              DC_SOURCE +
                                              " last ";

    /**
     *  A test URI.
     *  @since  Version 0.4 2005-01-07
     */
    protected static final String URI_1 =
    "http://www.remote.example/uri/test1";

    /**
     *  A test URI.
     *  @since  Version 0.4 2005-01-07
     */
    protected static final String URI_2 =
    "http://www.remote.example/uri/test2";


    /**
     *  A mock RDF <code>ContentHandler</code> to test.
     *  @since  Version 0.1 2004-12-22
     */
    private MockRdfContentHandler rch = new MockRdfContentHandler();


    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2004-12-22
     */
    public AbstractRdfContentHandlerTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2004-
    protected void setUp() {

    }
     */

    /**
     *  Test set a profile.
     *  @since  1.2 2005-05-26
     */
    public final void testSetApplicationProfileNull() {

        try {

            rch.setApplicationProfile(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test set an e-GMS profile.
     *  @since  1.2 2005-05-26
     */
    public final void testSetApplicationProfile() {

        Schema eGMS = new UKeGMS();

        ApplicationProfile profile = new UKeGMSProfile();

        rch.setApplicationProfile(profile);

        String predicate = "eGMS.Status";

        String expanded = rch.expandPredicate(predicate);

        assertTrue("Handler should expand an e-GMS predicate, " +
                   "actual expanded value was '" +
                   expanded + "'.",
                   expanded.startsWith(eGMS.getUri()));

    }

    /**
     *  Test adding an e-GMS schema.
     *  @since  1.2 2005-05-26
     */
    public final void testExpandSchemeEGMS() {

        Schema eGMS = new UKeGMS();

        ApplicationProfile profile = new UKeGMSProfile();

        rch.setApplicationProfile(profile);

        String expanded = rch.expandScheme(EGMS_AES);

        assertTrue("Handler should expand an e-GMS scheme, " +
                   "actual expanded value was '" +
                   expanded + "'.",
                   expanded.startsWith(eGMS.getUri()));

    }


    /**
     *  Expand a null list of predicates should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.7 2005-02-14
     */
    public final void testExpandPredicatesNullList() {

        try {

            rch.expandPredicates(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Purely generic link types should not be expanded.
     *  @since  Version 0.7 2005-02-14
     */
    public final void testExpandPredicatesPureGeneric() {

        String[] predicates = rch.expandPredicates(LNK_GENERIC);

        assertTrue(predicates.length == 0);
    }

    /**
     *  All Dublin Core predicates should be expanded.
     *  @since  Version 0.7 2005-02-14
     */
    public final void testExpandPredicatesDublinCore() {

        String[] predicates = rch.expandPredicates(LINKS_DC);

        assertTrue("Array should contain 3 items, actually contains " +
                   predicates.length + " items.",
                   predicates.length == 3);

        boolean hasId = false;

        boolean hasRel = false;

        boolean hasSrc = false;

        for (int i = 0; i < predicates.length; i++) {

            if (predicates[i].equals(URI_ID)) {

                hasId = true;
            }
            if (predicates[i].equals(URI_RELATION)) {

                hasRel = true;
            }
            if (predicates[i].equals(URI_SOURCE)) {

                hasSrc = true;
            }
        }

        assertTrue("Should contain a DC identifier predicate. ",
                   hasId);
        assertTrue("Should contain a DC relation predicate. ",
                   hasRel);
        assertTrue("Should contain a DC source predicate. ",
                   hasSrc);
    }

    /**
     *  Only Dublin Core predicates should be expanded.
     *  @since  Version 0.7 2005-02-14
     */
    public final void testExpandPredicatesMixed() {

        String[] predicates = rch.expandPredicates(LINKS_MIXED);

        assertTrue("Array should contain 3 items, actually contains " +
                   predicates.length + " items.",
                   predicates.length == 3);

        boolean hasId = false;

        boolean hasRel = false;

        boolean hasSrc = false;

        for (int i = 0; i < predicates.length; i++) {

            if (predicates[i].equals(URI_ID)) {

                hasId = true;
            }
            if (predicates[i].equals(URI_RELATION)) {

                hasRel = true;
            }
            if (predicates[i].equals(URI_SOURCE)) {

                hasSrc = true;
            }
        }

        assertTrue("Should contain a DC identifier predicate. ",
                   hasId);
        assertTrue("Should contain a DC relation predicate. ",
                   hasRel);
        assertTrue("Should contain a DC source predicate. ",
                   hasSrc);
    }

    /**
     *  Set null subject should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.4 2005-01-07
     */
    public final void testSetSubjectNull() {

        try {

            rch.setSubject(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Should adopt the new URI reference.
     *  @since  Version 0.4 2005-01-07
     */
    public final void testSetGetSubject() {

        rch.setSubject(URI_1);

        assertTrue(URI_1.equals(rch.getSubject()));

        rch.setSubject(URI_2);

        assertTrue(URI_2.equals(rch.getSubject()));
    }

    /**
     *  Test expand with a null argument.
     *  @since  Version 0.3 2005-01-06
     */
    public final void testExpandSchemeNullComponent() {

        try {

            rch.expandScheme(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test expand with a generic metadata scheme.
     *  @since  Version 0.3 2005-01-06
     */
    public final void testExpandSchemeGeneric() {

        String expanded = rch.expandScheme(GENERIC);

        assertTrue("Scheme should not be expanded, " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(GENERIC));
    }

    /**
     *  Test expand with an unknown metadata scheme.
     *  @since  Version 0.3 2005-01-06
     */
    public final void testExpandSchemeUnrecognisedSchema() {

        String expanded = rch.expandScheme(UNKNOWN);

        assertTrue("Scheme should not be expanded, " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(UNKNOWN));
    }

    /**
     *  Test expand with a new Dublin Core metadata scheme.
     *  @since  Version 0.3 2005-01-06
     */
    public final void testExpandSchemeNewDCTermsScheme() {

        DublinCoreTerms dct = new DublinCoreTerms();

        String expanded = rch.expandScheme(DCT_W3CDTF);

        assertTrue("Encoded scheme should  be expanded to " +
                   dct.BASE + W3CDTF + ", " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(dct.BASE + W3CDTF));
    }

    /**
     *  Test expand with a new Dublin Core metadata scheme.
     *  @since  Version 0.3 2005-01-06
     */
    public final void testExpandSchemeNewDCTermsSchemeNotMapped() {

        DublinCoreTerms dct = new DublinCoreTerms();

        String expanded = rch.expandScheme(DCT_W3CDTF);

        assertTrue("Encoded scheme should  be expanded to " +
                   dct.BASE + W3CDTF + ", " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(dct.BASE + W3CDTF));
    }

    /**
     *  Test expand with a new Dublin Core metadata scheme.
     *  @since  Version 0.3 2005-01-06
     */
// @REFACTORING@
/*
    public final void testExpandSchemeNewDCTermsNoSchemeMapped() {

        DublinCoreElements dce = new DublinCoreElements();
        DublinCoreTerms dct = new DublinCoreTerms();

        rch.removeSchema(DCT_PREFIX);
        rch.removeSchema(DC_PREFIX);

        String expanded = rch.expandScheme(DCT_W3CDTF);

        assertTrue("Encoded scheme should not be expanded, " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(DCT_W3CDTF));

        rch.addSchema(DCT_PREFIX, dct);
        rch.addSchema(DC_PREFIX, dce);

    }
*/

    /**
     *  Test expand with a new Dublin Core metadata scheme.
     *  @since  Version 0.3 2005-01-06
     */
    public final void testExpandSchemeOldDCTermsScheme() {

        DublinCoreTerms dct = new DublinCoreTerms();

        String expanded = rch.expandScheme(W3CDTF);

        assertTrue("Encoded scheme should  be expanded to " +
                   dct.BASE + W3CDTF + ", " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(dct.BASE + W3CDTF));
    }

    /**
     *  Test expand with a new Dublin Core metadata scheme.
     *  @since  Version 0.3 2005-01-06
     */
// @REFACTORING@
/*
    public final void testExpandSchemeOldDCTermsSchemeNotMapped() {

        DublinCoreElements dce = new DublinCoreElements();
        DublinCoreTerms dct = new DublinCoreTerms();

        rch.removeSchema(DCT_PREFIX);
        rch.removeSchema(DC_PREFIX);

        String expanded = rch.expandScheme(W3CDTF);

        assertTrue("Encoded scheme should not be expanded, " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(W3CDTF));


        rch.addSchema(DCT_PREFIX, dct);
        rch.addSchema(DC_PREFIX, dce);

    }
*/

    /**
     *  Get a series of bNode references.
     *  @since  Version 0.3 2005-01-06
     */
    public final void testGetBNode() {

        String bNode = null;

        bNode = rch.getBNode();

        int first = Integer.parseInt(bNode.substring(7));

        bNode = rch.getBNode();

        int next = Integer.parseInt(bNode.substring(7));

        assertTrue(next == (first + 1));

        bNode = rch.getBNode();

        next = Integer.parseInt(bNode.substring(7));

        assertTrue(next == (first + 2));
    }

    /**
     *  Get a series of bNode identifiers.
     *  @since  Version 0.3 2005-01-06
     */
    public final void testGetBNodeId() {

        String bNode = null;

        bNode = rch.getBNodeId();

        int first = Integer.parseInt(bNode.substring(5));

        bNode = rch.getBNodeId();

        int next = Integer.parseInt(bNode.substring(5));

        assertTrue("BNode index should equal '" + (first + 1) + "', " +
                   "actual index is '" + next + "'.",
                   next == (first + 1));

        bNode = rch.getBNodeId();

        next = Integer.parseInt(bNode.substring(5));

        assertTrue("BNode index should equal '" + (first + 2) + "', " +
                   "actual index is '" + next + "'.",
                   next == (first + 2));
    }

    /**
     *  Test expand with a null argument.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandPredicateNullComponent() {

        try {

            rch.expandPredicate(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test expand with a generic metadata predicate.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandPredicateGeneric() {

        String expanded = rch.expandPredicate(GENERIC);

        assertTrue("Predicate should not be expanded, " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(GENERIC));
    }

    /**
     *  Test expand with a generic metadata predicate.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandPredicateUnrecognisedSchema() {

        String expanded = rch.expandPredicate(UNKNOWN);

        assertTrue("Predicate should not be expanded, " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(UNKNOWN));
    }

    /**
     *  Test expand with a generic metadata predicate.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandPredicateRecognisedSchema() {

        DublinCoreElements dce = new DublinCoreElements();

        String expanded = rch.expandPredicate(DC_TITLE);

        assertTrue("Encoded predicate should  be expanded to " +
                   dce.BASE + dce.NAME_TITLE + ", " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_TITLE));
    }

    /**
     *  Test escape with literal.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testEscapeLiteral() {

        String literal = "Test string";

        String escape = rch.escape(literal);

        assertTrue(escape.equals("\"" + literal + "\""));
    }

    /**
     *  Test escape with URI.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testEscapeURI() {

        String uri = "http://www.remote.example/uri/test";

        String escape = rch.escape(uri);

        assertTrue(escape.equals("<" + uri + ">"));
    }

    /**
     *  Test escape with null component.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testEscapeNull() {

        try {

            rch.escape(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test isUri with null component.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testIsUriNull() {


        assertFalse(rch.isUri(null));

    }

    /**
     *  Test isUri with unrecognised scheme.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testIsUriFalse() {

        assertFalse(rch.isUri("telnet://uri"));
    }

    /**
     *  Test isUri with mailto scheme.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testIsUriMailto() {

        assertTrue(rch.isUri("mailto:phil@mkdoc.com"));
    }

    /**
     *  Test isUri with FTP scheme.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testIsUriFTP() {

        assertTrue(rch.isUri("ftp://ftp.remote.example/"));
    }

    /**
     *  Test isUri with HTTPS scheme.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testIsUriHTTPS() {

        assertTrue(rch.isUri("https://secure.remote.example/"));
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2004-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2004-12-22
     */
    public static Test suite() {

        return new TestSuite(AbstractRdfContentHandlerTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2004-12-22
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
