package com.mkdoc.sax;

/*
    Created:    2005-02-14
    Issued:     2005-02-14
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.model.Graph;
import org.openrdf.model.URI;
import org.openrdf.model.Value;
import org.openrdf.model.ValueFactory;

import org.openrdf.model.impl.GraphImpl;

import com.mkdoc.schema.ApplicationProfile;
import com.mkdoc.schema.Schema;
import com.mkdoc.schema.UKeGMS;
import com.mkdoc.schema.UKeGMSProfile;

import com.mkdoc.store.MockStoreManager;

import java.io.IOException;

import java.net.URL;

import org.xml.sax.SAXException;

import org.xml.sax.helpers.AttributesImpl;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>LinkStoreWriter</code>.
 *  @version    0.8 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class LinkStoreWriterTest extends TestCase {

    /**
     *  An expected graph content message.
     *  @since  Version 0.8 2005-11-01
     */
    public static final String GRAPH_SHOULD =
    "Graph should contain a statement with subject '";

    /**
     *  An expected literal content message.
     *  @since  Version 0.8 2005-11-01
     */
    public static final String AND_LITERAL =
    "', and literal value '";

    /**
     *  An expected predicate message.
     *  @since  Version 0.8 2005-11-01
     */
    public static final String AND_PRED = "' and predicate '";

    /**
     *  An expected object message.
     *  @since  Version 0.8 2005-11-01
     */
    public static final String AND_OBJECT = "', and object URI '";

    /**
     *  An actual graph content prefix.
     *  @since  Version 0.8 2005-11-01
     */
    public static final String ACTUAL_GRAPH =
    "Actual graph =\n";

    /**
     *  An end quote string.
     *  @since  Version 0.8 2005-11-01
     */
    public static final String ENDQUOTE = "'. ";

    /**
     *  An expected bNode content message.
     *  @since  Version 0.8 2005-11-01
     */
    public static final String EXPECT_BNODE =
    "Graph should contain a bNode statement with predicate '";

    /**
     *  The XHTML namespace URI.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String NAMESPACE = "http://www.w3.org/1999/xhtml";

    /**
     *  A test subject URI.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String URI_SUBJ_1 =
    "http://www.mkdoc.org/uri/test/LinkStoreWriter#one";

    /**
     *  A test subject URI.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String URI_SUBJ_2 =
    "http://www.mkdoc.org/uri/test/LinkStoreWriter#two";

    /**
     *  The DC rights URI.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String URI_RIGHTS =
    "http://purl.org/dc/elements/1.1/rights";

    /**
     *  The DC qualified URI type.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String URI_DCTERMS = "http://purl.org/dc/terms/URI";

    /**
     *  The RDF type URI.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String RDF_TYPE =
    "http://www.w3.org/1999/02/22-rdf-syntax-ns#type";

    /**
     *  The RDF value URI.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String RDF_VALUE =
    "http://www.w3.org/1999/02/22-rdf-syntax-ns#value";

    /**
     *  An XHTML <code>head</code> element local name.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String HEAD = "head";

    /**
     *  An XHTML <code>link</code> element local name.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String LINK = "link";

    /**
     *  An XHTML <code>rel</code> attribute local name.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String REL = "rel";

    /**
     *  An XHTML <code>rev</code> attribute local name.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String REV = "rev";

    /**
     *  A qualified XHTML <code>rel</code> attribute.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String QUAL_REL = NAMESPACE +
                                            ":" +
                                            REL;

    /**
     *  A qualified XHTML <code>rev</code> attribute.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String QUAL_REV = NAMESPACE +
                                           ":" +
                                           REV;

    /**
     *  An XHTML <code>href</code> attribute local name.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String HREF = "href";

    /**
     *  A qualified XHTML <code>rel</code> attribute.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String QUAL_HREF = NAMESPACE +
                                               ":" +
                                               HREF;

    /**
     *  A generic XHTML <code>copyright</code> rel value.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String REL_GENERIC = "copyright";

    /**
     *  A generic XHTML <code>copyright</code> href value.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String HREF_COPY =
    "http://test.mksearch.mkdoc.org/test/copyright";

    /**
     *  A DC rights predicate.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String DC_RIGHTS = "DC.Rights";

    /**
     *  A DC rights object URI value.
     *  @since  Version 0.7 2005-09-09
     */
    private static final String HREF_RIGHTS =
    "http://test.mksearch.mkdoc.org/test/rights";

    /**
     *  A DC rights object URI value.
     *  @since  Version 0.6 2005-09-09
     */
    private static final String HREF_PADDED =
    " http://test.mksearch.mkdoc.org/test/rights ";

    /**
     *  A qualified XHTML <code>link</code> element name.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String QUAL_LINK = NAMESPACE +
                                            ":" +
                                            LINK;

    /**
     *  A qualified XHTML <code>head</code> element name.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String QUAL_HEAD = NAMESPACE +
                                            ":" +
                                            HEAD;

    /**
     *  A CDATA type string.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String CDATA = "CDATA";

    /**
     *  An expected <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-02-14
     */
    private static final String EXPECT_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  An expected <code>SAXException</code>.
     *  @since  Version 0.2 2005-02-15
     */
    private static final String EXPECT_SE =
    "Should throw a SAXException.";

    /**
     *  An empty attribute set for testing.
     *  @since  Version Version 0.1 2005-02-14
     */
    private AttributesImpl attributes;

    /**
     *  Instance to test.
     *  @since  Version 0.1 2005-02-14
     */
    protected LinkStoreWriter storeWriter;

    /**
     *  An instance with default construction to test.
     *  @since  Version 0.1 2005-02-14
     */
    protected LinkStoreWriter basicWriter;

    /**
     *  A <code>MockSesameRepository</code>.
     *  @since  Version 0.1 2005-02-14
     */
    protected MockStoreManager manager;

    /**
     *  A value factory for generating statements.
     *  @since  Version 0.1 2005-02-14
     */
    protected ValueFactory factory;

    /**
     *  A graph implementation.
     *  @since  Version 0.2 2005-02-15
     */
    protected Graph graph;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-02-14
     */
    public LinkStoreWriterTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @exception  IOException Should not be thrown.
     *  @since  Version 0.1 2005-02-14
     */
    protected final void setUp() throws IOException {

        graph = new GraphImpl();

        factory = graph.getValueFactory();

        manager = new MockStoreManager();

        storeWriter = new LinkStoreWriter(manager, new URL(URI_SUBJ_1));

        attributes = new AttributesImpl();

        basicWriter = new LinkStoreWriter();
    }

    /**
     *  Test constructor with custom schema map.
     *  @since  Version 0.5 2005-05-25
     */
    public final void testConstructorNullApplicationProfile() {

        ApplicationProfile profile = null;

        try {

            new LinkStoreWriter(profile);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test constructor with custom schema map.
     *  @since  Version 0.5 2005-05-25
     */
    public final void testConstructorCustomApplicationProfile() {

        Schema egms = new UKeGMS();

        ApplicationProfile profile = new UKeGMSProfile();

        LinkStoreWriter customWriter = new LinkStoreWriter(profile);

        String predicate = "egms.Mandate";

        String expanded = customWriter.expandPredicate(predicate);

        String uri = egms.getUri() + "mandate";

        assertTrue("Should expand the predicate to the URI " +
                   uri + ", actually returned '" +
                   expanded + "'.",
                   expanded.equals(uri));
    }

    /**
     *  Check whether an element is accepted.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testAcceptElementNullUri() {

        try {

            storeWriter.acceptElement(null,
                                      LINK);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Check whether an element is accepted.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testAcceptElementNullLocalName() {

        try {

            storeWriter.acceptElement("http://www.w3.org/1999/xhtml",
                                      null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Check whether an element is accepted.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testAcceptElementFalse() {


        assertFalse(storeWriter.acceptElement("http://example.org/1999/xhtml",
                                              LINK));
    }

    /**
     *  Check whether an element is accepted.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testAcceptElementTrue() {


        assertTrue(storeWriter.acceptElement(NAMESPACE,
                                             LINK));
    }

    /**
     *  Add triples with null argument.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testAddTriplesNullSourceUri() {

        try {

            storeWriter.addTriples(null,
                                   attributes,
                                   graph);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Add triples with null argument.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testAddTriplesNullAttributes() {

        URI subjectUri = factory.createURI(URI_SUBJ_1);

        try {

            storeWriter.addTriples(subjectUri,
                                   null,
                                   graph);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Add triples with null argument.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testAddTriplesNullGraph() {

        URI subjectUri = factory.createURI(URI_SUBJ_1);

        try {

            storeWriter.addTriples(subjectUri,
                                   attributes,
                                   null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Start a <code>link</code> element with a default instance.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testStartElementDefaultInstance() {

        try {

            basicWriter.startElement(NAMESPACE,
                                       LINK,
                                       QUAL_LINK,
                                       attributes);

            fail(EXPECT_SE);
        }
        catch (SAXException se) { }

    }

    /**
     *  Start a <code>link</code> element with a default instance.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testStartElementDefaultInstanceWithSubject() {

        URI subjectUri = factory.createURI(URI_SUBJ_1);

        basicWriter.setSubjectUri(subjectUri);

        try {

            basicWriter.startElement(NAMESPACE,
                                       LINK,
                                       QUAL_LINK,
                                       attributes);

            fail(EXPECT_SE);
        }
        catch (SAXException se) { }

    }

    /**
     *  Start a non-<code>link</code> element should throw
     *  <code>SAXException</code>.
     *  @since  Version 0.1 2005-02-14
     */
    public final void testStartElementNotLink() {

        try {

            storeWriter.startElement(NAMESPACE,
                                     HEAD,
                                     QUAL_HEAD,
                                     attributes);

            fail(EXPECT_SE);
        }
        catch (SAXException se) { }

    }

    /**
     *  XHTML <code>link</code> no <code>href</code>
     *  attribute, should not generate a statement.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2005-02-14
     */
    public final void testStartElementNoHrefAttr() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                REL,
                                QUAL_REL,
                                CDATA,
                                DC_RIGHTS);

        storeWriter.startElement(NAMESPACE,
                                 LINK,
                                 QUAL_LINK,
                                 attributes);

        URI subject = factory.createURI(URI_SUBJ_1);

        storeWriter.endDocument();

        graph = manager.getGraph();

        assertFalse(graph.contains(subject, null, null));
    }

    /**
     *  XHTML <code>link</code> no <code>rel</code>
     *  attribute, should not generate a statement.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2005-02-14
     */
    public final void testStartElementNoRelAttr() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                HREF,
                                QUAL_HREF,
                                CDATA,
                                HREF_RIGHTS);

        storeWriter.startElement(NAMESPACE,
                                 LINK,
                                 QUAL_LINK,
                                 attributes);

        URI subject = factory.createURI(URI_SUBJ_1);

        storeWriter.endDocument();

        graph = manager.getGraph();

        assertFalse(graph.contains(subject, null, null));

    }

    /**
     *  XHTML <code>link</code> basic attributes.
     *  @exception  SAXException    If there is a problem
     *          parsing the content, indicates an error.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testStartElementGeneric() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                REL,
                                QUAL_REL,
                                CDATA,
                                REL_GENERIC);

        attributes.addAttribute(NAMESPACE,
                                HREF,
                                QUAL_HREF,
                                CDATA,
                                HREF_COPY);

        storeWriter.startElement(NAMESPACE,
                                 LINK,
                                 QUAL_LINK,
                                 attributes);

        storeWriter.endDocument();

        graph = manager.getGraph();

        URI subject = factory.createURI(URI_SUBJ_1);

        assertFalse(graph.contains(subject, null, null));
    }

    /**
     *  XHTML <code>link</code> with schema prefix -- forward relation.
     *  @exception  SAXException    If there is a problem
     *          parsing the content, indicates an error.
     *  @since  Version 0.6 2005-01-07
     */
    public final void testStartElementSchemaRel() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                REL,
                                QUAL_REL,
                                CDATA,
                                DC_RIGHTS);

        attributes.addAttribute(NAMESPACE,
                                HREF,
                                QUAL_HREF,
                                CDATA,
                                HREF_RIGHTS);

        storeWriter.startElement(NAMESPACE,
                                 LINK,
                                 QUAL_LINK,
                                 attributes);

        storeWriter.endDocument();

        graph = manager.getGraph();

        String graphString = manager.getGraphString();


        URI subject = factory.createURI(URI_SUBJ_1);

        URI predicate = factory.createURI(URI_RIGHTS);

        assertTrue(GRAPH_SHOULD +
                   subject + AND_PRED + predicate + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(subject, predicate, null));

        predicate = factory.createURI(RDF_TYPE);

        Value object = factory.createURI(URI_DCTERMS);

        assertTrue(EXPECT_BNODE +
                   predicate + AND_OBJECT + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));

        predicate = factory.createURI(RDF_VALUE);

        object = factory.createURI(HREF_RIGHTS);

        assertTrue(EXPECT_BNODE +
                   predicate + AND_OBJECT + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));
    }


    /**
     *  XHTML <code>link</code> with excessive white space.
     *  @exception  SAXException    If there is a problem
     *          parsing the content, indicates an error.
     *  @since  Version 0.7 2005-09-09
     */
    public final void testStartElementSchemaRelTrim() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                REL,
                                QUAL_REL,
                                CDATA,
                                DC_RIGHTS);

        attributes.addAttribute(NAMESPACE,
                                HREF,
                                QUAL_HREF,
                                CDATA,
                                HREF_PADDED);

        storeWriter.startElement(NAMESPACE,
                                 LINK,
                                 QUAL_LINK,
                                 attributes);

        storeWriter.endDocument();

        graph = manager.getGraph();

        String graphString = manager.getGraphString();


        URI subject = factory.createURI(URI_SUBJ_1);

        URI predicate = factory.createURI(URI_RIGHTS);

        assertTrue(GRAPH_SHOULD +
                   subject + AND_PRED + predicate + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(subject, predicate, null));

        predicate = factory.createURI(RDF_TYPE);

        Value object = factory.createURI(URI_DCTERMS);

        assertTrue(EXPECT_BNODE +
                   predicate + AND_OBJECT + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));

        predicate = factory.createURI(RDF_VALUE);

        object = factory.createURI(HREF_RIGHTS);

        assertTrue(EXPECT_BNODE +
                   predicate + AND_OBJECT + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));
    }

    /**
     *  XHTML <code>link</code> with schema prefix -- reverse relation.
     *  @exception  SAXException    If there is a problem
     *          parsing the content, indicates an error.
     *  @since  Version 0.6 2005-01-07
     */
    public final void testStartElementSchemaRev() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                REV,
                                QUAL_REV,
                                CDATA,
                                DC_RIGHTS);

        attributes.addAttribute(NAMESPACE,
                                HREF,
                                QUAL_HREF,
                                CDATA,
                                HREF_RIGHTS);

        storeWriter.startElement(NAMESPACE,
                                 LINK,
                                 QUAL_LINK,
                                 attributes);

        storeWriter.endDocument();

        graph = manager.getGraph();

        String graphString = manager.getGraphString();


        URI subject = factory.createURI(HREF_RIGHTS);

        URI predicate = factory.createURI(URI_RIGHTS);

        assertTrue(GRAPH_SHOULD +
                   subject + AND_PRED + predicate + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(subject, predicate, null));

        predicate = factory.createURI(RDF_TYPE);

        Value object = factory.createURI(URI_DCTERMS);

        assertTrue(EXPECT_BNODE +
                   predicate + AND_OBJECT + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));

        predicate = factory.createURI(RDF_VALUE);

        object = factory.createURI(URI_SUBJ_1);

        assertTrue(EXPECT_BNODE +
                   predicate + AND_OBJECT + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));
    }

    /**
     *  XHTML <code>link</code> with excessive white space.
     *  @exception  SAXException    If there is a problem
     *          parsing the content, indicates an error.
     *  @since  Version 0.7 2005-09-09
     */
    public final void testStartElementSchemaRevTrim() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                REV,
                                QUAL_REV,
                                CDATA,
                                DC_RIGHTS);

        attributes.addAttribute(NAMESPACE,
                                HREF,
                                QUAL_HREF,
                                CDATA,
                                HREF_PADDED);

        storeWriter.startElement(NAMESPACE,
                                 LINK,
                                 QUAL_LINK,
                                 attributes);

        storeWriter.endDocument();

        graph = manager.getGraph();

        String graphString = manager.getGraphString();


        URI subject = factory.createURI(HREF_RIGHTS);

        URI predicate = factory.createURI(URI_RIGHTS);

        assertTrue(GRAPH_SHOULD +
                   subject + AND_PRED + predicate + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(subject, predicate, null));

        predicate = factory.createURI(RDF_TYPE);

        Value object = factory.createURI(URI_DCTERMS);

        assertTrue(EXPECT_BNODE +
                   predicate + AND_OBJECT + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));

        predicate = factory.createURI(RDF_VALUE);

        object = factory.createURI(URI_SUBJ_1);

        assertTrue(EXPECT_BNODE +
                   predicate + AND_OBJECT + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));
    }

    /**
     *  Set a null URI subject should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-02-14
     */
    public final void testSetSubjectUri() {

        try {

            storeWriter.setSubjectUri(null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Set and get a subject URI.
     *  @since  Version 0.1 2005-02-14
     */
    public final void testSetGetSubjectUri() {

        URI uri = storeWriter.getSubjectUri();

        assertTrue(URI_SUBJ_1.equals(uri.getURI()));

        uri = factory.createURI(URI_SUBJ_2);

        storeWriter.setSubjectUri(uri);

        uri = storeWriter.getSubjectUri();

        assertTrue(URI_SUBJ_2.equals(uri.getURI()));
    }

    /**
     *  End document adds a graph to the store.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2005-02-14
     */
    public final void testEndDocument() throws SAXException {

        graph = manager.getGraph();

        assertNull(graph);

        storeWriter.endDocument();

        graph = manager.getGraph();

        assertNotNull(graph);
    }

    /**
     *  Constructor with null subject manager should throw an
     *  <code>IllegalArgumentException</code>.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-02-14
     */
    public final void testConstructorNullStoreManager() throws IOException {

        try {

            new LinkStoreWriter(null,
                                new URL(URI_SUBJ_1));

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Constructor with null subject URI should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-02-14
     */
    public final void testConstructorNullUri() {

        try {

            new LinkStoreWriter(manager,
                                null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-02-14
     */
    public static Test suite() {

        return new TestSuite(LinkStoreWriterTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-02-14
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
