package com.mkdoc.sax;

/*
    Created:    2005-01-13
    Issued:     2005-01-13
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.model.Graph;
import org.openrdf.model.URI;
import org.openrdf.model.Value;
import org.openrdf.model.ValueFactory;

import org.openrdf.model.impl.GraphImpl;

import com.mkdoc.sesame.MockSesameRepository;
import com.mkdoc.sesame.MockRdfSource;
import com.mkdoc.sesame.MockLocalService;

import com.mkdoc.schema.ApplicationProfile;
import com.mkdoc.schema.Schema;
import com.mkdoc.schema.UKeGMS;
import com.mkdoc.schema.UKeGMSProfile;

import com.mkdoc.store.MockStoreManager;

import java.io.IOException;

import java.net.URL;

import org.xml.sax.SAXException;

import org.xml.sax.helpers.AttributesImpl;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>MetaStoreWriter</code>.
 *  @version    0.8 2005-11-01
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class MetaStoreWriterTest extends TestCase {

    /**
     *  A repository identifier.
     *  @since  Version 0.3 2005-03-29
     */
    protected static final String REPO_ID = "Test repository";

    /**
     *  An expected graph content message.
     *  @since  Version 0.8 2005-11-01
     */
    public static final String GRAPH_SHOULD =
    "Graph should contain a statement with subject '";

    /**
     *  An expected literal content message.
     *  @since  Version 0.8 2005-11-01
     */
    public static final String AND_LITERAL =
    "', and literal value '";

    /**
     *  An actual graph content prefix.
     *  @since  Version 0.8 2005-11-01
     */
    public static final String ACTUAL_GRAPH =
    "Actual graph =\n";

    /**
     *  An end quote string.
     *  @since  Version 0.8 2005-11-01
     */
    public static final String ENDQUOTE = "'. ";

    /**
     *  An expected bNode content message.
     *  @since  Version 0.8 2005-11-01
     */
    public static final String EXPECT_BNODE =
    "Graph should contain a bNode statement with predicate '";

    /**
     *  The XHTML namespace URI.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String NAMESPACE = "http://www.w3.org/1999/xhtml";

    /**
     *  A test subject URI.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String URI_SUBJ_1 =
    "http://www.mkdoc.org/uri/test/MetaStoreWriter#one";

    /**
     *  A test subject URI.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String URI_SUBJ_2 =
    "http://www.mkdoc.org/uri/test/MetaStoreWriter#two";

    /**
     *  The DC date URI.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String URI_DC_DATE =
    "http://purl.org/dc/elements/1.1/date";

    /**
     *  The DC terms W3C DTF URI.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String URI_W3CDTF =
    "http://purl.org/dc/terms/W3CDTF";

    /**
     *  The RDF type URI.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String RDF_TYPE =
    "http://www.w3.org/1999/02/22-rdf-syntax-ns#type";

    /**
     *  The RDF value URI.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String RDF_VALUE =
    "http://www.w3.org/1999/02/22-rdf-syntax-ns#value";

    /**
     *  An XHTML <code>head</code> element local name.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String HEAD = "head";

    /**
     *  An XHTML <code>meta</code> element local name.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String META = "meta";

    /**
     *  An XHTML <code>name</code> attribute local name.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String NAME = "name";

    /**
     *  A qualified XHTML <code>name</code> attribute.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String QUAL_NAME = NAMESPACE +
                                            ":" +
                                            NAME;

    /**
     *  An XHTML <code>scheme</code> attribute local name.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String SCHEME = "scheme";

    /**
     *  A qualified XHTML <code>scheme</code> attribute.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String QUAL_SCHEME = NAMESPACE +
                                              ":" +
                                              SCHEME;

    /**
     *  An XHTML <code>content</code> attribute local name.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String CONTENT = "content";

    /**
     *  A qualified XHTML <code>name</code> attribute.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String QUAL_CONTENT = NAMESPACE +
                                               ":" +
                                               CONTENT;

    /**
     *  A generic XHTML <code>description</code> name value.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String NAME_DESC = "description";

    /**
     *  A generic XHTML <code>description</code> content
     *  value.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String VALUE_DESC =
    "Generic description content";

    /**
     *  A DC title predicate.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String DC_DATE = "DC.Date";

    /**
     *  The DC W3C-DTF encoding name.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String W3CDTF = "W3CDTF";

    /**
     *  A W3C-DTF date value.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String DATE_W3CDTF = "2005-01-13";

    /**
     *  A W3C-DTF date value with excess white space.
     *  @since  Version 0.7 2005-09-09
     */
    private static final String DATE_PADDED = " 2005-01-13 ";

    /**
     *  A qualified XHTML <code>meta</code> element name.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String QUAL_META = NAMESPACE +
                                            ":" +
                                            META;

    /**
     *  A qualified XHTML <code>head</code> element name.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String QUAL_HEAD = NAMESPACE +
                                            ":" +
                                            HEAD;

    /**
     *  A CDATA type string.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String CDATA = "CDATA";

    /**
     *  An expected <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-01-13
     */
    private static final String EXPECT_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  An expected <code>SAXException</code>.
     *  @since  Version 0.2 2005-02-15
     */
    private static final String EXPECT_SE =
    "Should throw a SAXException.";

    /**
     *  An empty attribute set for testing.
     *  @since  Version Version 0.1 2005-01-13
     */
    private AttributesImpl attributes;

    /**
     *  Instance to test.
     *  @since  Version 0.1 2005-01-13
     */
    protected MetaStoreWriter storeWriter;

    /**
     *  An instance with default construction to test.
     *  @since  Version 0.1 2005-01-13
     */
    protected MetaStoreWriter basicWriter;

    /**
     *  A <code>MockSesameRepository</code>.
     *  @since  Version 0.1 2005-01-13
     */
    protected MockSesameRepository repository;

    /**
     *  A <code>MockStoreManager</code>.
     *  @since  Version 0.4 2005-04-29
     */
    protected MockStoreManager manager;

    /**
     *  A value factory for generating statements.
     *  @since  Version 0.1 2005-01-13
     */
    protected ValueFactory factory;

    /**
     *  A graph instance for testing.
     *  @since  Version 0.2 2005-02-15
     */
    protected Graph graph;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-01-13
     */
    public MetaStoreWriterTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @exception  IOException Should not be thrown.
     *  @since  Version 0.1 2005-01-13
     */
    protected final void setUp() throws IOException {

        graph = new GraphImpl();

        factory = graph.getValueFactory();

        repository = new MockSesameRepository(REPO_ID,
                                              new MockRdfSource(),
                                              new MockLocalService());

        manager = new MockStoreManager();

        storeWriter = new MetaStoreWriter(manager,
                                          new URL(URI_SUBJ_1));

        attributes = new AttributesImpl();

        basicWriter = new MetaStoreWriter();
    }

    /**
     *  Test constructor with custom application profile.
     *  @since  Version 0.6 2005-05-25
     */
    public final void testConstructorNullApplicationProfile() {

        ApplicationProfile profile = null;

        try {

            new MetaStoreWriter(profile);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test constructor with custom application profile.
     *  @since  Version 0.6 2005-05-25
     */
    public final void testConstructorCustomApplicationProfile() {

        Schema egms = new UKeGMS();

        ApplicationProfile profile = new UKeGMSProfile();

        MetaStoreWriter customWriter = new MetaStoreWriter(profile);

        String predicate = "egms.Mandate";

        String expanded = customWriter.expandPredicate(predicate);

        String uri = egms.getUri() + "mandate";

        assertTrue("Should expand the predicate to the URI " +
                   uri + ", actually returned '" +
                   expanded + "'.",
                   expanded.equals(uri));
    }

    /**
     *  Check whether an element is accepted.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testAcceptElementNullUri() {

        try {

            storeWriter.acceptElement(null,
                                      META);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Check whether an element is accepted.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testAcceptElementNullLocalName() {

        try {

            storeWriter.acceptElement("http://www.w3.org/1999/xhtml",
                                      null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Check whether an element is accepted.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testAcceptElementFalse() {


        assertFalse(storeWriter.acceptElement("http://example.org/1999/xhtml",
                                              META));
    }

    /**
     *  Check whether an element is accepted.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testAcceptElementTrue() {


        assertTrue(storeWriter.acceptElement(NAMESPACE,
                                             META));
    }

    /**
     *  Add triples with null argument.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testAddTriplesNullSourceUri() {

        try {

            storeWriter.addTriples(null,
                                   attributes,
                                   graph);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Add triples with null argument.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testAddTriplesNullAttributes() {

        URI subjectUri = factory.createURI(URI_SUBJ_1);

        try {

            storeWriter.addTriples(subjectUri,
                                   null,
                                   graph);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Add triples with null argument.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testAddTriplesNullGraph() {

        URI subjectUri = factory.createURI(URI_SUBJ_1);

        try {

            storeWriter.addTriples(subjectUri,
                                   attributes,
                                   null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Start a <code>meta</code> element with a default instance.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testStartElementDefaultInstance() {

        try {

            basicWriter.startElement(NAMESPACE,
                                       META,
                                       QUAL_META,
                                       attributes);

            fail(EXPECT_SE);
        }
        catch (SAXException se) { }

    }

    /**
     *  Start a <code>meta</code> element with a default instance.
     *  @since  Version 0.2 2005-02-15
     */
    public final void testStartElementDefaultInstanceWithSubject() {

        URI subjectUri = factory.createURI(URI_SUBJ_1);

        basicWriter.setSubjectUri(subjectUri);

        try {

            basicWriter.startElement(NAMESPACE,
                                       META,
                                       QUAL_META,
                                       attributes);

            fail(EXPECT_SE);
        }
        catch (SAXException se) { }

    }


    /**
     *  Start a non-<code>meta</code> element should throw
     *  <code>SAXException</code>.
     *  @since  Version 0.1 2005-01-13
     */
    public final void testStartElementNotMeta() {

        try {

            storeWriter.startElement(NAMESPACE,
                                     HEAD,
                                     QUAL_HEAD,
                                     attributes);

            fail(EXPECT_SE);
        }
        catch (SAXException se) { }

    }

    /**
     *  XHTML <code>meta</code> no <code>content</code>
     *  attribute, should not generate a statement.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2005-01-13
     */
    public final void testStartElementNoContentAttr() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                DC_DATE);

        storeWriter.startElement(NAMESPACE,
                                 META,
                                 QUAL_META,
                                 attributes);

        URI subject = factory.createURI(URI_SUBJ_1);

        storeWriter.endDocument();

        graph = manager.getGraph();

        assertFalse(graph.contains(subject, null, null));
    }

    /**
     *  XHTML <code>meta</code> no <code>name</code>
     *  attribute, should not generate a statement.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2005-01-13
     */
    public final void testStartElementNoNameAttr() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                CONTENT,
                                QUAL_CONTENT,
                                CDATA,
                                DATE_W3CDTF);

        storeWriter.startElement(NAMESPACE,
                                 META,
                                 QUAL_META,
                                 attributes);

        URI subject = factory.createURI(URI_SUBJ_1);

        storeWriter.endDocument();

        graph = manager.getGraph();

        assertFalse(graph.contains(subject, null, null));

    }

    /**
     *  XHTML <code>meta</code> basic attributes.
     *  @exception  SAXException    If there is a problem
     *          parsing the content, indicates an error.
     *  @since  Version 0.1 2004-12-17
     */
    public final void testStartElementGeneric() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                NAME_DESC);

        attributes.addAttribute(NAMESPACE,
                                CONTENT,
                                QUAL_CONTENT,
                                CDATA,
                                VALUE_DESC);

        storeWriter.startElement(NAMESPACE,
                                 META,
                                 QUAL_META,
                                 attributes);

        storeWriter.endDocument();

        graph = manager.getGraph();

        URI subject = factory.createURI(URI_SUBJ_1);

        assertFalse(graph.contains(subject, null, null));
    }

    /**
     *  XHTML <code>meta</code> with schema prefix.
     *  @exception  SAXException    If there is a problem
     *          parsing the content, indicates an error.
     *  @since  Version 0.6 2005-01-07
     */
    public final void testStartElementSchema() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                DC_DATE);

        attributes.addAttribute(NAMESPACE,
                                CONTENT,
                                QUAL_CONTENT,
                                CDATA,
                                DATE_W3CDTF);

        storeWriter.startElement(NAMESPACE,
                                 META,
                                 QUAL_META,
                                 attributes);

        storeWriter.endDocument();

        graph = manager.getGraph();

        String graphString = manager.getGraphString();

        URI subject = factory.createURI(URI_SUBJ_1);

        URI predicate = factory.createURI(URI_DC_DATE);

        Value object = factory.createLiteral(DATE_W3CDTF);

        assertTrue(GRAPH_SHOULD +
                   subject + "', predicate '" +
                   predicate + AND_LITERAL +
                   object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(subject, predicate, object));
    }

    /**
     *  XHTML <code>meta</code> with excess whitespace.
     *  @exception  SAXException    If there is a problem
     *          parsing the content, indicates an error.
     *  @since  Version 0.7 2005-09-09
     */
    public final void testStartElementSchemaTrim() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                DC_DATE);

        attributes.addAttribute(NAMESPACE,
                                CONTENT,
                                QUAL_CONTENT,
                                CDATA,
                                DATE_PADDED);

        storeWriter.startElement(NAMESPACE,
                                 META,
                                 QUAL_META,
                                 attributes);

        storeWriter.endDocument();

        graph = manager.getGraph();

        String graphString = manager.getGraphString();

        URI subject = factory.createURI(URI_SUBJ_1);

        URI predicate = factory.createURI(URI_DC_DATE);

        Value object = factory.createLiteral(DATE_W3CDTF);

        assertTrue(GRAPH_SHOULD +
                   subject + "', predicate '" +
                   predicate + AND_LITERAL +
                   object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(subject, predicate, object));
    }

    /**
     *  XHTML <code>meta</code> with <code>scheme</code>
     *  attribute.
     *  @exception  SAXException    If there is a problem
     *          parsing the content, indicates an error.
     *  @since  Version 0.1 2005-01-13
     */
    public final void testStartElementWithScheme() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                DC_DATE);

        attributes.addAttribute(NAMESPACE,
                                SCHEME,
                                QUAL_SCHEME,
                                CDATA,
                                W3CDTF);

        attributes.addAttribute(NAMESPACE,
                                CONTENT,
                                QUAL_CONTENT,
                                CDATA,
                                DATE_W3CDTF);

        storeWriter.startElement(NAMESPACE,
                                 META,
                                 QUAL_META,
                                 attributes);

        storeWriter.endDocument();

        graph = manager.getGraph();

        String graphString = manager.getGraphString();


        URI subject = factory.createURI(URI_SUBJ_1);

        URI predicate = factory.createURI(URI_DC_DATE);

        assertTrue(GRAPH_SHOULD +
                   subject + "' and predicate '" + predicate + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(subject, predicate, null));

        predicate = factory.createURI(RDF_TYPE);

        Value object = factory.createURI(URI_W3CDTF);

        assertTrue(EXPECT_BNODE +
                   predicate + "', and object URI '" + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));

        predicate = factory.createURI(RDF_VALUE);

        object = factory.createLiteral(DATE_W3CDTF);

        assertTrue(EXPECT_BNODE +
                   predicate + AND_LITERAL + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));
    }

    /**
     *  XHTML <code>meta</code> with <code>scheme</code>
     *  attribute.
     *  @exception  SAXException    If there is a problem
     *          parsing the content, indicates an error.
     *  @since  Version 0.7 2005-09-09
     */
    public final void testStartElementWithSchemeTrim() throws SAXException {

        attributes.addAttribute(NAMESPACE,
                                NAME,
                                QUAL_NAME,
                                CDATA,
                                DC_DATE);

        attributes.addAttribute(NAMESPACE,
                                SCHEME,
                                QUAL_SCHEME,
                                CDATA,
                                W3CDTF);

        attributes.addAttribute(NAMESPACE,
                                CONTENT,
                                QUAL_CONTENT,
                                CDATA,
                                DATE_PADDED);

        storeWriter.startElement(NAMESPACE,
                                 META,
                                 QUAL_META,
                                 attributes);

        storeWriter.endDocument();

        graph = manager.getGraph();

        String graphString = manager.getGraphString();


        URI subject = factory.createURI(URI_SUBJ_1);

        URI predicate = factory.createURI(URI_DC_DATE);

        assertTrue(GRAPH_SHOULD +
                   subject + "' and predicate '" + predicate + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(subject, predicate, null));

        predicate = factory.createURI(RDF_TYPE);

        Value object = factory.createURI(URI_W3CDTF);

        assertTrue(EXPECT_BNODE +
                   predicate + "', and object URI '" + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));

        predicate = factory.createURI(RDF_VALUE);

        object = factory.createLiteral(DATE_W3CDTF);

        assertTrue(EXPECT_BNODE +
                   predicate + AND_LITERAL + object + ENDQUOTE +
                   ACTUAL_GRAPH + graphString,
                   graph.contains(null, predicate, object));
    }

    /**
     *  Set a null URI subject should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-01-13
     */
    public final void testSetSubjectUri() {

        try {

            storeWriter.setSubjectUri(null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Set and get a subject URI.
     *  @since  Version 0.1 2005-01-13
     */
    public final void testSetGetSubjectUri() {

        URI uri = storeWriter.getSubjectUri();

        assertTrue(URI_SUBJ_1.equals(uri.getURI()));

        uri = factory.createURI(URI_SUBJ_2);

        storeWriter.setSubjectUri(uri);

        uri = storeWriter.getSubjectUri();

        assertTrue(URI_SUBJ_2.equals(uri.getURI()));
    }

    /**
     *  End document adds a graph to the manager.
     *  @exception  SAXException    Should not be thrown by
     *          this test.
     *  @since  Version 0.1 2005-01-13
     */
    public final void testEndDocument() throws SAXException {

        graph = manager.getGraph();

        assertNull(graph);

        storeWriter.endDocument();

        graph = manager.getGraph();

        assertNotNull(graph);
    }

    /**
     *  Repository throws an exception adding the graph.
     *  @since  Version 0.1 2005-01-13
     */
    public final void testEndDocumentStoreException() {

        manager.setFailOnAdd(true);

        try {

            storeWriter.endDocument();

            fail(EXPECT_SE);
        }
        catch (SAXException saxe) { }

    }

    /**
     *  Constructor with null manager should throw an
     *  <code>IllegalArgumentException</code>.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-01-13
     */
    public final void testConstructorNullRepository() throws IOException {

        try {

            new MetaStoreWriter(null,
                                new URL(URI_SUBJ_1));

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Constructor with null subject URI should throw an
     *  <code>IllegalArgumentException</code>.
     *  @since  Version 0.1 2005-01-13
     */
    public final void testConstructorNullUri() {

        try {

            new MetaStoreWriter(manager,
                                null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-01-13
     */
    public static Test suite() {

        return new TestSuite(MetaStoreWriterTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-01-13
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
