package com.mkdoc.sax;

/*
    Created:    2005-03-01
    Issued:     2005-03-01
    Modified:   2005-03-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import org.xml.sax.helpers.AttributesImpl;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>XhtmlMetadataFilter</code>.
 *  @version    0.1 2005-03-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class XhtmlMetadataFilterTest extends TestCase {

    /**
     *  The XHTML namespace URI.
     *  @since  Version 0.1 2005-03-01
     */
    private static final String NAMESPACE = "http://www.w3.org/1999/xhtml";

    /**
     *  An XHTML <code>head</code> element local name.
     *  @since  Version 0.1 2005-03-01
     */
    private static final String HEAD = "head";

    /**
     *  An XHTML <code>meta</code> element local name.
     *  @since  Version 0.1 2005-03-01
     */
    private static final String META = "meta";

    /**
     *  An XHTML <code>link</code> element local name.
     *  @since  Version 0.1 2005-03-01
     */
    private static final String LINK = "link";

    /**
     *  A qualified XHTML <code>meta</code> element name.
     *  @since  Version 0.1 2005-03-01
     */
    private static final String QUAL_META = NAMESPACE +
                                            ":" +
                                            META;

    /**
     *  A qualified XHTML <code>link</code> element name.
     *  @since  Version 0.1 2005-03-01
     */
    private static final String QUAL_LINK = NAMESPACE +
                                            ":" +
                                            LINK;

    /**
     *  A qualified XHTML <code>head</code> element name.
     *  @since  Version 0.1 2005-03-01
     */
    private static final String QUAL_HEAD = NAMESPACE +
                                            ":" +
                                            HEAD;

    /**
     *  An empty attribute set for testing.
     *  @since  Version 0.1 2005-03-01
     */
    private Attributes attributes;

    /**
     *  A filter instance for testing.
     *  @since  Version 0.1 2005-03-01
     */
    private XhtmlMetadataFilter filter;

    /**
     *  A mock content handler to check throughput.
     *  @since  Version 0.1 2005-03-01
     */
    private MockContentHandler contentHandler;


    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-03-01
     */
    public XhtmlMetadataFilterTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-
     */
    protected final void setUp() {

        filter = new XhtmlMetadataFilter();

        contentHandler = new MockContentHandler();

        filter.setContentHandler(contentHandler);

        attributes = new AttributesImpl();
    }

    /**
     *  Not a meta element, should ignore.
     *  @exception  SAXException    If the filter method
     *          encounters a problem processing the
     *          arguments.
     *  @since  Version 0.1 2005-03-01
     */
    public final void testStartElementHeadLocalName() throws SAXException {


        filter.startElement(NAMESPACE,
                            HEAD,
                            QUAL_HEAD,
                            attributes);

        String eventLog = contentHandler.getEventLog();

        assertTrue(eventLog.equals(""));
    }

    /**
     *  With meta element, should pass.
     *  @exception  SAXException    If the filter method
     *          encounters a problem processing the
     *          arguments.
     *  @since  Version 0.1 2005-03-01
     */
    public final void testStartElementMetaLocalName() throws SAXException {


        filter.startElement(NAMESPACE,
                            META,
                            QUAL_META,
                            attributes);

        String eventLog = contentHandler.getEventLog();

        assertTrue("Filter should pass the meta element, " +
                   "actual log content: '" + eventLog + "'.",
                   eventLog.indexOf("meta") > -1);
    }

    /**
     *  With link element, should pass.
     *  @exception  SAXException    If the filter method
     *          encounters a problem processing the
     *          arguments.
     *  @since  Version 0.1 2005-03-01
     */
    public final void testStartElementLinkLocalName() throws SAXException {


        filter.startElement(NAMESPACE,
                            LINK,
                            QUAL_LINK,
                            attributes);

        String eventLog = contentHandler.getEventLog();

        assertTrue("Filter should pass the link element, " +
                   "actual log content: '" + eventLog + "'.",
                   eventLog.indexOf("link") > -1);
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-03-01
     */
    public static Test suite() {

        return new TestSuite(XhtmlMetadataFilterTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-03-01
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
