package com.mkdoc.schema;

/*
    Created:    2005-06-23
    Issued:     2005-06-28
    Modified:   2005-06-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Map;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>AbastractApplicationProfile</code>.
 *  @version    0.1 2005-06-23
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class AbstractApplicationProfileTest extends TestCase {

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.1 2005-06-28
     */
    protected static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  An application profile to test.
     *  @since  Version 0.1 2005-06-28
     */
    protected MockApplicationProfile profile;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-06-23
     */
    public AbstractApplicationProfileTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-
     */
    protected final void setUp() {

        profile = new MockApplicationProfile();
    }

    /**
     *  Get a predicate property by name.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testGetPredicatePropertyNullPredicate() {

        try {

            profile.getPredicateProperty(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Get a predicate property by name.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testGetPredicatePropertyNotRecognised() {

        SchemaProperty property = profile.getPredicateProperty("dc.title");

        assertNull(property);
    }

    /**
     *  Get a predicate property by name.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testGetPredicatePropertyIsRecognised() {

        SchemaProperty property = profile.getPredicateProperty("mock.examine");

        assertNotNull(property);
    }

    /**
     *  Get a predicate property by name.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testGetPredicatePropertyNoPrefix() {

        SchemaProperty property = profile.getPredicateProperty("examine");

        assertNull(property);
    }

    /**
     *  Expand a predicate  by name.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testExpandPredicateNullPredicate() {

        try {

            profile.expandPredicate(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Expand a predicate  by name.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testExpandPredicateNotRecognised() {

        String predicate = "dc.title";

        String expanded = profile.expandPredicate(predicate);

        assertTrue(expanded.equals(predicate));
    }

    /**
     *  Expand a predicate  by name.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testExpandPredicateIsRecognised() {

        String predicate = "mock.examine";

        String expanded = profile.expandPredicate(predicate);

        assertFalse(expanded.equals(predicate));

        String expected = "http://www.mkdoc.com/uri/test/MockSchema/examine";

        assertTrue(expanded.equals(expected));
    }

    /**
     *  Expand a predicate  by name.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testExpandPredicateNoPrefix() {

        String predicate = "examine";

        String expanded = profile.expandPredicate(predicate);

        assertTrue(expanded.equals(predicate));
    }

    /**
     *  Expand a profile extension.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testExpandPredicateProfileExtensionMatched() {

        String predicate = "mock.true";

        String expanded = profile.expandPredicate(predicate);

        String expected =
        "http://www.mkdoc.com/uri/test/MockSchema/profile/uri";

        assertTrue(expanded.equals(expected));
    }

    /**
     *  Expand a profile extension.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testExpandPredicateProfileExtensionUnmatched() {

        String predicate = "mock.false";

        String expanded = profile.expandPredicate(predicate);

        assertTrue(expanded.equals(predicate));
    }


    /**
     *  Expand a scheme  by name.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testExpandSchemeNullScheme() {

        try {

            profile.expandScheme(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Expand a scheme  by name.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testExpandSchemeNotRecognised() {

        String scheme = "dc.title";

        String expanded = profile.expandScheme(scheme);

        assertTrue(expanded.equals(scheme));
    }

    /**
     *  Expand a scheme  by name.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testExpandSchemeIsRecognised() {

        String scheme = "mock.check";

        String expanded = profile.expandScheme(scheme);

        assertFalse(expanded.equals(scheme));

        String expected = "http://www.mkdoc.com/uri/test/MockSchema/check";

        assertTrue(expanded.equals(expected));
    }

    /**
     *  Expand a scheme  by name.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testExpandSchemeNoPrefix() {

        String scheme = "check";

        String expanded = profile.expandScheme(scheme);

        assertTrue(expanded.equals(scheme));
    }

    /**
     *  Expand a profile extension.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testExpandSchemeProfileExtensionMatched() {

        String scheme = "true";

        String expanded = profile.expandScheme(scheme);

        String expected =
        "http://www.mkdoc.com/uri/test/MockSchema/profile/uri";

        assertTrue(expanded.equals(expected));
    }

    /**
     *  Expand a profile extension.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testExpandSchemeProfileExtensionUnmatched() {

        String scheme = "false";

        String expanded = profile.expandScheme(scheme);

        String expected =
        "http://www.mkdoc.com/uri/test/MockSchema/profile/uri";

        assertFalse(expanded.equals(expected));
    }

    /**
     *  Check for a schema by prefix.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testHasPrimarySchemaPrefixNull() {

        try {

            profile.hasPrimarySchema(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Check for a schema by prefix.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testHasPrimarySchemaPrefixTrue() {

        assertTrue(profile.hasPrimarySchema("mock"));

    }

    /**
     *  Check for a schema by prefix.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testHasPrimarySchemaPrefixFalse() {

        assertFalse(profile.hasPrimarySchema("dc"));
    }

    /**
     *  Get a schema by prefix.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testGetPrimarySchemaPrefixNull() {

        try {

            profile.getPrimarySchema(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Get a schema by prefix.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testGetPrimarySchemaPrefixTrue() {

        assertNotNull(profile.getPrimarySchema("mock"));

    }

    /**
     *  Get a schema by prefix.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testGetPrimarySchemaPrefixFalse() {

        assertNull(profile.getPrimarySchema("dc"));
    }

    /**
     *  Add a schema.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testAddSchemaNull() {

        try {

            profile.addSchema(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Get a schema map.
     *  @since  Version 0.1 2005-06-28
     */
    public final void testGetSchemaMap() {


        Map map = profile.getSchemaMap();

        assertNotNull(map);

        assertTrue(map.containsKey("mock"));
        assertFalse(map.containsKey("dc"));
        assertTrue(map.size() == 1);
    }



    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-06-23
     */
    public static Test suite() {

        return new TestSuite(AbstractApplicationProfileTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-06-23
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
