package com.mkdoc.schema;

/*
    Created:    2004-12-22
    Issued:     2004-12-22
    Modified:   2005-09-14

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2004 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Collection;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>DublinCoreElements</code>.
 *  @version    1.1 2005-09-14
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class DublinCoreElementsTest extends TestCase {

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.3 2005-02-28
     */
    public static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  A generic description predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String GENERIC = "description";

    /**
     *  A bogus Dublin Core encoded predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String BOGUS = "DC.bogus";

    /**
     *  Old form of Dublin Core qualified predicate.
     *  @since  Version 0.2 2004-12-23
     */
    private static final String OLD_FORM = "DC.Date.modified";

    /**
     *  Dublin Core qualified audience predicate.
     *  @since  Version 0.3 2005-02-28
     */
    public static final String NAME_AUDNCE = "DCTERMS.audience";

    /**
     *  Dublin Core encoded scheme.
     *  @since  Version 0.2 2004-12-23
     */
    private static final String SCHEME = "DCTERMS.NAME_W3CDTF";

    /**
     *  An encoded Title predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String TITLE = "DC.Title";

    /**
     *  An encoded Creator predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String CREATOR = "DC.Creator";

    /**
     *  An encoded Subject predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String SUBJECT = "DC.Subject";

    /**
     *  An encoded Description predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String DESCRIPTION = "DC.Description";

    /**
     *  An encoded Publisher predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String PUBLISHER = "DC.Publisher";

    /**
     *  An encoded Contributor predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String CONTRIBUTOR = "DC.Contributor";

    /**
     *  An encoded Date predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String DATE = "DC.Date";

    /**
     *  An encoded Type predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String TYPE = "DC.Type";

    /**
     *  An encoded Format predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String FORMAT = "DC.Format";

    /**
     *  An encoded Identifier predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String IDENTIFIER = "DC.Identifier";

    /**
     *  An encoded Source predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String SOURCE = "DC.Source";

    /**
     *  An encoded Language predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String LANGUAGE = "DC.Language";

    /**
     *  An encoded Relation predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String RELATION = "DC.Relation";

    /**
     *  An encoded Coverage predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String COVERAGE = "DC.Coverage";

    /**
     *  An encoded Rights predicate.
     *  @since  Version 0.1 2004-12-22
     */
    private static final String RIGHTS = "DC.Rights";

    /**
     *  A schema instance to test.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2004-12-22
     */
    private DublinCoreElements dce = new DublinCoreElements();

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2004-12-22
     */
    public DublinCoreElementsTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2004-
    protected void setUp() {

    }
     */

    /**
     *  Get the schema name.
     *  @since  Version 1.0 2005-06-17
     */
    public final void testGetName() {

        String expected = "Dublin Core Elements";

        assertTrue(expected.equals(dce.getName()));
    }

    /**
     *  Get a predicate URI.
     *  @since  Version 1.0 2005-06-17
     */
    public final void testGetPredicateUriUnmatched() {

        assertNull(dce.getPredicateUri(BOGUS));
    }

    /**
     *  Get a predicate URI.
     *  @since  Version 1.0 2005-06-17
     */
    public final void testGetPredicateUriMatched() {

        assertNotNull(dce.getPredicateUri(GENERIC));
    }

    /**
     *  Get a predicate URI.
     *  @since  Version 1.0 2005-06-17
     */
    public final void testGetPredicateUriNull() {

        try {

            dce.getPredicateUri(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Get the set of predicate names.
     *  @since  Version 0.8 2005-06-10
     */
    public final void testGetPredicates() {

        Collection predicates = dce.getPredicates();

        assertTrue(predicates.size() == 15);

        SchemaProperty property = null;

        property = dce.getNamedPredicateProperty("title");

        assertTrue(predicates.contains(property));

        property = dce.getNamedPredicateProperty("creator");

        assertTrue(predicates.contains(property));

        property = dce.getNamedPredicateProperty("subject");

        assertTrue(predicates.contains(property));

        property = dce.getNamedPredicateProperty("description");

        assertTrue(predicates.contains(property));

        property = dce.getNamedPredicateProperty("publisher");

        assertTrue(predicates.contains(property));

        property = dce.getNamedPredicateProperty("contributor");

        assertTrue(predicates.contains(property));

        property = dce.getNamedPredicateProperty("date");

        assertTrue(predicates.contains(property));

        property = dce.getNamedPredicateProperty("type");

        assertTrue(predicates.contains(property));

        property = dce.getNamedPredicateProperty("format");

        assertTrue(predicates.contains(property));

        property = dce.getNamedPredicateProperty("identifier");

        assertTrue(predicates.contains(property));

        property = dce.getNamedPredicateProperty("source");

        assertTrue(predicates.contains(property));

        property = dce.getNamedPredicateProperty("language");

        assertTrue(predicates.contains(property));

        property = dce.getNamedPredicateProperty("relation");

        assertTrue(predicates.contains(property));

        property = dce.getNamedPredicateProperty("coverage");

        assertTrue(predicates.contains(property));

        property = dce.getNamedPredicateProperty("rights");

    }

    /**
     *  Test newSchemaProperty with encoding scheme.
     *  @since  Version 1.1 2005-09-14
     */
    public final void testNewSchemaPropertyWithEncoding() {

        String url = "http://remote.example/uri/schema/test#";

        String label = "Test property";

        String name = "testProperty";

        SchemaProperty property = dce.newSchemaProperty(url, label, name, true);

        assertTrue(property.hasEncodingScheme());
    }

    /**
     *  Test newSchemaProperty without encoding scheme.
     *  @since  Version 1.1 2005-09-14
     */
    public final void testNewSchemaPropertyWithoutEncoding() {

        String url = "http://remote.example/uri/schema/test#";

        String label = "Test property";

        String name = "testProperty";

        SchemaProperty property = dce.newSchemaProperty(url, label, name);

        assertFalse(property.hasEncodingScheme());
    }

    /**
     *  Get the schema URI.
     *  @since  Version 0.6 2005-05-24
     */
    public final void testGetUri() {

       String uri = dce.getUri();

       assertTrue(uri.equals("http://purl.org/dc/elements/1.1/"));
    }

    /**
     *  Test the provides qualifiers methods.
     *  @since  Version 0.4 2005-03-11
     */
    public final void testProvidesQualifiers() {

        assertFalse(dce.providesQualifiers());

        assertFalse(dce.providesQualifiersFor("http://www.remote.example"));
    }

    /**
     *  Test the prefix method.
     *  @since  Version 0.4 2005-03-01
     */
    public final void testGetDefaultPrefix() {

        assertTrue("DC".equals(dce.getDefaultPrefix()));
    }

    /**
     *  Test the prefixes method.
     *  @since  Version 0.7 2005-05-27
     */
    public final void testGetAllPrefixes() {

        String[] prefixes = dce.getAllPrefixes();

        assertTrue(prefixes.length == 1);
        assertTrue("DC".equals(prefixes[0]));
    }

    /**
     *  Test expand null predicate.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandDualNullPredicate() {

        try {

            dce.expandPredicate(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test expand scheme, null scheme.
     *  @since  Version 0.2 2004-12-23
     */
    public final void testExpandSchemeNull() {

        try {

            dce.expandScheme(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }


    /**
     *  Test expand scheme, old form.
     *  @since  Version 0.2 2004-12-23
     */
    public final void testExpandScheme() {

        String expanded = dce.expandScheme(SCHEME);

        assertTrue("Predicate should not be expanded to, '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_W3CDTF + "', " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(SCHEME));
    }


    /**
     *  Test expand null predicate name.
     *  @since  Version 0.2 2004-12-23
     */
    public final void testExpandNullName() {

        try {

            dce.expandPredicate(null, TITLE);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test expand null predicate name.
     *  @since  Version 0.2 2004-12-23
     */
    public final void testExpandNullPredicate() {

        try {

            dce.expandPredicate(TITLE, null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test expand generic predicate.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandGenericPredicate() {


        String expanded = dce.expandPredicate(GENERIC);

        assertTrue("Predicate should not be expanded, " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(GENERIC));
    }

    /**
     *  Test expand a bogus Dublin Core predicate.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandDCBogusPredicate() {


        String expanded = dce.expandPredicate(BOGUS);

        assertTrue("Predicate should not be expanded, " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(BOGUS));
    }

    /**
     *  Test expand an old format Dublin Core qualified predicate.
     *  @since  Version 0.2 2004-12-23
     */
    public final void testExpandDCOldForm() {

        String expanded = dce.expandPredicate(OLD_FORM);

        assertTrue("Predicate should not be expanded to, '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MODFID + "', " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(OLD_FORM));
    }

    /**
     *  Test expand a new format Dublin Core qualified predicate.
     *  @since  Version 0.2 2004-12-23
     */
    public final void testExpandDCNewForm() {

        String expanded = dce.expandPredicate(NAME_AUDNCE);

        assertTrue("Predicate should not be expanded to, '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_AUDNCE + "', " +
                   "actually returned: '" + expanded + "'.",
                   expanded.equals(NAME_AUDNCE));
    }

    /**
     *  Test expand Title, mixed case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandTitleMixedCase() {


        String expanded = dce.expandPredicate(TITLE);

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_TITLE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_TITLE));
    }

    /**
     *  Test expand Title, lower case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandTitleLowerCase() {


        String expanded = dce.expandPredicate(TITLE.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_TITLE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_TITLE));
    }

    /**
     *  Test expand Title, upper case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandTitleUpperCase() {


        String expanded = dce.expandPredicate(TITLE.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_TITLE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_TITLE));
    }

    /**
     *  Test expand Creator, mixed case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandCreatorMixedCase() {


        String expanded = dce.expandPredicate(CREATOR);

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_CREATR + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_CREATR));
    }

    /**
     *  Test expand Creator, lower case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandCreatorLowerCase() {


        String expanded = dce.expandPredicate(CREATOR.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_CREATR + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_CREATR));
    }

    /**
     *  Test expand Creator, upper case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandCreatorUpperCase() {


        String expanded = dce.expandPredicate(CREATOR.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_CREATR + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_CREATR));
    }

    /**
     *  Test expand Subject, mixed case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandSubjectMixedCase() {


        String expanded = dce.expandPredicate(SUBJECT);

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_SUBJCT + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_SUBJCT));
    }

    /**
     *  Test expand Subject, lower case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandSubjectLowerCase() {


        String expanded = dce.expandPredicate(SUBJECT.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_SUBJCT + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_SUBJCT));
    }

    /**
     *  Test expand Subject, upper case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandSubjectUpperCase() {


        String expanded = dce.expandPredicate(SUBJECT.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_SUBJCT + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_SUBJCT));
    }

    /**
     *  Test expand Description, mixed case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandDescriptionMixedCase() {


        String expanded = dce.expandPredicate(DESCRIPTION);

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_DESCRN + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_DESCRN));
    }

    /**
     *  Test expand Description, lower case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandDescriptionLowerCase() {


        String expanded = dce.expandPredicate(DESCRIPTION.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_DESCRN + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_DESCRN));
    }

    /**
     *  Test expand Description, upper case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandDescriptionUpperCase() {


        String expanded = dce.expandPredicate(DESCRIPTION.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_DESCRN + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_DESCRN));
    }

    /**
     *  Test expand Publisher, mixed case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandPublisherMixedCase() {


        String expanded = dce.expandPredicate(PUBLISHER);

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_PUBSHR + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_PUBSHR));
    }

    /**
     *  Test expand Publisher, lower case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandPublisherLowerCase() {


        String expanded = dce.expandPredicate(PUBLISHER.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_PUBSHR + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_PUBSHR));
    }

    /**
     *  Test expand Publisher, upper case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandPublisherUpperCase() {


        String expanded = dce.expandPredicate(PUBLISHER.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_PUBSHR + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_PUBSHR));
    }

    /**
     *  Test expand Contributor, mixed case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandContributorMixedCase() {


        String expanded = dce.expandPredicate(CONTRIBUTOR);

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_CONTRB + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_CONTRB));
    }

    /**
     *  Test expand Contributor, lower case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandContributorLowerCase() {


        String expanded = dce.expandPredicate(CONTRIBUTOR.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_CONTRB + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_CONTRB));
    }

    /**
     *  Test expand Contributor, upper case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandContributorUpperCase() {


        String expanded = dce.expandPredicate(CONTRIBUTOR.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_CONTRB + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_CONTRB));
    }

    /**
     *  Test expand Date, mixed case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandDateMixedCase() {


        String expanded = dce.expandPredicate(DATE);

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_DATE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_DATE));
    }

    /**
     *  Test expand Date, lower case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandDateLowerCase() {


        String expanded = dce.expandPredicate(DATE.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_DATE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_DATE));
    }

    /**
     *  Test expand Date, upper case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandDateUpperCase() {


        String expanded = dce.expandPredicate(DATE.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_DATE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_DATE));
    }

    /**
     *  Test expand Type, mixed case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandTypeMixedCase() {


        String expanded = dce.expandPredicate(TYPE);

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_TYPE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_TYPE));
    }

    /**
     *  Test expand Type, lower case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandTypeLowerCase() {


        String expanded = dce.expandPredicate(TYPE.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_TYPE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_TYPE));
    }

    /**
     *  Test expand Type, upper case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandTypeUpperCase() {


        String expanded = dce.expandPredicate(TYPE.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_TYPE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_TYPE));
    }

    /**
     *  Test expand Format, mixed case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandFormatMixedCase() {


        String expanded = dce.expandPredicate(FORMAT);

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_FORMAT + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_FORMAT));
    }

    /**
     *  Test expand Format, lower case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandFormatLowerCase() {


        String expanded = dce.expandPredicate(FORMAT.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_FORMAT + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_FORMAT));
    }

    /**
     *  Test expand Format, upper case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandFormatUpperCase() {


        String expanded = dce.expandPredicate(FORMAT.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_FORMAT + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_FORMAT));
    }

    /**
     *  Test expand Identifier, mixed case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandIdentifierMixedCase() {


        String expanded = dce.expandPredicate(IDENTIFIER);

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_IDNTFR + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_IDNTFR));
    }

    /**
     *  Test expand Identifier, lower case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandIdentifierLowerCase() {


        String expanded = dce.expandPredicate(IDENTIFIER.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_IDNTFR + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_IDNTFR));
    }

    /**
     *  Test expand Identifier, upper case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandIdentifierUpperCase() {


        String expanded = dce.expandPredicate(IDENTIFIER.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_IDNTFR + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_IDNTFR));
    }

    /**
     *  Test expand Source, mixed case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandSourceMixedCase() {


        String expanded = dce.expandPredicate(SOURCE);

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_SOURCE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_SOURCE));
    }

    /**
     *  Test expand Source, lower case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandSourceLowerCase() {


        String expanded = dce.expandPredicate(SOURCE.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_SOURCE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_SOURCE));
    }

    /**
     *  Test expand Source, upper case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandSourceUpperCase() {


        String expanded = dce.expandPredicate(SOURCE.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_SOURCE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_SOURCE));
    }

    /**
     *  Test expand Language, mixed case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandLanguageMixedCase() {


        String expanded = dce.expandPredicate(LANGUAGE);

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_LNGAGE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_LNGAGE));
    }

    /**
     *  Test expand Language, lower case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandLanguageLowerCase() {


        String expanded = dce.expandPredicate(LANGUAGE.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_LNGAGE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_LNGAGE));
    }

    /**
     *  Test expand Language, upper case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandLanguageUpperCase() {


        String expanded = dce.expandPredicate(LANGUAGE.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_LNGAGE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_LNGAGE));
    }

    /**
     *  Test expand Relation, mixed case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandRelationMixedCase() {


        String expanded = dce.expandPredicate(RELATION);

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_RELATN + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_RELATN));
    }

    /**
     *  Test expand Relation, lower case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandRelationLowerCase() {


        String expanded = dce.expandPredicate(RELATION.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_RELATN + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_RELATN));
    }

    /**
     *  Test expand Relation, upper case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandRelationUpperCase() {


        String expanded = dce.expandPredicate(RELATION.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_RELATN + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_RELATN));
    }

    /**
     *  Test expand Coverage, mixed case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandCoverageMixedCase() {


        String expanded = dce.expandPredicate(COVERAGE);

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_COVRGE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_COVRGE));
    }

    /**
     *  Test expand Coverage, lower case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandCoverageLowerCase() {


        String expanded = dce.expandPredicate(COVERAGE.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_COVRGE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_COVRGE));
    }

    /**
     *  Test expand Coverage, upper case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandCoverageUpperCase() {


        String expanded = dce.expandPredicate(COVERAGE.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_COVRGE + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_COVRGE));
    }

    /**
     *  Test expand Rights, mixed case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandRightsMixedCase() {


        String expanded = dce.expandPredicate(RIGHTS);

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_RIGHTS + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_RIGHTS));
    }

    /**
     *  Test expand Rights, lower case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandRightsLowerCase() {


        String expanded = dce.expandPredicate(RIGHTS.toLowerCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_RIGHTS + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_RIGHTS));
    }

    /**
     *  Test expand Rights, upper case.
     *  @since  Version 0.1 2004-12-22
     */
    public final void testExpandRightsUpperCase() {


        String expanded = dce.expandPredicate(RIGHTS.toUpperCase());

        assertTrue("Encoded predicate should expand to " +
                   dce.BASE + dce.NAME_RIGHTS + ", actually returned: '" +
                   expanded + "'.",
                   expanded.equals(dce.BASE + dce.NAME_RIGHTS));
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2004-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2004-12-22
     */
    public static Test suite() {

        return new TestSuite(DublinCoreElementsTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2004-12-22
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
