package com.mkdoc.schema;

/*
    Created:    2005-01-06
    Issued:     2005-01-06
    Modified:   2005-06-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import java.util.Hashtable;

/**
 *  A mock <code>Schema</code> type for testing.
 *  @version    1.1 2005-06-28
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockSchema extends AbstractSchema implements Schema {

    /**
     *  A map of predicate names and URI.
     *  @since  Version 0.6 2005-05-25
     */
    protected static final Map PREDICATES = new Hashtable();

    /**
     *  A map of scheme names and URI.
     *  @since  Version 0.6 2005-05-25
     */
    protected static final Map SCHEMES = new Hashtable();

    /**
     *  An encoded predicate name.
     *  @since  Version 0.1 2005-01-06
     */
    protected static final String EXAMINE = "examine";

    /**
     *  An encoded scheme name.
     *  @since  Version 0.1 2005-01-06
     */
    protected static final String CHECK = "check";

    /**
     *  A schema URI.
     *  @since  Version 0.1 2005-01-06
     */
    protected static final String URI =
    "http://www.mkdoc.com/uri/test/MockSchema/";

    /**
     *  A schema prefix.
     *  @since  Version 0.2 2005-03-01
     */
    protected static final String[] PREFIXES = new String[] {"mock"};

    /**
     *  Initialise the mapping.
     *  @since  Version 0.6 2005-05-25
     */
    static {

        addPredicate(newSchemaProperty(URI + EXAMINE,
                                       "Examine",
                                       EXAMINE));


        addScheme(newSchemaProperty(URI + CHECK,
                                    "Check",
                                    CHECK));

    }

    /**
     *  Get a new schema property.
     *  @param  uri The resource URI.
     *  @param  label   The resource label.
     *  @param  name    The unique resource name.
     *  @return A new schema property constructed from the arguments.
     *  @since  Version 1.0 2005-06-17
     */
    protected static final SchemaProperty newSchemaProperty(final String uri,
                                                            final String label,
                                                            final String name) {

        return new DCElementProperty(uri, label, name);
    }

    /**
     *  Add a predicate to the schema mapping.
     *  @param  predicate   The schema property to add to the mapping.
     *  @since  Version 1.0 2005-06-17
     */
    protected static final void addPredicate(final SchemaProperty predicate) {

        String name = predicate.getName();

        PREDICATES.put(name.toLowerCase(), predicate);
    }

    /**
     *  Add a predicate to the schema mapping.
     *  @param  scheme  The schema property to add to the mapping.
     *  @since  Version 1.0 2005-06-17
     */
    protected static final void addScheme(final SchemaProperty scheme) {

        String name = scheme.getName();

        SCHEMES.put(name.toLowerCase(), scheme);
    }

    /**
     *  Get the base URI for the metadata schema.
     *  @return The namespace URI for the metadata schema.
     *  @since  Version 0.1 2005-01-06
     */
    public final String getUri() {

        return URI;
    }

    /**
     *  Expand an encoded scheme value to a URI.
     *  @param  name    The encoding scheme name to expand. Initially, this may
     *          include a schema prefix (i.e. <code>DCTERMS.URI</code>), but
     *          may be processed recursively without its schema prefix.
     *  @param  scheme  The scheme to expand.
     *  @return The expanded URI reference for the scheme,
     *          or the original string if it is not
     *          matched.
     *  @since  Version 0.1 2005-01-06
     */
    public final String expandScheme(final String name,
                                     final String scheme) {

        int dotIdx = name.indexOf(".");

        if (dotIdx > -1) {

            String term = name.substring(dotIdx + 1,
                                         name.length());

            String lower = term.toLowerCase();

            if (SCHEMES.containsKey(lower)) {

                SchemaProperty property = (SchemaProperty) SCHEMES.get(lower);

                return property.getUri();
            }
        }

        return scheme;
    }

    /**
     *  Expand an encoded predicate name to a URI.
     *  <p>
     *    Allows recursive handling of schema prefixes.
     *  </p>
     *  @param  name    The predicate name to expand,
     *          <strong>without</strong> its schema prefix.
     *  @param  predicate   The original predicate to fall
     *          back to if there is no match.
     *  @return The expanded URI reference for the
     *          predicate name, or the original predicate
     *          if the name is not matched.
     *  @since  Version 0.2 2004-12-23
     */
    public final String expandPredicate(final String name,
                                        final String predicate) {

        int dotIdx = name.indexOf(".");

        if (dotIdx > -1) {

            String element = name.substring(dotIdx + 1,
                                            name.length());

            String lower = element.toLowerCase();

            if (PREDICATES.containsKey(lower)) {

                SchemaProperty property = null;

                property = (SchemaProperty) PREDICATES.get(lower);

                return property.getUri();
            }
        }

        return predicate;
    }

    /**
     *  Get the default namespace prefix for the schema.
     *  @return The default namepace prefix for the schema.
     *  @since  Version 0.3 2005-03-01
     */
    public final String getDefaultPrefix() {

        return PREFIXES[0];
    }

    /**
     *  Get the HTML metadata prefixes recognised by this schema.
     *  @return An array of one or more schema prefixes in mixed case.
     *  @since  Version 0.7 2005-05-27
     */
    public final String[] getAllPrefixes() {

        return copyPrefixes(PREFIXES);
    }

    /**
     *  Signal whether the schema provides element qualifiers.
     *  @return <code>true</code> if the schema
     *  @since  Version 0.4 2005-03-10
     */
    public final boolean providesQualifiers() {

        return false;
    }

    /**
     *  Check whether the schema provides qualifiers for a specific schema.
     *  @param  uri The URI of another schema to check for extensions.
     *  @return <code>true</code> if the schema provieds qualifiers for the
     *        schema signified by the URI.
     *  @since  Version 0.4 2005-03-10
     */
    public final boolean providesQualifiersFor(final String uri) {

        return false;
    }

    /**
     *  Get a collection of predicate names for this schema.
     *  @return A collection of predicate names.
     *  @since  Version 0.8 2005-06-10
     */
    public final Collection getPredicates() {

        return Collections.unmodifiableCollection(PREDICATES.values());
    }

    /**
     *  Get the schema property for a given predicate.
     *  @param  name    The predicate name without prefix.
     *  @return The schema property for the predicate or <code>null</code> if no
     *          mapping exists.
     *  @since  Version 0.9 2005-06-16
     */
    public final SchemaProperty getNamedPredicateProperty(final String name) {

        return (SchemaProperty) PREDICATES.get(name.toLowerCase());
    }

    /**
     *  Get the common name for the schema.
     *  @return The common name for the schema.
     *  @since  Version 0.9 2005-06-17
     */
    public final String getName() {

        return "Mock Schema";
    }

}
