package com.mkdoc.schema;

/*
    Created:    2005-02-28
    Issued:     2005-02-28
    Modified:   2005-06-23

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Collection;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>UKeGMS</code>.
 *  @version    0.7 2005-06-17
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class UKeGMSTest extends TestCase {

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.3 2005-03-11
     */
    public static final String EXPECT_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  Generic predicate without dot separator.
     *  @since  Version 0.1 2005-02-28
     */
    public static final String GENERIC = "genericPredicate";

    /**
     *  A test fallback value.
     *  @since  Version 0.1 2005-02-28
     */
    public static final String FALLBACK = "testFallback";

    /**
     *  A bogus Dublin Core encoded predicate.
     *  @since  Version 0.1 2005-02-28
     */
    private static final String DC_BOGUS = "DC.bogus";

    /**
     *  A bogus e-GMS encoded predicate.
     *  @since  Version 0.1 2005-02-28
     */
    private static final String EGMS_BOGUS = "eGMS.bogus";

    /**
     *  Old form of Dublin Core qualified predicate.
     *  @since  Version 0.1 2005-02-28
     */
    private static final String OLD_FORM = "DC.Date.modified";

    /**
     *  Dublin Core encoded W3C-DTF predicate.
     *  @since  Version 0.1 2005-02-28
     */
    public static final String W3CDTF = "DCTERMS.NAME_W3CDTF";

    /**
     *  An e-GMS W3C-DTF scheme value in the original DC in XHTML format.
     *  @since  Version 0.1 2005-02-28
     */
    public static final String W3CDTF_OLD = "W3CDTF";

    /**
     *  An e-GMS Audience Encoding scheme value in the original DC in XHTML
     *  format.
     *  @since  Version 0.1 2005-02-28
     */
    public static final String EGMSSAES = "egmsaes";

    /**
     *  Dublin Core encoded Audience predicate.
     *  @since  Version 0.1 2005-02-28
     */
    public static final String NAME_AUDNCE = "DCTERMS.audience";

    /**
     *  The e-GMS Addressee encoded element.
     *  @since  Version 0.1 2005-02-28
     */
    public static final String EGMS_ADDREE = "eGMS.Addressee";

    /**
     *  The e-GMS Addressee.addresseeCopy encoded refinement.
     *  @since  Version 0.1 2005-02-28
     */
    public static final String EGMS_ADDCPY = "eGMS.Addressee.addresseeCopy";

    /**
     *  The e-GMS Addressee.addresseeCopy URI suffix.
     *  @since  Version 0.1 2005-02-28
     */
    public static final String SUFF_ADDCPY = "addresseeAddresseeCopy";

    /**
     *  The e-GMS Addressee.addresseeCopy encoded refinement.
     *  @since  Version 0.1 2005-02-28
     */
    public static final String EGMS_UPFREQ = "DC.Date.updatingFrequency";

    /**
     *  A bogus element qualifier using the dot separator scheme.
     *  @since  Version 0.1 2005-02-28
     */
    public static final String BOGUS_REFN = "DC.Date.endOfTheUniverse";

    /**
     *  A Dublin Core Terms schema for reference.
     *  @since  Version 0.1 2005-02-28
     */
    protected DublinCoreTerms dcTerms = new DublinCoreTerms();

    /**
     *  An e-GMS schema for testing.
     *  @since  Version 0.1 2005-02-28
     */
    protected UKeGMS eGMS = new UKeGMS();

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-02-28
     */
    public UKeGMSTest(final String name) {

        super(name);
    }

    /**
     *  Get the schema name.
     *  @since  Version 0.7 2005-06-17
     */
    public final void testGetName() {

        String expected = "UK e-GMS Terms";

        assertTrue(expected.equals(eGMS.getName()));
    }

    /**
     *  Get a predicate URI.
     *  @since  Version 0.7 2005-06-17
     */
    public final void testGetPredicateUriUnmatched() {

        assertNull(eGMS.getPredicateUri(EGMS_BOGUS));
    }

    /**
     *  Get a predicate URI.
     *  @since  Version 0.7 2005-06-17
     */
    public final void testGetPredicateUriMatched() {

        assertNotNull(eGMS.getPredicateUri("mandate"));
    }

    /**
     *  Get a predicate URI.
     *  @since  Version 0.7 2005-06-17
     */
    public final void testGetPredicateUriNull() {

        try {

            eGMS.getPredicateUri(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Get a scheme URI.
     *  @since  Version 0.7 2005-06-17
     */
    public final void testGetSchemeUriUnmatched() {

        assertNull(eGMS.getSchemeUri(EGMS_BOGUS));
    }

    /**
     *  Get a scheme URI.
     *  @since  Version 0.7 2005-06-17
     */
    public final void testGetSchemeUriMatched() {

        assertNotNull(eGMS.getSchemeUri(EGMSSAES));
    }

    /**
     *  Get a scheme URI.
     *  @since  Version 0.7 2005-06-17
     */
    public final void testGetSchemeUriNull() {

        try {

            eGMS.getSchemeUri(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Get the set of predicate names.
     *  @since  Version 0.5 2005-06-10
     */
    public final void testGetPredicates() {

        Collection predicates = eGMS.getPredicates();

        SchemaProperty property = null;

        property = eGMS.getNamedPredicateProperty("addressee");

        assertTrue(predicates.contains(property));

        property = eGMS.getNamedPredicateProperty("aggregation");

        assertTrue(predicates.contains(property));

        property = eGMS.getNamedPredicateProperty("digitalsignature");

        assertTrue(predicates.contains(property));

        property = eGMS.getNamedPredicateProperty("disposal");

        assertTrue(predicates.contains(property));

        property = eGMS.getNamedPredicateProperty("location");

        assertTrue(predicates.contains(property));

        property = eGMS.getNamedPredicateProperty("mandate");

        assertTrue(predicates.contains(property));

        property = eGMS.getNamedPredicateProperty("preservation");

        assertTrue(predicates.contains(property));

        property = eGMS.getNamedPredicateProperty("status");
    }


    /**
     *  Test the provides qualifiers methods.
     *  @since  Version 0.3 2005-03-11
     */
    public final void testProvidesQualifiers() {

        assertTrue(eGMS.providesQualifiers());

        assertTrue(eGMS.providesQualifiersFor(DublinCoreElements.BASE));

        assertFalse(eGMS.providesQualifiersFor("http://www.remote.example/"));
    }

    /**
     *  Test the provides qualifiers methods.
     *  @since  Version 0.3 2005-03-11
     */
    public final void testProvidesQualifiersForNullUri() {

        try {

            eGMS.providesQualifiersFor(null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test the prefix method.
     *  @since  Version 0.2 2005-03-01
     */
    public final void testGetDefaultPrefix() {

        assertTrue("eGMS".equals(eGMS.getDefaultPrefix()));
    }

    /**
     *  Test the prefixes method.
     *  @since  Version 0.6 2005-05-27
     */
    public final void testGetAllPrefixes() {

        String[] prefixes = eGMS.getAllPrefixes();

        assertTrue(prefixes.length == 2);
        assertTrue("eGMS".equals(prefixes[0]));
        assertTrue("DC".equals(prefixes[1]));
    }

    /**
     *  Test expand a null scheme.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandSchemeNull() {

        try {

            eGMS.expandScheme(null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test expand a generic scheme.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandSchemeGeneric() {

        String expanded = eGMS.expandScheme(GENERIC);

        assertTrue("Generic scheme should not be expanded, '" +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(GENERIC));

    }

    /**
     *  Test expand a scheme using the original DC in XHTML format.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandSchemeOriginal() {

        String expanded = eGMS.expandScheme(W3CDTF_OLD);

        assertTrue("Encoded scheme should not be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_W3CDTF + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(W3CDTF_OLD));
    }

    /**
     *  Test expand a scheme using the new DC in XHTML format.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandSchemeNew() {

        String expanded = eGMS.expandScheme(W3CDTF);

        assertTrue("Encoded scheme should not be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_W3CDTF + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(W3CDTF));
    }

    /**
     *  Test expand a scheme using an e-GMS scheme.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandSchemeGMS() {

        String expanded = eGMS.expandScheme(EGMSSAES);

        assertTrue("Encoded scheme should be expanded to '" +
                   eGMS.BASE + eGMS.NAME_GMSAES + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(eGMS.BASE + eGMS.NAME_GMSAES));
    }

    /**
     *  Test expand a null predicate with fallback.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateNull() {

        try {

            eGMS.expandPredicate(null,
                                 NAME_AUDNCE);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test expand a predicate with null fallback.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateNullFallback() {

        try {

            eGMS.expandPredicate(NAME_AUDNCE,
                                 null);

            fail(EXPECT_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test expand a generic predicate with fallback.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateGeneric() {

        String expanded = eGMS.expandPredicate(GENERIC,
                                               FALLBACK);

        assertTrue("Generic predicate should not be expanded, '" +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(FALLBACK));

    }

    /**
     *  Test expand an audience predicate.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateBogusDC() {

        String expanded = eGMS.expandPredicate(DC_BOGUS);

        assertTrue("Encoded predicate not should be expanded. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(DC_BOGUS));
    }

    /**
     *  Test expand an audience predicate.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateBogusGMS() {

        String expanded = eGMS.expandPredicate(EGMS_BOGUS);

        assertTrue("Encoded predicate not should be expanded. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(EGMS_BOGUS));
    }

    /**
     *  Test expand a modified predicate, old form.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateModifiedOldForm() {

        String expanded = eGMS.expandPredicate(OLD_FORM);

        assertTrue("Encoded predicate should not be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MODFID + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(OLD_FORM));
    }

    /**
     *  Test expand a modified predicate, old form.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateModifiedOldFormOverloaded() {

        String expanded = eGMS.expandPredicate(OLD_FORM, FALLBACK);

        assertTrue("Encoded predicate should not be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_MODFID + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(FALLBACK));
    }

    /**
     *  Test expand an e-GMS Adressee predicate.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateAddressee() {

        String expanded = eGMS.expandPredicate(EGMS_ADDREE);

        assertTrue("Encoded predicate should be expanded to '" +
                   eGMS.BASE + eGMS.NAME_ADDREE + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(eGMS.BASE + eGMS.NAME_ADDREE));
    }

    /**
     *  Test expand an e-GMS Adressee predicate.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateAddresseeOverloaded() {

        String expanded = eGMS.expandPredicate(EGMS_ADDREE, EGMS_ADDREE);

        assertTrue("Encoded predicate should be expanded to '" +
                   eGMS.BASE + eGMS.NAME_ADDREE + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(eGMS.BASE + eGMS.NAME_ADDREE));
    }

    /**
     *  Test expand an e-GMS adresseeCopy refined predicate.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateAddresseeCopy() {

        String expanded = eGMS.expandPredicate(EGMS_ADDCPY);

        assertTrue("Encoded predicate should be expanded to '" +
                   eGMS.BASE + eGMS.SUFF_ADDCPY + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(eGMS.BASE + eGMS.SUFF_ADDCPY));
    }

    /**
     *  Test expand an e-GMS adresseeCopy refined predicate.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateAddresseeCopyOverloaded() {

        String expanded = eGMS.expandPredicate(EGMS_ADDCPY, EGMS_ADDCPY);

        assertTrue("Encoded predicate should be expanded to '" +
                   eGMS.BASE + eGMS.SUFF_ADDCPY + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(eGMS.BASE + eGMS.SUFF_ADDCPY));
    }

    /**
     *  Test expand an e-GMS updadingFrequency refinement to a DC predicate.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateUpdatingFrequency() {

        String expanded = eGMS.expandPredicate(EGMS_UPFREQ);

        assertTrue("Encoded predicate should be expanded to '" +
                   eGMS.BASE + eGMS.NAME_UPFREQ + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(eGMS.BASE + eGMS.NAME_UPFREQ));
    }

    /**
     *  Test expand an e-GMS updadingFrequency refinement to a DC predicate.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateUpdatingFrequencyOverloaded() {

        String expanded = eGMS.expandPredicate(EGMS_UPFREQ, EGMS_UPFREQ);

        assertTrue("Encoded predicate should be expanded to '" +
                   eGMS.BASE + eGMS.NAME_UPFREQ + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(eGMS.BASE + eGMS.NAME_UPFREQ));
    }

    /**
     *  Test expand an audience predicate.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateAudience() {

        String expanded = eGMS.expandPredicate(NAME_AUDNCE);

        assertTrue("Encoded predicate should not be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_AUDNCE + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(NAME_AUDNCE));
    }

    /**
     *  Test expand an audience predicate.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateAudienceOverloaded() {

        String expanded = eGMS.expandPredicate(NAME_AUDNCE, FALLBACK);

        assertTrue("Encoded predicate should not be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_AUDNCE + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(FALLBACK));
    }

    /**
     *  Test expand a bogus predicate.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateBogusQualifierOldScheme() {

        String expanded = eGMS.expandPredicate(BOGUS_REFN, FALLBACK);

        assertTrue("Encoded predicate should not be expanded. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(FALLBACK));
    }

    /**
     *  Test expand an audience predicate, lower case.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateAudienceLowerCase() {

        String expanded = eGMS.expandPredicate(NAME_AUDNCE.toLowerCase());

        assertTrue("Encoded predicate should be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_AUDNCE + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(NAME_AUDNCE.toLowerCase()));
    }

    /**
     *  Test expand an audience predicate, upper case.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testExpandPredicateAudienceUpperCase() {

        String expanded = eGMS.expandPredicate(NAME_AUDNCE.toUpperCase());

        assertTrue("Encoded predicate should not be expanded to '" +
                   DublinCoreTerms.BASE + DublinCoreTerms.NAME_AUDNCE + "'. " +
                   "Actually returned '" + expanded + "'.",
                   expanded.equals(NAME_AUDNCE.toUpperCase()));
    }

    /**
     *  Get the base URI for the schema.
     *  @since  Version 0.1 2005-02-28
     */
    public final void testGetUri() {

        String uri = eGMS.getUri();

        assertTrue(uri.equals(eGMS.BASE));
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void setUp() {

    }
     */


    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-02-28
     */
    public static Test suite() {

        return new TestSuite(UKeGMSTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-02-28
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
