package com.mkdoc.servlet;

/*
    Created:    2005-01-21
    Issued:     2005-01-21
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.schema.ApplicationProfile;
import com.mkdoc.schema.DublinCoreProfile;

import com.mkdoc.sesame.SeRQLQueryBuilder;

import com.mkdoc.util.ApplicationContext;

import java.io.File;
import java.io.IOException;

import javax.servlet.ServletException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>HttpQuery</code> servlet.
 *  @version    1.8 2005-11-01
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class HttpQueryTest extends TestCase {

    /**
     *  An expected <code>ServletException</code> message.
     *  @since  Version 1.2 2005-07-25
     */
    public static final String EXPECTED_SE =
    "Should throw a ServletException.";

    /**
     *  The string &quot;true&quot;.
     *  @since  Version 1.7 2005-10-14
     */
    public static final String TRUE = "true";

    /**
     *  The servlet name.
     *  @since  Version 1.2 2005-07-22
     */
    public static final String CLASS_NAME = "HttpQueryTest";

    /**
     *  The HTML result page.
     *  @since  Version 1.2 2005-07-22
     */
    private static final String HTML_RESULT = "/XhtmlQueryResult.jsp";

    /**
     *  The <code>text/html</code> content type.
     *  @since  Version 1.5 2005-09-12
     */
//    private static final String TYPE_HTML = "text/html";

    /**
     *  The text result page.
     *  @since  Version 1.2 2005-07-22
     */
    private static final String TEXT_RESULT = "/TextQueryResult.jsp";

    /**
     *  The <code>text/plain</code> content type.
     *  @since  Version 1.5 2005-09-12
     */
//    private static final String TYPE_TEXT = "text/plain";

    /**
     *  The RSS result page.
     *  @since  Version 1.4 2005-07-29
     */
//    private static final String RSS_RESULT = "/RssQueryResult.jsp";

    /**
     *  The <code>application/rss+xml</code> content type.
     *  @since  Version 1.5 2005-09-12
     */
    private static final String TYPE_RSS = "application/rss+xml";

    /**
     *  The <code>dc</code> namespace prefix.
     *  @since  Version 0.3 2005-01-25
     */
    public static final String DC_PREFIX = "dc";

    /**
     *  The <code>dcterms</code> namespace prefix.
     *  @since  Version 0.3 2005-01-25
     */
    public static final String DCTERMS = "dcterms";

    /**
     *  The <code>dc</code> namespace URI.
     *  @since  Version 0.3 2005-01-25
     */
    public static final String DC_URI = "http://purl.org/dc/elements/1.1/";

    /**
     *  The <code>dcterms</code> namespace URI.
     *  @since  Version 0.3 2005-01-25
     */
    public static final String DCTERMS_URI = "http://purl.org/dc/terms/";

    /**
     *  An echo parameter name.
     *  @since  Version 0.5 2005-01-26
     */
    private static final String PARAM_ECHO = "echo";

    /**
     *  A type parameter name.
     *  @since  Version 0.7 2005-02-02
     */
    private static final String PARAM_TYPE = "type";

    /**
     *  A <code>dc:subject</code> parameter name.
     *  @since  Version 0.1 2005-01-21
     */
    private static final String PARAM_SUBJ = "dc.subject";

    /**
     *  A single word subject query parameter.
     *  @since  Version 0.1 2005-01-21
     */
    private static final String SUBJ_BASIC = "Angiostatins";

    /**
     *  The subject path expression.
     *  @since  Version 0.1 2005-01-21
     */
    public static final String SUBJECT_EXPR =
    "{targetUri} dc:subject {dcSubject}";

    /**
     *  The title path expression.
     *  @since  Version 0.4 2005-01-26
     */
    public static final String TITLE_EXPR =
    "{targetUri} dc:title {dcTitle}";

    /**
     *  The description path expression.
     *  @since  Version 0.4 2005-01-26
     */
    public static final String DESC_EXPR =
    "{targetUri} dc:description {dcDescription}";

    /**
     *  The target URI variable.
     *  @since  Version 0.1 2005-01-21
     */
    public static final String TARGET_VAR = "targetUri";

    /**
     *  The subject variable.
     *  @since  Version 0.1 2005-01-21
     */
    public static final String SUBJECT_VAR = "dcSubject";

    /**
     *  The title variable.
     *  @since  Version 0.4 2005-01-26
     */
    public static final String TITLE_VAR = "dcTitle";

    /**
     *  The description variable.
     *  @since  Version 0.4 2005-01-26
     */
    public static final String DESC_VAR = "dcDescription";

    /**
     *  A SeRQL FROM statement.
     *  @since  Version 0.4 2005-01-26
     */
    private static final String FROM = " FROM ";

    /**
     *  A preliminary SeRQL select subject statement with DISTICT keyword.
     *  @since  Version 0.1 2005-01-21
     */
    private static final String Q_DISTINCT = "SELECT DISTINCT ";

    /**
     *  An SeRQL query namespace declaration.
     *  @since  Version 0.3 2005-01-25
     */
    private static final String Q_NS_DC = " USING NAMESPACE " +
                                          DC_PREFIX +
                                          " = <" +
                                          DC_URI +
                                          ">";

    /**
     *  The project home directory.
     *  @since  Version 0.1 2005-01-21
     */
    private static final String HOME = System.getProperty("mkHome");

    /**
     *  The test directory name under HOME.
     *  @since  Version 0.1 2005-01-21
     */
    private static final String TEST = System.getProperty("testDir");

    /**
     *  The RDF directory name under TEST.
     *  @since  Version 0.1 2005-01-21
     */
    private static final String RDF = System.getProperty("rdfDir");

    /**
     *  The Sesame repository file in the RDF directory.
     *  @since  Version 0.1 2005-01-21
     */
    private static final String STORE = System.getProperty("storeFile");

    /**
     *  A non-existent Sesame repository file name.
     *  @since  Version 0.1 2005-01-21
     */
    private static final String STORE_BAD = "~not*exist^";

    /**
     *  The Sesame repository file path.
     *  @since  Version 0.1 2005-01-21
     */
    private static final String STORE_PATH = HOME +
                                             File.separator +
                                             TEST +
                                             File.separator +
                                             RDF;

    /**
     *  A SeRQL query builder for testing.
     *  @since  Version 0.3 2005-01-25
     */
    protected SeRQLQueryBuilder builder;

    /**
     *  A basic servlet instance for testing.
     *  @since  Version 0.1 2005-01-21
     */
    protected HttpQuery servlet;

    /**
     *  An initialized servlet instance for testing.
     *  @since  Version 0.1 2005-01-21
     */
    protected HttpQuery servletInit;

    /**
     *  An application profile.
     *  @since  Version 0.8 2005-06-16
     */
    protected ApplicationProfile profile = new DublinCoreProfile();

    /**
     *  A mock servlet configuration for testing.
     *  @since  Version 1.0 2005-07-14
     */
    protected MockServletConfig config;

    /**
     *  A mock servlet context for testing.
     *  @since  Version 1.2 2005-07-22
     */
    protected MockServletContext context;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-01-21
     */
    public HttpQueryTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @exception  ServletException    If the initialization of
     *          <code>servletInit</code> fails.
     *  @since  Version 0.1 2005-01-21
     */
    protected final void setUp() throws ServletException {

        servlet = new HttpQuery();

        servletInit = new HttpQuery();

        config = new MockServletConfig(CLASS_NAME);

        config.setInitParameter("htmlResult",
                                HTML_RESULT);
        config.setInitParameter("textResult",
                                TEXT_RESULT);

        context = new MockServletContext();

        context.setBasePath(HOME);

        context.setInitParameter(ApplicationContext.REPOSITORY, STORE);
        context.setInitParameter(ApplicationContext.ABSOLUTE, TRUE);
        context.setInitParameter(ApplicationContext.DIRECTORY, STORE_PATH);

        context.setInitParameter("manager",
                                 "com.mkdoc.store.LocalStoreManager");

        config.setServletContext(context);

        servletInit.init(config);

        builder = new SeRQLQueryBuilder();
    }


    /**
     *  Get with no parameters, should be forwarded to the HTML result page.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  ServletException Should not be thrown by this test.
     *  @since  Version 0.3 2005-01-25
     */
    public final void testDoGetNoParameters()
        throws IOException, ServletException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        MockHttpServletResponse msres = new MockHttpServletResponse();

        servletInit.doGet(msreq, msres);

        String path = msres.getPath();

        assertTrue(path.equals(HTML_RESULT));
    }

    /**
     *  Call doPost with subject and echo parameter will currently result in a
     *  print out of the SeRQL query.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  ServletException Should not be thrown by this test.
     *  @since  Version 0.3 2005-01-25
     */
    public final void testDoPost() throws IOException, ServletException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter(PARAM_SUBJ, SUBJ_BASIC);
        msreq.setParameter(PARAM_ECHO, TRUE);

        MockHttpServletResponse msres = new MockHttpServletResponse();

        MockServletOutputStream outStream = new MockServletOutputStream();

        msres.setOutputStream(outStream);

        servletInit.doPost(msreq, msres);

        String output = outStream.getOutput();

        assertTrue("Query should include a DISTINCT keyword. " +
                   "Actual output was: '" +
                   output + "'.",
                   (output.indexOf(Q_DISTINCT) > -1));

        // The order of variables is not significant, nor controlled
        assertTrue("Query should include the target URI and subject " +
                   "variables. Actual output was: '" +
                   output + "'.",
                   ((output.indexOf(TARGET_VAR + ", " + SUBJECT_VAR) > -1) ||
                    (output.indexOf(SUBJECT_VAR + ", " + TARGET_VAR) > -1)));

        assertTrue("Query should include the title path expression:\n\n" +
                   FROM + SUBJECT_EXPR +
                   "\n\nActual output was:\n\n" +
                   output,
                   output.indexOf(FROM + SUBJECT_EXPR) > -1);

        assertTrue("Query should include the DC namespace declaration:\n\n" +
                   Q_NS_DC +
                   "\n\nActual output was:\n\n" +
                   output,
                   output.indexOf(Q_NS_DC) > -1);

    }

    /**
     *  Call doPost with no query and echo parameter.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  ServletException Should not be thrown by this test.
     *  @since  Version 0.3 2005-01-25
     */
    public final void testDoPostEchoNoQuery()
        throws IOException, ServletException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter(PARAM_ECHO, TRUE);

        MockHttpServletResponse msres = new MockHttpServletResponse();

        servletInit.doPost(msreq, msres);

        int status = msres.getStatusCode();

        assertTrue("Status code should be 400, actual status code was " +
                   status,
                   status == msres.SC_BAD_REQUEST);
    }

    /**
     *  Call doPost with no query and echo parameter.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  ServletException Should not be thrown by this test.
     *  @since  Version 0.3 2005-01-25
     */
    public final void testDoPostEchoIOException()
        throws IOException, ServletException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter(PARAM_ECHO, TRUE);
        msreq.setParameter(PARAM_SUBJ, SUBJ_BASIC);

        MockHttpServletResponse msres = new MockHttpServletResponse();

        MockServletOutputStream outStream = new MockServletOutputStream();

        outStream.setFailOnWrite(true);

        msres.setOutputStream(outStream);

        servletInit.doPost(msreq, msres);

        int status = msres.getStatusCode();

        assertTrue("Status code should be 500, actual status code was " +
                   status,
                   status == msres.SC_INTERNAL_SERVER_ERROR);
    }

    /**
     *  Call doPost with a <code>type=html</code> parameter will generate an
     *  HTML response.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  ServletException Should not be thrown by this test.
     *  @since  Version 0.7 2005-02-02
     */
    public final void testDoPostTypeHtml()
        throws IOException, ServletException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter(PARAM_SUBJ, SUBJ_BASIC);
        msreq.setParameter(PARAM_ECHO, "false");
        msreq.setParameter(PARAM_TYPE, "html");

        MockHttpServletResponse msres = new MockHttpServletResponse();

        servletInit.doPost(msreq, msres);

        String path = msres.getPath();

        assertTrue(path.equals(HTML_RESULT));

    }

    /**
     *  Call doPost with a text <code>type</code> parameter will generate a
     *  plain text response.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  ServletException Should not be thrown by this test.
     *  @since  Version 0.7 2005-02-02
     */
    public final void testDoPostTypeText()
        throws IOException, ServletException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter(PARAM_SUBJ, SUBJ_BASIC);
        msreq.setParameter(PARAM_TYPE, "text");

        MockHttpServletResponse msres = new MockHttpServletResponse();

        servletInit.doPost(msreq, msres);

        String path = msres.getPath();

        assertTrue(path.equals(TEXT_RESULT));
    }

    /**
     *  Call doPost with an RSS <code>type</code> parameter will generate an
     *  RSS content type response.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  ServletException Should not be thrown by this test.
     *  @since  Version 1.4 2005-07-29
     */
    public final void testDoPostTypeRss()
        throws IOException, ServletException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter(PARAM_SUBJ, SUBJ_BASIC);
        msreq.setParameter(PARAM_TYPE, "rss");

        MockHttpServletResponse msres = new MockHttpServletResponse();

        servletInit.doPost(msreq, msres);

        String contentType = msres.getContentType();

        assertTrue(contentType.equals(TYPE_RSS));
    }

    /**
     *  Call doPost with an empty RSS query will give a bad request error.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  ServletException Should not be thrown by this test.
     *  @since  Version 1.5 2005-09-12
     */
    public final void testDoPostTypeRssNoQuery()
        throws IOException, ServletException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter(PARAM_TYPE, "rss");

        MockHttpServletResponse msres = new MockHttpServletResponse();

        servletInit.doPost(msreq, msres);

        int status = msres.getStatusCode();

        assertTrue(status == msres.SC_BAD_REQUEST);
    }

    /**
     *  Handle an IOException for an RSS query.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  ServletException Should not be thrown by this test.
     *  @since  Version 1.5 2005-09-12
     */
    public final void testDoPostTypeRssIOException()
        throws IOException, ServletException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter(PARAM_SUBJ, SUBJ_BASIC);
        msreq.setParameter(PARAM_TYPE, "rss");

        MockHttpServletResponse msres = new MockHttpServletResponse();

        MockServletOutputStream out = new MockServletOutputStream();

        out.setFailOnWrite(true);

        msres.setOutputStream(out);

        servletInit.doPost(msreq, msres);

        int status = msres.getStatusCode();

        assertTrue("Should give status code 500, actual status was " +
                   status + ".",
                   status == msres.SC_INTERNAL_SERVER_ERROR);

    }


    /**
     *  Initialize without repository parameter should throw a
     *  <code>ServletException</code>.
     *  @since  Version 0.1 2005-01-21
     */
    public final void testInitNoRepositoryParam() {

        MockServletContext badContext = new MockServletContext();

        badContext.setBasePath(HOME);
        badContext.setInitParameter("manager", "LocalRepositoryQueryManager");

        config.setServletContext(badContext);

        try {

            servlet.init(config);

            fail(EXPECTED_SE);
        }
        catch (ServletException se) { }
    }

    /**
     *  Initialize witha repository that doesn't exist should throw a
     *  <code>ServletException</code>.
     *  @since  Version 0.1 2005-01-21
     */
    public final void testInitRepositoryNoStoreQueryManager() {

        MockServletContext badContext = new MockServletContext();

        badContext.setBasePath(HOME);
        badContext.setInitParameter("repository", STORE_PATH);

        config.setServletContext(badContext);

        try {

            servlet.init(config);

            fail(EXPECTED_SE);
        }
        catch (ServletException se) { }
    }

    /**
     *  Initialize witha repository that doesn't exist should throw a
     *  <code>ServletException</code>.
     *  @since  Version 0.1 2005-01-21
     */
    public final void testInitRepositoryNotExist() {

        MockServletContext badContext = new MockServletContext();

        badContext.setBasePath(HOME);
        badContext.setInitParameter("repository", STORE_BAD);
        badContext.setInitParameter("manager", "LocalRepositoryQueryManager");

        config.setServletContext(badContext);

        try {

            servlet.init(config);

            fail(EXPECTED_SE);
        }
        catch (ServletException se) { }
    }

    /**
     *  Initialize with no text result page
     *  @since  Version 0.1 2005-01-21
     */
    public final void testInitRepositoryNoTextParam() {

        MockServletConfig badConfig = new MockServletConfig(CLASS_NAME);

        badConfig.setInitParameter("htmlResult",
                                   HTML_RESULT);

        badConfig.setServletContext(context);

        try {

            servlet.init(badConfig);

            fail(EXPECTED_SE);
        }
        catch (ServletException se) { }
    }

    /**
     *  Initialize with no HTML result page
     *  @since  Version 0.1 2005-01-21
     */
    public final void testInitRepositoryNoHtmlParam() {

        MockServletConfig badConfig = new MockServletConfig(CLASS_NAME);

        badConfig.setInitParameter("textResult",
                                   TEXT_RESULT);

        badConfig.setServletContext(context);

        try {

            servlet.init(badConfig);

            fail(EXPECTED_SE);
        }
        catch (ServletException se) { }
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-01-21
     */
    public static Test suite() {

        return new TestSuite(HttpQueryTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-01-21
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
