package com.mkdoc.servlet;

/*
    Created:    2005-01-20
    Issued:     2005-01-20
    Modified:   2005-10-14

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.security.Principal;

import java.util.Enumeration;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

/**
 *  A mock HTTP servlet request for testing.
 *  <p>
 *    Most fields are initially <code>null</code> and must be configured using
 *    the relevant mutators if their accessors are expected to be called. Some
 *    accessors issue fixed values. Most mutators do not have validation, so
 *    <code>null</code> values can generally be assigned.
 *  </p>
 *  @version    0.4 2005-10-14
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockHttpServletRequest extends MockServletRequest
                                 implements HttpServletRequest {

    /**
     *  The default HTTP port number.
     *  @since  Version 0.1 2005-01-20
     */
    private static final int PORT = 80;

    /**
     *  The authentication type.
     *  @since  Version 0.1 2005-01-20
     */
    protected String authType;

    /**
     *  The remote user.
     *  @since  Version 0.1 2005-01-20
     */
    protected String remoteUser;

    /**
     *  The user role.
     *  @since  Version 0.1 2005-01-20
     */
    protected String role;

    /**
     *  The security principal associated with the user.
     *  @since  Version 0.1 2005-01-20
     */
    protected Principal principal;

    /**
     *  The context path.
     *  @since  Version 0.1 2005-01-20
     */
    protected String contextPath = "";

    /**
     *  The request method.
     *  @since  Version 0.1 2005-01-20
     */
    protected String method;

    /**
     *  The path info.
     *  @since  Version 0.1 2005-01-20
     */
    protected String pathInfo;

    /**
     *  The session object.
     *  @since  Version 0.1 2005-01-20
     */
    protected HttpSession session;

    /**
     *  The servlet path.
     *  @since  Version 0.1 2005-01-20
     */
    protected String servletPath;

    /**
     *  Whether the requested session ID is valid.
     *  @since  Version 0.1 2005-01-20
     */
    protected boolean isValid;

    /**
     *  Whether the session ID is from a cookie, false implies a URL session.
     *  @since  Version 0.1 2005-01-20
     */
    protected boolean isCookie;

    /**
     *  Set protocol, scheme and port to HTTP defaults.
     *  @since  Version 0.1 2005-01-20
     */
    public MockHttpServletRequest() {

        super();

        protocol = "HTTP/1.1";

        scheme = "http";

        serverPort = PORT;
    }

    /**
     *  Get the authentication type.
     *  @return The authentication type used by this request, or
     *          <code>null</code>.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getAuthType() {

        return authType;
    }

    /**
     *  Set the authentication type.
     *  @param  authType   The authentication type to assign.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setAuthType(final String authType) {

        this.authType = authType;
    }

    /**
     *  Get the cookies.
     *  @return An array of cookies attached to this request.
     *  @since  Version 0.1 2005-01-20
     */
    public final Cookie[] getCookies() {

        return properties.getCookies();
    }

    /**
     *  Set a cookie.
     *  @param  cookie  A cookie to be added.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setCookie(final Cookie cookie) {

        properties.setCookie(cookie);
    }

    /**
     *  Get a date header by name.
     *  <p>
     *    Simple version expects date to be a <code>long</code> string.
     *  </p>
     *  @param  name    The name of the parameter value to get.
     *  @return The date value of the header or <code>-1L</code> if not set or
     *          not a number.
     *  @since  Version 0.1 2005-01-20
     */
    public final long getDateHeader(final String name) {

        String value = getHeader(name);

        if (value == null) {

            return -1L;
        }
        else {

            try {

                return Long.parseLong(value);
            }
            catch (NumberFormatException nfe) {

                return -1L;
            }

        }
    }

    /**
     *  Get an integer header by name.
     *  <p>
     *    Simple version expects the integer to be an <code>int</code> string.
     *  </p>
     *  @param  name    The name of the parameter value to get.
     *  @return The integer value of the header or <code>-1</code> if not set or
     *          not a number.
     *  @since  Version 0.1 2005-01-20
     */
    public final int getIntHeader(final String name) {

        String value = getHeader(name);

        if (value == null) {

            return -1;
        }
        else {

            try {

                return Integer.parseInt(value);
            }
            catch (NumberFormatException nfe) {

                return -1;
            }
        }
    }

    /**
     *  Get a header by name.
     *  @param  name    The name of the parameter value to get.
     *  @return The value of the header or <code>null</code> if not set.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getHeader(final String name) {

        return properties.getHeader(name);
    }

    /**
     *  Get a set of header values by name.
     *  @param  name    The name of the parameter values to get.
     *  @return The header values or <code>null</code> if not set.
     *  @since  Version 0.1 2005-01-20
     */
    public final Enumeration getHeaders(final String name) {

        return properties.getHeaders(name);
    }

    /**
     *  Set a header by name.
     *  <p>
     *    Expects all headers to be string literals.
     *  </p>
     *  @param  name    The name of the parameter.
     *  @param  value   The value to assign to the parameter.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setHeader(final String name,
                                final String value) {

        properties.setHeader(name, value);
    }

    /**
     *  Get all header names.
     *  @return All header names in lower case.
     *  @since  Version 0.1 2005-01-20
     */
    public final Enumeration getHeaderNames() {

        return properties.getHeaderNames();
    }

    /**
     *  Get the HTTP method.
     *  @return The HTTP method, e.g. &quot;get&quot; or &quot;post&quot;.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getMethod() {

        return method;
    }

    /**
     *  Set the HTTP method.
     *  @param  method  The HTTP method used for the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setMethod(final String method) {

        this.method = method;
    }

    /**
     *  Get the context path.
     *  @return The context path for the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getContextPath() {

        return contextPath;
    }

    /**
     *  Set the context path.
     *  @param  contextPath The context path for the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setContextPath(final String contextPath) {

        this.contextPath = contextPath;
    }

    /**
     *  Get the path info.
     *  @return The path info part of the request URL.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getPathInfo() {

        return pathInfo;
    }

    /**
     *  Set the path info.
     *  @param  pathInfo    The path info part of the request URL.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setPathInfo(final String pathInfo) {

        this.pathInfo = pathInfo;
    }

    /**
     *  Get the path translated.
     *  @return This version always returns <code>null</code>.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getPathTranslated() {

        return null;
    }

    /**
     *  Get the query string.
     *  @return The request query string for HTTP <code>GET</code> requests or
     *          <code>null</code>.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getQueryString() {

        if ((method != null) &&
            (method.equalsIgnoreCase("get"))) {

            return queryString.toString();
        }
        else {

            return null;
        }
    }

    /**
     *  Set the query string.
     *  @param  queryString A query string to assign, starting which should
     *          start with a &quot;?&quot;.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setQueryString(final String queryString) {

        this.queryString.setLength(0);

        this.queryString.append(queryString);

        firstParam = false;
    }

    /**
     *  Get the remote user.
     *  @return The remote user or <code>null</code> if not set.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getRemoteUser() {

        return remoteUser;
    }

    /**
     *  Set the query string.
     *  @param  remoteUser  The remote user name for the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setRemoteUser(final String remoteUser) {

        this.remoteUser = remoteUser;
    }

    /**
     *  Check if the user is in a named role.
     *  @param  role    The role name to compare with.
     *  @return <code>true</code> if the role matches that assigned to the
     *          request.
     *  @since  Version 0.1 2005-01-20
     */
    public final boolean isUserInRole(final String role) {

        return this.role.equalsIgnoreCase(role);
    }

    /**
     *  Set the query string.
     *  @param  role    The user role to attach to the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setUserRole(final String role) {

        this.role = role;
    }

    /**
     *  Get the user security principal.
     *  @return The security principal attached to the the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final Principal getUserPrincipal() {

        return principal;
    }

    /**
     *  Set the user security principal.
     *  @param  principal   The security principal to attach to the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setUserPrincipal(final Principal principal) {

        this.principal = principal;
    }

    /**
     *  Get the requested session ID.
     *  @return The session ID associated with the request or <code>null</code>
     *          if not set.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getRequestedSessionId() {

        if (session != null) {

            return session.getId();
        }

        return null;
    }

    /**
     *  Set the HTTP session object.
     *  @param  session The session associated with the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setSession(final HttpSession session) {

        this.session = session;
    }

    /**
     *  Get the request URL, e.g.
     *  <code>http://example.com/path/to/Servlet/pathInfo;jsessionid=123</code>
     *  @return An absolute URL except for the query string.
     *  @since  Version 0.1 2005-01-20
     */
    public final StringBuffer getRequestURL() {

        StringBuffer buffer = new StringBuffer();

        buffer.append(scheme);
        buffer.append("://");
        buffer.append(serverName);

        if (serverPort != PORT) {

            buffer.append(":");
            buffer.append(serverPort);
        }

        buffer.append(getRequestURI());

        return buffer;
    }

    /**
     *  Get the request URI, e.g.
     *  <code>/path/to/Servlet/pathInfo;jsessionid=123</code>
     *  @return The request URI in the form of an absolute path relative to
     *          the base URL, no query string.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getRequestURI() {

        StringBuffer buffer = new StringBuffer();

        if (servletPath != null) {

            buffer.append(servletPath);
        }

        if (pathInfo != null) {

            buffer.append(pathInfo);
        }

        if ((session != null) &&
            (!isCookie)) {

            buffer.append(";jsessionid=");
            buffer.append(session.getId());
        }

        if ((method != null) &&
            (method.equalsIgnoreCase("get")) &&
            (queryString.length() > 0)) {

            buffer.append(queryString);
        }

        return buffer.toString();
    }

    /**
     *  Get the servlet path.
     *  @return The absolute path of the requested servlet only.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getServletPath() {

        return servletPath;
    }

    /**
     *  Set the servlet path.
     *  @param  servletPath The absolute path of the servlet, should start with
     *          a &quot;/&quot;.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setServletPath(final String servletPath) {

        this.servletPath = servletPath;
    }

    /**
     *  Get the HTTP session.
     *  @return The HTTP session associated with the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final HttpSession getSession() {

        return session;
    }

    /**
     *  Get the HTTP session.
     *  @param  create  Whether to create a session if not already present.
     *          The current version throws an
     *          <code>IllegalStateException</code> in this case.
     *  @return A HTTP session associated with the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final HttpSession getSession(final boolean create) {

        if (session == null) {

            if (create) {

                throw new IllegalStateException(
                            "Session requested, but not pre-configured.");
            }
            else {

                return null;
            }
        }
        else {

            return session;
        }
    }

    /**
     *  Get the HTTP session.
     *  @return <code>true</code> if the request's session ID matches a valid
     *          session in the servlet container. Must be pre-configured.
     *  @since  Version 0.1 2005-01-20
     */
    public final boolean isRequestedSessionIdValid() {

        return isValid;
    }

    /**
     *  Set whether the HTTP session ID is treated as valid.
     *  @param  isValid Whether the associated session should be treated as
     *          valid.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setSessionIdValid(final boolean isValid) {

        this.isValid = isValid;
    }

    /**
     *  Check whether the session is cookie-based.
     *  @return <code>true</code> if the session ID originated from a cookie.
     *          Inverse with URL-based requests.
     *  @since  Version 0.1 2005-01-20
     */
    public final boolean isRequestedSessionIdFromCookie() {

        return isCookie;
    }

    /**
     *  Check whether the session is URL-based.
     *  @return <code>true</code> if the session ID originated from a URL.
     *          Inverse with cookie-based requests.
     *  @since  Version 0.1 2005-01-20
     */
    public final boolean isRequestedSessionIdFromURL() {

        return !isCookie;
    }

    /**
     *  Check whether the session is URL-based.
     *  @return <code>true</code> if the session ID originated from a URL.
     *          Inverse with cookie-based requests.
     *  @since  Version 0.1 2005-01-20
     */
    public final boolean isRequestedSessionIdFromUrl() {

        return !isCookie;
    }

}
