package com.mkdoc.servlet;

/*
    Created:    2005-01-21
    Issued:     2005-01-21
    Modified:   2005-10-19

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.IOException;

import java.util.Enumeration;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;

/**
 *  Mock <code>HttpServletResponse</code> for testing.
 *  @version    0.5 2005-10-19
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockHttpServletResponse extends MockServletResponse
                                  implements HttpServletResponse {

    /**
     *  The default status code is 200.
     *  @since  Version 0.1 2005-01-21
     */
    public static final int STATUS = 200;

    /**
     *  The HTTP status code.
     *  @since  Version 0.1 2005-01-21
     */
    protected int statusCode = STATUS;

    /**
     *  The session ID.
     *  @since  Version 0.1 2005-01-21
     */
    protected String sessionId;

    /**
     *  Whether the session is maintained by URL rewriting.
     *  @since  Version 0.1 2005-01-21
     */
    protected boolean rewrite;

    /**
     *  An error status message.
     *  @since  Version 0.1 2005-01-21
     */
    protected String message = "OK";

    /**
     *  Whether the response should fail upon redirection.
     *  @since  Version 0.5 2005-10-19
     */
    protected boolean failOnRedirect = false;

    /**
     *  Response without session handling.
     *  @since  Version 0.1 2005-01-21
     */
    public MockHttpServletResponse() {

        super();
    }

    /**
     *  Create a response with URL rewriting.
     *  @param  sessionId   An arbitrary session ID value to use.
     *  @param  rewrite Whether the session is being maintained by URL
     *          rewriting.
     *  @since  Version 0.1 2005-01-21
     */
    public MockHttpServletResponse(final String sessionId,
                                   final boolean rewrite) {

        super();

        this.sessionId = sessionId;
        this.rewrite = rewrite;
    }

    /**
     *  Set whether the response should fail upon redirection.
     *  @param  setting Whether the response should fail.
     *  @since  Version 0.5 2005-10-19
     */
    public final void setFailOnRedirect(final boolean setting) {

        failOnRedirect = setting;
    }

    /**
     *  Add a cookie to the response.
     *  @param  cookie  The cookie to add.
     *  @since  Version 0.1 2005-01-21
     */
    public final void addCookie(final Cookie cookie) {

        properties.setCookie(cookie);
    }

    /**
     *  Get all cookies.
     *  @return An array of cookies.
     *  @since  Version 0.1 2005-01-21
     */
    public final Cookie[] getCookies() {

        return properties.getCookies();
    }

    /**
     *  Check whether the response contains a named header.
     *  @param  name    The name of the header to check.
     *  @return <code>true</code> if a header with the given name has been set.
     *  @since  Version 0.1 2005-01-21
     */
    public final boolean containsHeader(final String name) {

        return properties.containsHeader(name);
    }

    /**
     *  Encode a redirect URL as required for session tracking.
     *  <p>
     *    Uses the session identifier &quot;jsessionid&quot;.
     *  </p>
     *  @param  url The URL to encode.
     *  @return The original URL encoded with the relevant session identifier
     *          where necessary.
     *  @since  Version 0.1 2005-01-21
     */
    public final String encodeRedirectURL(final String url) {

        return encodeURL(url);
    }

    /**
     *  Encode a redirect URL as required for session tracking.
     *  <p>
     *    Uses the session identifier &quot;jsessionid&quot;.
     *  </p>
     *  @param  url The URL to encode.
     *  @return The original URL encoded with the relevant session identifier
     *          where necessary.
     *  @since  Version 0.1 2005-01-21
     */
    public final String encodeRedirectUrl(final String url) {

        return encodeURL(url);
    }

    /**
     *  Encode a URL as required for session tracking.
     *  <p>
     *    Uses the session identifier &quot;jsessionid&quot;.
     *  </p>
     *  @param  url The URL to encode.
     *  @return The original URL encoded with the relevant session identifier
     *          where necessary.
     *  @since  Version 0.1 2005-01-21
     */
    public final String encodeURL(final String url) {

        if (rewrite) {

            int queryIndex = url.indexOf("?");

            if (queryIndex > -1) {

                String prefix = url.substring(0, queryIndex);

                String suffix = url.substring(queryIndex + 1);

                return prefix +
                       ";jsessionid=" + sessionId +
                       suffix;
            }
            else {

                return url + ";jsessionid=" + sessionId;
            }
        }
        else {

            return url;
        }
    }

    /**
     *  Encode a URL as required for session tracking.
     *  <p>
     *    Uses the session identifier &quot;jsessionid&quot;.
     *  </p>
     *  @param  url The URL to encode.
     *  @return The original URL encoded with the relevant session identifier
     *          where necessary.
     *  @since  Version 0.1 2005-01-21
     */
    public final String encodeUrl(final String url) {

        return encodeURL(url);
    }

    /**
     *  Send an error status code and message.
     *  <p>
     *    Sets a false URL path for test verification.
     *  </p>
     *  @param  statusCode  The HTTP status code to assign.
     *  @param  message The status message to display.
     *  @exception  IOException Not thrown by this implementation.
     *  @since  Version 0.1 2005-01-21
     */
    public final void sendError(final int statusCode,
                                final String message) throws IOException {

        if (failOnRedirect) {

            throw new IOException(
                        "MockHttpServletResponse fails on redirection.");
        }

        this.statusCode = statusCode;

        this.message = message;

        // Make a false URL path for test verification
        path = "/error/" + statusCode;
    }

    /**
     *  Send an error status code with default message.
     *  <p>
     *    Sets a false URL path for test verification.
     *  </p>
     *  @param  statusCode  The HTTP status code to assign.
     *  @exception  IOException Not thrown by this implementation.
     *  @since  Version 0.1 2005-01-21
     */
    public final void sendError(final int statusCode) throws IOException {

        if (failOnRedirect) {

            throw new IOException(
                        "MockHttpServletResponse fails on redirection.");
        }

        this.statusCode = statusCode;

        this.message = "Error, " + statusCode;

        // Make a false URL path for test verification
        path = "/error/" + statusCode;
    }

    /**
     *  Send a redirect header and set the new path.
     *  @param  url The absolute URL to refer clients to.
     *  @exception  IOException Not thrown by this implementation.
     *  @since  Version 0.1 2005-01-21
     */
    public final void sendRedirect(final String url) throws IOException {

        if (failOnRedirect) {

            throw new IOException(
                        "MockHttpServletResponse fails on redirection.");
        }

        statusCode = SC_MOVED_TEMPORARILY;
    }

    /**
     *  Set a date header.
     *  @param  name    The name of the header to set.
     *  @param  value   The date value to set.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setDateHeader(final String name,
                                    final long value) {

        properties.replaceDateHeader(name, value);
    }

    /**
     *  Set a general header.
     *  @param  name    The name of the header to set.
     *  @param  value   The header value to set.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setHeader(final String name,
                                final String value) {

        properties.replaceHeader(name, value);
    }

    /**
     *  Add a general header without replacing any existing value.
     *  @param  name    The name of the header to add.
     *  @param  value   The header value to add.
     *  @since  Version 0.1 2005-01-21
     */
    public final void addHeader(final String name,
                                final String value) {

        properties.setHeader(name, value);
    }

    /**
     *  Add a date header without replacing any existing value.
     *  @param  name    The name of the header to add.
     *  @param  date    The date value to add.
     *  @since  Version 0.1 2005-01-21
     */
    public final void addDateHeader(final String name,
                                    final long date) {

        properties.setDateHeader(name, date);
    }

    /**
     *  Add an int header without replacing any existing value.
     *  @param  name    The name of the header to add.
     *  @param  value   The integer value to add.
     *  @since  Version 0.1 2005-01-21
     */
    public final void addIntHeader(final String name,
                                   final int value) {

        properties.setIntHeader(name, value);
    }

    /**
     *  Set an int header.
     *  @param  name    The name of the header to set.
     *  @param  value   The integer value to set.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setIntHeader(final String name,
                                   final int value) {

        properties.replaceIntHeader(name, value);
    }

    /**
     *  Get a header by name.
     *  @param  name    The name of the header value to get.
     *  @return The value of the header or <code>null</code> if not set.
     *  @since  Version 0.1 2005-01-21
     */
    public final String getHeader(final String name) {

        return properties.getHeader(name);
    }

    /**
     *  Get a set of header values by name.
     *  @param  name    The name of the header values to get.
     *  @return The header values or <code>null</code> if not set.
     *  @since  Version 0.1 2005-01-21
     */
    public final Enumeration getHeaders(final String name) {

        return properties.getHeaders(name);
    }

    /**
     *  Set the response status code.
     *  @param  statusCode  The HTTP status code to assign.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setStatus(final int statusCode) {

        this.statusCode = statusCode;
    }

    /**
     *  Set the response status with a message.
     *  @param  statusCode  The HTTP status code to assign.
     *  @param  message A message to associate with the response.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setStatus(final int statusCode,
                                final String message) {

        this.statusCode = statusCode;

        this.message = message;
    }

    /**
     *  Get the status code of the response.
     *  @return The HTTP status code for the response.
     *  @since  Version 0.2 2005-01-25
     */
    public final int getStatusCode() {

        return statusCode;
    }

    /**
     *  Get the status message for the response.
     *  @return The status message for the response.
     *  @since  Version 0.3 2005-01-26
     */
    public final String getMessage() {

        return message;
    }

}
