package com.mkdoc.servlet;

/*
    Created:    2005-01-21
    Issued:     2005-01-21
    Modified:   2005-07-22

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.servlet.http.Cookie;

/**
 *  Store for servlet properties common to request and response types.
 *  <p>
 *    Intended for use by composition amongst the mock servlet API classes in
 *    the <code>com.mkdoc.servlet</code> package. Could be used for general
 *    purposes.
 *  </p>
 *  @version    0.2 2005-07-14
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockServiceProperties {

    /**
     *  A set of attributes.
     *  <p>
     *    Used in <code>MockServletRequest</code>. Key strings are stored in
     *    lower case.
     *  </p>
     *  @since  Version 0.1 2005-01-21
     */
    protected Dictionary attributes = new Hashtable();

    /**
     *  A set of parameters.
     *  <p>
     *    Used in <code>MockServletRequest</code>. Has the facility for one to
     *    many name to value relations. Key strings are stored in lower case.
     *  </p>
     *  @since  Version 0.1 2005-01-21
     */
    protected Dictionary parameters = new Hashtable();

    /**
     *  A set of HTTP headers.
     *  <p>
     *    Used in both <code>MockServletRequest</code> and
     *    <code>MockServletResponse</code>. Has the facility for one to
     *    many name to value relations. Key strings are stored in lower case.
     *  </p>
     *  @since  Version 0.1 2005-01-21
     */
    protected Dictionary headers = new Hashtable();

    /**
     *  A set of HTTP cookies.
     *  <p>
     *    Used in both <code>MockHttpServletRequest</code> and
     *    <code>MockHttpServletResponse</code>.
     *  </p>
     *  @since  Version 0.1 2005-01-21
     */
    protected List cookies = new Vector();

    /**
     *  Get an attribute value by name.
     *  @param  name    The attribute name.
     *  @return The object stored with the given name key.
     *  @since  Version 0.1 2005-01-21
     */
    public final Object getAttribute(final String name) {

        String lower = name.toLowerCase();

        synchronized (attributes) {

            return attributes.get(lower);
        }
    }

    /**
     *  Set an attribute value by name.
     *  @param  name    The attribute name.
     *  @param  value   The object to store with the attribute name.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setAttribute(final String name,
                                   final Object value) {

        String lower = name.toLowerCase();

        synchronized (attributes) {

            attributes.put(lower, value);
        }
    }

    /**
     *  Get all attribute names.
     *  @return All the attribute names.
     *  @since  Version 0.1 2005-01-21
     */
    public final Enumeration getAttributeNames() {

        synchronized (attributes) {

            int size = attributes.size();

            if (size > 0) {

                return attributes.keys();
            }
            else {

                return null;
            }
        }
    }

    /**
     *  Get all attributes.
     *  @return All the attribute names.
     *  @since  Version 0.1 2005-01-21
     */
    public final Enumeration getAttributes() {

        synchronized (attributes) {

            int size = attributes.size();

            if (size > 0) {

                return attributes.elements();
            }
            else {

                return null;
            }
        }
    }

    /**
     *  Remove an attribute value by name.
     *  @param  name    The name of the attribute to remove.
     *  @since  Version 0.1 2005-01-21
     */
    public final void removeAttribute(final String name) {

        String lower = name.toLowerCase();

        synchronized (attributes) {

            attributes.remove(lower);
        }
    }

    /**
     *  Get a parameter value by name.
     *  @param  name    The parameter name.
     *  @return The parameter value or <code>null</code> if not matched.
     *  @since  Version 0.1 2005-01-21
     */
    public final String getParameter(final String name) {

        String[] values = getParameterValues(name);

        if (values != null) {

            return values[0];
        }
        else {

            return null;
        }
    }


    /**
     *  Set a parameter value by name.
     *  @param  name    The parameter name.
     *  @param  value   The value of the parameter.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setParameter(final String name,
                                   final String value) {

        String lower = name.toLowerCase();

        Object valueObject = null;

        Vector values = null;

        synchronized (parameters) {

            valueObject = parameters.get(lower);

            if (valueObject != null) {

                values = (Vector) valueObject;

                values.add(value);
            }
            else {

                values = new Vector();

                values.add(value);

                parameters.put(lower, values);
            }
        }
    }

    /**
     *  Get a parameter map.
     *  @return A mapping of the request parameter names and values.
     *  @since  Version 0.1 2005-01-21
     */
    public final Map getParameterMap() {

        synchronized (parameters) {

            return new Hashtable((Map) parameters);
        }
    }

    /**
     *  Get an array of parameter values by name.
     *  @param  name    The parameter name.
     *  @return An array of values for the given parameter name, or
     *          <code>null</code> if not matched.
     *  @since  Version 0.1 2005-01-21
     */
    public final String[] getParameterValues(final String name) {

        String lower = name.toLowerCase();

        Object valueObject = null;

        synchronized (parameters) {

            valueObject = parameters.get(lower);

        }

        if (valueObject != null) {

            Vector values = (Vector) valueObject;

            int size = values.size();

            if (size > 0) {

                return (String[]) values.toArray(new String[size]);
            }
            // Fall through
        }
        // Fall through

        return null;
    }

    /**
     *  Get all parameter names.
     *  @return All parameter names.
     *  @since  Version 0.1 2005-01-21
     */
    public final Enumeration getParameterNames() {

        synchronized (parameters) {

            return parameters.keys();
        }
    }

    /**
     *  Get all cookies.
     *  @return An array of cookies.
     *  @since  Version 0.1 2005-01-21
     */
    public final Cookie[] getCookies() {

        synchronized (cookies) {

            int size = cookies.size();

            if (size > 0) {

                return (Cookie[]) cookies.toArray(new Cookie[size]);
            }
            else {

                return null;
            }
        }
    }

    /**
     *  Set a cookie.
     *  @param  cookie  A cookie to be added.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setCookie(final Cookie cookie) {

        synchronized (cookies) {

            cookies.add(cookie);
        }
    }

    /**
     *  Get a header by name.
     *  @param  name    The name of the header value to get.
     *  @return The value of the header or <code>null</code> if not set.
     *  @since  Version 0.1 2005-01-21
     */
    public final String getHeader(final String name) {

        String lower = name.toLowerCase();

        Object valueObject = null;

        synchronized (headers) {

            valueObject = headers.get(lower);

        }

        if (valueObject != null) {

            Vector values = (Vector) valueObject;

            int size = values.size();

            if (size > 0) {

                return (String) values.elementAt(0);
            }
            // Fall through
        }
        // Fall through

        return null;
    }

    /**
     *  Get a set of header values by name.
     *  @param  name    The name of the header values to get.
     *  @return The header values or <code>null</code> if not set.
     *  @since  Version 0.1 2005-01-21
     */
    public final Enumeration getHeaders(final String name) {

        String lower = name.toLowerCase();

        Object valueObject = null;

        synchronized (headers) {

            valueObject = headers.get(lower);

        }

        if (valueObject != null) {

            Vector values = (Vector) valueObject;

            int size = values.size();

            if (size > 0) {

                return values.elements();
            }
            // Fall through
        }
        // Fall through

        return null;
    }

    /**
     *  Set a header by name.
     *  <p>
     *    Expects all headers to be string literals.
     *  </p>
     *  @param  name    The name of the parameter.
     *  @param  value   The value to assign to the parameter.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setHeader(final String name,
                                final String value) {

        String lower = name.toLowerCase();

        Object valueObject = null;

        Vector values = null;

        synchronized (headers) {

            valueObject = headers.get(lower);

            if (valueObject != null) {

                values = (Vector) valueObject;

                values.add(value);
            }
            else {

                values = new Vector();

                values.add(value);

                headers.put(lower, values);
            }
        }
    }

    /**
     *  Replace a header by name.
     *  <p>
     *    Normally, headers are added, but the <code>ServletResponse</code>
     *    &quot;set&quot; methods imply replacement with respect to its
     *    &quot;add&quot; methods.
     *  </p>
     *  @param  name    The name of the parameter.
     *  @param  value   The value to assign to the parameter.
     *  @since  Version 0.1 2005-01-21
     */
    public final void replaceHeader(final String name,
                                    final String value) {

        String lower = name.toLowerCase();

        Object valueObject = null;

        Vector values = null;

        synchronized (headers) {

            valueObject = headers.get(lower);

            if (valueObject != null) {

                headers.remove(lower);
            }

            values = new Vector();

            values.add(value);

            headers.put(lower, values);
        }
    }

    /**
     *  Check whether a named header value is stored.
     *  @param  name    The name of the parameter.
     *  @return <code>true</code> if a header exists with the given name.
     *  @since  Version 0.1 2005-01-21
     */
    public final boolean containsHeader(final String name) {

        String lower = name.toLowerCase();

        Object valueObject = null;

        synchronized (headers) {

            valueObject = headers.get(lower);
        }

        return (valueObject != null);
    }

    /**
     *  Set a date header by name.
     *  <p>
     *    All headers are stored as string literals.
     *  </p>
     *  @param  name    The name of the parameter.
     *  @param  date    The date value to assign to the parameter.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setDateHeader(final String name,
                                    final long date) {

        setHeader(name, String.valueOf(date));
    }

    /**
     *  Replace a date header by name.
     *  <p>
     *    All headers are stored as string literals.
     *  </p>
     *  @param  name    The name of the parameter.
     *  @param  date    The date value to assign to the parameter.
     *  @since  Version 0.1 2005-01-21
     */
    public final void replaceDateHeader(final String name,
                                        final long date) {

        replaceHeader(name, String.valueOf(date));
    }

    /**
     *  Set an integer header by name.
     *  <p>
     *    All headers are stored as string literals.
     *  </p>
     *  @param  name    The name of the parameter.
     *  @param  value   The <code>int</code>value to assign to the parameter.
     *  @since  Version 0.1 2005-01-21
     */
    public final void setIntHeader(final String name,
                                   final int value) {

        setHeader(name, String.valueOf(value));
    }

    /**
     *  Replace an integer header by name.
     *  <p>
     *    All headers are stored as string literals.
     *  </p>
     *  @param  name    The name of the parameter.
     *  @param  value   The <code>int</code>value to assign to the parameter.
     *  @since  Version 0.1 2005-01-21
     */
    public final void replaceIntHeader(final String name,
                                       final int value) {

        replaceHeader(name, String.valueOf(value));
    }

    /**
     *  Get all header names.
     *  @return All header names in lower case.
     *  @since  Version 0.1 2005-01-21
     */
    public final Enumeration getHeaderNames() {

        synchronized (headers) {

            int size = headers.size();

            if (size > 0) {

                return headers.keys();
            }
            else {

                return null;
            }
        }
    }

}
