package com.mkdoc.servlet;

/*
    Created:    2005-01-20
    Issued:     2005-01-20
    Modified:   2005-07-25

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;

import java.util.Collections;
import java.util.Enumeration;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Vector;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletInputStream;
import javax.servlet.ServletRequest;

/**
 *  A mock servlet request for testing.
 *  <p>
 *    Most fields are initially <code>null</code> and must be configured using
 *    the relevant mutators if their accessors are expected to be called. Some
 *    accessors issue fixed values. Most mutators do not have validation, so
 *    <code>null</code> values can generally be assigned.
 *  </p>
 *  @version    0.4 2005-07-22
 *  @author Philip Shaw
 */
public class MockServletRequest implements ServletRequest {

    /**
     *  The default server name, &quot;localhost&quot;.
     *  @since  Version 0.1 2005-01-20
     */
    private static final String SERVER_NAME = "localhost";

    /**
     *  Common service properties.
     *  @since  Version 0.2 2005-01-21
     */
    protected MockServiceProperties properties = new MockServiceProperties();

    /**
     *  The character encoding.
     *  @since  Version 0.1 2005-01-20
     */
    protected String encoding;

    /**
     *  The content length.
     *  @since  Version 0.1 2005-01-20
     */
    protected int length;

    /**
     *  The content type.
     *  @since  Version 0.1 2005-01-20
     */
    protected String contentType;

    /**
     *  The servlet input stream.
     *  @since  Version 0.1 2005-01-20
     */
    protected ServletInputStream inputStream;

    /**
     *  A latch for reading input.
     *  @since  Version 0.1 2005-01-20
     */
    protected boolean read = false;

    /**
     *  The locales.
     *  @since  Version 0.1 2005-01-20
     */
    protected List locales = new Vector();

    /**
     *  The remote address.
     *  @since  Version 0.1 2005-01-20
     */
    protected String remoteAddr;

    /**
     *  The remote host.
     *  @since  Version 0.1 2005-01-20
     */
    protected String remoteHost;

    /**
     *  A request dispatcher to issue.
     *  @since  Version 0.1 2005-01-20
     */
    protected RequestDispatcher dispatcher;

    /**
     *  The request protocol.
     *  @since  Version 0.1 2005-01-20
     */
    protected String protocol;

    /**
     *  The request scheme.
     *  @since  Version 0.1 2005-01-20
     */
    protected String scheme;

    /**
     *  The server name.
     *  @since  Version 0.1 2005-01-20
     */
    protected String serverName = SERVER_NAME;

    /**
     *  The server port.
     *  @since  Version 0.1 2005-01-20
     */
    protected int serverPort;

    /**
     *  Whether a secure request.
     *  @since  Version 0.1 2005-01-20
     */
    protected boolean secure;

    /**
     *  The real path.
     *  @since  Version 0.1 2005-01-20
     */
    protected String realPath;

    /**
     *  The query string.
     *  @since  Version 0.1 2005-01-20
     */
    protected StringBuffer queryString = new StringBuffer();

    /**
     *  A flag to latch the first query parameter.
     *  @since  Version 0.1 2005-01-20
     */
    protected boolean firstParam = true;

    /**
     *  The local port number.
     *  @since  Version 0.3 2005-07-07
     */
    protected int localPort;

    /**
     *  The remote port number.
     *  @since  Version 0.3 2005-07-07
     */
    protected int remotePort;

    /**
     *  The local IP address.
     *  @since  Version 0.3 2005-07-07
     */
    protected String localAddr;

    /**
     *  The local host name.
     *  @since  Version 0.3 2005-07-07
     */
    protected String localName;

    /**
     *  Get the content length.
     *  @return The bytte length of the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final int getContentLength() {

        return length;
    }

    /**
     *  Set the content length.
     *  @param  length  The byte length to set.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setContentLength(final int length) {

        this.length = length;
    }

    /**
     *  Get the content type.
     *  @return The content type of the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getContentType() {

        return contentType;
    }

    /**
     *  Set the content type.
     *  @param  contentType The content type to set.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setContentType(final String contentType) {

        this.contentType = contentType;
    }

    /**
     *  Get the protocol.
     *  @return The protocol element of the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getProtocol() {

        return protocol;
    }

    /**
     *  Set the protocol.
     *  @param  protocol    The protocol name.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setProtocol(final String protocol) {

        this.protocol = protocol;
    }

    /**
     *  Get the scheme.
     *  @return The scheme part of the request URL.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getScheme() {

        return scheme;
    }

    /**
     *  Set the scheme.
     *  @param  scheme  The scheme part of the request URL.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setScheme(final String scheme) {

        this.scheme = scheme;
    }

    /**
     *  Get the server name.
     *  @return The server name.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getServerName() {

        return serverName;
    }

    /**
     *  Set the server name.
     *  @param  serverName  The server name or host for the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setServerName(final String serverName) {

        this.serverName = serverName;
    }

    /**
     *  Get the server port.
     *  @return The port number of the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final int getServerPort() {

        return serverPort;
    }

    /**
     *  Set the server port.
     *  @param  serverPort  The port number of the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setServerPort(final int serverPort) {

        this.serverPort = serverPort;
    }

    /**
     *  Get the remote address.
     *  @return The remote address of the request client.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getRemoteAddr() {

        return remoteAddr;
    }

    /**
     *  Set the remote address.
     *  @param  remoteAddr  The remote address of the request client.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setRemoteAddr(final String remoteAddr) {

        this.remoteAddr = remoteAddr;
    }

    /**
     *  Get the remote host.
     *  @return The host name of the request client, which may fall back to the
     *          remote address.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getRemoteHost() {

        if (remoteHost == null) {

            return remoteAddr;
        }
        else {

            return remoteHost;
        }
    }

    /**
     *  Set the remote host.
     *  @param  remoteHost  The remote host name.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setRemoteHost(final String remoteHost) {

        this.remoteHost = remoteHost;
    }

    /**
     *  Get the real path, always returns the same stored value.
     *  @param  path    The public path of a resource.
     *  @return The real path assignment for the request. The current version
     *          has no regard for the path argument.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getRealPath(final String path) {

        return realPath;
    }

    /**
     *  Set the real path.
     *  @param  realPath   The real path to assign for any given request.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setRealPath(final String realPath) {

        this.realPath = realPath;
    }

    /**
     *  Get the request input stream.
     *  @return An input stream for the request.
     *  @exception  IOException If the stream has already been opened.
     *  @since  Version 0.1 2005-01-20
     */
    public final ServletInputStream getInputStream() throws IOException {

        if (read) {

            throw new IOException("Already read from this request.");
        }
        else {

            read = true;

            return inputStream;
        }
    }

    /**
     *  Set the servlet input stream.
     *  @param  inputStream An input stream to assign to the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setInputStream(final ServletInputStream inputStream) {

        this.inputStream = inputStream;
    }

    /**
     *  Get a parameter value by name.
     *  @param  name    The parameter name.
     *  @return The parameter value or <code>null</code> if not matched.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getParameter(final String name) {

        return properties.getParameter(name);
    }

    /**
     *  Set a parameter value by name.
     *  @param  name    The parameter name.
     *  @param  value   The value of the parameter.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setParameter(final String name,
                                   final String value) {

        properties.setParameter(name, value);

        if (firstParam) {

            queryString.append("?");

            firstParam = false;
        }
        else {

            queryString.append("&");
        }
        queryString.append(name);
        queryString.append("=");
        queryString.append(value);
    }


    /**
     *  Get a parameter map.
     *  @return A mapping of the request parameter names and values.
     *  @since  Version 0.1 2005-01-20
     */
    public final Map getParameterMap() {

        return properties.getParameterMap();
    }

    /**
     *  Get an array of parameter values by name.
     *  @param  name    The parameter name.
     *  @return An array of values for the given parameter name, or
     *          <code>null</code> if not matched.
     *  @since  Version 0.1 2005-01-20
     */
    public final String[] getParameterValues(final String name) {

        return properties.getParameterValues(name);
    }

    /**
     *  Get the parameter names.
     *  @return All parameter names.
     *  @since  Version 0.1 2005-01-20
     */
    public final Enumeration getParameterNames() {

        return properties.getParameterNames();
    }

    /**
     *  Get an attribute value by name.
     *  @param  name    The attribute name.
     *  @return The object stored with the given name key.
     *  @since  Version 0.1 2005-01-20
     */
    public final Object getAttribute(final String name) {

        return properties.getAttribute(name);
    }

    /**
     *  Set an attribute value by name.
     *  @param  name    The attribute name.
     *  @param  value   The object to store with the attribute name.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setAttribute(final String name,
                                   final Object value) {

        properties.setAttribute(name, value);
    }

    /**
     *  Get the attribute names.
     *  @return All the attribute names.
     *  @since  Version 0.1 2005-01-20
     */
    public final Enumeration getAttributeNames() {

        return properties.getAttributeNames();
    }

    /**
     *  Remove an attribute value by name.
     *  @param  name    The name of the attribute to remove.
     *  @since  Version 0.1 2005-01-20
     */
    public final void removeAttribute(final String name) {

        properties.removeAttribute(name);
    }

    /**
     *  Get the locale (first in the list of locales).
     *  @return The primary locale.
     *  @since  Version 0.1 2005-01-20
     */
    public final Locale getLocale() {

        if (locales.size() > 0) {

            return (Locale) locales.get(0);
        }
        else {

            return null;
        }
    }

    /**
     *  Set an attribute value by name.
     *  @param  locale  A locale to add to the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setLocale(final Locale locale) {

        locales.add(locale);
    }

    /**
     *  Get all locales.
     *  @return All locales.
     *  @since  Version 0.1 2005-01-20
     */
    public final Enumeration getLocales() {

        return Collections.enumeration(locales);
    }

    /**
     *  Get a buffered reader of the servlet request.
     *  @return The servlet input stream wrapped in a buffered reader.
     *  @exception  IOException If the input stream has already been opened.
     *  @since  Version 0.1 2005-01-20
     */
    public final BufferedReader getReader() throws IOException {

        return new BufferedReader(new InputStreamReader(getInputStream()));
    }

    /**
     *  Get the character encoding.
     *  @return The character encoding of the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getCharacterEncoding() {

        return encoding;
    }

    /**
     *  Set the character encoding.
     *  @param  encoding    The character encoding to set.
     *  @exception  UnsupportedEncodingException    Never thrown by this
     *          implementation.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setCharacterEncoding(final String encoding)
        throws UnsupportedEncodingException {

        this.encoding = encoding;
    }

    /**
     *  Check if a secure request.
     *  @return Whether the request is a secure one.
     *  @since  Version 0.1 2005-01-20
     */
    public final boolean isSecure() {

        return secure;
    }

    /**
     *  Set the secure status.
     *  @param  secure  Whether the request is a secure one.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setSecure(final boolean secure) {

        this.secure = secure;
    }

    /**
     *  Get a request dispatcher, only takes a pre-configured dispatcher path.
     *  @param  path    The resource path to associate with the dispatcher. Not
     *          used in the current implementation, the path must be assigned
     *          with the dispatcher.
     *  @return A pre-configured request dispatcher.
     *  @since  Version 0.1 2005-01-20
     */
    public final RequestDispatcher getRequestDispatcher(final String path) {

        if (dispatcher == null) {

            return new MockRequestDispatcher(path);
        }
        else {

            return dispatcher;
        }
    }

    /**
     *  Set the request dispatcher.
     *  @param  dispatcher  A dispatcher instance to assign to the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setRequestDispatcher(final RequestDispatcher dispatcher) {

        this.dispatcher = dispatcher;
    }

    /**
     *  Get the local port number
     *  @return The local port number for the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final int getLocalPort() {

        return localPort;
    }

    /**
     *  Set the local port number
     *  @param  port    The local port number for the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setLocalPort(final int port) {

        localPort = port;
    }

    /**
     *  Get the remote port number
     *  @return The remote port number for the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final int getRemotePort() {

        return remotePort;
    }

    /**
     *  Set the remote port number
     *  @param  port    The remote port number for the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setRemotePort(final int port) {

        remotePort = port;
    }

    /**
     *  Get the local IP address.
     *  @return The local IP address for the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getLocalAddr() {

        return localAddr;
    }

    /**
     *  Set the local port number
     *  @param  addr    The local port number for the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setLocalAddr(final String addr) {

        localAddr = addr;
    }

    /**
     *  Get the local host name.
     *  @return The local host name for the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final String getLocalName() {

        return localName;
    }

    /**
     *  Set the local host name.
     *  @param  name    The local host name for the request.
     *  @since  Version 0.1 2005-01-20
     */
    public final void setLocalName(final String name) {

        localName = name;
    }



}
