package com.mkdoc.servlet;

/*
    Created:    2005-10-04
    Issued:     2005-10-04
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import com.mkdoc.util.ApplicationContext;
import com.mkdoc.util.ApplicationContextException;

import java.io.File;

/**
 *  Test suite for <code>ServletApplicationContext</code>.
 *  @version    0.3 2005-10-28
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class ServletApplicationContextTest extends TestCase {

    /**
     *  The home directory property.
     *  @since  Version 0.2 2005-10-19
     */
    public static final String HOME = System.getProperty("mkHome");

    /**
     *  The test directory name property.
     *  @since  Version 0.2 2005-10-19
     */
    public static final String TEST_DIR = System.getProperty("testDir");

    /**
     *  The RDF directory name property.
     *  @since  Version 0.2 2005-10-19
     */
    public static final String RDF_DIR = System.getProperty("rdfDir");

    /**
     *  The base test directory path.
     *  @since  Version 0.2 2005-10-19
     */
    public static final String BASE_PATH = HOME +
                                           File.separator +
                                           TEST_DIR;

    /**
     *  The default servlet RDF store directory.
     *  @since  Version 0.2 2005-10-19
     */
    public static final String STORE_DIR = File.separator +
                                            "WEB-INF" +
                                            File.separator +
                                            RDF_DIR;

    /**
     *  The repository store file name.
     *  @since  Version 0.2 2005-10-19
     */
    public static final String STORE_FILE = System.getProperty("storeFile");

    /**
     *  The servlet repository store file path.
     *  @since  Version 0.2 2005-10-19
     */
    public static final String STORE_PATH = STORE_DIR +
                                            File.separator +
                                            STORE_FILE;

    /**
     *  The absolute path to the test repository.
     *  @since  Version 0.2 2005-10-19
     */
    public static final String FULL_PATH = BASE_PATH + STORE_PATH;

    /**
     *  A servlet context to test.
     *  @since  Version 0.2 2005-10-19
     */
    protected MockServletContext context;

    /**
     *  An application context to test.
     *  @since  Version 0.2 2005-10-19
     */
    protected ApplicationContext appContext;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-10-04
     */
    public ServletApplicationContextTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-10-04
     */
    protected final void setUp() {

        context = new MockServletContext();

        context.setBasePath(BASE_PATH);

        context.setInitParameter(ApplicationContext.REPOSITORY, STORE_FILE);

        appContext = new ServletApplicationContext(context);
    }

    /**
     *  Test get parameter.
     *  @since  Version 0.2 2005-10-19
     */
    public final void testGetStringParameterPresent() {

        String param = appContext.getStringParameter(
                                                ApplicationContext.REPOSITORY);

        assertNotNull(param);

        assertTrue(param.equals(STORE_FILE));

    }

    /**
     *  Test get parameter.
     *  @since  Version 0.2 2005-10-19
     */
    public final void testGetStringParameterNotPresent() {

        String param = appContext.getStringParameter("Scotch-Mist");

        assertNull(param);

    }

    /**
     *  Test get parameter.
     *  @since  Version 0.2 2005-10-19
     */
    public final void testGetBooleanParameterPresent() {

        String name = "test";

        context.setInitParameter(name, "true");

        boolean param = appContext.getBooleanParameter(name);

        assertTrue(param);
    }

    /**
     *  Test get parameter.
     *  @since  Version 0.2 2005-10-19
     */
    public final void testGetBooleanParameterNotPresent() {

        boolean param = appContext.getBooleanParameter("Scotch-Mist");

        assertFalse(param);
    }

    /**
     *  Test get parameter.
     *  @since  Version 0.2 2005-10-19
     */
    public final void testGetClassParameterPresent() {

        String name = "test";

        String className = "java.lang.StringBuffer";

        context.setInitParameter(name, className);

        Class param = appContext.getClassParameter(name);

        assertNotNull(param);

        String testClass = param.getName();

        assertTrue(testClass.equals(className));
    }

    /**
     *  Test get parameter.
     *  @since  Version 0.2 2005-10-19
     */
    public final void testGetClassParameterNotPresent() {

        Class param = appContext.getClassParameter("Scotch-Mist");

        assertNull(param);
    }

    /**
     *  Test get parameter.
     *  @since  Version 0.2 2005-10-19
     */
    public final void testGetClassParameterInvalid() {

        Class param = appContext.getClassParameter(
                                                ApplicationContext.REPOSITORY);

        assertNull(param);
    }



    /**
     *  Test constructor with a null argument.
     *  @since  Version 0.2 2005-10-19
     */
    public final void testConstructorNullContext() {

        try {

            new ServletApplicationContext(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test get store path.
     *  @since  Version 0.1 2005-10-04
     */
    public final void testGetStorePath() {

        String storePath = appContext.getStorePath();

        assertNotNull(storePath);

        File expectedFile = new File(FULL_PATH);

        String expected = expectedFile.getAbsolutePath();

        assertTrue("Store path should equal '" +
                   expected + "', actual path was '" +
                   storePath + "'.",
                   storePath.equals(expected));
    }

    /**
     *  Test get store path.
     *  @since  Version 0.1 2005-10-04
     */
    public final void testGetStorePathNoWebAppBase() {

        context = new MockServletContext();

        context.setInitParameter(ApplicationContext.REPOSITORY, STORE_FILE);

        ServletApplicationContext badContext = new ServletApplicationContext(
                                                                      context);

        try {

            badContext.getStorePath();

            fail("Should throw an ApplicationContextException.");
        }
        catch (ApplicationContextException ace) { }
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-10-04
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-10-04
     */
    public static Test suite() {

        return new TestSuite(ServletApplicationContextTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-10-04
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
