package com.mkdoc.sesame;

/*
    Created:    2005-01-13
    Issued:     2005-01-13
    Modified:   2005-05-12

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.model.Graph;
import org.openrdf.model.Resource;
import org.openrdf.model.Statement;
import org.openrdf.model.URI;
import org.openrdf.model.Value;

import org.openrdf.model.impl.GraphImpl;

import org.openrdf.sesame.admin.AdminListener;

import org.openrdf.sesame.config.AccessDeniedException;

import org.openrdf.sesame.constants.QueryLanguage;
import org.openrdf.sesame.constants.RDFFormat;

import org.openrdf.sesame.query.MalformedQueryException;
import org.openrdf.sesame.query.GraphQueryResultListener;
import org.openrdf.sesame.query.QueryResultsTable;
import org.openrdf.sesame.query.TableQueryResultListener;

import org.openrdf.sesame.repository.SesameRepository;
import org.openrdf.sesame.repository.local.LocalRepository;
import org.openrdf.sesame.repository.local.LocalService;

import org.openrdf.sesame.sail.RdfSource;
import org.openrdf.sesame.sail.StatementIterator;

import com.mkdoc.io.MockInputStream;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.IOException;
import java.io.Reader;

import java.net.URL;

/**
 *  A mock <code>SesameRepository</code> type for testing.
 *  <p>
 *    Includes a simple system for triggering checked
 *    exceptions by class name.
 *  </p>
 *  @version    0.4 2005-05-12
 *  @author     Philip Shaw <phil@mkdoc.com>
 */
public class MockSesameRepository extends LocalRepository
                              implements SesameRepository {

    /**
     *  The <code>IOException</code> class name.
     *  @since  Version 0.1 2005-01-13
     */
    public static final String IOEXCEPTION = "IOException";

    /**
     *  The <code>MalformedQueryException</code> class
     *  name.
     *  @since  Version 0.1 2005-01-13
     */
    public static final String MALFORMED = "MalformedQueryException";

    /**
     *  The <code>AccessDeniedException</code> class name.
     *  @since  Version 0.1 2005-01-13
     */
    public static final String DENIED = "AccessDeniedException";

    /**
     *  An exception class name to throw.
     *  @since  Version 0.1 2005-01-13
     */
    private String exception = null;

    /**
     *  Store the most recent graph added.
     *  @since  Version 0.1 2005-01-13
     */
    private Graph graph;

    /**
     *  Whether the repository is shut down.
     *  @since  Version 0.1 2005-01-13
     */
    protected boolean shutDown = false;

    /**
     *  Public constructor.
     *  @param  storeId A repository ID reference.
     *  @param  rdfSource   An RDF source for the repository.
     *  @param  service A local service reference for the repository.
     *  @since  Version 0.3 2005-03-29
     */
    public MockSesameRepository(final String storeId,
                                final RdfSource rdfSource,
                                final LocalService service) {

        super(storeId, rdfSource, service);
    }

    /**
     *  Specify an exception to be thrown where relevant.
     *  @param  exception   The un-qualified class name for
     *          the exception to throw.
     *  @since  Version 0.1 2005-01-13
     */
    public final void setException(final String exception) {

        this.exception = exception;
    }

    /**
     *  Evaluates a query and reports the results as rows
     *  in a table to the supplied listener.
     *  <p>
     *    <em>This implementation does nothing.</em>
     *  </p>
     *  @param  language    The query language in which the
     *          query is formulated.
     *  @param  query   The query.
     *  @param  listener    The result listener.
     *  @exception  IOException In case an I/O error occurs
     *          while sending the query to, or retrieving
     *          the result from a Sesame server.
     *  @exception  MalformedQueryException If the supplied
     *          query is malformed.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have read access on
     *          this repository.
     *  @since  Version 0.1 2005-01-13
     */
    public final void performTableQuery(
                                    final QueryLanguage language,
                                    final String query,
                                    final TableQueryResultListener listener)
        throws IOException, MalformedQueryException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(MALFORMED)) {

                throw new MalformedQueryException(
                            "MockSesameRepository throws " +
                            "MalformedQueryException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        // Empty
    }

    /**
     *  Evaluates a query and returns the results in a
     *  <code>QueryResultsTable</code>.
     *  <p>
     *    <em>This implementation issues a table with zero
     *    columns.</em>
     *  </p>
     *  @param  language    The query language in which the
     *          query is formulated.
     *  @param  query   The query.
     *  @return A new results table for the query.
     *  @exception  IOException In case an I/O error occurs
     *          while sending the query to, or retrieving
     *          the result from a Sesame server.
     *  @exception  MalformedQueryException If the supplied
     *          query is malformed.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have read access on
     *          this repository.
     *  @since  Version 0.1 2005-01-13
     */
    public final QueryResultsTable performTableQuery(
                                                final QueryLanguage language,
                                                final String query)
        throws IOException, MalformedQueryException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(MALFORMED)) {

                throw new MalformedQueryException(
                            "MockSesameRepository throws " +
                            "MalformedQueryException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        return new QueryResultsTable(0);
    }

    /**
     *  Evaluates a graph query and reports the resulting
     *  statements to the specified listener.
     *  <p>
     *    <em>This implementation does nothing.</em>
     *  </p>
     *  @param  language    The query language in which the
     *          query is formulated. Currently, only SeRQL
     *          supports graph queries and
     *          <code>QueryLanguage.SERQL</code> is the
     *          only legal value.
     *  @param  query   The query.
     *  @param  listener    A listener for the query
     *          result.
     *  @exception  IOException In case an I/O error occurs
     *          while sending the query to, or retrieving
     *          the result from a Sesame server.
     *  @exception  MalformedQueryException If the supplied
     *          query is malformed.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have read access on
     *          this repository.
     *  @since  Version 0.1 2005-01-13
     */
    public final void performGraphQuery(
                                    final QueryLanguage language,
                                    final String query,
                                    final GraphQueryResultListener listener)
        throws IOException, MalformedQueryException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(MALFORMED)) {

                throw new MalformedQueryException(
                            "MockSesameRepository throws " +
                            "MalformedQueryException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        // Empty
    }

    /**
     *  Evaluates a graph query and returns the resulting
     *  statements.
     *  <p>
     *    <em>This implementation issues a graph with zero
     *    statements.</em>
     *  </p>
     *  @param  language    The query language in which the
     *          query is formulated. Currently, only SeRQL
     *          supports graph queries and
     *          <code>QueryLanguage.SERQL</code> is the
     *          only legal value.
     *  @param  query   The query.
     *  @return A new result graph for the query.
     *  @exception  IOException In case an I/O error occurs
     *          while sending the query to, or retrieving
     *          the result from a Sesame server.
     *  @exception  MalformedQueryException If the supplied
     *          query is malformed.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have read access on
     *          this repository.
     *  @since  Version 0.1 2005-01-13
     */
    public final Graph performGraphQuery(final QueryLanguage language,
                                         final String query)
        throws IOException, MalformedQueryException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(MALFORMED)) {

                throw new MalformedQueryException(
                            "MockSesameRepository throws " +
                            "MalformedQueryException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        // Empty
        return new GraphImpl();
    }

    /**
     *  Adds the RDF data from the specified URL.
     *  <p>
     *    <em>This implementation does nothing.</em>
     *  </p>
     *  @param  dataURL The URL of the RDF data.
     *  @param  baseURI The base URI to resolve any
     *          relative URIs against. This defaults to the
     *          value of dataURL if the value is set to
     *          null.
     *  @param  format  The serialization format of the
     *          data (one of <code>RDFFormat.RDFXML</code>,
     *          <code>RDFFormat.NTRIPLES</code>).
     *  @param  verifyData  If <code>true</code>, the RDF
     *          parser will verify the data before adding
     *          it to the repository.
     *  @param  listener    An <code>AdminListener</code>
     *          that will receive error and progress
     *          messages.
     *  @exception  IOException In case an I/O error
     *          occurs.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have write access on
     *          this repository.
     *  @since  Version 0.1 2005-01-13
     */
    public final void addData(final URL dataURL,
                              final String baseURI,
                              final RDFFormat format,
                              final boolean verifyData,
                              final AdminListener listener)
        throws IOException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        // Empty
    }

    /**
     *  Adds RDF data from the specified file to the
     *  repository.
     *  <p>
     *    <em>This implementation does nothing.</em>
     *  </p>
     *  @param  dataFile    A file containing RDF data.
     *  @param  baseURI The base URI to resolve any
     *          relative URIs against.
     *  @param  format  The serialization format of the
     *          data (one of <code>RDFFormat.RDFXML</code>,
     *          <code>RDFFormat.NTRIPLES</code>).
     *  @param  verifyData  If <code>true</code>, the RDF
     *          parser will verify the data before adding
     *          it to the repository.
     *  @param  listener    An <code>AdminListener</code>
     *          that will receive error and progress
     *          messages.
     *  @exception  IOException In case an I/O error
     *          occurs, including file not found.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have write access on
     *          this repository.
     *  @since  Version 0.1 2005-01-13
     */
    public final void addData(final File dataFile,
                              final String baseURI,
                              final RDFFormat format,
                              final boolean verifyData,
                              final AdminListener listener)
        throws IOException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals("FileNotFoundException")) {

                throw new FileNotFoundException(
                            "MockSesameRepository throws " +
                            "FileNotFoundException.");
            }
            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        // Empty
    }

    /**
     *  Adds an RDF document to the repository.
     *  <p>
     *    <em>This implementation does nothing.</em>
     *  </p>
     *  @param  data    An RDF document.
     *  @param  baseURI The base URI to resolve any
     *          relative URIs against.
     *  @param  format  The serialization format of the
     *          data (one of <code>RDFFormat.RDFXML</code>,
     *          <code>RDFFormat.NTRIPLES</code>).
     *  @param  verifyData  If <code>true</code>, the RDF
     *          parser will verify the data before adding
     *          it to the repository.
     *  @param  listener    An <code>AdminListener</code>
     *          that will receive error and progress
     *          messages.
     *  @exception  IOException In case an I/O error
     *          occurs.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have write access on
     *          this repository.
     *  @since  Version 0.1 2005-01-13
     */
    public final void addData(final String data,
                              final String baseURI,
                              final RDFFormat format,
                              final boolean verifyData,
                              final AdminListener listener)
        throws IOException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        // Empty
    }

    /**
     *  Adds the contents of the supplied repository.
     *  <p>
     *    <em>This implementation does nothing.</em>
     *  </p>
     *  @param  repository  The source repository.
     *  @param  listener    An <code>AdminListener</code>
     *          that will receive error and progress
     *          messages.
     *  @exception  IOException - In case an I/O error
     *          occurs.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have write access on
     *          this repository.
     *  @since  Version 0.1 2005-01-13
     */
    public final void addData(final SesameRepository repository,
                              final AdminListener listener)
        throws IOException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        // Empty
    }

    /**
     *  Adds RDF data from a <code>Reader</code>.
     *  <p>
     *    <em>This implementation does nothing.</em>
     *  </p>
     *  @param  data    A Reader from which RDF data can be
     *          read.
     *  @param  baseURI The base URI to resolve any
     *          relative URIs against.
     *  @param  format  The serialization format of the
     *          data (one of <code>RDFFormat.RDFXML</code>,
     *          <code>RDFFormat.NTRIPLES</code>).
     *  @param  verifyData  If <code>true</code>, the RDF
     *          parser will verify the data before adding
     *          it to the repository.
     *  @param  listener    An <code>AdminListener</code>
     *          that will receive error and progress messages.
     *  @exception  IOException In case an I/O error occurs.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have write access
     *          on this repository.
     *  @since  Version 0.1 2005-01-13
     */
    public final void addData(final Reader data,
                              final String baseURI,
                              final RDFFormat format,
                              final boolean verifyData,
                              final AdminListener listener)
        throws IOException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        // Empty
    }

    /**
     *  Adds RDF data from an InputStream.
     *  <p>
     *    <em>This implementation does nothing.</em>
     *  </p>
     *  @param  data    An InputStream from which RDF data
     *          can be read.
     *  @param  baseURI The base URI to resolve any
     *          relative URIs against.
     *  @param  format  The serialization format of the
     *          data (one of <code>RDFFormat.RDFXML</code>,
     *          <code>RDFFormat.NTRIPLES</code>).
     *  @param  verifyData  If <code>true</code>, the RDF
     *          parser will verify the data before adding
     *          it to the repository.
     *  @param  listener    An <code>AdminListener</code> that
     *          will receive error and progress messages.
     *  @exception  IOException In case an I/O error occurs.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have write access on
     *          this repository.
     */
    public final void addData(final InputStream data,
                              final String baseURI,
                              final RDFFormat format,
                              final boolean verifyData,
                              final AdminListener listener)
        throws IOException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        // Empty
    }

    /**
     *  Adds the contents of a graph.
     *  <p>
     *    <em>This implementation does nothing.</em>
     *  </p>
     *  @param  graph   The <code>Graph</code> statements to be
     *          added.
     *  @exception  IOException In case an I/O error occurs.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have write access on
     *          this repository.
     *  @since  Version 0.1 2005-01-13
     */
    public final void addGraph(final Graph graph)
        throws IOException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        this.graph = graph;
    }

    /**
     *  Get the most recently added graph.
     *  @return The most recently added <code>Graph</code>.
     *  @since  Version 0.1 2005-01-13
     */
    public final Graph getGraph() {

        return graph;
    }

    /**
     *  Get the most recently added graph.
     *  @return A line per statement list of the most
     *          recently added <code>Graph</code>.
     *  @since  Version 0.1 2005-01-13
     */
    public final String getGraphString() {

        StatementIterator iterator = graph.getStatements();

        StringBuffer buffer = new StringBuffer();

        Statement statement = null;

        Resource subject = null;

        URI predicate = null;

        Value object = null;

        while (iterator.hasNext()) {

            statement = iterator.next();

            subject = statement.getSubject();

            predicate = statement.getPredicate();

            object = statement.getObject();

            buffer.append("Subject: ");
            buffer.append(subject);
            buffer.append(", Predicate: ");
            buffer.append(predicate);
            buffer.append(", Object: ");
            buffer.append(object);
            buffer.append(".\n");
        }

        return buffer.toString();
    }

    /**
     *  Adds the graph result of a query to the repository.
     *  <p>
     *    <em>This implementation does nothing.</em>
     *  </p>
     *  @param  language    The query language in which the
     *          query is formulated.
     *  @param  query   The query. Note that the query must
     *          be of a form that has an RDF graph as its
     *          result (e.g. a SeRQL CONSTRUCT query).
     *  @exception  IOException In case an I/O error occurs.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have write access on
     *          this repository.
     *  @since  Version 0.1 2005-01-13
     */
    public final void addGraph(final QueryLanguage language,
                               final String query)
        throws IOException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        // Empty
    }

    /**
     *  Removes the contents of graph from the repository.
     *  <p>
     *    <em>This implementation does nothing.</em>
     *  </p>
     *  @param  graph   A <code>Graph</code> containing the
     *          RDF statements to be removed.
     *  @exception  IOException In case an I/O error occurs.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have write access on
     *          this repository.
     *  @since  Version 0.1 2005-01-13
     */
    public final void removeGraph(final Graph graph)
        throws IOException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        // Empty
    }

    /**
     *  Removes the graph result of a query from the
     *  repository.
     *  <p>
     *    <em>This implementation does nothing.</em>
     *  </p>
     *  @param  language    The query language in which
     *          the query is formulated.
     *  @param  query   The query. Note that the query must
     *          be of a form that has an RDF graph as its
     *          result (e.g. a SeRQL CONSTRUCT query).
     *  @exception  IOException In case an I/O error occurs.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have write access on
     *          this repository.
     *  @since  Version 0.1 2005-01-13
     */
    public final void removeGraph(final QueryLanguage language,
                                  final String query)
        throws IOException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        // Empty
    }

    /**
     *  Return the content of the repository in the
     *  requested format.
     *  <p>
     *    <em>This implementation returns an empty input
     *    stream.</em>
     *  </p>
     *  @param  format  The serialization format that
     *          should be used (one of
     *          <code>RDFFormat.RDFXML</code>,
     *          <code>RDFFormat.NTRIPLES</code>,
     *          <code>RDFFormat.N3</code>).
     *  @param  ontology    If <code>true</code>, the
     *          ontological part of the data will be
     *          extracted.
     *  @param  instances   If <code>true</code>, the
     *          instances part of the data will be
     *          extracted.
     *  @param  explicitOnly    If <code>true</code>, only
     *          explicitely added statements will be
     *          extracted, i.e. the inferred statements
     *          will be skipped.
     *  @param  niceOutput  If <code>true</code>, the
     *          server will try its best to make the data
     *          as much readable for humans as possible.
     *  @return An input stream of the repository contents
     *          in the specified format.
     *  @exception  IOException In case an I/O error occurs.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have read access on
     *          this repository.
     *  @since  Version 0.1 2005-01-13
     */
    public final InputStream extractRDF(final RDFFormat format,
                                        final boolean ontology,
                                        final boolean instances,
                                        final boolean explicitOnly,
                                        final boolean niceOutput)
        throws IOException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        return new MockInputStream();
    }

    /**
     *  Removes statements about a resource from the repository.
     *  <p>
     *    <em>This implementation does nothing.</em>
     *  </p>
     *  @param  subject The subject of the statements that
     *          should be removed, or <code>null</code> if
     *          the subject is allowed to be anything.
     *  @param  predicate   The predicate of the statements
     *          that should be removed, or
     *          <code>null</code> if the predicate is
     *          allowed to be anything.
     *  @param  object  The object of the statements that
     *          should be removed, or <code>null</code> if
     *          the object is allowed to be anything.
     *  @param  listener    An <code>AdminListener</code>
     *          that will receive error and progress
     *          messages.
     *  @exception  IOException In case an I/O error occurs.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have write access on
     *          this repository.
     *  @since  Version 0.1 2005-01-13
     */
    public final void removeStatements(final Resource subject,
                                       final URI predicate,
                                       final Value object,
                                       final AdminListener listener)
        throws IOException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        // Empty
    }

    /**
     *  Clears the repository. All statements will be
     *  removed.
     *  <p>
     *    <em>This implementation does nothing.</em>
     *  </p>
     *  @param  listener    An <code>AdminListener</code>
     *          that will receive error- and progress
     *          messages.
     *  @exception  IOException In case an I/O error
     *          occurs.
     *  @exception  AccessDeniedException   In case the
     *          current user does not have write access
     *          on this repository.
     *  @since  Version 0.1 2005-01-13
     */
    public final void clear(final AdminListener listener)
        throws IOException, AccessDeniedException {

        if (exception != null) {

            if (exception.equals(IOEXCEPTION)) {

                throw new IOException(
                            "MockSesameRepository throws " +
                            "IOException.");
            }
            if (exception.equals(DENIED)) {

                throw new AccessDeniedException(
                            "MockSesameRepository throws " +
                            "AccessDeniedException.");
            }
        }
        // Empty
    }

    /**
     *  Shutdown the repository.
     *  @since  Version 0.2 2005-03-15
     */
    public final void shutDown() {

        shutDown = true;
    }

    /**
     *  Check whether the repository is shut down.
     *  @return <code>true</code> if the <code>shutDown</code> method has been
     *          called.
     *  @since  Version 0.2 2005-03-15
     */
    public final boolean isShutDown() {

        return shutDown;
    }

    /**
     *  Get the repository ID.
     *  @return A unique identifier for the repository, currently the hash code
     *          for the instance.
     *  @since  Version 0.3 2005-03-29
     */
    public final String getRepositoryId() {

        return String.valueOf(hashCode());
    }

}
