package com.mkdoc.store;

/*
    Created:    2005-05-10
    Issued:     2005-05-10
    Modified:   2005-10-21

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.jspider.MockPropertySet;
import com.mkdoc.jspider.PluginApplicationContext;

import com.mkdoc.util.MockApplicationContext;
import com.mkdoc.util.ApplicationContext;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

import java.net.URL;
import java.net.MalformedURLException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>AbstractFileStoreManager</code>.
 *  @version    0.6 2005-10-21
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class AbstractFileStoreManagerTest extends TestCase {

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.1 2005-05-10
     */
    public static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  A test range of bytes to write.
     *  @since  Version 0.1 2005-05-10
     */
    public static final int BYTE_RANGE = 256;

    /**
     *  A mock file system manager to test.
     *  @since  Version 0.1 2005-05-10
     */
    protected MockFileStoreManager manager;

    /**
     *  A mock file system manager to test.
     *  @since  Version 0.1 2005-05-10
     */
    protected MockFileStoreManager managerInit;

    /**
     *  The project source directory.
     *  @since  Version 0.1 2005-05-10
     */
    protected static final String HOME = System.getProperty("mkHome");

    /**
     *  A test domain.
     *  @since  Version 0.1 2005-05-10
     */
    protected static final String DOMAIN = System.getProperty("testDomain");

    /**
     *  The HTTP protocol scheme for failure reports.
     *  @since  Version 0.6 2005-10-21
     */
    protected static final String HTTP = "http://";

    /**
     *  The &quot;actual output path&quot; for failure reports.
     *  @since  Version 0.6 2005-10-21
     */
    public static final String ACTUAL = "actual output path = '";

    /**
     *  A comma and new line for failure reports.
     *  @since  Version 0.6 2005-10-21
     */
    public static final String COMMA_NL = "',\n";

    /**
     *  A URL without file extension.
     *  @since  Version 0.1 2005-05-10
     */
    protected static final String URL_NO_EXT = HTTP +
                                               DOMAIN +
                                               "/";

    /**
     *  A URL with file extension.
     *  @since  Version 0.1 2005-05-10
     */
    protected static final String URL_EXT = HTTP +
                                            DOMAIN +
                                            "/test.html";

    /**
     *  The domain root trimmed.
     *  @since  Version 0.1 2005-05-10
     */
    protected static final String ROOT_TRIM = "/";

    /**
     *  A URL with file extension trimmed.
     *  @since  Version 0.1 2005-05-10
     */
    protected static final String EXT_TRIM = "/test";

    /**
     *  The default JSpider output directory.
     *  @since  Version 0.1 2005-05-10
     */
    protected static final String OUTPUT = "output";

    /**
     *  A custom output directory.
     *  @since  Version 0.1 2005-05-10
     */
    protected static final String CUSTOM = "custom";

    /**
     *  The default JSpider output path.
     *  @since  Version 0.1 2005-05-10
     */
    protected static final String PATH_OUT = HOME +
                                             File.separator +
                                             OUTPUT;
    /**
     *  An index source file.
     *  @since  Version 0.1 2005-05-10
     */
    protected static final String INDEX = "/index.html";

    /**
     *  A temporary output file.
     *  @since  Version 0.1 2005-05-10
     */
    protected static final String INDEX_TMP = "index.tmp";

    /**
     *  A test domain URL.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String URL_TEST = HTTP +
                                             DOMAIN +
                                             INDEX;

    /**
     *  A bad test URL.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String URL_BAD = HTTP +
                                             DOMAIN +
                                             "/\\:*?\"<>|.BAD";


    /**
     *  A test file path.
     *  @since  Version 0.1 2004-12-17
     */
    protected static final String PATH_TEST = PATH_OUT +
                                              File.separator +
                                              DOMAIN +
                                              File.separator +
                                              INDEX_TMP;

    /**
     *  The custom output path.
     *  @since  Version 0.1 2005-05-10
     */
    protected static final String PATH_CUSTOM = HOME +
                                                File.separator +
                                                OUTPUT +
                                                File.separator +
                                                CUSTOM;

    /**
     *  A <code>File</code> representation of the custom
     *  output directory for comparisons.
     *  @since  Version 0.1 2005-05-10
     */
    protected static final File DIR_CUSTOM = new File(PATH_CUSTOM);

    /**
     *  A <code>File</code> representation of the default
     *  output path for comparisons.
     *  @since  Version 0.1 2005-05-10
     */
    protected static final File DIR_OUT = new File(PATH_OUT);

    /**
     *  A file reference to the test output directory.
     *  @since  Version 0.1 2004-12-16
     */
    protected static final File DIR_TEST = new File(HOME +
                                                    File.separator +
                                                    OUTPUT);

    /**
     *  A file reference for invalid output file.
     *  @since  Version 0.1 2004-12-16
     */
    protected static final File FILE_OUT = new File(DIR_TEST,
                                                    File.separator +
                                                    DOMAIN +
                                                    "/index.tmp");

    /**
     *  A basic application context.
     *  @since  Version 0.1 2004-12-16
     */
    protected ApplicationContext context;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-05-10
     */
    public AbstractFileStoreManagerTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-05-10
     */
    protected final void setUp() {

        manager = new MockFileStoreManager();

        managerInit = new MockFileStoreManager();

        context = new MockApplicationContext(new File(PATH_OUT));

        managerInit.configureOutput(context);
    }

    /**
     *  Purge statements by subject URI.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  StoreException  Should not be thrown by this test.
     *  @since  Version 0.4 2005-05-12
     */
    public final void testPurgeDocumentFile()
        throws IOException, StoreException {

        manager.configureOutput(context);

        manager.createDirectories(FILE_OUT);

        if ((!FILE_OUT.exists()) &&
            (!FILE_OUT.createNewFile())) {

            fail("Could not create the test file, '" +
                 FILE_OUT.getAbsolutePath() + "'.");
        }

        FileOutputStream outStream = null;

        try {

            outStream = new FileOutputStream(FILE_OUT);

            for (int i = 0; i < BYTE_RANGE; i++) {

                outStream.write(i);
            }
        }
        catch (IOException ioe) {

            fail("Could not write test file, '" +
                 FILE_OUT.getAbsolutePath() + "'.");
        }
        finally {

            outStream.close();
        }

        assertTrue("Test file should exist but does not, '" +
                    FILE_OUT.getAbsolutePath() + "'.",
                    FILE_OUT.exists());

        File testFile = manager.getStorageFile(new URL(URL_TEST));

        String testPath = testFile.getAbsolutePath();

        assertTrue("Storage file path should equal the test file but does " +
                   "not , manager's storage file path was '" +
                   testPath + "', test file path is '" +
                   FILE_OUT.getAbsolutePath() + "'.",
                   testPath.equals(FILE_OUT.getAbsolutePath()));


        manager.purgeDocumentFile(URL_TEST);

        assertFalse("Should have deleted the file but has not, '" +
                    "file path was '" +
                    FILE_OUT.getAbsolutePath() + "', URL was '" +
                    URL_TEST + "'.",
                    FILE_OUT.exists());
    }

    /**
     *  Purge statements by subject URI.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  StoreException  Should not be thrown by this test.
     *  @since  Version 0.4 2005-05-12
     */
    public final void testPurgeDocumentFileNotStored()
        throws IOException, StoreException {

        manager.configureOutput(context);

        if ((FILE_OUT.exists()) &&
            (!FILE_OUT.delete())) {

            fail("Could not delete the test file, '" +
                 FILE_OUT.getAbsolutePath() + "'.");
        }

        assertFalse("Test file should not exist but does, '" +
                    FILE_OUT.getAbsolutePath() + "'.",
                    FILE_OUT.exists());

        manager.purgeDocumentFile(URL_TEST);

        // No side effects, no assertion
    }

    /**
     *  Purge statements with null subject URI.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  StoreException  Should not be thrown by this test.
     *  @since  Version 0.4 2005-05-12
     */
    public final void testPurgeDocumentFileNullSubjectUri()
        throws IOException, StoreException {

        try {

            manager.purgeDocumentFile(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Bad URL means was not stored in first place.
     *  @exception  StoreException  Should not be thrown by this test.
     *  @since  Version 0.4 2005-05-12
     */
    public final void testPurgeDocumentFileBadUrl()
        throws StoreException {

        try {

            manager.purgeDocumentFile("bogus-protocol://phil@mkdoc.com");

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }


    /**
     *  Configure output should run successfully with a default context.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-05-10
     */
    public final void testConfigureOutputDefaultContext()
        throws IOException {

        manager.configureOutput(context);

        String outPath = manager.outputDir.getCanonicalPath();

        assertTrue("Should use default output directory, '" +
                   DIR_OUT.getCanonicalPath() + COMMA_NL +
                   ACTUAL +
                   outPath + "'",
                   outPath.equals(DIR_OUT.getCanonicalPath()));
    }

    /**
     *  With absolute file path but no path specification,
     *  should try the default output directory.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-05-10
     */
    public final void testConfigureOutputAbsolutePathNullPath()
        throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, true);

        context = new PluginApplicationContext(propertySet);

        manager.configureOutput(context);

        String outPath = manager.outputDir.getCanonicalPath();

        assertTrue("Should use the default output directory, '" +
                   DIR_OUT.getCanonicalPath() + COMMA_NL +
                   ACTUAL +
                   outPath + "'",
                   outPath.equals(DIR_OUT.getCanonicalPath()));
    }

    /**
     *  With absolute file path and a valid path specification,
     *  should use the specified path.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-05-10
     */
    public final void testConfigureOutputAbsolutePathWithPath()
        throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, true);
        propertySet.setString(AbstractFileStoreManager.FOLDER, PATH_CUSTOM);

        context = new PluginApplicationContext(propertySet);

        manager.configureOutput(context);

        String outPath = manager.outputDir.getCanonicalPath();

        assertTrue("Should use test output directory, '" +
                   DIR_CUSTOM.getCanonicalPath() + COMMA_NL +
                   ACTUAL +
                   outPath + "'",
                   outPath.equals(DIR_CUSTOM.getCanonicalPath()));
    }

    /**
     *  With relative file path but no path specification,
     *  should try the default output directory.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-05-10
     */
    public final void testConfigureOutputRelativePathNullPath()
        throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, false);

        context = new PluginApplicationContext(propertySet);

        manager.configureOutput(context);

        String outPath = manager.outputDir.getCanonicalPath();

        assertTrue("Should use the default output directory, '" +
                   DIR_OUT.getCanonicalPath() + COMMA_NL +
                   ACTUAL +
                   outPath + "'",
                   outPath.equals(DIR_OUT.getCanonicalPath()));
    }

    /**
     *  With relative file path and a valid path specification,
     *  should use the specified path.
     *  @exception  IOException In check for canonical
     *          path, or read write operations.
     *  @since  Version 0.1 2005-05-10
     */
    public final void testConfigureOutputRelativePathWithPath()
        throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setBoolean(AbstractFileStoreManager.ABSOLUTE, false);
        propertySet.setString(AbstractFileStoreManager.FOLDER, CUSTOM);

        context = new PluginApplicationContext(propertySet);

        manager.configureOutput(context);

        String outPath = manager.outputDir.getCanonicalPath();

        assertTrue("Should use test output directory, '" +
                   DIR_CUSTOM.getCanonicalPath() + COMMA_NL +
                   ACTUAL +
                   outPath + "'",
                   outPath.equals(DIR_CUSTOM.getCanonicalPath()));
    }

    /**
     *  Configure output should not accept a null argument.
     *  @since  Version 0.1 2005-05-10
     */
    public final void testConfigureOutputNullContext() {

        try {

            manager.configureOutput(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Get an output stream and check it is attached to a file.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  StoreException Should not be thrown by this test.
     *  @since  Version 0.1 2005-05-10
     */
    public final void testGetOutputStream()
        throws IOException, StoreException {

        File testFile = new File(PATH_TEST);

        if (testFile.exists()) {

            File parent = testFile.getParentFile();

            if (!testFile.delete()) {

                fail("Could not delete test file, " +
                     testFile.getAbsolutePath() + ".");
            }
            if (!parent.delete()) {

                fail("Could not delete test directory, " +
                     parent.getAbsolutePath() + ".");
            }
        }

        assertFalse(testFile.exists());

        URL url = new URL(URL_TEST);

        OutputStream outStream = managerInit.getOutputStream(url);

        try {

            for (int i = 0; i < BYTE_RANGE; i++) {

                outStream.write(i);
            }
        }
        catch (IOException ioe) {

            fail("Should not throw an IOException " +
                 ioe.getMessage() + ".");
        }
        finally {

            outStream.close();
        }

        assertTrue(testFile.exists());

        if (!testFile.delete()) {

            fail("Could not delete test file, " +
                 testFile.getAbsolutePath() + ".");
        }
    }

    /**
     *  Get an output stream and throw an exception.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  StoreException Should not be thrown by this test.
     *  @since  Version 0.1 2005-05-10
     */
    public final void testGetOutputStreamThrowsException()
        throws IOException, StoreException {

        URL url = new URL(URL_BAD);

        try {

            managerInit.getOutputStream(url);

            fail("Should throw a StoreException.");
        }
        catch (StoreException se) { }

    }


    /**
     *  Get an output file for a subject URL without file part.
     *  @exception  MalformedURLException   Should not be thrown by this test.
     *  @since  Version 0.1 2005-05-10
     */
    public final void testGetStorageFileWithoutExtension()
        throws MalformedURLException {

        File expected = new File(PATH_TEST);

        URL url = new URL(URL_NO_EXT);

        File file = managerInit.getStorageFile(url);

        String path = file.getAbsolutePath();

        assertTrue("Storage file path should be '" +
                   expected.getAbsolutePath() + ", actual path was '" +
                   path + "'.",
                   path.equals(expected.getAbsolutePath()));

    }

    /**
     *  If URL has a dot file extension, should return
     *  <code>true</code>.
     *  @exception  MalformedURLException   If the test URL
     *          is malformed, should not be thrown.
     *  @since  Version 0.1 2005-05-10
     */
    public final void testIncludesFileTrue() throws MalformedURLException {

        URL url = new URL(URL_EXT);

        assertTrue(manager.includesFile(url));
    }

    /**
     *  If URL has no dot file extension, should return
     *  <code>false</code>.
     *  @exception  MalformedURLException   If the test URL
     *          is malformed, should not be thrown.
     *  @since  Version 0.1 2005-05-10
     */
    public final void testIncludesFileFalse() throws MalformedURLException {

        URL url = new URL(URL_NO_EXT);

        assertFalse(manager.includesFile(url));
    }

    /**
     *  Domain root URL should truncate to just "/".
     *  @exception  MalformedURLException   If the test URL
     *          is malformed, should not be thrown.
     *  @since  Version 0.1 2005-05-10
     */
    public final void testTrimExtensionNoExtension()
        throws MalformedURLException {

        URL url = new URL(URL_NO_EXT);

        String trimmed = manager.trimExtension(url);

        assertTrue("Trimmed root URL should equal '" + ROOT_TRIM + "', " +
                   "actually '" + trimmed + "'.",
                   trimmed.equals(ROOT_TRIM));

    }

    /**
     *  Should truncate to file part without the extension.
     *  @exception  MalformedURLException   If the test URL
     *          is malformed, should not be thrown.
     *  @since  Version 0.1 2004-12-16
     */
    public final void testTrimExtensionWithExtension()
        throws MalformedURLException {

        URL url = new URL(URL_EXT);

        String trimmed = manager.trimExtension(url);

        assertTrue("Trimmed file URL should equal '" + EXT_TRIM + "', " +
                   "actually '" + trimmed + "'.",
                   trimmed.equals(EXT_TRIM));

    }

    /**
     *  If parent does not exist, should create.
     *  @since  Version 0.1 2005-05-10
     */
    public final void testCreateDirectoriesNotExist() {

        File parent = FILE_OUT.getParentFile();

        if ((parent.exists()) &&
            (!parent.delete())) {

            fail("Could not delete parent directory, " +
                 parent.getAbsolutePath() + ".");
        }

        assertFalse("Parent directory should not exist but does, '" +
                    parent.getAbsolutePath() + "'.",
                    parent.exists());

        managerInit.createDirectories(FILE_OUT);

        assertTrue("Parent directory should exist but does not, '" +
                   parent.getAbsolutePath() + "'.",
                   parent.exists());

        if (!parent.delete()) {

            fail("Could not delete parent directory, " +
                 parent.getAbsolutePath() + ".");
        }
    }

    /**
     *  Create directories null file.
     *  @since  Version 0.1 2005-05-10
     */
    public final void testCreateDirectoriesNullFile() {

        try {

            managerInit.createDirectories(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Create directories null parent.
     *  @since  Version 0.1 2005-05-10
     */
    public final void testCreateDirectoriesNullParent() {

        File file = new File("NoParent.txt");

        try {

            managerInit.createDirectories(file);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  If parent already exists, should return immediately.
     *  @since  Version 0.1 2005-05-10
     */
    public final void testCreateDirectoriesParentExists() {

        managerInit.createDirectories(FILE_OUT);

        File parent = FILE_OUT.getParentFile();

        assertTrue(parent.exists());

        long modified1 = parent.lastModified();

        managerInit.createDirectories(FILE_OUT);

        long modified2 = parent.lastModified();

        assertTrue(modified1 == modified2);

        if (!parent.delete()) {

            fail("Could not delete parent directory, " +
                 parent.getAbsolutePath() + ".");
        }
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-05-10
    protected void tearDown() throws IOException {


    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-05-10
     */
    public static Test suite() {

        return new TestSuite(AbstractFileStoreManagerTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-05-10
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
