package com.mkdoc.store;

/*
    Created:    2005-05-11
    Issued:     2005-05-11
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.model.Graph;
import org.openrdf.model.URI;
import org.openrdf.model.Value;
import org.openrdf.model.ValueFactory;

import org.openrdf.model.impl.GraphImpl;

import org.openrdf.sesame.config.AccessDeniedException;

import org.openrdf.sesame.constants.QueryLanguage;

import org.openrdf.sesame.query.QueryResultsTable;
import org.openrdf.sesame.query.MalformedQueryException;
import org.openrdf.sesame.query.QueryEvaluationException;

import org.openrdf.sesame.repository.SesameRepository;

import com.mkdoc.io.MockInputStream;

import com.mkdoc.jspider.AbstractMKDocPlugin;
import com.mkdoc.jspider.MockPropertySet;

import com.mkdoc.query.SimpleQuery;
import com.mkdoc.query.QueryResult;
import com.mkdoc.query.NullQueryResult;
import com.mkdoc.query.XhtmlQueryContext;
import com.mkdoc.query.Query;

import com.mkdoc.sesame.MockSesameRepository;
import com.mkdoc.sesame.MockLocalService;
import com.mkdoc.sesame.MockRdfSource;

import com.mkdoc.util.ApplicationContext;
import com.mkdoc.util.ApplicationContextException;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import java.net.URL;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>LocalStoreManager</code>.
 *  @version    0.5 2005-11-01
 */
public class LocalStoreManagerTest extends TestCase {

    /**
     *  An expected <code>IllegalArgumentException</code> message.
     *  @since  Version 0.3 2005-06-09
     */
    protected static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  A repository manager to test.
     *  @since  Version 0.1 2005-05-11
     */
    protected LocalStoreManager manager;

    /**
     *  A repository manager to test.
     *  @since  Version 0.1 2005-05-11
     */
    protected LocalStoreManager managerInit;

    /**
     *  A home directory path.
     *  @since  Version 0.1 2005-05-11
     */
    public static final String HOME = System.getProperty("mkHome");

    /**
     *  A test directory name.
     *  @since  Version 0.1 2005-05-11
     */
    public static final String TEST_DIR = System.getProperty("testDir");

    /**
     *  An RDF directory name.
     *  @since  Version 0.1 2005-05-11
     */
    public static final String RDF_DIR = System.getProperty("rdfDir");

    /**
     *  The Sesame repository file in the RDF directory.
     *  @since  Version 0.5 2005-11-01
     */
    protected static final String STORE = System.getProperty("storeFile");

    /**
     *  An RDF test file.
     *  @since  Version 0.1 2005-05-11
     */
    public static final String RDF_FILE = "minimal.rdf";

    /**
     *  A test SeRQL query for the minimal RDF input file.
     *  @since  Version 0.3 2005-06-06
     */
    public static final String TEST_QUERY =
    "SELECT * FROM {subject} <dc:contributor> {object} " +
    "USING NAMESPACE dc = <http://purl.org/dc/elements/1.1/> ";

    /**
     *  The RDF test file path.
     *  @since  Version 0.1 2005-05-11
     */
    public static final String RDF_PATH = HOME +
                                          File.separator +
                                          TEST_DIR +
                                          File.separator +
                                          RDF_DIR +
                                          File.separator +
                                          RDF_FILE;

    /**
     *  The Sesame repository file path.
     *  @since  Version 0.4 2005-07-19
     */
    protected static final String STORE_PATH = File.separator +
                                               RDF_DIR +
                                               File.separator +
                                               STORE;

    /**
     *  The default JSpider output directory name.
     *  @since  Version 0.1 2005-05-11
     */
    protected static final String OUTPUT = "output";

    /**
     *  The default JSpider output path.
     *  @since  Version 0.1 2005-05-11
     */
    protected static final String OUT_PATH = HOME +
                                             File.separator +
                                             OUTPUT;

    /**
     *  The store file path.
     *  @since  Version 0.4 2005-07-19
     */
    protected static final String BAD_PATH = OUT_PATH +
                                             File.separator +
                                             "bogus-not-exist.duff";

    /**
     *  A test subject URL.
     *  @since  Version 0.1 2005-05-12
     */
    protected static final String TEST_URL = "http://www.remote.example/";


    /**
     *  The default page index.
     *  @since  Version 0.5 2005-11-01
     */
    public static final int PAGE_INDEX = 1;

    /**
     *  The default results per page limit.
     *  <p>
     *    Low to make &quot;extreme&quot; paging possible with fewer results.
     *  </p>
     *  @since  Version 0.5 2005-11-01
     */
    public static final int RESULT_LIMIT = 2;

    /**
     *  An SeRQL query.
     *  @since  Version 0.5 2005-11-01
     */
    public static final String QUERY_STRING =
    "SELECT DISTINCT targetUri, createdDate, dcSubject, targetTitle, " +
    "targetSummary FROM {targetUri} dcterms:created {createdDate}, " +
    "     {targetUri} dc:subject {dcSubject}," +
    "     [ {targetUri} dc:title {targetTitle} ], " +
    "     [ {targetUri} dc:description {targetSummary} ] " +
    "WHERE (label(dcSubject) LIKE \"GCJ\" OR " +
    "       label(dcSubject) LIKE \"JSpider\") " +
    "AND label(createdDate) LIKE \"2005-*\" " +
    "UNION " +
    "SELECT DISTINCT targetUri, createdDate, dcSubject, targetTitle,  " +
    "targetSummary FROM {targetUri} dcterms:created {} rdf:value  " +
    "{createdDate}, {targetUri} dc:subject {dcSubject}, " +
    "     [ {targetUri} dc:title {targetTitle} ], " +
    "     [ {targetUri} dc:description {targetSummary} ] " +
    "WHERE (label(dcSubject) LIKE \"GCJ\" OR " +
    "       label(dcSubject) LIKE \"JSpider\") " +
    "AND label(createdDate) LIKE \"2005-*\" " +
    "USING NAMESPACE dc = <http://purl.org/dc/elements/1.1/>, " +
    "            dcterms = <http://purl.org/dc/terms/> ";

    /**
     *  A result key for testing.
     *  @since  Version 0.5 2005-11-01
     */
    public static final String RESULT_KEY =
    "http://localhost/HttpQuery?dcterms.created=2005-*&amp;dc.subject=" +
    "GCJ+JSpider";

    /**
     *  An XHTML query context.
     *  @since  Version 0.5 2005-11-01
     */
    protected XhtmlQueryContext queryContext;

    /**
     *  A query object.
     *  @since  Version 0.5 2005-11-01
     */
    protected Query query;

    /**
     *  A graph implementation.
     *  @since  Version 0.1 2005-05-11
     */
    protected Graph graph;

    /**
     *  A value factory.
     *  @since  Version 0.1 2005-05-11
     */
    protected ValueFactory factory;

    /**
     *  The store file path.
     *  @since  Version 0.1 2005-05-11
     */
    protected static final String FILE_PATH =
                                  OUT_PATH +
                                  File.separator +
                                  "com.mkdoc.store.LocalStoreManager" +
                                  ".rdf";

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-05-11
     */
    public LocalStoreManagerTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-05-11
     */
    protected final void setUp() {

        graph = new GraphImpl();

        factory = graph.getValueFactory();

        manager = new LocalStoreManager();

        managerInit = new LocalStoreManager();

        MockPropertySet propertySet = new MockPropertySet();

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        managerInit.initialize(context);


        queryContext = new XhtmlQueryContext(PAGE_INDEX,
                                             RESULT_LIMIT);

        query = new SimpleQuery(QUERY_STRING,
                                RESULT_KEY,
                                queryContext);
    }

    /**
     *  Test the execute query method.
     *  @since  Version 0.5 2005-11-01
     */
    public final void testExecuteQueryNullQuery() {

        try {

            managerInit.executeQuery(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test the execute query method.
     *  @since  Version 0.5 2005-11-01
     */
    public final void testExecuteQuerySuccessful() {

        QueryResult result = managerInit.executeQuery(query);

        assertNotNull(result);

        assertFalse(result instanceof NullQueryResult);
    }

    /**
     *  Test the execute query method.
     *  @since  Version 0.5 2005-11-01
     */
    public final void testExecuteQueryException() {

        managerInit.shutdown();

        assertFalse(managerInit.isActive());

        QueryResult result = managerInit.executeQuery(query);

        assertNotNull(result);

        assertTrue(result instanceof NullQueryResult);
    }




    /**
     *  Configure repository with null property set.
     *  @since  Version 0.5 2005-11-01
     */
    public final void testConfigureRepositoryNullContext() {

        try {

            manager.configureRepository(null);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Configure repository with a repository file.
     *  @since  Version 0.5 2005-11-01
     */
    public final void testConfigureRepositoryBadPath() {

        MockPropertySet propertySet = new MockPropertySet();

        propertySet.setString(StoreManager.REPOSITORY,
                              BAD_PATH);

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        try {

            manager.configureRepository(context);

            fail("Should throw an ApplicationContextException.");
        }
        catch (ApplicationContextException ace) { }

    }

    /**
     *  Configure repository with a repository file.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.5 2005-11-01
     */
    public final void testConfigureRepositoryGoodPath() throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        File testFile = new File(HOME +
                                 File.separator +
                                 TEST_DIR +
                                 File.separator +
                                 RDF_DIR +
                                 File.separator +
                                 STORE);

        propertySet.setString(StoreManager.REPOSITORY,
                              STORE_PATH);

        propertySet.setBoolean(ApplicationContext.ABSOLUTE,
                               true);

        propertySet.setString(ApplicationContext.DIRECTORY,
                              HOME +
                              File.separator +
                              TEST_DIR);

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        String storePath = context.getStorePath();

        assertTrue("Context store path should equal the test file path." +
                   "Context store path was " +
                   storePath +
                   ", test file path was " +
                   testFile.getAbsolutePath() +
                   ".",
                   storePath.equals(testFile.getAbsolutePath()));

        String message = manager.configureRepository(context);

        String expected = testFile.getAbsolutePath();


        assertTrue("Should use specified repository, '" +
                   STORE_PATH +
                   "', actual message was '" +
                   message + "'.",
                   message.indexOf(expected) > -1);

    }


    /**
     *  Configure repository should run successfully with empty
     *  property set.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.5 2005-11-01
     */
    public final void testConfigureRepositoryEmptyPropertySet()
        throws IOException {

        MockPropertySet propertySet = new MockPropertySet();

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        assertNull(manager.repository);

        manager.configureOutput(context);

        manager.configureRepository(context);

        assertNotNull(manager.repository);
    }

    /**
     *  Configure repository, check store file path.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.5 2005-11-01
     */
    public final void testConfigureRepository()
        throws IOException {

        File expected = new File(FILE_PATH);

        MockPropertySet propertySet = new MockPropertySet();

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        manager.configureOutput(context);

        String message = manager.configureRepository(context);

        assertTrue("Should report store file path, '" +
                   expected.getAbsolutePath() + "'. Actual message was '" +
                   message + "'.",
                   message.indexOf(expected.getAbsolutePath()) > -1);
    }




    /**
     *  Minimal test for add XML RDF stream method, no RDF content.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  StoreException Should not be thrown by this test.
     *  @exception  MalformedQueryException Should not be thrown by this test.
     *  @exception  QueryEvaluationException    Should not be thrown by this
     *          test.
     *  @exception  AccessDeniedException   Should not be thrown by this test.
     *  @since  Version 0.2.1 2005-06-06
     */
    public final void testAddXmlRdfStreamNotRDF()
        throws IOException, StoreException, MalformedQueryException,
        QueryEvaluationException, AccessDeniedException {

        managerInit.addXmlRdfStream(new MockInputStream(),
                                    new URL(TEST_URL));

        SesameRepository repository = managerInit.repository;

        QueryResultsTable table = null;

        table = repository.performTableQuery(QueryLanguage.SERQL,
                                             TEST_QUERY);

        assertTrue("Results table should contain no rows, " +
                   "actually contained " +
                   table.getRowCount() + " rows.",
                   table.getRowCount() == 0);
    }

    /**
     *  Minimal test for add XML RDF stream method, no RDF content.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  StoreException Should not be thrown by this test.
     *  @exception  MalformedQueryException Should not be thrown by this test.
     *  @exception  QueryEvaluationException    Should not be thrown by this
     *          test.
     *  @exception  AccessDeniedException   Should not be thrown by this test.
     *  @since  Version 0.5 2005-11-01
     */
    public final void testAddXmlRdfStreamNotInitialized()
        throws IOException, StoreException, MalformedQueryException,
        QueryEvaluationException, AccessDeniedException {

        try {

            manager.addXmlRdfStream(new MockInputStream(),
                                    new URL(TEST_URL));

            fail("Should throw a StoreException.");
        }
        catch (StoreException se) { }

    }

    /**
     *  Minimal test for add XML RDF stream method, RDF content.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  StoreException Should not be thrown by this test.
     *  @exception  MalformedQueryException Should not be thrown by this test.
     *  @exception  QueryEvaluationException    Should not be thrown by this
     *          test.
     *  @exception  AccessDeniedException   Should not be thrown by this test.
     *  @since  Version 0.2.1 2005-06-06
     */
    public final void testAddXmlRdfStreamIsRDF()
        throws IOException, StoreException, MalformedQueryException,
        QueryEvaluationException, AccessDeniedException {

        File rdfFile = new File(RDF_PATH);

        FileInputStream inStream = new FileInputStream(rdfFile);

        managerInit.addXmlRdfStream(inStream,
                                    new URL(TEST_URL));


        SesameRepository repository = managerInit.repository;

        QueryResultsTable table = null;

        table = repository.performTableQuery(QueryLanguage.SERQL,
                                             TEST_QUERY);

        assertTrue("Results table should contain one row, " +
                   "actually contained " +
                   table.getRowCount() + " rows.",
                   table.getRowCount() == 1);
    }

    /**
     *  Minimal test for add XML RDF stream method, throws IOException.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  StoreException Should not be thrown by this test.
     *  @since  Version 0.2.1 2005-06-06
     */
    public final void testAddXmlRdfStreamThrowsIOException()
        throws IOException, StoreException {

        MockInputStream inStream = new MockInputStream();

        inStream.setFailOnRead(true);

        try {

            managerInit.addXmlRdfStream(inStream,
                                        new URL(TEST_URL));

            fail("Should throw a StoreException.");
        }
        catch (StoreException se) { }
    }


    /**
     *  Add an XML RDF stream, null stream.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  StoreException Should not be thrown by this test.
     *  @since  Version 0.2.1 2005-06-06
     */
    public final void testAddXmlRdfStreamNullStream()
        throws IOException, StoreException {

        try {

            managerInit.addXmlRdfStream(null,
                                        new URL(TEST_URL));

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Add an XML RDF stream, null URL.
     *  @exception  IOException Should not be thrown by this test.
     *  @exception  StoreException Should not be thrown by this test.
     *  @since  Version 0.2.1 2005-06-06
     */
    public final void testAddXmlRdfStreamNullUrl()
        throws IOException, StoreException {

        try {

            managerInit.addXmlRdfStream(new MockInputStream(),
                                        null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Initialize and check the store file path.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-05-11
     */
    public final void testInitialize() throws IOException {

        File expected = new File(OUT_PATH);

        MockPropertySet propertySet = new MockPropertySet();

        ApplicationContext context = null;

        context = AbstractMKDocPlugin.getApplicationContext(propertySet);

        assertNull(manager.outputDir);

        assertNull(manager.repository);


        manager.initialize(context);

        assertNotNull(manager.outputDir);

        String outPath = manager.outputDir.getAbsolutePath();

        assertTrue("Output directory path should be '" +
                   expected.getAbsolutePath() + "'. Actual path was '" +
                   outPath + "'.",
                   outPath.equals(expected.getAbsolutePath()));
    }

    /**
     *  Initialize and check the store file path.
     *  @exception  IOException Should not be thrown by this test.
     *  @since  Version 0.1 2005-05-11
     */
    public final void testInitializeNullPropertySet() throws IOException {

        try {

            manager.initialize(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }

    }

    /**
     *  Test shutdown not set up, should silently ignore.
     *  @since  Version 0.1 2005-05-11
     */
    public final void testShutdownNotInitialized() {

        assertFalse(manager.isActive());

        assertNull(manager.repository);

        manager.shutdown();

        assertFalse(manager.isActive());

    }

    /**
     *  Test add document statements, not set up.
     *  @exception  StoreException  Should not be thrown by this test.
     *  @since  Version 0.1 2005-05-11
     */
    public final void testAddDocumentStatementsSuccess()
        throws StoreException {

        String storeId = "LocalStoreManagerTest";

        MockRdfSource source = new MockRdfSource();

        MockLocalService service = new MockLocalService();


        MockSesameRepository repository = new MockSesameRepository(storeId,
                                                                   source,
                                                                   service);

        managerInit.repository = repository;


        URI subject = factory.createURI(TEST_URL);

        URI predicate = factory.createURI(
                                        "http://www.remote.example/predicate");

        Value object = factory.createLiteral("Test object");

        graph.add(subject, predicate, object);

        managerInit.addDocumentStatements(subject, graph);

        Graph testGraph = repository.getGraph();

        assertTrue(testGraph.contains(subject, predicate, object));
    }

    /**
     *  Test add document statements, not set up.
     *  @exception  StoreException  Should not be thrown by this test.
     *  @since  Version 0.1 2005-05-11
     */
    public final void testAddDocumentStatementsManagerThrowsException()
        throws StoreException {

        String storeId = "LocalStoreManagerTest";

        MockRdfSource source = new MockRdfSource();

        MockLocalService service = new MockLocalService();


        MockSesameRepository repository = new MockSesameRepository(storeId,
                                                                   source,
                                                                   service);

        repository.setException(MockSesameRepository.DENIED);


        managerInit.repository = repository;


        URI subject = factory.createURI(TEST_URL);

        URI predicate = factory.createURI(
                                        "http://www.remote.example/predicate");

        Value object = factory.createLiteral("Test object");

        graph.add(subject, predicate, object);

        try {

            managerInit.addDocumentStatements(subject, graph);

            fail("Should throw a StoreException.");
        }
        catch (StoreException se) { }

    }

    /**
     *  Test add document statements, not set up.
     *  @since  Version 0.1 2005-05-11
     */
    public final void testAddDocumentStatementsNotInitialized() {

        URI subject = factory.createURI(TEST_URL);

        try {

            manager.addDocumentStatements(subject, graph);

            fail("Should throw an StoreException.");
        }
        catch (StoreException se) { }
    }

    /**
     *  Test add document statements, null subject URI.
     *  @exception  StoreException  Should not be thrown by this test.
     *  @since  Version 0.1 2005-05-11
     */
    public final void testAddDocumentStatementsNullSubjectURI()
        throws StoreException {

        try {

            managerInit.addDocumentStatements(null, graph);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test add document statements, null graph.
     *  @exception  StoreException  Should not be thrown by this test.
     *  @since  Version 0.1 2005-05-11
     */
    public final void testAddDocumentStatementsNullGraph()
        throws StoreException {

        URI subject = factory.createURI(TEST_URL);

        try {

            managerInit.addDocumentStatements(subject, null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test shutdown should shut down the repository.
     *  @since  Version 0.1 2005-05-11
     */
    public final void testShutdown() {

        String storeId = "LocalStoreManagerTest";

        MockLocalService service = new MockLocalService();

        MockRdfSource source = new MockRdfSource();

        MockSesameRepository repository = new MockSesameRepository(storeId,
                                                                   source,
                                                                   service);

        assertFalse(repository.isShutDown());

        manager.repository = repository;

        manager.shutdown();

        assertTrue(repository.isShutDown());

    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-05-11
     */
    public static Test suite() {

        return new TestSuite(LocalStoreManagerTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-05-11
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
