package com.mkdoc.store;

/*
    Created:    2005-10-26
    Issued:     2005-10-26
    Modified:   2005-10-26

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import org.openrdf.model.Value;

import org.openrdf.model.impl.LiteralImpl;
import org.openrdf.model.impl.URIImpl;


import org.openrdf.sesame.query.QueryResultsTable;

import com.mkdoc.query.Query;

import java.util.List;
import java.util.Vector;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>StoreQueryResult</code>.
 *  @version    0.1 2005-10-26
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class StoreQueryResultTest extends TestCase {

    /**
     *  An expected IllegalArgumentException.
     *  @since  Version 0.1 2005-10-26
     */
    private static final String EXPECTED_IAE =
    "Should throw an IllegalArgumentException.";

    /**
     *  An array of basic column headings.
     *  @since  Version 0.1 2005-10-26
     */
    public static final String[] HEADERS = new String[] {Query.VAR_URI,
                                                         Query.VAR_SUMMARY,
                                                         Query.VAR_TITLE};

    /**
     *  A test URI.
     *  @since  Version 0.1 2005-10-26
     */
    public static final String URI =
    "http://www.remote.exaple/uri/StoreQueryResult";

    /**
     *  A test URI.
     *  @since  Version 0.1 2005-10-26
     */
    public static final Value URI_OBJECT = new URIImpl(URI);

    /**
     *  A test summary.
     *  @since  Version 0.1 2005-10-26
     */
    public static final String SUMMARY =
    "This is a test URI, there is no content.";

    /**
     *  A test summary.
     *  @since  Version 0.1 2005-10-26
     */
    public static final Value SUMMARY_LITERAL = new LiteralImpl(SUMMARY);

    /**
     *  A test title.
     *  @since  Version 0.1 2005-10-26
     */
    public static final String TITLE = "Test document";

    /**
     *  A test title.
     *  @since  Version 0.1 2005-10-26
     */
    public static final Value TITLE_LITERAL = new LiteralImpl(TITLE);

    /**
     *  A test title.
     *  @since  Version 0.1 2005-10-26
     */
    protected static List row1;

    /**
     *  A query results table with which to instantiate the query result.
     *  @since  Version 0.1 2005-10-26
     */
    protected static QueryResultsTable table;

    /**
     *  A store query result to test.
     *  @since  Version 0.1 2005-10-26
     */
    protected static StoreQueryResult result;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-10-26
     */
    public StoreQueryResultTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-10-26
     */
    protected final void setUp() {

        table = new QueryResultsTable(HEADERS.length, HEADERS);

        row1 = new Vector();

        row1.add(URI_OBJECT);

        row1.add(SUMMARY_LITERAL);

        row1.add(TITLE_LITERAL);

        table.addRow(row1);

        result = new StoreQueryResult(table);
    }

    /**
     *  Test the constructor with a null results table.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testContructorNullResultsTable() {

        try {

            new StoreQueryResult(null);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test the getRows method.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testGetRows() {

        assertTrue(result.getRows() == 1);
    }

    /**
     *  Test the getColumns method.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testGetColumns() {

        assertTrue(result.getColumns() == HEADERS.length);
    }

    /**
     *  Test exceptions in with bad results rows.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testGetItemUriException() {

        QueryResultsTable badTable = new QueryResultsTable(HEADERS.length,
                                                           HEADERS);

        List badRow = new Vector();

        badRow.add(null);

        badRow.add(SUMMARY_LITERAL);

        badRow.add(TITLE_LITERAL);

        badTable.addRow(badRow);

        StoreQueryResult badResult = new StoreQueryResult(badTable);

        try {

            badResult.getItemTitle(0);

            badResult.getItemSummary(0);
        }
        catch (Exception e) {

            fail("Should not throw an exception getting title or summary.");
        }

        try {

            badResult.getItemUri(0);

            fail("Should throw an IllegalStateException.");
        }
        catch (IllegalStateException ise) { }
    }

    /**
     *  Test exceptions in with bad results rows.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testGetItemSummaryException() {

        QueryResultsTable badTable = new QueryResultsTable(HEADERS.length,
                                                           HEADERS);

        List badRow = new Vector();

        badRow.add(URI_OBJECT);

        badRow.add(null);

        badRow.add(TITLE_LITERAL);

        badTable.addRow(badRow);

        StoreQueryResult badResult = new StoreQueryResult(badTable);

        String summary = badResult.getItemSummary(0);

        assertTrue(summary.equals("No summary available for this item."));
    }

    /**
     *  Test correct result values.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testGetItemSummaryCorrect() {

        String summary = result.getItemSummary(0);

        assertTrue("Summary should equal '" +
                   SUMMARY + "', actual summary was '" +
                   summary + "'.",
                   summary.equals(SUMMARY));
    }

    /**
     *  Test correct result values.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testGetItemTitleCorrect() {

        String title = result.getItemTitle(0);

        assertTrue("Title should equal '" +
                   TITLE + "', actual title was '" +
                   title + "'.",
                   title.equals(TITLE));
    }

    /**
     *  Test correct result values.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testGetItemUriCorrect() {

        String uri = result.getItemUri(0);

        assertTrue("URI should equal '" +
                   URI + "', actual URI was '" +
                   uri + "'.",
                   uri.equals(URI));
    }

    /**
     *  Test exceptions in with bad results rows.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testGetItemTitleException() {

        QueryResultsTable badTable = new QueryResultsTable(HEADERS.length,
                                                           HEADERS);

        List badRow = new Vector();

        badRow.add(URI);

        badRow.add(SUMMARY);

        badRow.add(null);

        badTable.addRow(badRow);

        StoreQueryResult badResult = new StoreQueryResult(badTable);

        String title = badResult.getItemTitle(0);

        assertTrue(title.equals("No title available for this item."));
    }

    /**
     *  Test the getItemUri method.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testGetItemUriOutOfRange() {

        try {

            result.getItemUri(2);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test the getItemUri method.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testGetItemUriNegative() {

        try {

            result.getItemUri(-1);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test the getItemSummary method.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testGetItemSummaryOutOfRange() {

        try {

            result.getItemSummary(2);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test the getItemSummary method.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testGetItemSummaryNegative() {

        try {

            result.getItemSummary(-1);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test the getItemTitle method.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testGetItemTitleOutOfRange() {

        try {

            result.getItemTitle(2);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Test the getItemTitle method.
     *  @since  Version 0.1 2005-10-26
     */
    public final void testGetItemTitleNegative() {

        try {

            result.getItemTitle(-1);

            fail(EXPECTED_IAE);
        }
        catch (IllegalArgumentException iae) { }
    }





    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-10-26
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-10-26
     */
    public static Test suite() {

        return new TestSuite(StoreQueryResultTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-10-26
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
