package com.mkdoc.tag;

/*
    Created:    2005-10-18
    Issued:     2005-10-18
    Modified:   2005-11-01

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import com.mkdoc.jsp.MockPageContext;
import com.mkdoc.jsp.MockJspWriter;

import com.mkdoc.query.ResultRenderer;

import com.mkdoc.servlet.MockServletContext;
import com.mkdoc.servlet.MockHttpServletRequest;
import com.mkdoc.servlet.MockHttpServletResponse;

import com.mkdoc.store.StoreManagerFactory;
import com.mkdoc.store.StoreManager;

import com.mkdoc.util.ApplicationContext;

import java.io.File;

import javax.servlet.jsp.JspException;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>QueryResultTag</code>.
 *  @version    0.2 2005-11-02
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class QueryResultTagTest extends TestCase {

    /**
     *  The home directory.
     *  @since  Version 0.1 2005-10-18
     */
    protected static final String HOME = System.getProperty("mkHome");

    /**
     *  The test directory.
     *  @since  Version 0.1 2005-10-18
     */
    protected static final String TEST_DIR = System.getProperty("testDir");

    /**
     *  The RDF directory.
     *  @since  Version 0.1 2005-10-18
     */
    protected static final String RDF_DIR = System.getProperty("rdfDir");

    /**
     *  The RDF store file.
     *  @since  Version 0.1 2005-10-18
     */
    protected static final String STORE_FILE = System.getProperty("storeFile");

    /**
     *  The RDF directory path.
     *  @since  Version 0.1 2005-10-18
     */
    protected static final String RDF_PATH = HOME +
                                             File.separator +
                                             TEST_DIR +
                                             File.separator +
                                             RDF_DIR;


    /**
     *  The query manager class name.
     *  @since  Version 0.1 2005-10-18
     */
    protected static final String STORE_MANAGER =
    "com.mkdoc.store.LocalStoreManager";

    /**
     *  A bad query manager class name.
     *  @since  Version 0.1 2005-10-18
     */
    protected static final String BAD_MANAGER = "java.lang.Object";

    /**
     *  A DC subject parameter name.
     *  @since  Version 0.1 2005-10-19
     */
    public static final String DC_SUBJECT = "dc.subject";

    /**
     *  A DC subject search value.
     *  @since  Version 0.1 2005-10-19
     */
    public static final String SUBJECT_1 = "GCJ";

    /**
     *  An invalid ResultRenderer mode.
     *  @since  Version 0.1 2005-10-19
     */
    public static final int INVALID = 24;

    /**
     *  A null StoreManager for resetting the tag.
     *  @since  Version 0.1 2005-10-19
     */
    public static final StoreManager NULL_MANAGER = null;

    /**
     *  A tag to test.
     *  @since  Version 0.1 2005-10-18
     */
    protected QueryResultTag tag;

    /**
     *  A page context.
     *  @since  Version 0.1 2005-10-18
     */
    protected MockPageContext pageContext;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-10-18
     */
    public QueryResultTagTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-10-18
     */
    protected final void setUp() {

        tag = new QueryResultTag();

        pageContext = new MockPageContext();

        MockServletContext context = new MockServletContext();

        context.setInitParameter(ApplicationContext.ABSOLUTE, "true");
        context.setInitParameter(ApplicationContext.DIRECTORY, RDF_PATH);
        context.setInitParameter(ApplicationContext.REPOSITORY, STORE_FILE);
        context.setInitParameter(StoreManagerFactory.MANAGER_KEY,
                                 STORE_MANAGER);
        context.setBasePath(HOME);

        pageContext.setServletContext(context);

        tag.setPageContext(pageContext);
    }

    /**
     *  Test the doStartTag method.
     *  @exception  JspException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-10-19
     */
    public final void testDoStartTagSuccess() throws JspException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter(DC_SUBJECT, SUBJECT_1);

        pageContext.setRequest(msreq);


        MockHttpServletResponse msres = new MockHttpServletResponse();

        pageContext.setResponse(msres);


        MockJspWriter jspWriter = new MockJspWriter();

        pageContext.setOut(jspWriter);


        tag.doStartTag();

        int statusCode = msres.getStatusCode();

        assertTrue(statusCode == msres.SC_OK);
    }

    /**
     *  Test the doStartTag method.
     *  @exception  JspException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-10-19
     */
    public final void testDoStartTagNoQuery() throws JspException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        pageContext.setRequest(msreq);


        MockHttpServletResponse msres = new MockHttpServletResponse();

        pageContext.setResponse(msres);


        MockJspWriter jspWriter = new MockJspWriter();

        pageContext.setOut(jspWriter);


        tag.doStartTag();

        int statusCode = msres.getStatusCode();

        assertTrue(statusCode == msres.SC_BAD_REQUEST);
    }

    /**
     *  Test the doStartTag method.
     *  @exception  JspException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-10-19
     */
    public final void testDoStartTagNoQueryFailOnRedirect()
        throws JspException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        pageContext.setRequest(msreq);


        MockHttpServletResponse msres = new MockHttpServletResponse();

        msres.setFailOnRedirect(true);

        pageContext.setResponse(msres);


        MockJspWriter jspWriter = new MockJspWriter();

        pageContext.setOut(jspWriter);

        try {

            tag.doStartTag();

            fail("Should throw a JspException.");
        }
        catch (JspException jspe) { }

    }

    /**
     *  Test the doStartTag method.
     *  @exception  JspException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-10-19
     */
    public final void testDoStartTagFailOnWrite() throws JspException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter(DC_SUBJECT, SUBJECT_1);

        pageContext.setRequest(msreq);


        MockHttpServletResponse msres = new MockHttpServletResponse();

        pageContext.setResponse(msres);


        MockJspWriter jspWriter = new MockJspWriter();

        jspWriter.setFailOnPrint(true);

        pageContext.setOut(jspWriter);


        tag.doStartTag();

        int statusCode = msres.getStatusCode();

        assertTrue(statusCode == msres.SC_INTERNAL_SERVER_ERROR);
    }

    /**
     *  Test the doStartTag method.
     *  @exception  JspException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-10-19
     */
    public final void testDoStartTagFailOnWriteFailOnRedirect()
        throws JspException {

        MockHttpServletRequest msreq = new MockHttpServletRequest();

        msreq.setParameter(DC_SUBJECT, SUBJECT_1);

        pageContext.setRequest(msreq);


        MockHttpServletResponse msres = new MockHttpServletResponse();

        msres.setFailOnRedirect(true);

        pageContext.setResponse(msres);


        MockJspWriter jspWriter = new MockJspWriter();

        jspWriter.setFailOnPrint(true);

        pageContext.setOut(jspWriter);

        try {

            tag.doStartTag();

            fail("Should throw a JspException.");
        }
        catch (JspException jspe) { }

    }


    /**
     *  Test init without assigning page context.
     *  @exception  JspException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-10-18
     */
    public final void testInitNoServletContext() throws JspException {

        tag.manager = NULL_MANAGER;

        MockPageContext badPageContext = new MockPageContext();

        tag.setPageContext(badPageContext);

        try {

            tag.init();

            fail("Should throw an IllegalStateException.");
        }
        catch (IllegalStateException ise) { }
    }

    /**
     *  Test init with basic page context.
     *  @since  Version 0.1 2005-10-18
     */
    public final void testInitServletContextNotConfigured() {

        tag.manager = NULL_MANAGER;

        MockServletContext badContext = new MockServletContext();

        pageContext.setServletContext(badContext);

        try {

            tag.init();

            fail("Should throw a JspException.");
        }
        catch (JspException jspe) { }
    }

    /**
     *  Test init with basic page context.
     *  @exception  JspException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-10-18
     */
    public final void testInitSuccessful() throws JspException {

        tag.manager = NULL_MANAGER;

        try {

            tag.init();
        }
        catch (JspException jspe) {

            fail("Should not throw an exception. " +
                 jspe.getRootCause().getMessage());

        }
    }

    /**
     *  Test init with basic page context.
     *  @exception  JspException    Should not be thrown by this test.
     *  @since  Version 0.1 2005-10-18
     */
    public final void testInitBadRepositoryClass() throws JspException {

        tag.manager = NULL_MANAGER;

        MockServletContext badContext = new MockServletContext();

        badContext.setInitParameter(ApplicationContext.ABSOLUTE, "true");
        badContext.setInitParameter(ApplicationContext.DIRECTORY, RDF_PATH);
        badContext.setInitParameter(ApplicationContext.REPOSITORY, STORE_FILE);
        badContext.setInitParameter(StoreManagerFactory.MANAGER_KEY,
                                    BAD_MANAGER);
        badContext.setBasePath(HOME);

        pageContext.setServletContext(badContext);

        try {

            tag.init();

            fail("Should throw JspException.");
        }
        catch (JspException jspe) { }
    }

    /**
     *  Test the set and get mode methods.
     *  @since  Version 0.1 2005-10-19
     */
    public final void testSetGetModeFull() {

        int initialMode = tag.getMode();

        assertTrue(initialMode == ResultRenderer.FULL);

        tag.setMode(ResultRenderer.INSERT);

        int testMode = tag.getMode();

        assertTrue(testMode == ResultRenderer.INSERT);

        tag.setMode(ResultRenderer.FULL);

        testMode = tag.getMode();

        assertTrue(testMode == ResultRenderer.FULL);
    }

    /**
     *  Test the set and get mode methods.
     *  @since  Version 0.1 2005-10-19
     */
    public final void testSetModeInvalid() {

        try {

            tag.setMode(INVALID);

            fail("Should throw an IllegalArgumentException.");
        }
        catch (IllegalArgumentException iae) { }
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-10-18
     */
    public final void testSetGetExtended() {

        assertFalse(tag.getExtended());

        tag.setExtended(true);

        assertTrue(tag.getExtended());
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-10-18
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-10-18
     */
    public static Test suite() {

        return new TestSuite(QueryResultTagTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-10-18
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
