package com.mkdoc.util;

/*
    Created:    2005-10-18
    Issued:     2005-10-18
    Modified:   2005-10-28

    Author:     Philip Shaw <phil@mkdoc.com>
    Copyright:  (c) 2005 MKDoc Ltd.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA
*/

import java.io.File;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *  Test suite for <code>AbstractApplicationContext</code>.
 *  @version    0.2 2005-10-28
 *  @author Philip Shaw <phil@mkdoc.com>
 */
public class AbstractApplicationContextTest extends TestCase {

    /**
     *  The home directory.
     *  @since  Version 0.1 2005-10-18
     */
    protected static final String HOME = System.getProperty("mkHome");

    /**
     *  The output directory name.
     *  @since  Version 0.1 2005-10-18
     */
    protected static final String OUTPUT = System.getProperty("outputDir");

    /**
     *  The test directory name.
     *  @since  Version 0.1 2005-10-18
     */
    protected static final String TEST = System.getProperty("testDir");

    /**
     *  The output directory path.
     *  @since  Version 0.1 2005-10-18
     */
    protected static final String OUT_PATH = HOME +
                                             File.separator +
                                             OUTPUT;

    /**
     *  A non-default relative path.
     *  @since  Version 0.1 2005-10-18
     */
    protected static final String REL_PATH = OUT_PATH +
                                             File.separator +
                                             TEST;


    /**
     *  A non-default absolute path.
     *  @since  Version 0.1 2005-10-18
     */
    protected static final String ABS_PATH = HOME +
                                             File.separator +
                                             TEST;

    /**
     *  The output directory reference.
     *  @since  Version 0.1 2005-10-18
     */
    protected File defaultDir;

    /**
     *  A mock context to test.
     *  @since  Version 0.1 2005-10-18
     */
    protected MockApplicationContext context;

    /**
     *  Named test instance.
     *  @param  name    The name of the test case.
     *  @since  Version 0.1 2005-10-18
     */
    public AbstractApplicationContextTest(final String name) {

        super(name);
    }

    /**
     *  Set up a test scenario as required.
     *  @since  Version 0.1 2005-10-18
     */
    protected final void setUp() {

        defaultDir = new File(OUT_PATH);

        context = new MockApplicationContext(defaultDir);

    }

    /**
     *  Get the store directory.
     *  @since  Version 0.1 2005-10-18
     */
    public final void testGetStoreDirectoryDefault() {

        File testDir = context.getStoreDirectory();

        assertTrue(testDir.equals(defaultDir));
    }

    /**
     *  Get the store directory.
     *  @since  Version 0.1 2005-10-18
     */
    public final void testGetStoreDirectoryRelative() {

        context.setParameter(ApplicationContext.DIRECTORY, TEST);

        File testDir = context.getStoreDirectory();

        File expected = new File(REL_PATH);

        assertTrue(testDir.equals(expected));
    }

    /**
     *  Get the store directory.
     *  @since  Version 0.1 2005-10-18
     */
    public final void testGetStoreDirectoryAbsolute() {

        context.setParameter(ApplicationContext.DIRECTORY, ABS_PATH);

        context.setParameter(ApplicationContext.ABSOLUTE, "true");

        File testDir = context.getStoreDirectory();

        File expected = new File(ABS_PATH);

        assertTrue(testDir.equals(expected));
    }

    /**
     *  Get the store directory.
     *  @since  Version 0.1 2005-10-18
     */
    public final void testGetStoreDirectoryAbsoluteNoDirectory() {

        context.setParameter(ApplicationContext.ABSOLUTE, "true");

        File testDir = context.getStoreDirectory();

        assertTrue("Store directory should equal '" +
                   defaultDir.getAbsolutePath() +
                   "', actual path was '" +
                   testDir.getAbsolutePath() +
                   "'.",
                   testDir.equals(defaultDir));
    }


    /**
     *  Get the store path.
     *  @since  Version 0.1 2005-10-18
     */
    public final void testGetStorePathIsNull() {

        String testPath = context.getStorePath();

        assertNull(testPath);
    }

    /**
     *  Get the store path.
     *  @since  Version 0.1 2005-10-18
     */
    public final void testGetStorePath() {

        String repoName = "test.tmp";

        context.setParameter(ApplicationContext.REPOSITORY, repoName);

        String testPath = context.getStorePath();

        File expectedFile = new File(OUT_PATH +
                                     File.separator +
                                     repoName);

        String expected = expectedFile.getAbsolutePath();

        assertTrue("Store path should equal '" +
                   expected + "', actual path was '" +
                   testPath + "'.",
                   testPath.equals(expected));
    }

    /**
     *  Tear down a test scenario as required.
     *  @since  Version 0.1 2005-10-18
    protected void tearDown() {

    }
     */

    /**
     *  Create a test suite for this test case.
     *  @return A test suite.
     *  @since  Version 0.1 2005-10-18
     */
    public static Test suite() {

        return new TestSuite(AbstractApplicationContextTest.class);
    }

    /**
     *  Run this test suite as a stand alone application.
     *  @param  args    Any command line arguments to pass
     *          to the test. None required or expected.
     *  @since  Version 0.1 2005-10-18
     */
    public static void main(final String[] args) {

        junit.textui.TestRunner.run(suite());
    }

}
